/*****************************************************************/

/*  Copyright (c) 2010-2011 by SAS Institute Inc, Cary, NC, USA. */

/*             All Rights Reserved.                              */

/*  This software is protected by copyright laws and             */

/*               international treaties                          */

/*                                                               */

/*              U.S. Government Restricted Rights                */

/* Use, duplication, or disclosure of this software and related  */

/*  documentation by the U.S. government is subject to the       */

/*   Agreement with SAS Institute and the restrictions set forth */

/*  in FAR 52.227-19, Commercial Computer Software -             */

/*              Restricted Rights ( June 1987 )                  */

/*                                                               */

/*****************************************************************/

/* NAME: mamoch                                                  */

/* VERSION: 5.4                                                  */

/* DESCRIPTION: MAMO Contact History                             */

/*                                                               */

/* PRODUCT: MA                                                   */

/* USAGE: %mamoch( );                                            */

/*****************************************************************/

/*****************************************************************************************/

/*History:                                                                               */

/* DATE       Defect No  Comments                                                        */

/* 05Apr2010  S0736824   Syntax error due to using dbtmplib directly instead of length   */

/*****************************************************************************************/

%macro mamoch(DEBUGFILE, SubjectKeys, SasFormats, WORKLIB, MOMALIBREF, INTABLE, ConnectStmt,

              CHLIBREF, SUBJECT_CH_TABLE, UNIQUE_KEY_COLUMN, COMMUNICATION_TABLE, CONTACT_RECORD_TABLE );

    %global MADebug MAError;



    %madmpdbg(&DEBUGFILE,Dumping Debug Log SYSCC eq &SYSCC);



    %if %length(&SUBJECT_CH_TABLE) = 0 %then %do;

       %madmpdbg(&DEBUGFILE,No Contact History Table passed.);

       %goto gt_createEmptyTables;

    %end;



    /* pre processing */

    %let SUBJ_KEY_COMMA=;

    data _null_;

       call symput('SUBJ_KEY_COMMA',prxchange('s/\s+/, t0./i', -1, "t0.%trim(%superq(SUBJECTKEYS))"));

    run;

    %madmpdbg(&DEBUGFILE,SUBJ_KEY_COMMA eq %superq(SUBJ_KEY_COMMA));



    %let dbg_ts=; /* debug timestamp */

    %let hasUDF = %sysfunc(exist(UDF_TABLES));





    %let eVar=0;

    proc sql noprint;

       create table SUBJECT_TABLE as

       select distinct &SUBJ_KEY_COMMA from &MOMALIBREF..&INTABLE t0;

    quit;

    %maCheckReturn(eVar,Failed to create SUBJECT_TABLE,1160,&mName);

    %if &eVar %then %return;



    %let tLib=&WORKLIB;

    %let tName=SUBJECT_TABLE;



    %if &DOUPLOAD %then %do;

       %maTimestamp(dbg_ts);

       %maupload(&libStmt, &tName, &SubjectKeys, UploadTable, &DataBase, &ConnectStmt, &UploadTableNames, &SasFormats, LibRef=&tLib);

       %maDiffTimestamp(dbg_ts,dbg_ts);

       %maCheckDbtmplib(&UploadTableNames,tLib,tName);

       %madmpdbg(&DEBUGFILE,Upload subject table - resulting libref was &tLib: &dbg_ts);

    %end;





    /* create the join condition and store it in 'joinCondition' */

    %let joinCondition=;

    %maMakeJoin(t0,t1,&SubjectKeys,joinCondition);



    %maTimestamp(dbg_ts);

    proc sql noprint;

      create table &WORKLIB..TARGET_CONTACTS as

      select CELL_PACKAGE_SK, &SUBJ_KEY_COMMA, datepart(CONTACT_DTTM) as CONTACT_DTTM FORMAT=DATE9.

      from &tLib..&tName t0 inner join &CHLIBREF..&SUBJECT_CH_TABLE t1 on (&joinCondition)

      where t1.CONTACT_DTTM >= &TARGET_DATE;

    quit;

    %maCheckReturn(eVar,Failed to create &WORKLIB..TARGET_CONTACTS,1160,&mName);

    %if &eVar ne 1 %then %do;

       %maDiffTimestamp(dbg_ts,dbg_ts);

       %madmpdbg(&DEBUGFILE,Created TARGET_CONTACTS: &dbg_ts);





       /* Set up the UDF entries */

       %maTimestamp(dbg_ts);

       /* use SAS to limit it to the last occurrence */

       proc sql noprint;

       create table &WORKLIB..BASIC_COMMUNICATION as

          select distinct

          t1.CELL_PACKAGE_SK as CELL_PACKAGE_SK FORMAT=BEST8.,

          t2.communication_sk as COMMUNICATION_SK FORMAT=BEST12.,

          t2.campaign_cd as campaign_cd FORMAT=$32.,

          t2.communication_cd as communication_cd FORMAT=$32.,

          t2.marketing_cell_cd as cell_cd label='' FORMAT=$32.,

          t2.channel_cd as fixed_channel_code label='' FORMAT=$32.,

          t4.channel_nm as channel_cd label='' FORMAT=$32.,

          datepart(t1.contact_dttm) as occurrence_export_date FORMAT=DATE9.

          from

            &tLib..&tName t0 inner join &CHLIBREF..&SUBJECT_CH_TABLE t1 on (&joinCondition)

                             inner join &CHLIBREF..CI_CELL_PACKAGE t2 on (t1.CELL_PACKAGE_SK=t2.CELL_PACKAGE_SK)

                             inner join &CHLIBREF..CI_CHANNEL t4 on (t2.CHANNEL_CD=t4.CHANNEL_CD)

          where t1.CONTACT_DTTM >= &TARGET_DATE and

                not(t1.CONTACT_HISTORY_STATUS_CD in ('&CHSTAT_EXCLUDE','&CHSTAT_FAILED'))

       ;

       quit;

       %maCheckReturn(eVar,Failed to create &WORKLIB..BASIC_COMMUNICATION,1160,&mName);

    %end;



    %if &eVar ne 1 %then %do;

       %maDiffTimestamp(dbg_ts,dbg_ts);

       %madmpdbg(&DEBUGFILE,Created BASIC_COMMUNICATION: &dbg_ts);



       /* order by CCC, processed export date, cell_package_sk (which is based on occurrence!) */

       %maTimestamp(dbg_ts);

       proc sort data=&WORKLIB..BASIC_COMMUNICATION;

          by campaign_cd communication_cd cell_cd occurrence_export_date cell_package_sk;

       run;

       %maCheckReturn(eVar,Failed to create &WORKLIB..BASIC_COMMUNICATION,1160,&mName);

    %end;



    %if &eVar ne 1 %then %do;

       /* only use the *last* of each occurrence on a given day */

       data &WORKLIB..BASIC_COMMUNICATION;

          set &WORKLIB..BASIC_COMMUNICATION;

          by campaign_cd communication_cd cell_cd occurrence_export_date;

          if last .occurrence_export_date then output;

       run;

       %maCheckReturn(eVar,Failed to update &WORKLIB..BASIC_COMMUNICATION,1160,&mName);

    %end;



    %if &eVar ne 1 %then %do;

       /* re-order by COMMUNICATION_SK so that we can merge later */

       proc sort data=&WORKLIB..BASIC_COMMUNICATION;

          by COMMUNICATION_SK;

       run;

       %maCheckReturn(eVar,Failed to sort &WORKLIB..BASIC_COMMUNICATION,1160,&mName);

       %if &eVar %then %return;





       %maDiffTimestamp(dbg_ts,dbg_ts);

       %madmpdbg(&DEBUGFILE,Dedupped BASIC_COMMUNICATION: &dbg_ts);

    %end;



    %if &DOUPLOAD %then %do;

       %maupload(&libStmt, SUBJECT_TABLE, &SubjectKeys, DropTable, &DataBase, &ConnectStmt, &UploadTableNames, &SasFormats, LibRef=WORK);

       %madmpdbg(&DEBUGFILE,Dropped uploaded SUBJECT_TABLE);

    %end;



    /* drop the table and *then* abort so that we do not leave tables

       uploaded to the remote database. */

    %if &eVar %then %return;



    /* create an uploadable table to speed up UDFs */



    proc sql noprint;

       create view &WORKLIB..UNIQUE_COMMUNICATION_KEY as

       select distinct COMMUNICATION_SK from &WORKLIB..BASIC_COMMUNICATION;

    quit;

    %maCheckReturn(eVar,Failed to create &WORKLIB..UNIQUE_COMMUNICATION_KEY,1160,&mName);

    %if &eVar %then %return;



    /* upload the table we just made */

    %let tLib=&WORKLIB;

    %let tName=UNIQUE_COMMUNICATION_KEY;



    %if &DOUPLOAD %then %do;

       %maTimestamp(dbg_ts);

       %maupload(&libStmt, &tName, COMMUNICATION_SK, UploadTable, &DataBase, &ConnectStmt, &UploadTableNames, BEST12., LibRef=&tLib);

       %maDiffTimestamp(dbg_ts,dbg_ts);

       %maCheckDbtmplib(&UploadTableNames,tLib,tName);

       %madmpdbg(&DEBUGFILE,Upload UNIQUE_COMMUNICATION_KEY - resulting libref was &tLib: &dbg_ts );

    %end;



    %maTimestamp(dbg_ts);

    proc sql noprint;

       create table &WORKLIB..BASIC_COMM_EXTRA as

       select

          t0.communication_sk as COMMUNICATION_SK FORMAT=BEST8.,

          t3.communication_desc as communication_desc FORMAT=$256.,

          t3.communication_occurrence_no FORMAT=BEST8.,

          t3.budget_unit_cost_amt as comm_unit_cost LABEL='' FORMAT=BEST8.,

          ' ' as time_period FORMAT=$32.,

          t3.budget_unit_usage_amt as unit_usage label='' FORMAT=BEST8.,

          datepart(t3.start_dttm) as start_date FORMAT=DATE9.,

          datepart(t3.end_dttm) as end_date FORMAT=DATE9.

       from

          &tLib..&tName t0 inner join &CHLIBREF..CI_COMMUNICATION t3 on (t0.COMMUNICATION_SK=t3.COMMUNICATION_SK)

       order by COMMUNICATION_SK

    ;

    quit;

    %maCheckReturn(eVar,Failed to create &WORKLIB..BASIC_COMM_EXTRA,1160,&mName);



    %if &eVar ne 1 %then %do;

       %maDiffTimestamp(dbg_ts,dbg_ts);

       %madmpdbg(&DEBUGFILE,Created BASIC_COMM_EXTRA: &dbg_ts);



       %maTimestamp(dbg_ts);

       data &WORKLIB..BASIC_COMMUNICATION;

          merge &WORKLIB..BASIC_COMMUNICATION &WORKLIB..BASIC_COMM_EXTRA;

          by COMMUNICATION_SK;

       run;

       %maCheckReturn(eVar,Failed to merge &WORKLIB..BASIC_COMMUNICATION,1160,&mName);

    %end;



    %if &eVar ne 1 and &hasUDF %then %do;

       %maDiffTimestamp(dbg_ts,dbg_ts);

       %madmpdbg(&DEBUGFILE,Merged BASIC_COMMUNICATION and BASIC_COMM_EXTRA: &dbg_ts);



       filename chquery temp lrecl=32767;

       filename tincl temp lrecl=32767;

       data _null_;

          file chquery;

          put;

       run;



       /* put the columns in a nice order by type */

       data _null_;

          file tincl;

          set UDF_TABLES;

          length command $ 4000;

          if exist(inputTableName) then do;

             command="proc sort data="||trim(inputTableName)||" SORTSEQ=LINGUISTIC(NUMERIC_COLLATION=ON); by outColName; run;";

             put command;

          end;

       run;

       %include tincl;



       /* &WORKLIB..&COMMUNICATION_TABLE; */

       data _null_;

          file tincl;

          set UDF_TABLES;

          length command $ 4000;

          if exist(inputTableName) then do;

             command="%"||"maGenerateUDFTables(chquery,&tLib..&tName,&CHLIBREF,"||trim(inputTableName)||","||trim(nameCol)||","||

                          trim(valCol)||",%"||"str("||trim(fmtCol)||"));";

             put command;

          end;

       run;

       %maTimestamp(dbg_ts);

       %include tincl;

       %include chquery;

       %maCheckReturn(eVar,Failed to generate custom detail tables,1160,&mName);

    %end;



    %if &eVar ne 1 %then %do;

       %maDiffTimestamp(dbg_ts,dbg_ts);

       %madmpdbg(&DEBUGFILE,Built script and grabbed custom details: &dbg_ts);

    %end;



    filename tincl clear;

    filename chquery clear;





    %if &DOUPLOAD %then %do;

       %maupload(&libStmt, UNIQUE_COMMUNICATION_KEY, COMMUNICATION_SK, DropTable, &DataBase, &ConnectStmt, &UploadTableNames, BEST12.);

       %madmpdbg(&DEBUGFILE,Dropped uploaded UNIQUE_COMMUNICATION_KEY table);

    %end;



    /* drop the table and *then* abort so that we do not leave tables

       uploaded to the remote database. */

    %if &eVar %then %return;



    filename tincl temp lrecl=32767;

    filename chquery temp lrecl=32767;



    proc sort data=&WORKLIB..BASIC_COMMUNICATION; by COMMUNICATION_SK; run;



    /* now merge all of the tables into the target table */

    data _null_; file tincl; put; run;

    data _null_; file chquery; put; run;



    %maTimestamp(dbg_ts);

    %if &hasUDF %then %do;

       data _null_;

          file tincl;

          set UDF_TABLES;

          length command $ 4000;

          length masterTable $ 120;

          retain masterTable "&WORKLIB..BASIC_COMMUNICATION";

          if exist(inputTableName) then do;

             *maMergeTables(fName,TargetTable,DestinationTable,MasterTable);

             command="%"||"maMergeTables(chquery,"||trim(inputTableName)||",&WORKLIB..&COMMUNICATION_TABLE,"||trim(masterTable)||");";

             masterTable="&WORKLIB..&COMMUNICATION_TABLE";

             put command;

          end;

       run;

    %end; %else %do;

       * No UDFs so just copy the basic table directly to the worklib representation.;

       data &WORKLIB..&COMMUNICATION_TABLE;

          set &WORKLIB..BASIC_COMMUNICATION;

       run;

    %end;



    %include tincl;

    %include chquery;

    %maCheckReturn(eVar,Failed to merge custom detail tables,1160,&mName);



    filename tincl clear;

    filename chquery clear;



    /* clear filenames before returning */

    %if &eVar %then %return;

    %maDiffTimestamp(dbg_ts,dbg_ts);

    %madmpdbg(&DEBUGFILE,Merged custom details: &dbg_ts);



    /* for now, assuming that only one column is coming in */

    %maTimestamp(dbg_ts);

    data &WORKLIB..&COMMUNICATION_TABLE;

       format &UNIQUE_KEY_COLUMN 11.;

       %if &EXNCOLS ne

       %then format &EXNCOLS 11.;;



       set &WORKLIB..&COMMUNICATION_TABLE(drop=COMMUNICATION_SK);

       &UNIQUE_KEY_COLUMN=_n_;

    run;



    %maDiffTimestamp(dbg_ts,dbg_ts);

    %madmpdbg(&DEBUGFILE,Generated UNIQUE_KEY_COLUMN: &dbg_ts);



    %maTimestamp(dbg_ts);

    proc sql noprint;

       create table &MOMALIBREF..&CONTACT_RECORD_TABLE as

       select distinct

          &SUBJ_KEY_COMMA, &UNIQUE_KEY_COLUMN

       from

          &WORKLIB..TARGET_CONTACTS t0 inner join &WORKLIB..&COMMUNICATION_TABLE t1 on (t0.CELL_PACKAGE_SK=t1.CELL_PACKAGE_SK)

    ;

    quit;

    %maCheckReturn(eVar,Failed to create &MOMALIBREF..&CONTACT_RECORD_TABLE,1160,&mName);

    %if &eVar %then %return;



    %maDiffTimestamp(dbg_ts,dbg_ts);

    %madmpdbg(&DEBUGFILE,Created &MOMALIBREF..&CONTACT_RECORD_TABLE: &dbg_ts);



    %maTimestamp(dbg_ts);

    %maEnsureSameColumns(&UNIQUE_KEY_COLUMN,11.,&MOMALIBREF..COMMUNICATION,&WORKLIB..&COMMUNICATION_TABLE);

    %maCheckReturn(eVar,Failed to rectify missing columns,1160,&mName);

    %if &eVar %then %return;

    %maDiffTimestamp(dbg_ts,dbg_ts);

    %madmpdbg(&DEBUGFILE,Rectified missing columns: &dbg_ts);



    %maTimestamp(dbg_ts);

    data &MOMALIBREF..&COMMUNICATION_TABLE;

       set &WORKLIB..&COMMUNICATION_TABLE(drop=CELL_PACKAGE_SK);

    run;

    %maCheckReturn(eVar,Failed to create &MOMALIBREF..&COMMUNICATION_TABLE,1160,&mName);

    %if &eVar %then %return;



    %maDiffTimestamp(dbg_ts,dbg_ts);

    %madmpdbg(&DEBUGFILE,Moved comm historic data to &MOMALIBREF..&COMMUNICATION_TABLE: &dbg_ts);



    %return;



%gt_createEmptyTables:

    %madmpdbg(&DEBUGFILE,Creating empty tables.);



    data &MOMALIBREF..&COMMUNICATION_TABLE;

       format &UNIQUE_KEY_COLUMN BEST8.;

       set &MOMALIBREF..communication;

       &UNIQUE_KEY_COLUMN=0;

       stop;

    run;



    data &MOMALIBREF..&CONTACT_RECORD_TABLE;

       format &UNIQUE_KEY_COLUMN BEST8.;

       set &MOMALIBREF..&INTABLE(keep=&SubjectKeys);

       &UNIQUE_KEY_COLUMN=0;

       stop;

    run;

%mend;



%macro maTimestamp(outvarname);

   %if &MADebug ne Y %then %return;

   %local nom non;

   %if %index(%sysfunc(getoption(notes)),NON) le 0 %then %let non=1; %else %let non=0;

   %if %index(%sysfunc(getoption(mprint)),NO) le 0 %then %let nom=1; %else %let nom=0;

   options nomprint nonotes;

   data _null_;

      rv = datetime();

      call symput("&outvarname", rv);

   run;



   %if &non %then options notes;

   %if &nom %then options mprint;

%mend;



%macro maDiffTimestamp(invarname, outvarname);

   %if &MADebug ne Y %then %return;

   %local nom non;

   %if %index(%sysfunc(getoption(notes)),NON) le 0 %then %let non=1; %else %let non=0;

   %if %index(%sysfunc(getoption(mprint)),NO) le 0 %then %let nom=1; %else %let nom=0;



   options nomprint nonotes;

   data _null_;

      length outvar $ 20;

      invarval = symget("&invarname");

      tv = datetime();

      if ( tv >= invarval ) then do;

          diffvarval = tv - invarval;

      end;

      else do;

          diffvarval= 0.00;

      end;

      outvar=trim(left(put(diffvarval,F10.2)))||" s";

      call symput("&outvarname", outvar);

   run;

   %if &non %then options notes;

   %if &nom %then options mprint;

%mend;



%macro maMakeJoin( t0, t1, columns, targetVar );

   %let join=;

   %let i = 1;

   %let col = %scan(&columns,&i,%str( ));

   %do %while( &col ne );

      %if &i=1 %then %let join=%str(&t0..&col=&t1..&col);

      %else %let join=%str(%trim(&join) and &t0..&col=&t1..&col);



      %let i = %eval(&i+1);

      %let col = %scan(&columns,&i,%str( ));

   %end;

   data _null_; call symput("&targetVar","&join"); run;

%mend;



%macro maGenerateUDFTables(fName,SourceTable,TargetLib,TargetTable,NameCol,ValCol,Fmt);

data _null_;

   file &fName mod;

   length sel $ 4000;

   set &TargetTable;



   udfName = tranwrd(trim(udfName),"'","''");

   sel="proc sql noprint; create table "||trim(outColName)||" as "||

       "select t2.COMMUNICATION_SK, t2.&ValCol as "||trim(outColName)||" label='' format=%trim(&Fmt)"||

       " from &SourceTable t1 inner join &TargetLib..&TargetTable t2 on t1.COMMUNICATION_SK=t2.COMMUNICATION_SK"||

       " where upcase(t2.&NameCol)=upcase('"||trim(udfName)||"') order by 1;";

   put sel;

run;

%mend;



%macro maMergeTables(fName,TargetTable,DestinationTable,MasterTable);

data _null_;

   file &fName mod;

   set &TargetTable end=done;

   if ( _n_ = 1 ) then do;

      put "data &DestinationTable; merge &MasterTable ";

   end;



   put outColName;



   if done then do;

      put "; by communication_sk; run;";

   end;

run;

%mend;



/* if dbtmplib was set, put the library "dbtmplib" and passed upload table name

   into the passed variables (outLib, outTab) */

%macro maCheckDbtmpLib(uplTableName, outLib,outTab);

%let SYSCC=0; /* don't let errors in uploading propagate */

%let dbtmplib=;

data _null_;

   set dbtmplib;

   put libref=;

   call symput('dbtmplib',libref);

   stop;

run;

%if %length(&dbtmplib) > 0 %then %do;

  &dbtmplib;

  %madmpdbg(&DEBUGFILE,Upload Successful);

  %let &outLib=dbtmplib;

  %let &outTab=&uplTableName;

%end; %else %madmpdbg(&DEBUGFILE,Upload Failed);

%mend;



/* check the SYSCC and return codes, if there's a problem, set eVar to 1,

   set MAMsg to the text Macro[&sysmacroname] &eMsg.  SYSCC=&SYSCC, and MAError to eCode */

%macro maCheckReturn(inEVar,eMsg,eCode,macroName);

   %global MAError MAMsg;

   %if &SYSCC ne 0 and &SYSCC ne 4 %then %do;

      %let &inEVar=1;

      %let MAError=&eCode;

      %let MAMsg=Macro[&macroName.] &eMsg.   SYSCC=&SYSCC;

      %madmpdbg(&DEBUGFILE,%str(MAError=&MAError MAMsg=&MAMsg));

   %end;

%mend;



/*

  Take the fully qualified sourceTable and destTable, see if any columns

  from sourceTable are not in destTable.  If there are missing columns, add

  them directly to destTable.  destTable must already be sorted by uniqueKey.

*/

%macro maEnsureSameColumns(uniqueKey,uniKeyFmt,sourceTable,destTable);

proc contents data=&sourceTable out=_content_st noprint;

proc contents data=&destTable out=_content_dt noprint;

%local vars;

%let vars=;

proc sql noprint;

   select name into :vars separated by ' '

   from _content_st

   where upcase(name) not in (select upcase(name) from _content_dt)

;

quit;

%if %sysfunc(countw(&vars,' ')) gt 0 %then %do;

   data _additional_columns;

      format &uniqueKey &uniKeyFmt;

      set &sourceTable(keep=&vars);

      stop;

   run;

   data &destTable;

      merge &destTable _additional_columns; by &uniqueKey;

   run;

%end;

%mend;

