/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,    */
/*                    USA.                                       */
/*             All Rights Reserved.                              */
/*  This software is protected by copyright laws and             */
/*               international treaties                          */
/*                                                               */
/*              U.S. Government Restricted Rights                */
/* Use, duplication, or disclosure of this software and related  */
/*  documentation by the U.S. government is subject to the       */
/*   Agreement with SAS Institute and the restrictions set forth */
/*  in FAR 52.227-19, Commercial Computer Software -             */
/*              Restricted Rights ( June 1987 )                  */
/*                                                               */
/*****************************************************************/

/*****************************************************************/
/* NAME: maprnobs.sas                                            */
/* VERSION: 5.4                                                  */
/* DESCRIPTION: Common Data Interface macro to read BC and       */
/*              common model metadata from input stream/Payload  */
/*              URI to create the master table                   */
/*                                                               */
/* Parameters                                                    */
/* Macro variables:                                              */
/*     action - USTADTTM/UPOPDTTM                                */
/*     badrsp - Y/N                                              */
/*     expimplflag - E/I to specify implict or explicity         */
/*     fl_loc - Location for the CSV file                        */
/*     srctbl - CH/RH/PT CI_Intable or dm_meta_tbl               */
/*     srclib - dbtmplib                                         */
/*     sub_key - subject key list                                */
/*     stpcall - CH/RH/PT                                        */
/*     tgttbl - target table name                                */
/*     tgtlib  - target library name                             */
/*     updatemode - B/T Bulk or Trickling                        */
/*     xmlflag - Y/N to indicate whether to use XML Stream       */
/*                                                               */
/*                                                               */
/* PRODUCT: MA                                                   */
/* USAGE: %maprnobs                                              */
/*                                                               */
/*****************************************************************/

/*****************************************************************************************/
/*History:                                                                               */
/*dd-mon-yy                                                                              */
/* 14Sep2007 - First Version of the Code                                                 */
/*                                                                                       */
/* DATE      Defect No  Comments                                                         */
/* 04Sep2008 S0530203   Fix to handle the CDTTM correctly for SPDS                       */
/*                                                                                       */
/* 17Sep2008 S0521156   Date-time values for SASSPDS,ODBC connection needs to be handled */
/*                      separately, so that the datetime number value is passed to the   */
/*                      SQL processing                                                   */
/*                                                                                       */
/* 12Jan2009 S0472135   Changes to support the call from implicit pass-through macros    */
/*                      (macdichi/macdipti/macdirhi) inserts the records correct         */
/*                                                                                       */
/* 16Jan2008 S0472135   Code added to remove the temp tables in case of any error        */
/*                                                                                       */
/* 11Mar2009 S0567809   New _hash_val columns included for PT and RH inserts             */
/*                                                                                       */
/* 02Jul2009  S0591527  Keep the bad records file name as the stp call+date in case the  */
/*                      target table name metadata is not passed to this macro           */
/*                                                                                       */
/* 15Dec2010  S0715116  New External_ columns added to CH and PT table                   */
/*                      and CONTACT_DT on CH table                                       */
/*                                                                                       */
/* 09Dec2011 S0791301  create / refer all the table/column names in upper case           */
/*****************************************************************************************/

%macro maprnobs(action=, badrsp=, expimplflag=, fl_loc=, srclib=, srctbl=,stpcall=,sub_key=,
               tgtlib=,tgttbl=,updatemode=, xmlflag=);

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put Current Version of the code: 541MA02;
    %put Last Modified on 09Dec2011;
%end;

/* Use Case 1: SDM Updates failed on CH */

%let prnsrc=PrntSrc;
%let tgtfl=Tgtfl;

/* File reference for dynamic source code */
filename &prnsrc. catalog "work.CICDI.Retryfailures.Source";

/*File reference for target csv file */
data _null_;
    dt=date();
    call symput('fpath',pathname("&fl_loc."));
    call symput('dat',put(dt,date9.));
run;

%if &badrsp. eq Y %then
%do;
    filename &tgtfl. "&fpath./&stpcall._RESPTRACKING_LIST_&dat..csv";
%end;

/* S0591527 If target table name is missing use the STPCALL as the name */
%if &tgttbl. ne %str() %then
%do;
    filename &tgtfl. "&fpath./&tgttbl._&dat..csv";
%end;
%else
%do;
    filename &tgtfl. "&fpath./&stpcall._&dat..csv";
%end;

%if %sysfunc(fexist(&tgtfl.)) %then
%do;
    %let flpresent=1;
%end;
%else
%do;
    %let flpresent=0;
%end;

/* Construct the dynamic source file */
/* Use Case 1: Bad Response Tracking Code */
%if &badrsp. eq Y %then
%do;
    proc contents data=&srctbl. out=contents(keep=name) noprint;
    run;

    data _null_;
        set contents nobs=obscnt;
            call symput('obscnt',left(obscnt));
            call symput('var'||left(_n_),left(name) );
    run;

    %do i=1 %to &obscnt;
        %if &i=1 %then
        %do;
            %let title=&&var&i.;
            %let varlist=&&var&i.;
        %end;
        %else
        %do;
            %let title=&title. , &&var&i.;
            %let varlist=&varlist. ',' &&var&i.;
        %end;

    %end;

    data _null_;
        file &tgtfl. MOD lrecl=32767;
        set &srctbl.;
        if _n_=1 and &flpresent.= 0 then
        do;
            put "&title. ";
        end;

        put &varlist.;
    run;

%end;

%else %if &badrsp. eq N %then
%do;
    /* Create the Source and Target table name upfront to avoid the 2 dots issue
       if the input table is from work library */

    %if %upcase(&expimplflag.) eq E %then
    %do;
        %let srctbl=&srclib..&srctbl.;
        %let tgttbl=&tgtlib..&tgttbl.;
    %end;
    %else %if %upcase(&expimplflag.) eq I %then
    %do; /* Since the &srctbl already has the value <libnm>.<tablenm>, the work reference is removed from srctbl */
        %let srctbl=&srctbl.;
        %let tgttbl=&tgtlib..&tgttbl.;
    %end;

    /* Use case 2: SDM Update failures */
    %if (&action. eq USTADTTM and &updatemode. eq B) and &stpcall eq CH %then
    %do;
        %let ch_col_lst=%str(CELL_PACKAGE_SK, ) &sub_key. %str(, CONTACT_HISTORY_STATUS_CD,CONTACT_DTTM,CONTACT_DT);

        /* Create recordnum and cdttm storing integer value.  */
        data &srctbl._err;
            length RECORDNUM CDTTM_NUM CTDATE_NUM 8.;
            set &srctbl.;
            /* S0530203 Fix to handle the cdttm for SPDS is removed as the fix
               for defect S0521156 takes care of this issue */
                CDTTM_DT=scan(CDTTM,1,'');
                CDTTM_TM=scan(CDTTM,2,'');
                CDTTM_CHAR=put(input(CDTTM_DT,yymmdd10.),date9.)||':'||put(input(CDTTM_TM,time12.3),tod12.3);
                CDTTM_NUM=input(CDTTM_CHAR,datetime24.3);
                CTDATE_NUM=input(CTDATE,yymmdd10.); /* S0715116 - New contact_dt column in CH table */
                drop CDTTM_DT CDTTM_TM CDTTM_CHAR;
            RECORDNUM=_n_;
        run;

        /* Dynamic source code creation */
        data _null_;
            file &prnsrc. lrecl=32767;
            set &srctbl._err end=done;
            if _n_ eq 1 then
            do;
                put '%macro print_bad_obs;';
            end;
            put ' ';
            put ' proc sql noerrorstop;';
            put ' UPDATE '"&tgttbl. ch";
            put '     SET ';
            put '         CONTACT_HISTORY_STATUS_CD=';
            put '                                    (select CHSTATUS_CD';
            put '                                       from '"&srctbl._err inptbl ";
            put '                                       where ch.CELL_PACKAGE_SK = inptbl.CELL_PACKAGE_SK AND ';
            %do i=1 %to &maxnumkey.;
                %if &i. > 1 %then
                %do;
                    put '                                       AND';
                %end;
                    put "                                       ch.&&chsubjectidcol&i. =  inptbl.&&chsubjectidcol&i.";
            %end;
            put '                                        AND inptbl.RECORDNUM=' _n_ ' ),';
            put '         CONTACT_DTTM=';
            put '                      (select CDTTM_NUM';
            put '                         from '"&srctbl._err inptbl ";
            put '                         where ch.CELL_PACKAGE_SK = inptbl.CELL_PACKAGE_SK AND ';
            %do i=1 %to &maxnumkey.;
                %if &i. > 1 %then
                %do;
                    put '                        AND';
                %end;
                    put "                        ch.&&chsubjectidcol&i. =  inptbl.&&chsubjectidcol&i.";
            %end;
            put '                                AND inptbl.RECORDNUM=' _n_ ' ),';
            put '       ';
            put '         CONTACT_DT=';
            put '                      (select CTDATE_NUM';
            put '                         from '"&srctbl._err inptbl ";
            put '                         where ch.CELL_PACKAGE_SK = inptbl.CELL_PACKAGE_SK AND ';
            %do i=1 %to &maxnumkey.;
                %if &i. > 1 %then
                %do;
                    put '                        AND';
                %end;
                    put "                        ch.&&chsubjectidcol&i. =  inptbl.&&chsubjectidcol&i.";
            %end;
            put '                                AND inptbl.RECORDNUM=' _n_ ' )';
            put '       ';
            put '         where exists( select 1 from '"&srctbl._err inptbl ";
            put '                         where ch.CELL_PACKAGE_SK = inptbl.CELL_PACKAGE_SK AND ';
            %do i=1 %to &maxnumkey.;
                %if &i. > 1 %then
                %do;
                    put '                        AND';
                %end;
                    put "                        ch.&&chsubjectidcol&i. =  inptbl.&&chsubjectidcol&i.";
            %end;
            put '                                AND inptbl.RECORDNUM=' _n_ ' );';
            put '      ';
            put ' quit;';
            put '      ';
            put '      ';
            put '%if (%symexist(SYSDBRC) and &SYSDBRC ^=0) or &sqlrc > 4 %then';
            put '%do;';
            put '    data _null_ ;';
            put '        file '"&tgtfl. " ' MOD lrecl=32767;';
            put '        if _n_=1 and  &flpresent. = 0 then';
            put '        do;';
            put '            put "' " &ch_col_lst." '";';
            put '            %let flpresent= 1;';
            put '        end;';
            put '        ';
            put '        put '"'" CELL_PACKAGE_SK ',';
            %do i=1 %to &maxnumkey.;
                %do;
                    put ' ' &&chsubjectidcol&i. ',' ;
                %end;
            %end;
            put '   ' CHSTATUS_CD ', ' CDTTM_NUM ', ' CTDATE_NUM "' ;" ;
            put '      ';
            put '    run;';
            put '%end;';
            put '     ';

            if done then
            do;
                put '      ';
                put ' %mend;';
                put ' %print_bad_obs;';
            end;
        run;
    %end; /* End statement for SDM Contact History update failures */
    /* Use Case 3: CH Insert from RDM Client */
    %else %if (&action. eq UPOPDTTM and &updatemode. eq T) and &stpcall eq CH %then
    %do;
        /* S0567809 Package_Hash_Val included for RDM campaigns */
        /* S0715116 Contact_DT, External_Contact_Info_ID1 and ID2 added */
        %let ch_col_lst=%str(CELL_PACKAGE_SK, ) &sub_key. %str(, CONTACT_HISTORY_STATUS_CD,CONTACT_DTTM,PACKAGE_HASH_VAL,CONTACT_DT, EXTERNAL_CONTACT_INFO_ID1, EXTERNAL_CONTACT_INFO_ID2);

        /* Create recordnum and cdttm storing integer value.  */
        data &srctbl._err;
            length RECORDNUM CDTTM_NUM CTDATE_NUM 8.;
            set &srctbl.;
            /* S0530203 Fix to handle the cdttm for SPDS is removed as the fix
               for defect S0521156 takes care of this issue */
                CDTTM_DT=scan(CDTTM,1,'');
                CDTTM_TM=scan(CDTTM,2,'');
                CDTTM_CHAR=put(input(CDTTM_DT,yymmdd10.),date9.)||':'||put(input(CDTTM_TM,time12.3),tod12.3);
                CDTTM_NUM=input(CDTTM_CHAR,datetime24.3);
                CTDATE_NUM=input(CTDATE,yymmdd10.);
                drop CDTTM_DT CDTTM_TM CDTTM_CHAR;
            RECORDNUM=_n_;
        run;

        /* Dynamic source code creation */
        data _null_;
            file &prnsrc. lrecl=32767;
            set &srctbl._err end=done;
            if _n_ eq 1 then
            do;
                put '%macro print_bad_obs;';
            end;
            put ' ';
            put ' proc sql noerrorstop; ';
            put '   INSERT INTO '"&tgttbl."' (';
            put '       CELL_PACKAGE_SK, ';
            put "       &sub_key. , ";
            put '       CONTACT_HISTORY_STATUS_CD, ';
            put '       CONTACT_DTTM, ';
            put '       PACKAGE_HASH_VAL, ';
            put '       CONTACT_DT, ';
            put '       EXTERNAL_CONTACT_INFO_ID1, ';
            put '       EXTERNAL_CONTACT_INFO_ID2 )';
            put '     SELECT CELL_PACKAGE_SK,';
            put "            &sub_key. , ";
            put '            CHSTATUS_CD, ';
            put '            CDTTM_NUM, ';
            put '            PACKAGE_HASH_VAL, ';
            put '            CTDATE_NUM, ';
            put '            EXTERNAL_CONTACT_INFO_ID1, ';
            put '            EXTERNAL_CONTACT_INFO_ID2 ';
            put '      FROM '" &srctbl._err ";
            put '      WHERE RECORDNUM=' _n_ ' ;';
            put '    ';
            put '%if (%symexist(SYSDBRC) and &SYSDBRC ^=0) or &sqlrc > 4 %then';
            put '%do;';
            put '    data _null_ ;';
            put '        file '"&tgtfl. " ' MOD lrecl=32767;';
            put '        if _n_=1 and  &flpresent. = 0 then';
            put '        do;';
            put '            put "' " &ch_col_lst." '";';
            put '            %let flpresent= 1;';
            put '        end;';
            put '        ';
            put '        put '"'" CELL_PACKAGE_SK ',';
            %do i=1 %to &maxnumkey.;
                %do;
                    put ' ' &&chsubjectidcol&i. ',' ;
                %end;
            %end;
            put '   ' CHSTATUS_CD ', ' CDTTM_NUM ', ' PACKAGE_HASH_VAL ', ' CTDATE_NUM ', ' ;
            put '   ' EXTERNAL_CONTACT_INFO_ID1 ', ' EXTERNAL_CONTACT_INFO_ID2 "' ;" ;
            put '      ';
            put '    run;';
            put '%end;';
            put '     ';

            if done then
            do;
                put '      ';
                put ' %mend;';
                put ' %print_bad_obs;';
            end;
        run;
    %end; /* End statement for RDM Contact History Insert failures */
    /* Use Case 3: RH insert failed from SDM/RDM  */
    /* S0567809 Treatment_Hash_Val is included for SDM/RDM campaigns */
    %else %if &action. eq UPOPDTTM and &stpcall. eq RH %then
    %do;
        %let ch_col_lst=%str(CELL_PACKAGE_SK, TREATMENT_HASH_VAL) &sub_key. %str(, RESPONSE_CD, RESPONSE_DTTM, TREATMENT_SK, INFERRED_RESPONSE_FLG, RESPONSE_CHANNEL_CD, EXTERNAL_RESPONSE_INFO_ID1, EXTERNAL_RESPONSE_INFO_ID2, PROCESSED_DTTM);

        /* Create recordnum and cdttm storing integer value.  */
        data &srctbl._err;
            length RECORDNUM RDTTM_NUM PDTTM 8.;
            format PDTTM datetime24.3;
            set &srctbl.;
            /* S0530203 Fix to handle the cdttm for SPDS is removed as the fix
               for defect S0521156 takes care of this issue */
                RDTTM_DT=scan(RDTTM,1,'');
                RDTTM_TM=scan(RDTTM,2,'');
                RDTTM_CHAR=put(input(RDTTM_DT,yymmdd10.),date9.)||':'||put(input(RDTTM_TM,time12.3),tod12.3);
                RDTTM_NUM=input(RDTTM_CHAR,datetime24.3);
                drop RDTTM_DT RDTTM_TM RDTTM_CHAR;
            PDTTM=datetime();
            RECORDNUM=_n_;
        run;

        data _null_;
            file &prnsrc. lrecl=32767;
            set &srctbl._err end=done;
            if _n_ eq 1 then
            do;
                put '%macro print_bad_obs;';
            end;
            put ' ';
            put ' proc sql noerrorstop;';
            put '   INSERT INTO '"&tgttbl."' (';
            put '       CELL_PACKAGE_SK, ';
            put '       TREATMENT_HASH_VAL, ';
            put "       &sub_key. , ";
            put '       RESPONSE_CD , ';
            put '       RESPONSE_DTTM , ';
            put '       TREATMENT_SK, ';
            put '       INFERRED_RESPONSE_FLG, ';
            put '       RESPONSE_CHANNEL_CD, ';
            put '       EXTERNAL_RESPONSE_INFO_ID1, ';
            put '       EXTERNAL_RESPONSE_INFO_ID2, ';
            put '       PROCESSED_DTTM )';
            put '    SELECT ';
            put '       CELL_PACKAGE_SK ,';
            put '       TREATMENT_HASH_VAL, ';
            put "       &sub_key. , ";
            put '       RESPONSE_CD, ';
            put '       RDTTM_NUM, ';
            put '       TREATMENT_SK, ';
            put '       INFERRED_RESPONSE_FLG, ';
            put '       RESPONSE_CHANNEL_CD, ';
            put '       EXTERNAL_RESPONSE_INFO_ID1, ';
            put '       EXTERNAL_RESPONSE_INFO_ID2, ';
            put '       PDTTM ';
            put '    FROM '" &srctbl._err ";
            put '    WHERE RECORDNUM=' _n_ ' ;';
            put '        ';
            put '  quit; ';
            put '       ';
            put '       ';
            put '%if (%symexist(SYSDBRC) and &SYSDBRC ^=0) or &sqlrc > 4 %then';
            put '%do;';
            put '    data _null_ ;';
            put '        file '"&tgtfl. " ' MOD  lrecl=32767;';
            put '        if _n_=1 and  &flpresent.= 0 then';
            put '        do;';
            put '            put "' " &ch_col_lst." ' ";';
            put '            %let flpresent= 1;';
            put '        end;';
            put '        ';
            put '        put '"'" CELL_PACKAGE_SK ', ' TREATMENT_HASH_VAL ', ';
            %do i=1 %to &maxnumkey.;
                %do;
                    put '   ' &&rhsubjectidcol&i. ', ';
                %end;
            %end;
            put '      ' RESPONSE_CD ', ' RDTTM_NUM ', ' TREATMENT_Sk ', ' INFERRED_RESPONSE_FLG ', ' RESPONSE_CHANNEL_CD ', ';
            put '      ' EXTERNAL_RESPONSE_INFO_ID1 ', ' EXTERNAL_RESPONSE_INFO_ID2 ', '  PDTTM "';";
            put ' run;';
            put '%end;';
            put '     ';

            if done then
            do;
                put '      ';
                put ' %mend; ';
                put ' %print_bad_obs; ';
            end;
        run;
    %end; /* End statement for Response History Insert failures */
    /* Use Case 4: RDM Insert failed on PT */
    /* S0567809 Treatment_Hash_Val is included for RDM campaigns */
    %else %if &action. eq UPOPDTTM and &stpcall. eq PT %then
    %do;
        %let ch_col_lst=%str(CELL_PACKAGE_SK, TREATMENT_HASH_VAL) &sub_key. %str(, TREATMENT_SK, PRESENTED_TREATMENT_HIST_DTTM, EXTERNAL_PRESENTED_INFO_ID1, EXTERNAL_PRESENTED_INFO_ID2);

        /* Create recordnum and cdttm storing integer value.  */
        data &srctbl._err;
            length RECORDNUM PTDTTM_NUM 8.;
            set &srctbl.;
            /* S0530203 Fix to handle the cdttm for SPDS is removed as the fix
               for defect S0521156 takes care of this issue */
                PTDTTM_DT=scan(PTDTTM,1,'');
                PTDTTM_TM=scan(PTDTTM,2,'');
                PTDTTM_CHAR=put(input(PTDTTM_DT,yymmdd10.),date9.)||':'||put(input(PTDTTM_TM,time12.3),tod12.3);
                PTDTTM_NUM=input(PTDTTM_CHAR,datetime24.3);
                drop PTDTTM_DT PTDTTM_TM PTDTTM_CHAR;
            RECORDNUM=_n_;
        run;

        data _null_;
            file &prnsrc. lrecl=32767;
            set &srctbl._err end=done;
            if _n_ eq 1 then
            do;
                put '%macro print_bad_obs;';
            end;
            put ' ';
            put ' proc sql noerrorstop;';
            put '   INSERT INTO '" &tgttbl."' (';
            put '       CELL_PACKAGE_SK, ';
            put '       TREATMENT_HASH_VAL, ';
            put "         &&sub_key. , ";
            put '       TREATMENT_SK, ';
            put '       PRESENTED_TREATMENT_HIST_DTTM,';
            put '       EXTERNAL_PRESENTED_INFO_ID1, ';
            put '       EXTERNAL_PRESENTED_INFO_ID2 ) ';
            put '    SELECT  ';
            put '       CELL_PACKAGE_SK, ';
            put '       TREATMENT_HASH_VAL, ';
            put "       &sub_key. , ";
            put '       TREATMENT_SK, ';
            put '       PTDTTM_NUM,';
            put '       EXTERNAL_PRESENTED_INFO_ID1, ';
            put '       EXTERNAL_PRESENTED_INFO_ID2 ';
            put '    FROM '" &srctbl._err ";
            put '    WHERE RECORDNUM=' _n_ ' ;';
            put '        ';
            put '  quit; ';
            put '       ';
            put '       ';
            put '%if (%symexist(SYSDBRC) and &SYSDBRC ^=0) or &sqlrc > 4 %then';
            put '%do;';
            put '    data _null_ ;';
            put '        file '"&tgtfl. " ' MOD lrecl=32767;';
            put '        if _n_=1 and &flpresent. = 0 then';
            put '        do;';
            put '            put "' " &ch_col_lst." ' ";';
            put '            %let flpresent= 1;';
            put '        end;';
            put '        ';
            put '        put '"'" CELL_PACKAGE_SK ', ' TREATMENT_HASH_VAL ', ';
            %do i=1 %to &maxnumkey.;
                %if &i. = &maxnumkey.%then
                %do;
                    put '  ' &&ptsubjectidcol&i. ', ' ;
                %end;
            %end;
            put '      ' TREATMENT_Sk ', ' PTDTTM_NUM ', ' EXTERNAL_PRESENTED_INFO_ID1 ', ' EXTERNAL_PRESENTED_INFO_ID2 "' ;" ;
            put ' run;';
            put '%end;';
            put '     ';

            if done then
            do;
                put '      ';
                put ' %mend; ';
                put ' %print_bad_obs; ';
            end;
        run;
    %end; /* End statement for Presented Treatment insert failures */

%include &prnsrc.;

/* Drop the newly created table *_err */
%if %sysfunc( exist( &srctbl._err ) ) %then
%do;
    proc sql;
        drop table &srctbl._err;
    quit;
%end;

%end; /* End statement for &badrsp=N  block */

%mend;
