/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/*****************************************************************************************/
/* NAME: macamgp.sas                                                                     */
/* VERSION: 5.4                                                                        */
/*                                                                                       */
/* DESCRIPTION: Macro to update the following data model tables                          */
/*              CI_CAMPAIGN_GROUP                                                        */
/*              CI_CAMPAIGN_X_CAMPAIGN_GROUP                                             */
/*                                                                                       */
/*  Parameters:                                                                          */
/*             ccg          - Physical table name for Campaign Group                     */
/*             cg_char_udf  - Physical table name for Campaign_Group_Char_UDF            */
/*             cg_num_udf   - Physical table name for Campaign_Group_Num_UDF             */
/*             cg_dt_udf    - Physical table name for Campaign_Group_Dt_UDF              */
/*                                                                                       */
/* PRODUCT: MA                                                                           */
/* USAGE: %macamgp                                                                       */
/*                                                                                       */
/*****************************************************************************************/
/*****************************************************************************************/
/*                                                                                       */
/* History:                                                                              */
/*   01Sep2008 - First version of the macro.                                             */
/*                                                                                       */
/* DATE       Defect No  Comments/updated by                                             */
/* 01Sep2008  S0506508   Changes for Campaign_Group publish                              */
/*                                                                                       */
/* 14Oct2008  S0504678   PROCESSED_DTTM included part of 5.3 data model changes          */
/*                                                                                       */
/* 5Feb2009   S0551066   a) Remove Campaign_X_Campaign_Group table                       */
/*                       b) Add new Campaign_Group UDF tables                            */
/*                                                                                       */
/* 31Mar2009  S0569710   MO_Optimization_Scenario & CI_Camp_Grp_X_Mo_Opt_Scenario tables */
/*                                                                                       */
/* 18Jun2009  S0583163   Reset the SYSCC value to 100000 in case of error                */
/*                                                                                       */
/* 22Jun2009  S0575393   Allow the missing values from the input set to overwrite the    */
/*                       master table. This is mainly required when default UDF values   */
/*                       are reset to blank.                                             */
/*                                                                                       */
/* 06Jan2012  S0782404   Fix the UDF table population                                     */
/*****************************************************************************************/

%macro macamgp(ccg=, cg_char_udf=, cg_num_udf=, cg_dt_udf=, opt_sc=, cgxoptsc=);

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put Current Version of the code: F0MA541;
    %put Last Modified on 06Jan2012;
    %put MACAMGP macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

/* Step 1: Insert/Update the CI_CAMPAIGN_GROUP table */
data &libnm..&ccg.;
    modify &libnm..&ccg.(&dswriteopts.) work.&ccg. UPDATEMODE=NOMISSINGCHECK;
    by campaign_group_sk;
    select(_IORC_);
        when (%SYSRC(_SOK))
        do;
            processed_dttm="&pdttm"dt;
            replace;
        end;
        when (%SYSRC(_DSENMR))
        do;
            processed_dttm="&pdttm"dt;
            _error_= 0;
            output;
        end;
        otherwise
        do;
            stop;
        end;
    end;
run;

/* Exit the process if any error */
%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC. ^= 0 %then
    %do;
        %let SYSCC=100000;
        %let MAError = 869;
        %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
%end;
%else %if &SYSERR. > 4  %then
%do;
    %let MAError =869;
    %let MAMsg =Macro[&sysmacroname.]. Data Modfiy Stmt error while attempting to update Campaign_Group table . Check the log for details;
%end;

/* Update Campaign_Group_Udf records */
/* S0782404 */
%marowcnt( &cg_char_udf. , cg_cudf_cnt);

/* Campaign_Group_Char_Udf: Update or Insert */
%if &cg_cudf_cnt. > 0 %then
%do;
    data &libnm..&cg_char_udf.;
        modify &libnm..&cg_char_udf.(&dswriteopts.) work.&cg_char_udf. UPDATEMODE=NOMISSINGCHECK;
        by CAMPAIGN_GROUP_SK CHAR_UDF_NM;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                processed_dttm="&pdttm"dt;
                replace;
            end;
            when(%SYSRC(_DSENMR))
            do;
                processed_dttm="&pdttm"dt;
                _error_ = 0;
                output;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;

    /* Exit the process if any error */
    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC. ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError = 879;
            %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
        %end;
    %end;
    %else %if &SYSERR.> 4  %then
    %do;
        %let MAError =879;
        %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update CAMPAIGN_GROUP_CHAR_UDF records SYSERR[&SYSERR.]. Check the log for details;
    %end;
%end;

/* Campaign_Group_Num_Udf: Update or Insert */
%marowcnt( &cg_num_udf. , cg_nudf_cnt);

%if &cg_nudf_cnt. > 0 %then
%do;
    data &libnm..&cg_num_udf.;
        modify &libnm..&cg_num_udf.(&dswriteopts.) work.&cg_num_udf. UPDATEMODE=NOMISSINGCHECK;
        by CAMPAIGN_GROUP_SK NUM_UDF_NM;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                processed_dttm="&pdttm"dt;
                replace;
            end;
            when(%SYSRC(_DSENMR))
            do;
                processed_dttm="&pdttm"dt;
                _error_ = 0;
                output;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;

    /* Exit process if any error */
    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC. ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError = 880;
            %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
        %end;
    %end;
    %else %if &SYSERR.> 4  %then
    %do;
        %let MAError =880;
        %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update CAMPAIGN_GROUP_NUM_UDF records SYSERR[&SYSERR.]. Check the log for details;
    %end;
%end;

/* Campaign_Group_Date_Udf: Update or Insert */
%marowcnt( &cg_dt_udf. , cg_dudf_cnt);

%if &cg_dudf_cnt. > 0 %then
%do;
    data &libnm..&cg_dt_udf.;
        modify &libnm..&cg_dt_udf.(&dswriteopts.) work.&cg_dt_udf. UPDATEMODE=NOMISSINGCHECK;
        by CAMPAIGN_GROUP_SK DATE_UDF_NM;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                processed_dttm="&pdttm"dt;
                replace;
            end;
            when(%SYSRC(_DSENMR))
            do;
                processed_dttm="&pdttm"dt;
                _error_= 0;
                output;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;

    /* Exit process if any error */
    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC. ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError = 881;
            %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
        %end;
    %end;
    %else %if &SYSERR.> 4  %then
    %do;
        %let MAError =881;
        %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update CAMPAIGN_GROUP_DATE_UDF records SYSERR[&SYSERR.]. Check the log for details;
    %end;
%end;

/* MO_Optimization_Scenario : Update or Insert */
data &libnm..&opt_sc.;
    modify &libnm..&opt_sc.(&dswriteopts.) work.&opt_sc. UPDATEMODE=NOMISSINGCHECK;
    by MO_OPT_SCENARIO_SK;
    select (_IORC_);
        when(%SYSRC(_SOK))
        do;
            processed_dttm="&pdttm"dt;
            replace;
        end;
        when(%SYSRC(_DSENMR))
        do;
            processed_dttm="&pdttm"dt;
            _error_= 0;
            output;
        end;
        otherwise
        do;
            stop;
        end;
    end;
run;

/* Exit process if any error */
%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC. ^= 0 %then
    %do;
        %let SYSCC=100000;
        %let MAError = 882;
        %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
%end;
%else %if &SYSERR.> 4  %then
%do;
    %let MAError =882;
    %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update MO_OPTIMIZATION_SCENARIO records SYSERR[&SYSERR.]. Check the log for details;
%end;


/* CI_CAMP_GRP_X_MO_OPT_SCENARIO : Update or Insert */
data &libnm..&cgxoptsc.;
    modify &libnm..&cgxoptsc.(&dswriteopts.) work.&cgxoptsc. UPDATEMODE=NOMISSINGCHECK;
    by CAMPAIGN_GROUP_SK MO_OPT_SCENARIO_SK;
    select (_IORC_);
        when(%SYSRC(_DSENMR))
        do;
            _error_= 0;
            output;
        end;
        otherwise
        do;
            stop;
        end;
    end;
run;

/* Exit process if any error */
%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC. ^= 0 %then
    %do;
        %let SYSCC=100000;
        %let MAError = 883;
        %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
%end;
%else %if &SYSERR.> 4  %then
%do;
    %let MAError =883;
    %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update CI_CAMP_GRP_X_MO_OPT_SCENARIO records SYSERR[&SYSERR.]. Check the log for details;
%end;

%if &MADebug. eq Y %then
%do;
    %put MACAMGP macro End Time - %sysfunc(datetime(), datetime24.2);
%end;

%mend;
