/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/*****************************************************************************************/
/* NAME: macclst.sas                                                                     */
/* VERSION: 5.4                                                                        */
/*                                                                                       */
/* DESCRIPTION: Macro to update the following data model tables                          */
/*              CI_CAMP_CHECKLIST_ITEM                                                   */
/*              CI_CAMP_CHECKLIST_CHAR_UDF                                               */
/*              CI_CAMP_CHECKLIST_NUM_UDF                                                */
/*              CI_CAMP_CHECKLIST_DATE_UDF                                               */
/*                                                                                       */
/*  Parameters:                                                                          */
/*             c_clst_item - Physical table name for Campaign_Checklist_item             */
/*             c_clst_char_udf - Physical table name for clist_char_udf                  */
/*             c_clst_num_udf  - Physical table name for clist_num_udf                   */
/*             c_clst_date_udf   - Physical table name for clist_date_udf                */
/*                                                                                       */
/* PRODUCT: MA                                                                           */
/* USAGE: %macclst                                                                       */
/*                                                                                       */
/*****************************************************************************************/
/*****************************************************************************/
/* PRISM VERSION: $Revision: 1.1.4.3.2.4.4.1.4.3 $ ;
/* Revision Date: $Date: 2013/08/14 12:50:21 $  ;
/*****************************************************************************/
/*****************************************************************************************/
/*                                                                                       */
/* History:                                                                              */
/*   20Oct2008 - First version of the macro.                                             */
/*                                                                                       */
/* DATE       Defect No  Comments                                                        */
/* 20Oct2008  S0504678   New project to support CAMP_CHECKLIST publish                   */
/*                                                                                       */
/* 18Jun2009  S0583163   Reset the SYSCC value to 100000 in case of error                */
/*                                                                                       */
/* 22Jun2009  S0575393   Allow the missing values from the input set to overwrite the    */
/*                       master table. This is mainly required when default UDF values   */
/*                       are reset to blank.                                             */
/*                                                                                       */
/* 06Jan2012  S0782404   Fix the UDF table population                                    */
/* 14Aug2013  S0992864   Adding &dswriteopts to the data/modify statements               */
/*****************************************************************************************/

%macro macclst(c_clst_item=, c_clst_char_udf=, c_clst_num_udf=, c_clst_dt_udf=);

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put Current Version of the code: F0MA541;
    %put Last Modified on 14Aug2013;
    %put MACCLST macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

/* Step 1: Update the CI_CAMP_GRP_CHECKLIST_ITEM table */
data &libnm..&c_clst_item.;
    modify &libnm..&c_clst_item. (&dswriteopts.) work.&c_clst_item. UPDATEMODE=NOMISSINGCHECK;
    by CAMPAIGN_SK UDF_SEQ_NO;
    select(_IORC_);
        when (%SYSRC(_SOK))
        do;
            processed_dttm="&pdttm"dt;
            replace;
        end;
        when (%SYSRC(_DSENMR))
        do;
            processed_dttm="&pdttm"dt;
            _error_= 0;
            output;
        end;
        otherwise
        do;
            stop;
        end;
    end;
run;

/* Exit the process if any error */
%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC. ^= 0 %then
    %do;
        %let SYSCC=100000;
        %let MAError = 875;
        %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
%end;
%else %if &SYSERR. > 4  %then
%do;
    %let MAError =875;
    %let MAMsg =Macro[&sysmacroname.]. Data Modfiy Stmt error while attempting to update CI_CAMP_CHECKLIST_ITEM table . Check the log for details;
%end;

/* Step 2: Update all CAMP_GRP_CLIST_UDF tables */

/* CI_Camp_Grp_Clist_Char_Udf: Updates and Inserts for CI_Camp_Grp_Clist_Char_Udf */
%marowcnt( &c_clst_char_udf. , cclst_cudf_cnt);

%if &cclst_cudf_cnt. > 0 %then
%do;
    data &libnm..&c_clst_char_udf.;
        modify &libnm..&c_clst_char_udf. (&dswriteopts.) work.&c_clst_char_udf. UPDATEMODE=NOMISSINGCHECK;
        by CAMPAIGN_SK UDF_SEQ_NO CHAR_UDF_NM;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                processed_dttm="&pdttm"dt;
                replace;
            end;
            when(%SYSRC(_DSENMR))
            do;
                processed_dttm="&pdttm"dt;
                _error_ = 0;
                output;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;

    /* Exit the process if any error */
    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC. ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError = 876;
            %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
        %end;
    %end;
    %else %if &SYSERR.> 4  %then
    %do;
        %let MAError =876;
        %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update CI_CAMP_CHECKLIST_CHAR_UDF records SYSERR[&SYSERR.]. Check the log for details;
    %end;
%end;

/* CI_Camp_Grp_Clist_Num_Udf: Updates and Inserts for CI_Camp_Grp_Clist_Num_Udf */
%marowcnt( &c_clst_num_udf. , cclst_nudf_cnt);

%if &cclst_nudf_cnt. > 0 %then
%do;
    data &libnm..&c_clst_num_udf.;
        modify &libnm..&c_clst_num_udf. (&dswriteopts.) work.&c_clst_num_udf. UPDATEMODE=NOMISSINGCHECK;
        by CAMPAIGN_SK UDF_SEQ_NO NUM_UDF_NM;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                processed_dttm="&pdttm"dt;
                replace;
            end;
            when(%SYSRC(_DSENMR))
            do;
                processed_dttm="&pdttm"dt;
                _error_ = 0;
                output;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;

    /* Exit the process if any error */
    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC. ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError = 877;
            %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
        %end;
    %end;
    %else %if &SYSERR.> 4  %then
    %do;
        %let MAError =877;
        %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update CI_CAMP_CHECKLIST_NUM_UDF records SYSERR[&SYSERR.]. Check the log for details;
    %end;
%end;

/* CI_Camp_Grp_Clist_Date_Udf: Updates and Inserts for CI_Camp_Grp_Clist_Date_Udf */
%marowcnt( &c_clst_dt_udf. , cclst_dudf_cnt);

%if &cclst_dudf_cnt. > 0 %then
%do;
    data &libnm..&c_clst_dt_udf.;
        modify &libnm..&c_clst_dt_udf. (&dswriteopts.) work.&c_clst_dt_udf. UPDATEMODE=NOMISSINGCHECK;
        by CAMPAIGN_SK UDF_SEQ_NO DATE_UDF_NM;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                processed_dttm="&pdttm"dt;
                replace;
            end;
            when(%SYSRC(_DSENMR))
            do;
                processed_dttm="&pdttm"dt;
                _error_ = 0;
                output;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;

    /* Exit the process if any error */
    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC. ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError = 878;
            %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
        %end;
    %end;
    %else %if &SYSERR.> 4  %then
    %do;
        %let MAError =878;
        %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update CI_CHECKLIST_CLIST_DATE_UDF records SYSERR[&SYSERR.]. Check the log for details;
    %end;
%end;

%if &MADebug. eq Y %then
%do;
    %put MACCLST macro End Time - %sysfunc(datetime(), datetime24.2);
%end;

%mend;
