/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,    */
/*                    USA.                                       */
/*             All Rights Reserved.                              */
/*  This software is protected by copyright laws and             */
/*               international treaties                          */
/*                                                               */
/*              U.S. Government Restricted Rights                */
/* Use, duplication, or disclosure of this software and related  */
/*  documentation by the U.S. government is subject to the       */
/*   Agreement with SAS Institute and the restrictions set forth */
/*  in FAR 52.227-19, Commercial Computer Software -             */
/*              Restricted Rights ( June 1987 )                  */
/*                                                               */
/*****************************************************************/

/*****************************************************************/
/* NAME: macdirhw.sas                                            */
/* VERSION: 5.4                                                */
/* DESCRIPTION: Common Data Interface wrapper macro for RH       */
/*              process to create dynamic source code file       */
/*                                                               */
/* Parameters                                                    */
/* Macro variables:                                              */
/*     action - UPOPDTTM                                         */
/*     bulkdatalibref - XML Payload URI for bulkdata             */
/*     bulkdatamap - XML Map file for Payload URI                */
/*     xmlflag - Y/N to indicate whether to use XML Stream       */
/*     updatemode - B/T Bulk or Trickling                        */
/*     Retrials -No of retry attempts for SWA connection failure */
/*     RetryInterval-Interval between retrails                   */
/*                                                               */
/* PRODUCT: MA                                                   */
/* USAGE: %macdirhw                                              */
/*                                                               */
/*****************************************************************/

/*****************************************************************************************/
/*History:                                                                               */
/*dd-mon-yy                                                                              */
/* 27Apr2007 - First Version of the Code                                                 */
/*                                                                                       */
/* S0479890 - added new macro call macdibl to upload the subject level records for SDM   */
/* and RDM client.  Also added new param bcid to maconcnt macro call                     */
/*                                                                                       */
/* S0486525 - Modifications to support NETEZZA                                           */
/* a) added &dbengine. eq NETEZZA to call macro macdiche                                 */
/* b) Code added to call %machkdbc macro to check and create dbconnect statement         */
/*    macro variable is not passed                                                       */
/*                                                                                       */
/* S0486300 - Modifications to support SPDS                                              */
/*                                                                                       */
/* S0499640 - Fix to handle the DBMSTEMP=YES option (global temporary schema to upload   */
/* table) correct                                                                        */
/*                                                                                       */
/* S0513401 - Added engine check for NEOVIEW to call macdirhe                            */
/*                                                                                       */
/* S0513401 - passed targetschema,chschema as macro parameters to macdirhe,because for   */
/* Neoview the resource template is not available and we assign this parameter in the    */
/* madbconn macro                                                                        */
/*                                                                                       */
/* DATE       Defect No  Comments                                                        */
/* 19Sep2008  S0529138   Changes to handle the comma separated multipart-key correct in  */
/*                       SDM/RDM RH processing                                           */
/*                                                                                       */
/* 03Oct2008  S0536278   Assign correct formats to all key columns in the input table    */
/*                       to avoid mismatch in data type after bulkload                   */
/*                                                                                       */
/* 19Nov2008  S0504678   Dynamic Treatment Changes for CI5.3 development                 */
/*                                                                                       */
/* 01Dec2008  S0504678   Changes to support RH update from both 5.1 and 5.3 campaigns    */
/*                                                                                       */
/* 30Dec2008  S0472135   Code added to remove the temp tables in case of any error       */
/*                                                                                       */
/* 03Jun2009  S0584561   Deleteinput and Inputtable new param for SWA Client             */
/*                                                                                       */
/* 04Jun2009  S0583077   added new macro parameter(perfflag)                             */
/*            using new counts process marhcnt in place of marspcnt                      */
/*            calling marhcnt for each subject in each BC                                */
/*                                                                                       */
/* 30Jun2009  S0587037   Added 1. Retry logic for SWA                                    */
/*                             2. new parameters Retrials,RetryInterval                  */
/*                                                                                       */
/* 30Jun2009  S0587951   Modified to call madbirhw macro when i/p tbl is at db           */
/*                                                                                       */
/* 02Jul2009  S0591527   a) Subset input table to have only a particular BC specific recs*/
/*                       when multiple BC records are part of the single input set       */
/*                       b) Process every BC/Subject records part of the input set even  */
/*                       one BC/Subject fails from the single batch file and wirte the   */
/*                       execution status to a error report file                         */
/*                                                                                       */
/* 06May2010 S0663006   Add ASTER db support                                             */
/*                                                                                       */
/* 14Dec2011  S0799554   Reset the MAERROR code to the original one, so to handle the    */
/*                       SDM special handling correctly                                  */
/* 04Dec2012  S0894932  Delete uploaded table when perfflag = N                          */
/*****************************************************************************************/

%macro macdirhw(action=, bulkdatalibref=, bulkdatamap=, deleteinput=, inputtable=, perfflag=, retrials=, retryinterval=, updatemode=, xmlflag= );

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put MACDIRHW - Current Version of the code: 541MA06;
    %put Last Modified on 04Dec2012;
    %put MACDIRHW macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

/* Step 1: Definitions*/

/* Global variable to keep track of the error codes */
%global syscc_old maerror_old mamsg_old;

/* Define Stream Name */
%let Inputstream=RHStream;

/* Define Business Context Metadata table name */
%let bc_meta_tbl=%str(WORK.CCSConnect);

/* Define Data Model Metadata table name */
%let dm_meta_tbl=%str(WORK.CCSDM);

/* Define Data table name */
/* S0584561 - Support for SWA */
%if %length(&inputtable.) > 0 %then
%do;
    %let inputlib=%qscan(&inputtable.,1);
    %let inputtblnm=%qscan(&inputtable.,2);
%end;
%let data_tbl=%str(WORK.RHUpdate);

/* Define XML Payload fileref */
%let xml_pl_ref=RHUpdate;

/* Define XML Payload Mapfile */
%let xml_pl_map=plmap;

/* Define Combined Meta table */
%let cdi_meta_tbl=%str(WORK.CDIMetatbl);

/* Define Dynamic Source fileref */
%let cdisrc=CDIRHSrc;

/* Define the Report Counts table name*/
%let cp_tbl_nm=CI_CELL_PACKAGE;
%let cpt_tbl_nm=CI_CELL_PACKAGE_TREATMENT_CNT;
%let cr_tbl_nm=CI_RESPONSE;
%let crt_tbl_nm=CI_RESPONSE_TYPE;
%let crxcp_tbl_nm=CI_RESPONSE_X_CELL_PACKAGE;
%let crcr_tbl_nm=CI_RESPONSE_CHANNEL_RESPONSE;
%let ct_tbl_nm=CI_TREATMENT;
%let cdta_tbl_nm=CI_DYNAMIC_TREATMENT_ATTRIBUTE;

%let s_rsp_cd=_CV;

/* S0583077 - if perfflag is not set then set it to default(D)*/
%if &perfflag. eq %str() %then %let perfflag =D;

/* Fileref to create dynamic source code file */
filename &cdisrc. catalog "work.CICDI.&cdisrc..Source";

/* Exit process if any error */
%if &SYSERR. > 4 %then
%do;
    %let MAError =720;
    %let MAMsg=Macro[&sysmacroname.]. Error while attempting to define variables. Check the log for details.;
    %return;
%end;

/* S0587037 set the retry parameters to default if not set already */
%if &retrials. eq %str() %then %let retrials=0;             /* if retrials is not send in ,set it to 0 */
%if &retryinterval. eq %str() %then %let retryinterval=0;   /* if RetryInterval is not send in ,set it to 0 */
%let retryinterval = %eval(&retryinterval. * 60);           /* interval input is in minutes , hecne convert it to seconds*/

/* S0587951 - check if bulkdatalibref is a database engine  and ACTION is UPOPDTTM and UpdateMode =D and xmlFlag =Y */
/* then call the swa database processing logic else continue with common logic */
%let dbenginenm = %upcase(%qscan(&bulkdatalibref.,3,%str( )));/* Extract database engine name */

%let firstchar=%qsubstr(&dbenginenm.,1,1);/*Extract the first character of Engine Name */

/*if EngineName is BASE or EngineName Starts with single / double quote then proceed with the old way else call db input processing */
%if ( (%sysevalf(&dbenginenm. eq BASE) OR &firstchar. eq %str(%" ) or &firstchar. eq %str(%' ) )eq 0 )
    AND %upcase(&action.) eq UPOPDTTM
    AND %upcase(&xmlflag.) eq Y
    AND %upcase(&updatemode.)=D %then
%do;
    %madbirhw(bulkdatalibref=&bulkdatalibref., deleteinput=&deleteinput., inputtable=&inputtable.,
                perfflag=&perfflag., retrials=&retrials., retryinterval=&retryinterval.);
    %return;
%end;

/* Step 2: Code Block for Trickling and Bulk update */
/* Macro to read the BC Connect  & Subject meta table to create master table for dynamic
       source code generation */

/* S0587037- added new parameters retrials & retryinterval*/
    %mabcmeta(bc_meta_tbl=&bc_meta_tbl., bulkdatalibref=&bulkdatalibref., bulkdatamap=&bulkdatamap.,
              cdi_meta_tbl=&cdi_meta_tbl., Inputstream=&Inputstream., dm_meta_tbl=&dm_meta_tbl.,
              data_tbl=&data_tbl., retrials=&retrials., retryinterval=&retryinterval., stpcall=RH, xml_pl_map=&xml_pl_map., xml_pl_ref=&xml_pl_ref.,
              xmlflag=&xmlflag., updatemode=&updatemode.);

    /* Return to the calling macro if MAError is already set to a non-zero value */
    %if (&MAError. ne %str()) %then
    %do;
        %return;
    %end;

      /* Convert the Numeric column type to Character if need be and create other
         additional coloumns which is missing in the input table */

      /* S0536278 Apply formats to all key columns in input table */
      /* S0504678 a) CI5.3 Dynamic Treatment changes to add Treatment_Hash_Val
                  b) Add TMTTracking_Cd which will have either one of the following Treatment_Tracking_Cd
                     Treatment_hash_Val varchar(32)- for 5.3 Campaign with Dynamic Treatments
                     Treatment_Sk number(10)- for 5.3 campaign with Static Treatments
                     Treatment_Cd varchar(32)- for 5.1 campaigns. */
      /* S0583077  apply format to RESPONSE_DTTM*/
      data &data_tbl. (rename=(TMT_CDS=TMTTRACKING_CD
                               RESP_CD=RESPONSE_CD
                               RESP_CH_CD=RESPONSE_CHANNEL_CD
                               INF_RESP_FLG=INFERRED_RESPONSE_FLG
                               EXT_RESP_ID1=EXTERNAL_RESPONSE_INFO_ID1
                               EXT_RESP_ID2=EXTERNAL_RESPONSE_INFO_ID2)
                       );
          attrib RESPONSE_DTTM length=8 format=DATETIME24.3
                 CELL_PACKAGE_SK length=8 format=f10.
                 TREATMENT_SK length=8 format=f10.
                 TREATMENT_HASH_VAL length=$32 format=$32.
                 TMT_CDS length=$32 format=$32.
                 RESP_CD length=$30 format=$30.
                 RESP_CH_CD length=$3 format=$3.
                 INF_RESP_FLG length=$1 format=$1.
                 EXT_RESP_ID1 length=$32 format=$32.
                 EXT_RESP_ID2 length=$32 format=$32. ;
          set &data_tbl.;
              TMT_CDS=strip(TMTTRACKING_CD);
              RESP_CD=strip(RESPONSE_CD);
              RESP_CH_CD=strip(RESPONSE_CHANNEL_CD);
              INF_RESP_FLG=strip(INFERRED_RESPONSE_FLG);
              EXT_RESP_ID1=strip(EXTERNAL_RESPONSE_INFO_ID1);
              EXT_RESP_ID2=strip(EXTERNAL_RESPONSE_INFO_ID2);

          drop RESPONSE_CD RESPONSE_CHANNEL_CD INFERRED_RESPONSE_FLG
               EXTERNAL_RESPONSE_INFO_ID1 EXTERNAL_RESPONSE_INFO_ID2
               TMTTRACKING_CD;
      run;

    /* Exit the process if there is an error */
    %if &SYSERR. > 4 %then
    %do;
        %let MAError = 725;
        %let MAMsg = Macro[&sysmacroname.]. Data Step failed - while attempting to create input table. Check the log for details.;
        %return;
    %end;

   /* Source Code creation Data block and might be the last block in this script and
     do any necessary processing above this block or inside this itself*/
    data _null_;
        file &cdisrc. flowover lrecl=32767;
        put ' ';
        put ' %macro ' "&cdisrc." ';';
    run;

    /* S0591527: a) Apply the bc_data where ever needed in the dynamic source code
                 b) Added new macdists macro to create the error report */
    data _null_;
        file &cdisrc. flowover lrecl=32767 mod;
        set &cdi_meta_tbl. end=last;
        by bcid;
        if FIRST.BCID=1 then /* Only for the first BCID in the group */
        do;
            put ' %let targetschema =' targetschema';';
            put ' ';
            put ' /***** Macro Calls for Business Context ID: ' bcid ' and Subject ID: ' subject_id '*****/;';
            put ' ';
            /* S0587037 */
            %if &action.=UPOPDTTM and &updatemode.=D %then                      /* Retry Logic only for SWA */
            %do;
            put ' ';
            put '   %let RetryId=0;                                             /* Retry loop counter 0 - Default Attempt*/';
            put '   %let RetryAttempts=%eval(&retrials + 1);                    /* RetryAttempts = 1(default) + no of Retries */';
            put '   %do %while(&RetryId. lt &RetryAttempts.);                   /* in case of error loop until Retry Attempts */';
            put '       %put Attempt [&RetryId.];                               /* Print Retry Attempt number */';
            put ' ';
            put '       %macdiupl(action= ' "&action." ', bcid= ' bcid ', crcr_tbl_nm= ' "&crcr_tbl_nm." ', ct_tbl_nm=' "&ct_tbl_nm." ', dbconstmt= ' %nrbquote(dbconnect) ', ';
            put '           dbdsoptions=' %nrbquote(dbdsoptions)' , dbmstemp= ' dbmstemp ', dbtempschema= ' dbtempschema ', data_tbl=%str(' "&data_tbl." ') , libstmt=' %nrbquote(libstmt)', ';
            put '           stpcall= RH, sub_id= ' subject_id ', sub_key=%str(' subject_key '), updatemode= ' "&updatemode." ', xmlflag= ' "&xmlflag" '); ';
            put ' ';
            put '       %if (&MAError eq 001) AND (&RetryId. ne (&RetryAttempts. - 1)) %then';
            put '       %do;';
            put '           data _null_;';
            put '               call sleep(&RetryInterval.,1);                  /* Wait for Retry interval */';
            put '           run;    ';
            put '           %let RetryId=%eval(&RetryId. + 1);                  /* increment the retry counter */ ';
            put '           %let MAError=;                                      /* reset MAError before retry */';
            put '       %end;';
            put '       %else ';
            put '           %let RetryId=&RetryAttempts.;';
            put '   %end;       ';
            put ' ';
            %end;
            %else
            %do;
            put ' ';
            put ' %macdiupl(action= ' "&action." ', bcid= ' bcid ', crcr_tbl_nm= ' "&crcr_tbl_nm." ', ct_tbl_nm=' "&ct_tbl_nm." ', dbconstmt= ' %nrbquote(dbconnect) ', ';
            put '           dbdsoptions=' %nrbquote(dbdsoptions)' , dbmstemp= ' dbmstemp ', dbtempschema= ' dbtempschema ', data_tbl=%str(' "&data_tbl." ') , libstmt=' %nrbquote(libstmt)', ';
            put '           stpcall= RH, sub_id= ' subject_id ', sub_key=%str(' subject_key '), updatemode= ' "&updatemode." ', xmlflag= ' "&xmlflag" '); ';
            put ' ';
            %end;
            put ' ';
            put ' %if  &MAError.  ne %str() %then';
            put ' %do; ';
            put '     /* Print the Error report and reset the error code and messages */ ';
            put '     %macdists(bcnm=' bcname ', bcid= ' bcid ', inputnm= %str(' "&data_tbl." '), fl_loc=MAMISC, stpcall=RH);';
            put ' ';
            put '     /* Go To end of the BC processing */ ';
            put '     %goto ExitBCID_' bcid  '; ';
            put ' %end; ';
            put ' ';
            put ' %macdibl(action= ' "&action." ', bcid= ' bcid ', data_tbl=&bc_data., dbdsoptions=' %nrbquote(dbdsoptions)',';
            put '          stpcall=RH, sub_id= ' subject_id ', sub_key=%str(' subject_key'), sub_key_fmt= %str(' subject_key_fmt') ); ';
            put ' ';
            put ' ';
            put ' %if  &MAError.  ne %str() %then';
            put ' %do; ';
            put '     %if %length(&dbconnect.) > 0 and (&dbengine. ne SASSPDS) %then ';
            put '     %do; ';
            put '         /* S0472135: Drop all the uploaded tables in case error */ ';
            put '         %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                   tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put '     %end; ';
            put ' ';
            put '     /* Print the Error report and reset the error code and messages */ ';
            put '     %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=RH, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '     /* Go To end of the Subject processing */ ';
            put '     %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put ' %end; ';
            put ' ';
            put ' ';
            /* S0663006 - Use Explicit Passthrough INSERT for Aster */
            put ' %if %length(&dbconnect.) > 0 and (&dbengine. eq ORACLE or &dbengine. eq TERADATA or &dbengine. eq ASTER or ';
            put '     &dbengine. eq DB2 or &dbengine. eq SASSPDS or &dbengine. eq OLEDB or &dbengine. eq NETEZZA or &dbengine. eq NEOVIEW) %then ';
            put ' %do; ';
            /* S0587037 */
            %if &action.=UPOPDTTM and &updatemode.=D %then                  /* Retry Logic only for SWA */
            %do;
            put '       %let RetryId=0;                                             /* Retry loop counter 0 - Default Attempt*/';
            put '       %let RetryAttempts=%eval(&retrials + 1);                    /* RetryAttempts = 1(default) + no of Retries */';
            put '       %do %while(&RetryId. lt &RetryAttempts.);                   /* in case of error loop until Retry Attempts */';
            put '           %put Attempt [&RetryId.];                               /* Print Retry Attempt number */';
            put ' ';
            put '           %macdirhe(action= ' "&action." ',rhschema= &targetschema., rhtable= ' rhtable ', data_tbl= &bc_data. ,';
            put '               dbconnect=  %nrbquote(&dbconnect) , dbengine= &dbengine., libname= &libname., sub_id= ' subject_id ',';
            put '               sub_key=%str(' subject_key '), tempschema= &tempschema.);' ;
            put ' ';
            put '           %if (&MAError eq 753) AND (&RetryId. ne (&RetryAttempts. - 1)) %then';
            put '           %do;';
            put '               data _null_;';
            put '                   call sleep(&RetryInterval.,1);                  /* Wait for Retry interval */';
            put '               run;    ';
            put '               %let RetryId=%eval(&RetryId. + 1);                  /* increment the retry counter */ ';
            put '               %let MAError=;                                      /* reset MAError before retry */';
            put '               %let syscc=0;                                        /* reset SYSCC before retry */';
            put '           %end;';
            put '           %else ';
            put '               %let RetryId=&RetryAttempts.;';
            put '       %end;       ';
            %end;
            %else
            %do;
            put '     %macdirhe(action= ' "&action." ',rhschema= &targetschema., rhtable= ' rhtable ', data_tbl= &bc_data. ,';
            put '               dbconnect=  %nrbquote(&dbconnect) , dbengine= &dbengine., libname= &libname., sub_id= ' subject_id ',';
            put '               sub_key=%str(' subject_key '), tempschema= &tempschema.);' ;
            %end;
            put ' ';
            put ' ';
            put '     %if  &MAError.  ne %str() %then';
            put '     %do; ';
            put '         %if %length(&dbconnect.) > 0 and (&dbengine. ne SASSPDS) %then ';
            put '         %do; ';
            put '             /* S0472135: Drop all the uploaded tables in case error */ ';
            put '             %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                       tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put '         %end; ';
            put ' ';
            put '         /* Print the Error report and reset the error code and messages */ ';
            put '         %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=RH, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '         /* Go To end of the Subject processing */ ';
            put '         %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put '     %end; ';
            put ' %end; ';
            put ' %else ';
            put ' %do; ';
            put '     %macdirhi(action= ' "&action." ', rhtable= ' rhtable ', data_tbl= &bc_data. , dbengine= &dbengine.,' ;
            put '               libname= &libname., libstmt= ' %nrbquote(libstmt) ', sub_id= ' subject_id ', sub_key=%str(' subject_key ') );' ;
            put ' ';
            put '     %if  &MAError.  ne %str() %then';
            put '     %do; ';
            put '         /* S0472135: Drop all the uploaded tables in case error */ ';
            put '         %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                   tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put ' ';
            put '         /* Print the Error report and reset the error code and messages */ ';
            put '         %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=RH, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '         /* Go To end of the Subject processing */ ';
            put '         %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put '     %end; ';
            put ' %end; ';
            put ' ';
            put ' ';
            /* S0583077 - call the marhcnt for each BC and each subject in BC */
            put ' /***** Response Report count for the given BC *****/';
            put ' %if &perfflag eq D or &perfflag eq T %then';
            put ' %do; ';
            put '   %marhcnt(ch_tbl= ' chtable ', cp_tbl= ' "&cp_tbl_nm." ',cpt_tbl= ' "&cpt_tbl_nm." ', ';
            put '             crxcp_tbl= ' "&crxcp_tbl_nm." ',data_tbl=&unqintbl.' subject_id ' ,dbengine=&dbengine. ,libnm= &libname., ';
            put '             newid= &newid. , perfflag= ' "&perfflag." ', pth_tbl= ' pttable ' ,rh_tbl= ' rhtable ',sub_key_sp= &sub_key_sp. ,s_rsp_cd=' "&s_rsp_cd." ' ,tmplibnm=&templibname.); ';
            put ' ';
            put '   %if &MAError. ne %str() %then';
            put '   %do; ';
            put '       %if %length(&dbconnect.) > 0 and (&dbengine. eq SASSPDS) %then ';
            put '       %do; ';
            put '           /* S0472135: Drop all the uploaded tables in case error */ ';
            put '           %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                     tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put '       %end; ';
            put ' ';
            put '       /* Print the Error report and reset the error code and messages */ ';
            put '       %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=RH, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '       /* Go To end of the Subject processing */ ';
            put '       %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put '   %end; /* End for Error check */';
            put ' %end; /* End for perfflag check */ ';

            /* S0894932 - delete the uploaded table when perfflag= N */
            put ' %else %if &perfflag eq N %then';
            put ' %do; ';
            put '       %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '        tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put ' ';
            put ' %end; /* End for perfflag = N check */ ';

            put ' ';
            put ' /* End of Subject: ' subject ' in BC: ' bcname ' Processing */ ';
            put ' %ExitBC_' bcid +(-1) '_SubjectId_' subject_id +(-1) ': ' ; /* +(-1) used to move the pointer control bacward and avoid unwanted blank space between subject_id and colon */
            put ' ';
        end;
        else if FIRST.BCID=0 then /* Except the first BCID in the group and includes the last BCID */
        do;
            put ' ';
            put ' %let targetschema =' targetschema';';
            put ' /***** Macro Calls for Business Context ID: ' bcid ' and Subject ID: ' subject_id '*****/;';
            put ' ';
            put ' %macdibl(action= ' "&action." ', bcid= ' bcid ', data_tbl=&bc_data., dbdsoptions=' %nrbquote(dbdsoptions)',';
            put '          stpcall=RH, sub_id= ' subject_id ', sub_key=%str(' subject_key'), sub_key_fmt= %str(' subject_key_fmt') ); ';
            put ' ';
            put ' %if  &MAError.  ne %str() %then';
            put ' %do; ';
            put '     %if %length(&dbconnect.) > 0 and (&dbengine. ne SASSPDS) %then ';
            put '     %do; ';
            put '        /* S0472135: Drop all the uploaded tables in case error */ ';
            put '        %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                  tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put '     %end; ';
            put ' ';
            put '     /* Print the Error report and reset the error code and messages */ ';
            put '     %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=RH, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '     /* Go To end of the Subject processing */ ';
            put '     %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put ' %end; ';
            put ' ';
            /* S0663006 - Use Explicit Passthrough INSERT for Aster */
            put ' %if %length(&dbconnect.) > 0 and (&dbengine. eq ORACLE or &dbengine. eq TERADATA or &dbengine. eq ASTER or ';
            put '     &dbengine. eq DB2 or &dbengine. eq SASSPDS or &dbengine. eq OLEDB or &dbengine. eq NETEZZA or &dbengine. eq NEOVIEW) %then ';
            put ' %do; ';
            /* S0587037 */
            %if &action.=UPOPDTTM and &updatemode.=D %then                  /* Retry Logic only for SWA */
            %do;
            put '       %let RetryId=0;                                             /* Retry loop counter 0 - Default Attempt*/';
            put '       %let RetryAttempts=%eval(&retrials + 1);                    /* RetryAttempts = 1(default) + no of Retries */';
            put '       %do %while(&RetryId. lt &RetryAttempts.);                   /* in case of error loop until Retry Attempts */';
            put '           %put Attempt [&RetryId.];                               /* Print Retry Attempt number */';
            put ' ';
            put '           %macdirhe(action= ' "&action." ',rhschema= &targetschema. , rhtable= ' rhtable ', data_tbl= &bc_data. ,';
            put '               dbconnect= ' %nrbquote(dbconnect) ', dbengine= &dbengine., libname= &libname., sub_id= ' subject_id ',';
            put '               sub_key=%str(' subject_key '), tempschema= &tempschema.);' ;
            put ' ';
            put '           %if (&MAError eq 753) AND (&RetryId. ne (&RetryAttempts. - 1)) %then';
            put '           %do;';
            put '               data _null_;';
            put '                   call sleep(&RetryInterval.,1);                  /* Wait for Retry interval */';
            put '               run;    ';
            put '               %let RetryId=%eval(&RetryId. + 1);                  /* increment the retry counter */ ';
            put '               %let MAError=;                                      /* reset MAError before retry */';
            put '               %let syscc=0;                                        /* reset SYSCC before retry */';
            put '           %end;';
            put '           %else ';
            put '               %let RetryId=&RetryAttempts.;';
            put '       %end;       ';
            put ' ';
            %end;
            %else
            %do;
            put ' ';
            put '       %macdirhe(action= ' "&action." ',rhschema= &targetschema. , rhtable= ' rhtable ', data_tbl= &bc_data. ,';
            put '               dbconnect= ' %nrbquote(dbconnect) ', dbengine= &dbengine., libname= &libname., sub_id= ' subject_id ',';
            put '               sub_key=%str(' subject_key '), tempschema= &tempschema.);' ;
            put ' ';
            %end;
            put ' ';
            put '     %if  &MAError.  ne %str() %then';
            put '     %do; ';
            put '         %if %length(&dbconnect.) > 0 and (&dbengine. ne SASSPDS) %then ';
            put '         %do; ';
            put '             /* S0472135: Drop all the uploaded tables in case error */ ';
            put '             %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                       tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put '         %end; ';
            put ' ';
            put '        /* Print the Error report and reset the error code and messages */ ';
            put '        %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=RH, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '        /* Go To end of the Subject processing */ ';
            put '        %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put '     %end; ';
            put ' %end; ';
            put ' %else ';
            put ' %do; ';
            put '     %macdirhi(action= ' "&action." ', rhtable= ' rhtable ', data_tbl= &bc_data. , dbengine= &dbengine.,' ;
            put '               libname= &libname., libstmt= ' %nrbquote(libstmt) ', sub_id= ' subject_id ', sub_key=%str(' subject_key ') );' ;
            put ' ';
            put '     %if  &MAError.  ne %str() %then';
            put '     %do; ';
            put '         /* S0472135: Drop all the uploaded tables in case error */ ';
            put '         %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                   tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put ' ';
            put '         /* Print the Error report and reset the error code and messages */ ';
            put '         %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=RH, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '         /* Go To end of the Subject processing */ ';
            put '         %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put '     %end; ';
            put ' %end; ';
            put ' ';
            put ' ';
            /* S0583077 - call the marhcnt for each BC and each subject in BC */
            put ' /***** Response Report count for the given BC *****/';
            put ' %if &perfflag eq D or &perfflag eq T %then';
            put ' %do; ';
            put '    %marhcnt(ch_tbl= ' chtable ', cp_tbl= ' "&cp_tbl_nm." ',cpt_tbl= ' "&cpt_tbl_nm." ', ';
            put '              crxcp_tbl= ' "&crxcp_tbl_nm." ',data_tbl=&unqintbl.' subject_id ' ,dbengine=&dbengine. ,libnm= &libname., ';
            put '              newid= &newid. , perfflag= ' "&perfflag." ', pth_tbl= ' pttable ' ,rh_tbl= ' rhtable ',sub_key_sp= &sub_key_sp. ,s_rsp_cd=' "&s_rsp_cd." ' ,tmplibnm=&templibname.); ';
            put ' ';
            put '    %if &MAError. ne %str() %then';
            put '    %do; ';
            put '        %if %length(&dbconnect.) > 0 and (&dbengine. eq SASSPDS) %then ';
            put '        %do; ';
            put '            /* S0472135: Drop all the uploaded tables in case error */ ';
            put '            %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '                      tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put '        %end; ';
            put ' ';
            put '        /* Print the Error report and reset the error code and messages */ ';
            put '        %macdists(bcnm=' bcname ', bcid= ' bcid ', fl_loc=MAMISC, stpcall=RH, subnm= ' subject', subid= ' subject_id');';
            put ' ';
            put '        /* Go To end of the Subject processing */ ';
            put '        %goto ExitBC_' bcid +(-1) '_SubjectId_' subject_id '; ';
            put ' ';
            put '    %end; /* End for Error check */';
            put ' %end; /* End for perfflag check */ ';

            /* S0894932 - delete the uploaded table when perfflag= N */
            put ' %else %if &perfflag eq N %then';
            put ' %do; ';
            put '       %macdidel(dbmstemp= ' dbmstemp', dbtempschema= ' dbtempschema ', libexist=Y, libstmt=' %nrbquote(libstmt)', ';
            put '        tblnms=%str(&unqintbl.' subject_id'), templibname= &templibname.); ';
            put ' ';
            put ' %end; /* End for perfflag = N check */ ';

            put ' ';
            put ' /* End of Subject: ' subject ' in BC: ' bcname ' Processing */ ';
            put ' %ExitBC_' bcid +(-1) '_SubjectId_' subject_id +(-1) ': ' ; /* +(-1) used to move the pointer control bacward and avoid unwanted blank space between subject_id and colon */
            put ' ';
        end;
        if LAST.BCID=1 then /* Only for last BCID */
        do;
            put ' /* End of BC: ' bcname ' Processing */ ';
            put ' %ExitBCID_' bcid +(-1) ': '; /* +(-1) used to move the pointer control bacward and avoid unwanted blank space between bcid and colon */
            put ' ';
        end;
run;

data _null_;
file &cdisrc. flowover lrecl=32767 mod;
    put ' ';
    /* S0584561 - Drop the input dataset on successfull execution and based on the flag setting for SWA */
    /* S0591527 - Included MAERROR_old var check to delete input table */
    put ' %if &MAError. eq %str() and &MAError_old. eq %str() and %upcase(&deleteinput) eq Y %then';
    put ' %do; ';
    put '     proc datasets library=&inputlib. memtype=data nolist; ';
    put '         delete &inputtblnm; ';
    put '     run; ';
    put ' %end; ';
    put ' ';
    put '%mend;';
    put ' ';
    put ' %'"&cdisrc."';'; /* call to execute the dynamic macro */
run;

/* Exit process if any error */
%if &SYSERR. > 4 %then
%do;
     %let MAError = 721;
     %let MAMsg = Macro[&sysmacroname.]. Error in Data Step while attempting to create dynamic source code file. Check the log for details.;
     %return;
%end;

/* Include the Dynamic Source code file */
%include &cdisrc.;

/* S0799554: Reset the MAError code to MAError_old which is set from the macdists macro. */
/* Assign the error codes back */
%if &syscc_old. ne %str() %then
%do;
     %let SYSCC= &syscc_old.;
     %let MAError= &maerror_old.;
     %let MAMsg= Error processing History tables.  Check log and MAMISC.CI_HISTORY_ERROR_REPORT for details;
%end;

%mend;
