/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/************************************************************************************/
/* NAME: macelpub                                                                   */
/* VERSION: 5.4                                                                     */
/*                                                                                  */
/* DESCRIPTION: Macro to Update/Insert Campaign Cell records                        */
/* Parameter:                                                                       */
/*       ccell - Physical table name for Marketing_Cell table                       */
/*                                                                                  */
/* PRODUCT: MA                                                                      */
/* USAGE: %macelpub                                                                 */
/*                                                                                  */
/************************************************************************************/
/*****************************************************************************************/
/*                                                                                       */
/* History:                                                                              */
/*   27APR2007 - Revised first version of the macro.                                     */
/*                                                                                       */
/* DATE       Defect No  Comments                                                        */
/* 18Jun2009  S0583163   Reset the SYSCC value to 100000 in case of error                */
/*                                                                                       */
/* 22Jun2009  S0575393   Allow the missing values from the input set to overwrite the    */
/*                       master table. This is mainly required when default UDF values   */
/*                       are reset to blank.                                             */
/* 14Aug2013  S0992864   add dswriteopts in data modify                                  */
/*****************************************************************************************/

%macro macelpub(ccell=);
%put Start Time - %sysfunc(datetime(), datetime24.2);

/*Check in Marketing_Cell for records required to be inserted or updated separately - insertflg =0*/
data &libnm..&ccell.;
    modify &libnm..&ccell. (&dswriteopts.) work.&ccell. UPDATEMODE=NOMISSINGCHECK;
    by MARKETING_CELL_SK;
    select(_IORC_);
        when(%SYSRC(_SOK))
        do;
            processed_dttm="&pdttm"dt;
            replace;
        end;
        when(%SYSRC(_DSENMR))
        do;
            processed_dttm="&pdttm"dt;
            _error_ = 0;
            output;
        end;
        otherwise
        do;
            stop;
        end;
    end;
run;

/* Exit the process if any error */
%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC. ^= 0 %then
    %do;
        %let SYSCC=100000;
        %let MAError = 819;
        %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
%end;
%else %if &SYSERR. > 4  %then
%do;
    %let MAError =819;
    %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update MARKETING_CELL records SYSERR[&SYSERR.]. Check the log for details;;
%end;

%put end Time - %sysfunc(datetime(), datetime24.2);
%mend;
