/****************************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.           */
/*                     All Rights Reserved.                                 */
/*       This software is protected by copyright laws and                   */
/*                    international treaties                                */
/*                                                                          */
/*               U.S. Government Restricted Rights                          */
/* Use, duplication, or disclosure of this software and related             */
/*    documentation by the U.S. government is subject to the                */
/*  Agreement with SAS Institute and the restrictions set forth             */
/*       in FAR 52.227-19, Commercial Computer Software -                   */
/*                Restricted Rights ( June 1987 )                           */
/*                                                                          */
/****************************************************************************/
/* NAME: machast.sas                                                        */
/* VERSION: 5.41                                                            */
/* DESCRIPTION: Common Data Interface macro to compute contact counts       */
/*                                                                          */
/* PRODUCT: MA                                                              */
/* USAGE: %machast                                                          */
/****************************************************************************/
/* Macro variables:                                                         */
/* Name       Desc                                    Value                 */
/* action     UPOP/USTA/UPOPDTTM/USTADTTM                                   */
/* cp_tbl     Cell_Package table name                                       */
/* cpt_tbl    Cell_Package_Treatment_Cnt table name                         */
/* data_tbl   Input Data table                                              */
/* dbengine   Name of the database Engine,Update tables Statments are       */
/*            Engine Dependent                                              */
/* f_con_cd   Code for Failed Contacts                                      */
/* libnm      Library Name for CI Common Data Model                         */
/* newid      Unique new id for creating data processing tables in templib  */
/* popexists  Population in case of action=USTA                             */
/* tmplibnm   Temporary Library Name for data processing                    */
/****************************************************************************/
/* Change log                                                               */
/* Date    DefectNo  Comments                                               */
/* 24Mar10 S0651614  New Version for Aster Partition Key Support            */
/* 28Jun11 S0744863   Added support for GREENPLUM                           */
/* 28Jun11 S0755467   Modify where exists to use EXCEPT clause              */
/* 15Jun12 S0782605   use proc sql to get counts instead of proc summary    */
/* 06Dec12 S0779594   Modified Treat_cnt table update for UPOPDTTM          */
/****************************************************************************/
%macro machast(action=,cp_tbl=,cpt_tbl=,data_tbl=,dbengine=,f_con_cd=,libnm=,newid=,
       popexists=,tmplibnm=,);

    /*Setp2a generate summary when action = UPOPDTTM or UPOP or USTA*/
    %if &action. =UPOPDTTM or &action. = UPOP or  &action. = USTA %then
    %do;
        /*Step2a1 generate summary from input table*/
        %if &MADebug = Y %then  %put Step2a1 Start Time:%sysfunc(datetime(),datetime20);
        /*Append the tmp table to TblNames variable*/
        %let TblNames=&TblNames. %sysfunc(strip(CH_CP_COUNTS&newid.));
        %* S0782605 -  use proc sql to get counts instead of proc summary  ;
        proc sql;
            create table    &tmplibnm..CH_CP_COUNTS&newid. ( &MppDBKey. ) as
                select      CELL_PACKAGE_SK, PACKAGE_HASH_VAL , count(*) as ATTEMPTED_CNT
                from        &tmplibnm..&data_tbl.
                group by    CELL_PACKAGE_SK, PACKAGE_HASH_VAL ;
        quit;

        /*attempted_cnt='INT'*/
        %if &MADebug = Y %then  %put Step2a1 End Time:%sysfunc(datetime(),datetime20);

        /* S0558672 */
        /* if the input table does not have any of the cell_package_sk in dist_cpk  */
        /* table then add them in the ch_cp_counts with Attempted_cnt as 0 */

        %if &action. = UPOP or &action. = USTA %then
        %do;
            /* S0605419 S0755467*/
            %let TblNames=&TblNames. %sysfunc(strip(ch_cp_deleted&newid.));
            proc sql;
                create table    &tmplibnm..ch_cp_deleted&newid. ( &MppDBKey. )  as
                    select      t.cell_package_sk, 0 as attempted_cnt
                    from
                    (select     cell_package_sk from &tmplibnm..&unqintbl._CPK
                    except
                    select      cell_package_sk from &tmplibnm..CH_CP_COUNTS&newid.)t
            ;quit;

            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            proc sql;
                insert into     &tmplibnm..CH_CP_COUNTS&newid.(cell_package_sk,attempted_cnt)
                    select      cell_package_sk,attempted_cnt
                    from        &tmplibnm..ch_cp_deleted&newid.
            ;quit;
            %if &SQLRC ne 0  %then %do;%let MAError =739; %goto exit; %end;
        %end;

        /* Step2a2 Get the Track Preseneted Treatmetn flag for the input cell packages this is only for Action=UPOPDTTM */
        %if &action. =UPOPDTTM %then
        %do;
            %if &MADebug = Y %then  %put Step2a2 Start Time:%sysfunc(datetime(),datetime20);
            %let TblNames=&TblNames. %sysfunc(strip(ch_cp_tmt_cnt&newid.));
            proc sql;
                create table    &tmplibnm..ch_cp_tmt_cnt&newid. ( &MppDBKey. )  as
                    select      t.*,
                                cp.track_presented_treatments_flg as tpt_flg
                    from        &tmplibnm..CH_CP_COUNTS&newid. t
                    inner join  &libnm..&cp_tbl. cp
                    on          t.cell_package_sk = cp.cell_package_sk
            ;quit;
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            %let pxt_tbl = CI_PACKAGE_X_TREATMENT;
            %* S0779594 - modify ch_cpt_tmt_cnt table creation
            For offer arbitration campaign - all the treatments may not be offered, the offered treatment need to be figured out
            using the package_hash_val from the contact history input table
            1.if the package_hash_val is NULL then get the for the cell_package_sk get the list of treatments from pkg_x_treat table for matching contributing_cell_package_sk
            2.if the package_hash_val is not null then get the list of treatments for the matching package_hash_val from pkg_x_treat
            NOTE - following quert needs to be tested for different input data conditions ;
            %let TblNames=&TblNames. %sysfunc(strip(ch_cpt_tmt_cnt&newid.));
            proc sql;
                    create table    &tmplibnm..ch_cpt_tmt_cnt&newid. ( &MppDBKey. ) as
                        /* for matching package_hash_val get the treatment_sk from pkg_x_treatment*/
                        select      t.*,pxt.treatment_sk
                        from        &tmplibnm..ch_cp_tmt_cnt&newid. t
                        inner join  &libnm..&pxt_tbl. pxt
                        on          t.package_hash_val= pxt.package_hash_val
                        /* if there are no treatments added in the campaign then its published with NOT_TREATMENT and
                        package_hash_val for such cell is null hence , for null package_hash_val get the treatment_sk from pkg_x_treatment for matching contributing_cell_package_sk */
                        union
                        select      t.*,pxt.treatment_sk
                        from        &tmplibnm..ch_cp_tmt_cnt&newid. t
                        inner join  &libnm..&pxt_tbl. pxt
                        on          t.cell_package_sk= pxt.contributing_cell_package_sk
                        where       t.package_hash_val is null
            ;quit;
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;
            %if &MADebug = Y %then  %put Step2a2 End Time:%sysfunc(datetime(),datetime20);
        %end;
        %else
        %do;
            /* For Generic Update Get the Treatment_sk for each cell_package */
            %let TblNames=&TblNames. %sysfunc(strip(ch_cpt_counts&newid.));
            proc sql;
                create table    &tmplibnm..ch_cpt_counts&newid. ( &MppDBKey. )  as
                    select      t.*,cpt.treatment_sk
                    from        &tmplibnm..CH_CP_COUNTS&newid. t
                    inner join  &libnm..&cpt_tbl. cpt
                    on          t.cell_package_sk = cpt.cell_package_sk
            ;quit;
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;
        %end;
    %end;

    /*Setp2b generate summary when action = USTADTTM*/
    %if &action. =USTADTTM %then
    %do;
        %if &MADebug = Y %then  %put Step2b Start Time:%sysfunc(datetime(),datetime20);
        %let TblNames=&TblNames. %sysfunc(strip(CH_CP_FAILED_CNT&newid.));
        %* S0782605 -  use proc sql to get counts instead of proc summary  ;
        proc sql;
            create table    &tmplibnm..CH_CP_FAILED_CNT&newid. ( &MppDBKey.) as
                select      CELL_PACKAGE_SK , count(*) as  FAILED_CNT
                from        &tmplibnm..&data_tbl.
                where       CHSTATUS_CD="&f_con_cd."
                group by    CELL_PACKAGE_SK ;
        quit;

        %let TblNames=&TblNames. %sysfunc(strip(ch_cpt_failed_cnt&newid.));
        proc sql;
            create table    &tmplibnm..ch_cpt_failed_cnt&newid. ( &MppDBKey. ) as
                select      t.*,cpt.treatment_sk
                from        &tmplibnm..CH_CP_FAILED_CNT&newid. t
                inner join  &libnm..&cpt_tbl. cpt
                on          t.cell_package_sk = cpt.cell_package_sk
        ;quit;
        %if &SQLRC ne 0  %then %do;%let MAError =736;  %goto exit; %end;
        %if &MADebug = Y %then  %put Step2b End Time:%sysfunc(datetime(),datetime20);
    %end;
%exit:
%mend;
