/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,    */
/*                    USA.                                       */
/*             All Rights Reserved.                              */
/*  This software is protected by copyright laws and             */
/*               international treaties                          */
/*                                                               */
/*              U.S. Government Restricted Rights                */
/* Use, duplication, or disclosure of this software and related  */
/*  documentation by the U.S. government is subject to the       */
/*   Agreement with SAS Institute and the restrictions set forth */
/*  in FAR 52.227-19, Commercial Computer Software -             */
/*              Restricted Rights ( June 1987 )                  */
/*                                                               */
/*****************************************************************/
/*****************************************************************/
/* NAME: macheust.sas                                            */
/* VERSION: 5.4                                                  */
/* DESCRIPTION: Common Data Interface macro to Update Contact    */
/*              History table using Explicit Pass through for    */
/*              action usta and popexists=1                      */
/*                                                               */
/* Parameters                                                    */
/* Macro variables:                                              */
/*     dbengine - Data base engine from connect stmt             */
/*                                                               */
/* PRODUCT: MA                                                   */
/* USAGE: %macheust                                              */
/*                                                               */
/*****************************************************************************************/
/*History:                                                                               */
/* 20May10  Taskid-57851 MA5.4Iteration5-Update Stored Process for CH "exclude" status   */
/*                                                                                       */
/* DATE      Defect No  Comments                                                         */
/* 15Dec2010  S0715116  New External_ columns added to CH and PT table                   */
/*                                                                                       */
/* 28Jun2011 S0744863   Added support for GREENPLUM                                      */
/*                                                                                       */
/* 09Dec2011 S0791301    create/refer all the table/col names in upper case              */
/*****************************************************************************************/
%macro macheust(dbengine= );
/* DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put macheust Current Version of the code: 541MA02;
    %put Last Modified on 09Dec2011;
    %put MACHEUST macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;
/* Step 6: Explicit Passthrough block to process the updates */
proc sql noerrorstop;
    &dbconnect.;;
    /* Exit the process if the pass-through connect statement fails */
    /* Neoview: returns SQLXRC as -1 even after successful connection  */
    %if (&dbengine. eq NEOVIEW and %sysevalf(&SQLXRC eq -1) and &SQLXMSG eq %str()) %then
        %let SQLXRC=0;
    %if &SQLXRC. ne 0 %then
    %do;
        %let MAError=753;
        %let syscc=100000;
        %let MAMsg=Macro[&sysmacroname.] Error executing Explicit Pass-through CONNECT statement. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
        %put &MAMsg;
        %return;
    %end;
    /* Engine Specific update blocks */
    %if &dbengine. eq NETEZZA %then
    %do;
        execute(update &targettbl.
            set
                CONTACT_HISTORY_STATUS_CD= &sourcetbl._CPK.CHSTATUS_CD,
                CONTACT_DTTM= %STR(%')&cdttm%STR(%'),
                CONTACT_DT= %STR(%')&ctdate%STR(%')
            where &targettbl..CELL_PACKAGE_SK=&sourcetbl._CPK.CELL_PACKAGE_SK  and &sourcetbl._CPK.CHSTATUS_CD IS NOT NULL
          )by &dbengine.;

        /* Check for any errors in update process */
        %if &SQLXRC. ne 0 and &SQLRC. > 4 %then
        %do;
            %let MAError=756;
            %let SYSCC=100000;
            %let MAMsg=Macro[&sysmacroname.] Error in Updating Contact History target table. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %return;
        %end;
    %end;/* end of NETEZZA engine update */
    %else %if &dbengine. eq TERADATA %then
    %do; /* This syntax is specific to Teradata*/
        execute (update &targettbl. from &sourcetbl._CPK
                set CONTACT_HISTORY_STATUS_CD = &sourcetbl._CPK.CHSTATUS_CD,
                    CONTACT_DTTM= TIMESTAMP %STR(%')&cdttm%STR(%'),
                    CONTACT_DT= DATE %STR(%')&ctdate%STR(%')
                 where &targettbl..CELL_PACKAGE_SK=&sourcetbl._CPK.CELL_PACKAGE_SK  and CHSTATUS_CD IS NOT NULL
            )by &dbengine.;

        /* Check for any errors in update process */
        %if &SQLXRC. ne 0 and &SQLRC. > 4 %then
        %do;
            %let MAError=756;
            %let SYSCC=100000;
            %let MAMsg=Macro[&sysmacroname.] Error in Updating Contact History target table. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %return;
        %end;
         execute (commit) by &dbengine.;
    %end; /* end of Teradata update */
    %else %if &dbengine. eq NEOVIEW %then
    %do; /* This syntax is specific to NEOVIEW*/
        execute(update &targettbl.
            set(CONTACT_HISTORY_STATUS_CD ) =
                (select CHSTATUS_CD from &sourcetbl._CPK intbl where &targettbl..CELL_PACKAGE_SK=intbl.CELL_PACKAGE_SK and intbl.CHSTATUS_CD IS NOT NULL ),
            CONTACT_DTTM = TIMESTAMP %STR(%')&cdttm%STR(%'),
            CONTACT_DT = DATE %STR(%')&ctdate%STR(%')
            where exists
            (select 1 from &sourcetbl._CPK intbl where &targettbl..CELL_PACKAGE_SK=intbl.CELL_PACKAGE_SK and intbl.CHSTATUS_CD IS NOT NULL)
            ) by &dbengine.;

        /* Check for any errors in update process */
        %if &SQLXRC. ne 0 and &SQLRC. > 4 %then
        %do;
            %let MAError=756;
            %let SYSCC=100000;
            %let MAMsg=Macro[&sysmacroname.] Error in Updating Contact History target table. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %return;
        %end;
    %end; /* End of NEOVIEW update */
    %else %if &dbengine. eq OLEDB %then
    %do; /* This syntax is specific to SQL Server */
        execute (update &targettbl.
                set CONTACT_HISTORY_STATUS_CD = intbl.CHSTATUS_CD,
                    CONTACT_DTTM=CONVERT(DATETIME,%STR(%')&cdttm%STR(%'),121),
                    CONTACT_DT=CONVERT(DATE,REPLACE(%STR(%')&ctdate%STR(%'),'-','/'),111)
                from &targettbl. ch, &sourcetbl._CPK intbl
                where ch.CELL_PACKAGE_SK=intbl.CELL_PACKAGE_SK and CHSTATUS_CD IS NOT NULL
            )by &dbengine.;

        /* Check for any errors in update process */
        %if &SQLXRC. ne 0 and &SQLRC. > 4 %then
        %do;
            %let MAError=756;
            %let SYSCC=100000;
            %let MAMsg=Macro[&sysmacroname.] Error in Updating Contact History target table. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %return;
        %end;
    %end;/* end of OLEDB update */
    %else %if &dbengine. eq ASTER or &dbengine. eq GREENPLM %then
    %do;
        execute (update &targettbl.
            set     CONTACT_HISTORY_STATUS_CD = intbl.CHSTATUS_CD,
                    CONTACT_DTTM= %STR(%')&cdttm%STR(%'),
                    CONTACT_DT= %STR(%')&ctdate%STR(%')
            from    &sourcetbl._CPK intbl
            where   &targettbl..CELL_PACKAGE_SK=intbl.CELL_PACKAGE_SK and intbl.CHSTATUS_CD IS NOT NULL
            )by &dbengine.;
            /* Check for any errors in update process */
        %if &SQLXRC. ne 0 and &SQLRC. > 4 %then
        %do;
            %let MAError=756;
            %let SYSCC=100000;
            %let MAMsg=Macro[&sysmacroname.] Error in Updating Contact History target table. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %return;
        %end;
    %end; /* end of Aster/Greenplum update */
    %else %if &dbengine. eq SASSPDS %then
    %do;
        execute (update &targettbl.
            set
                CONTACT_HISTORY_STATUS_CD =
                    (select CHSTATUS_CD from &sourcetbl._CPK intbl
                        where &targettbl..CELL_PACKAGE_SK=intbl.CELL_PACKAGE_SK and CHSTATUS_CD IS NOT NULL),
                CONTACT_DTTM= &execdt.,
                CONTACT_DT= datepart(&execdt.)
            where exists
                (select 1 from &sourcetbl._CPK intbl where &targettbl..CELL_PACKAGE_SK=intbl.CELL_PACKAGE_SK and CHSTATUS_CD IS NOT NULL)
            )by &dbengine.;

        /* Check for any errors in update process: The condition check for SPDS is different
        compared to other DB as SPDS sets the SQLRC variable in case of errors */
        %if &SQLRC. > 4 %then
        %do;
            %let MAError=756;
            %let SYSCC=100000;
            %let MAMsg=Macro[&sysmacroname.] Error in Updating Contact History target table. SQLRC=[&SQLRC.]. Check the log for details.;
            %return;
        %end;
    %end;/* End of spds engine update */
    %else
    %do;
        execute(update &targettbl. ch
                set
                    (CONTACT_HISTORY_STATUS_CD)=
                            (select CHSTATUS_CD from &sourcetbl._CPK intbl
                            where ch.CELL_PACKAGE_SK=intbl.CELL_PACKAGE_SK and CHSTATUS_CD IS NOT NULL),
                    %if %upcase(&dbengine.) eq ORACLE %then
                    %do;
                        CONTACT_DTTM = TO_TIMESTAMP(%STR(%')&cdttm%STR(%'),'YYYY-MM-DD HH24:MI:SS.FF'),
                        CONTACT_DT = TO_DATE( %STR(%')&ctdate%STR(%'),'YYYY-MM-DD' )
                    %end;
                    %else
                    %do;
                        CONTACT_DTTM= %STR(%')&cdttm%STR(%'),
                        CONTACT_DT= %STR(%')&ctdate%STR(%')
                    %end;
                where exists
                (select 1 from &sourcetbl._CPK intbl
                where ch.CELL_PACKAGE_SK=intbl.CELL_PACKAGE_SK and CHSTATUS_CD IS NOT NULL)
           ) by &dbengine.;

        /* Check for any errors in update process */
        %if &SQLXRC. ne 0 and &SQLRC. > 4 %then
        %do;
            %let MAError=756;
            %let SYSCC=100000;
            %let MAMsg=Macro[&sysmacroname.] Error in Updating Contact History target table. SQLXRC=[&SQLXRC.] and SQLXMSG=[&SQLXMSG.]. Check the log for details.;
            %return;
        %end;
        execute (commit) by &dbengine.;
    %end;/* End of Update Contact record from Input table */
    /* Above changes are not committed until a disconnect or new connection */
    disconnect from &dbengine.;
quit; /* End of Explicit Pass-through Proc SQL block */
%if &MADebug. eq Y %then
%do;
    %put MACHEUST macro End Time - %sysfunc(datetime(), datetime24.2);
%end;
%mend;
