/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/*****************************************************************************************/
/* NAME: macompub.sas                                                                    */
/* VERSION: 5.4                                                                        */
/*                                                                                       */
/* DESCRIPTION: Macro to update or insert commuication records for the existing          */
/*              existing campaign.  This also inserts records to all susequent           */
/*              tables if any new records exisits for the new communication              */
/*              records inserted.                                                        */
/* Parameter:                                                                            */
/*    comm          - Physical table name for Communication                              */
/*    comm_char_udf - Physical table name for Communication_Char_udf                     */
/*    comm_num_udf  - Physical table name for Communication_Num_udf                      */
/*    comm_dt_udf   - Physical table name for Communication_Date_udf                     */
/*    comm_ext      - Physical table name for Communication_Ext                          */
/*    ccell         - Physical table name for Marketing_Cell                             */
/*    ccp           - Physical table name for Cell_Package                               */
/*    ccpt          - Physical table name for Cell_Package_Treatment_Count               */
/* PRODUCT: MA                                                                           */
/* USAGE: %macompub                                                                      */
/*                                                                                       */
/*****************************************************************************************/
/*****************************************************************************************/
/*                                                                                       */
/* History:                                                                              */
/*   27APR2007 - Revised first version of the macro.                                     */
/*                                                                                       */
/* S0490547 - 5 new Budget columns added to CI_Communication table                       */
/*                                                                                       */
/* DATE       Defect No  Comments                                                        */
/* 14Oct2008  S0504678   Added PACKAGE_SK and CONTROL_GROUP_TYPE_CD to treament_cnt      */
/*                       table part of the 5.3 data model changes.                       */
/*                                                                                       */
/* 18Jun2009  S0583163   Reset the SYSCC value to 100000 in case of error                */
/*                                                                                       */
/* 16Jul2009  S0554379   Eliminate the additional white space created from the           */
/*                       put statement for all character variable                        */
/*                                                                                       */
/* 29Jan2010  a2ma54     New Column COMMUNICATION_RECURR_TYPE_CD added                   */
/*                                                                                       */
/* 26Mar2010  a2ma54     a)New column added for Offer arbitration                        */
/*                       CI_CELL_PACKAGE#PACKAGE_HASH_VAL                                */
/*                       b)Moved out the Treatment_Cnt table processing to macptpub      */
/*                                                                                       */
/* 12May2010  5.4Iteration5 Publish DELETED_FLG to CI_COMMUNICATION table                */
/*                                                                                       */
/* 18May2010  S0670074   Remove Package_Hash_Val from cell_package table                 */
/*                                                                                       */
/* 17Aug2010  S0692895   use length of COMMUNICATION_DESC for comm_desc                  */
/*                                                                                       */
/* 16Sep2010  Task-TA559 New column CI_COMMUNICATIONS#BUDGET_UNIT_USAGE_AMT              */
/*                                                                                       */
/* 06Jan2012  S0782404   Fix the COMM UDF table population                               */
/* 26Feb2013  S0937584   541MA07: Publish error on DTTM fields for CI_CAMPAIGN...        */
/* 14Aug2013  S0996040   replace dswriteopts with dsinsopts                              */
/*****************************************************************************************/

%macro macompub(comm=, comm_char_udf=, comm_num_udf=, comm_dt_udf=,
                comm_ext=, ccell=, ccp=, ccpt=);


/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put Current Version of the code: 541MA09;
    %put Last Modified on 14Aug2013;
    %put MACOMPUB macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;


/* 5.4Iteration5 - Mark Deleted Communications if any */
/* Check if any communication is deleted */
%marowcnt( &comm_deleted. , DeletedComs);

%if &DeletedComs. > 0 %then
%do;
    proc sql;
        update  &libnm..&comm
        set     deleted_flg = 'Y',
                processed_dttm="&pdttm"dt
        where   communication_sk in (select communication_sk from &comm_deleted.)
    ;quit;

    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError=812;
            %let MAMsg = Macro[&sysmacroname.]. Publish failed with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
        %end;
    %end;
    %else %if &SQLRC > 4  %then
    %do;
        %let SYSCC=100000;
        %let MAError=812;
        %let MAMsg=Macro[&sysmacroname.]. PROC SQL error while attempting to update CI_COMMUNICATION SQLRC[&sqlrc]. Check the log for details;
    %end;
%end;

/* S0782404 - When all communication records are inserted part of the new campaign insert or
   no communicaiton record in incremental publish then exit this code */
%marowcnt (&comm., comm_cnt);

%if &comm_cnt.=0 or &initial_pub.=Y %then
%do;
    %return;
%end;

/* S0692895 use _comm_desc=COMMUNICATION_DESC such that it uses the length of COMMUNICATION_DESC column defined in publish map file */
/* Dynamic source code to udpate or insert communication records and subsequent tables */
data _null_;
    file &commsrc. lrecl=32767;
    set work.&comm. end=done;
    _comm_desc=COMMUNICATION_DESC;
    _comm_cd=tranwrd(COMMUNICATION_CD,'"','""');
    _comm_nm=tranwrd(COMMUNICATION_NM,'"','""');
    _comm_desc=tranwrd(COMMUNICATION_DESC,'"','""');

    if insertflg = 0 then
    do;
        if _n_=1 then
        do;
            put '%macro publish_comm; ';
        end;


        put ' proc sql; ';
        put '   SELECT COUNT(COMMUNICATION_SK) into :comm_upd_cnt FROM ' "&libnm..&comm" ' WHERE COMMUNICATION_SK = ' COMMUNICATION_SK ';';
        put ' quit; ';

        put '%if &comm_upd_cnt > 0 %then';
        put '%do ;';

        /* 5.4Iteration5 */
        /* Task-TA559 */
        put 'proc sql;';
        put '    update ' "&libnm..&comm";
        put '        set ';
        put '            CAMPAIGN_SK = ' CAMPAIGN_SK',';
        put '            COMMUNICATION_CD = "' _comm_cd +(-1) '",';
        put '            COMMUNICATION_OCCURRENCE_NO = ' COMMUNICATION_OCCURRENCE_NO',';
        put '            COMMUNICATION_RECURR_TYPE_CD = "' COMMUNICATION_RECURR_TYPE_CD +(-1) '",';
        put '            COMMUNICATION_NM = "' _comm_nm +(-1) '",';
        put '            COMMUNICATION_DESC = "' _comm_desc +(-1) '",';
        put '            COMMUNICATION_STATUS_CD = "' COMMUNICATION_STATUS_CD +(-1) '",';
        put '            SUBJECT_TYPE_NM = "' SUBJECT_TYPE_NM +(-1) '",';
        put '            MIN_BUDGET_OFFER_AMT = ' MIN_BUDGET_OFFER_AMT',';
        put '            MAX_BUDGET_OFFER_AMT = ' MAX_BUDGET_OFFER_AMT',';
        put '            MIN_BUDGET_AMT = ' MIN_BUDGET_AMT',';
        put '            MAX_BUDGET_AMT = ' MAX_BUDGET_AMT',';
        put '            BUDGET_UNIT_COST_AMT = ' BUDGET_UNIT_COST_AMT',';
        put '            BUDGET_UNIT_USAGE_AMT = ' BUDGET_UNIT_USAGE_AMT',';
        put '            END_DTTM = "' END_DTTM '"dt,';
        put '            START_DTTM = "' START_DTTM '"dt,';
        put '            EXPORT_DTTM = "' EXPORT_DTTM '"dt,';
        put '            ACTUAL_RESPONSE_RT = ' ACTUAL_RESPONSE_RT ',';
        put '            UPDATE_CONTACT_HISTORY_FLG = "' UPDATE_CONTACT_HISTORY_FLG +(-1) '",';
        put '            DELETED_FLG = "' DELETED_FLG +(-1) '",';
        put '            PROCESSED_DTTM =  "&pdttm"dt ';
        put '    where COMMUNICATION_SK = ' COMMUNICATION_SK';';
        Put 'quit;';
        put ' ';
        put ' ';

        /* If there are errors, initialize the global variables and return. */
        put '%if %symexist(SYSDBRC) %then';
        put '%do;';
        put '     %if &SYSDBRC ^= 0 %then';
        put '     %do;';
        put '         %let SYSCC=100000;';
        put '         %let MAError=812;';
        put '         %let MAMsg = Macro[&sysmacroname.]. Publish failed for ["'%nrbquote( _comm_nm )'"] with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];';
        put '         %goto eof_comm_' _n_ ';';
        put '     %end;';
        put '%end;';
        put '%else %if &SQLRC > 4  %then';
        put '%do;';
        put '    %let SYSCC=100000;';
        put '    %let MAError=812;';
        put '    %let MAMsg=Macro[&sysmacroname.]. PROC SQL error while attempting to update COMMUNICATION ["'%nrbquote( _comm_nm )'"] records SQLRC[&sqlrc]. Check the log for details;';
        put '    %goto eof_comm_' _n_ ';';
        put '%end;';
        put ' ';
        put ' ';

        put '%end;';

        put '%if &comm_upd_cnt = 0 %then';
        put '%do ;';
        /* insert into communication table , subsequently in all communication udfs and cell_package table */
        /* 5.4Iteration5 */
        /* Task-TA559 */
        put '    proc sql;';
        put '        insert into ' "&libnm..&comm" ' ( ';
        put '            COMMUNICATION_SK,';
        put '            CAMPAIGN_SK,';
        put '            COMMUNICATION_CD, ';
        put '            COMMUNICATION_OCCURRENCE_NO,';
        put '            COMMUNICATION_NM,';
        put '            COMMUNICATION_DESC,';
        put '            SUBJECT_TYPE_NM,';
        put '            ACTUAL_RESPONSE_RT,';
        put '            MIN_BUDGET_OFFER_AMT,';
        put '            MAX_BUDGET_OFFER_AMT,';
        put '            MIN_BUDGET_AMT,';
        put '            MAX_BUDGET_AMT,';
        put '            BUDGET_UNIT_COST_AMT,';
        put '            BUDGET_UNIT_USAGE_AMT,';
        put '            START_DTTM,';
        put '            END_DTTM,';
        put '            COMMUNICATION_STATUS_CD,';
        put '            EXPORT_DTTM,';
        put '            UPDATE_CONTACT_HISTORY_FLG,';
        put '            COMMUNICATION_RECURR_TYPE_CD,';
        put '            DELETED_FLG,';
        put '            PROCESSED_DTTM  )';
        put '        select ';
        put '            COMMUNICATION_SK,';
        put '            COMM.CAMPAIGN_SK,';
        put '            COMMUNICATION_CD,';
        put '            COMMUNICATION_OCCURRENCE_NO,';
        put '            COMMUNICATION_NM,';
        put '            COMMUNICATION_DESC,';
        put '            SUBJECT_TYPE_NM,';
        put '            ACTUAL_RESPONSE_RT,';
        put '            COMM.MIN_BUDGET_OFFER_AMT,';
        put '            COMM.MAX_BUDGET_OFFER_AMT,';
        put '            COMM.MIN_BUDGET_AMT,';
        put '            COMM.MAX_BUDGET_AMT,';
        put '            BUDGET_UNIT_COST_AMT,';
        put '            BUDGET_UNIT_USAGE_AMT,';
        put '            COMM.START_DTTM,';
        put '            COMM.END_DTTM,';
        put '            COMMUNICATION_STATUS_CD,';
        put '            EXPORT_DTTM,';
        put '            UPDATE_CONTACT_HISTORY_FLG,';
        put '            COMMUNICATION_RECURR_TYPE_CD,';
        put '            comm.DELETED_FLG,';
        put '            "&pdttm"dt ';
        put '        from ' "work.&comm" ' comm ';
        put '              where comm.COMMUNICATION_SK = ' COMMUNICATION_SK';';
        put '    quit;';
        put ' ';
        put ' ';

        /* If there are errors, initialize the global variables and return. */
        put '    %if %symexist(SYSDBRC) %then';
        put '    %do;';
        put '        %if &SYSDBRC ^= 0 %then';
        put '        %do;';
        put '            %let SYSCC=100000;';
        put '            %let MAError=813;';
        put '            %let MAMsg = Macro[&sysmacroname.]. Publish failed for ["'%nrbquote( _comm_nm )'"] with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];';
        put '            %goto eof_comm_' _n_ ';';
        put '        %end;';
        put '    %end;';
        put '    %else %if &SQLRC > 4  %then';
        put '    %do;';
        put '        %let SYSCC=100000;';
        put '        %let MAError =813;';
        put '        %let MAMsg =Macro[&sysmacroname.]. PROC SQL error while attempting to Insert COMMUNICATION ["'%nrbquote( _comm_nm )'"] records SQLRC[&sqlrc]. Check the log for details;';
        put '        %goto eof_comm_' _n_ ';';
        put '    %end;';
        put ' ';
        put ' ';

        /* Communication_Chr_Udf: Subset*/
        %if &comm_cudf_cnt. > 0 %then
        %do;
            /* S0782404: Added pdttm to the subset table so to use the subset table directly in append process */
            put '    data work.CommCudfSubset ' "work.&comm_char_udf" ';';
            put '        set ' "work.&comm_char_udf" ';';
            put '        if COMMUNICATION_SK= ' COMMUNICATION_SK ' and insertflg=0 then';
            put '        do;';
            put '            insertflg=1;';
            put '            processed_dttm="&pdttm"dt;';
            put '            output work.CommCudfSubset;';
            put '        end;';
            put '        else if insertflg=0 then';
            put '        do;';
            put '            processed_dttm="&pdttm"dt;';
            put '            output ' "work.&comm_char_udf" ';';
            put '        end;';
            put '    run;';
            put ' ';
            put ' ';

            /* If there are errors, initialize the global variables and return. */
            put '    %if &SYSERR > 4  %then ';
            put '    %do;';
            put '        %let MAError =723;';
            put '        %let MAMsg = Macro[&sysmacroname.]. Data Step failed - while attempting to create temp dataset. Check the log for details.;';
            put '        %goto eof_comm_' _n_ ';';
            put '    %end;';
            put ' ';
            put ' ';
            /* S0996040 */
            put 'PROC APPEND BASE=&libnm..&comm_char_udf (&dsinsopts.) DATA=work.CommCudfSubset(DROP=INSERTFLG) FORCE;';;
            put ' ';

            /* If there are errors, initialize the global variables and return. */
            put '    %if %symexist(SYSDBRC) %then';
            put '    %do;';
            put '        %if &SYSDBRC ^= 0 %then';
            put '        %do;';
            put '            %let SYSCC=100000;';
            put '            %let MAError=814;';
            put '            %let MAMsg = Macro[&sysmacroname.]. Publish failed for ["'%nrbquote( _comm_nm )'"] with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];';
            put '            %goto eof_comm_' _n_ ';';
            put '        %end;';
            put '    %end;';
            put '    %else %if &SQLRC > 4  %then';
            put '    %do;';
            put '        %let MAError =814;';
            put '        %let MAMsg =Macro[&sysmacroname.]. PROC SQL error while attempting to Insert COMMUNICATION_CHAR_UDF for Communication ["'%nrbquote( _comm_nm )'"] records SQLRC[&sqlrc]. Check the log for details;';
            put '        %goto eof_comm_' _n_ ';';
            put '    %end;';
            put ' ';
            put ' ';
        %end;

        /* Communication_Num_Udf: Subset*/
        %if &comm_nudf_cnt. > 0 %then
        %do;
            /* Communication_Num_Udf: Subset*/
            /* The Base data set created to make use in the Transpose step */
            put '    data work.CommNUdfSubset ' "work.&comm_num_udf" ';';
            put '        length insertflg 8;';
            put '        set ' "work.&comm_num_udf" ';';
            put '        if COMMUNICATION_SK=' COMMUNICATION_SK' and insertflag=0 then';
            put '        do;';
            put '            insertflg=1;';
            put '            processed_dttm="&pdttm"dt;';
            put '            output work.CommNUdfSubset;';
            put '        end;';
            put '        else if insertflg=0 then';
            put '        do;';
            put '            processed_dttm="&pdttm"dt;';
            put '            output ' "work.&comm_num_udf" ';';
            put '        end;';
            put '    run;';
            put ' ';
            put ' ';

            /* If there are errors, initialize the global variables and return. */
            put '    %if &SYSERR > 4  %then ';
            put '    %do;';
            put '        %let MAError =723;';
            put '        %let MAMsg = Macro[&sysmacroname.]. Data Step failed - while attempting to create temp dataset. Check the log for details.;';
            put '        %goto eof_comm_' _n_ ';';
            put '    %end;';
            put ' ';
            put ' ';
            /* S0996040 */
            put 'PROC APPEND BASE=&libnm..&comm_num_udf (&dsinsopts.) DATA=work.CommNUdfSubset(DROP=INSERTFLG) FORCE;';
            put ' ';

            /* If there are errors, initialize the global variables and return. */
            put '    %if %symexist(SYSDBRC) %then';
            put '    %do;';
            put '        %if &SYSDBRC ^= 0 %then';
            put '        %do;';
            put '            %let SYSCC=100000;';
            put '            %let MAError=815;';
            put '            %let MAMsg = Macro[&sysmacroname.]. Publish failed for ["'%nrbquote( _comm_nm )'"] with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];';
            put '            %goto eof_comm_' _n_ ';';
            put '        %end;';
            put '    %end;';
            put '    %else %if &SQLRC > 4  %then';
            put '    %do;';
            put '        %let SYSCC=100000;';
            put '        %let MAError =815;';
            put '        %let MAMsg =Macro[&sysmacroname.]. PROC SQL error while attempting to Insert COMMUNICATION_NUM_UDF for Communication ["'%nrbquote( _comm_nm )'"] records SQLRC[&sqlrc]. Check the log for details;';
            put '        %goto eof_comm_' _n_ ';';
            put '    %end;';
            put ' ';
            put ' ';
        %end;

        /* Communication_Date_Udf: Subset*/
        %if &comm_dudf_cnt. > 0 %then
        %do;
            /* Communication_Date_Udf: Subset*/
            /* The Base data set created to make use in the Transpose step */
            put '    data work.CommDUdfSubset ' "work.&comm_dt_udf" ';';
            put '        length insertflg 8;';
            put '        set ' "work.&comm_dt_udf" ';';
            put '        if COMMUNICATION_SK=' COMMUNICATION_SK ' and insertflg=0 then';
            put '        do;';
            put '            insertflg=1;';
            put '            processed_dttm="&pdttm"dt;';
            put '            output work.CommDUdfSubset;';
            put '        end;';
            put '        else if insertflg=0 then';
            put '        do;';
            put '            processed_dttm="&pdttm"dt;';
            put '            output ' "work.&comm_dt_udf" ';';
            put '        end;';
            put '    run;';
            put ' ';
            put ' ';

            /* If there are errors, initialize the global variables and return. */
            put '    %if &SYSERR > 4  %then ';
            put '    %do;';
            put '        %let MAError =723;';
            put '        %let MAMsg = Macro[&sysmacroname.]. Data Step failed - while attempting to create temp dataset. Check the log for details.;';
            put '        %goto eof_comm_' _n_ ';';
            put '    %end;';
            put ' ';
            put ' ';


            /* S0996040 */
            put 'PROC APPEND BASE=&libnm..&comm_dt_udf (&dsinsopts.) DATA=work.CommDUdfSubset(DROP=INSERTFLG) FORCE;';
            put ' ';

            /* If there are errors, initialize the global variables and return. */
            put '    %if %symexist(SYSDBRC) %then';
            put '    %do;';
            put '        %if &SYSDBRC ^= 0 %then';
            put '        %do;';
            put '            %let SYSCC=100000;';
            put '            %let MAError=816;';
            put '            %let MAMsg = Macro[&sysmacroname.]. Publish failed for ["'%nrbquote( _comm_nm )'"] with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];';
            put '            %goto eof_comm_' _n_ ';';
            put '        %end;';
            put '    %end;';
            put '    %else %if &SQLRC > 4  %then';
            put '    %do;';
            put '        %let SYSCC=100000;';
            put '        %let MAError =816;';
            put '        %let MAMsg =Macro[&sysmacroname.]. PROC SQL error while attempting to Insert COMMUNICATION_DATE_UDF for Communication ["'%nrbquote( _comm_nm )'"] records SQLRC[&sqlrc]. Check the log for details;';
            put '        %goto eof_comm_' _n_ ';';
            put '    %end;';
            put ' ';
            put ' ';
        %end;

        /* Cell_Package: Subset Cell_Package based on communication_sk */
        put '    data work.ccpSubset ' "work.&ccp" ';';
        put '        set ' "work.&ccp" ';';
        put '        if COMMUNICATION_SK = ' COMMUNICATION_SK 'then ';
        put '        do;';
        put '            insertflg=1;';
        put '            output work.ccpsubset;';
        put '        end;';
        put '        else';
        put '        do;';
        put '            insertflg=0;';
        put '            output ' "work.&ccp" ';';
        put '        end;';
        put '    run;';
        put ' ';
        put ' ';

        /* If there are errors, initialize the global variables and return. */
        put '    %if &SYSERR > 4  %then ';
        put '    %do;';
        put '        %let MAError =723;';
        put '        %let MAMsg = Macro[&sysmacroname.]. Data Step failed - while attempting to create temp dataset. Check the log for details.;';
        put '        %goto eof_comm_' _n_ ';';
        put '    %end;';
        put ' ';
        put ' ';

        /* Cell_Package: Insert subset table to target Cell_Package table */
        put 'data work.ccpInsert;';
        put '   set work.ccpsubset;';
        put '   TOTAL_MARKETING_CONTACT_CNT = 0;';
        put '   TOTAL_ATTEMPTED_CONTACT_CNT = 0;';
        put '   TOTAL_RESPONSE_CNT = 0;';
        put '   TOTAL_SUCCESSFUL_RESPONSE_CNT = 0;';
        put '   PROCESSED_DTTM = "&pdttm"dt;';
        put 'run;';

        /* S0996040 */
        put 'proc append base=&libnm..&ccp (&dsinsopts.) data=work.ccpInsert(DROP=INSERTFLG) FORCE;';

        /* If there are errors, initialize the global variables and return. */
        put '    %if %symexist(SYSDBRC) %then';
        put '    %do;';
        put '        %if &SYSDBRC ^= 0 %then';
        put '        %do;';
        put '            %let SYSCC=100000;';
        put '            %let MAError=817;';
        put '            %let MAMsg = Macro[&sysmacroname.]. Publish failed for ["'%nrbquote( _comm_nm )'"] with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];';
        put '            %goto eof_comm_' _n_ ';';
        put '        %end;';
        put '    %end;';
        put '    %else %if &SQLRC > 4  %then';
        put '    %do;';
        put '        %let SYSCC=100000;';
        put '        %let MAError =817;';
        put '        %let MAMsg =Macro[&sysmacroname.]. PROC SQL error while attempting to insert CELL_PACKAGE records for COMMUNICATION ["'%nrbquote( _comm_nm )'"] SQLRC[&sqlrc]. Check the log for details;';
        put '        %goto eof_comm_' _n_ ';';
        put '    %end;';
        put ' ';
        put ' ';
        /* The CI_CELL_PACKAGE_TREATMENT_CNT table update moved out from this place to macptpub macro processing. Now instead
           populating this table per campaign/communication it will be processed only once at the end of publish process.
           This is introduced populate the ci_package_x_treatment once after populating all cell_package records and
           create the correct input set for treatment_cnt table.
        */

        put ' %end; '; /* End of comm_upd_cnt=0 block */

        put ' %eof_comm_' _n_ ':';
        put ' ';
        put ' ';

        if done then
        do;
             put '%mend;';
             put '%publish_comm;';
        end;
    end;
run;

%include &commsrc.;

%if &SYSERR. > 4  %then
%do;
    %let MAError =721;
    %let MAMsg =Macro[&sysmacroname.]. Error in Data Step while attempting to create dynamic source code file. Check the log for details.;
%end;

%if &MADebug. eq Y %then
%do;
    %put MACOMPUB End Time - %sysfunc(datetime(), datetime24.2);
%end;

%mend;
