/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/************************************************************************************/
/* NAME: macomudf.sas                                                               */
/* VERSION: 5.4                                                                     */
/*                                                                                  */
/* DESCRIPTION: Macro to update or insert Communication UDF records which are not   */
/*              included part of the new campaign record insert process             */
/*    comm_char_udf - Physical table name for Communication_Char_udf                */
/*    comm_num_udf  - Physical table name for Communication_Num_udf                 */
/*    comm_dt_udf   - Physical table name for Communication_Date_udf                */
/*                                                                                  */
/* PRODUCT: MA                                                                      */
/* USAGE: %macomudf                                                                 */
/*                                                                                  */
/************************************************************************************/
/*****************************************************************************************/
/*                                                                                       */
/* History:                                                                              */
/*   27Apr2007 - Revised first version of the macro.                                     */
/*                                                                                       */
/* DATE       Defect No  Comments                                                        */
/* 18Jun2009  S0583163   Reset the SYSCC value to 100000 in case of error                */
/*                                                                                       */
/* 22Jun2009  S0575393   Allow the missing values from the input set to overwrite the    */
/*                       master table. This is mainly required when default UDF values   */
/*                       are reset to blank.                                             */
/*                                                                                       */
/* 06Jan2012  S0782404   Fix the COMM UDF table population                               */
/* 14Aug2013  S1000142   add  dswriteopts in data modify                                 */
/*****************************************************************************************/

%macro macomudf(comm_char_udf=, comm_num_udf=, comm_dt_udf=);

%put macomudf Start Time - %sysfunc(datetime(), datetime24.2);


/* S0782404 During initial publish all the communication udf records are inserted by the dynamic source created in mapub script.
   Hence nothing to be processed from this macro */
%if &initial_pub=Y %then
%do;
    %return;
%end;

/* Communication UDF records can be added to the existing communication node by changing the definition. Hence
   re-counting the comm udf records after the communicaiton insert process is required here */
%if &comm_cudf_cnt. > 0 %then
%do;
    %marowcnt( &comm_char_udf. , comm_cudf_cnt);
%end;

%if &comm_cudf_cnt. > 0 %then
%do;
    /* Communication_Char_Udf: Updates and Inserts for already existing Communication */
    data &libnm..&comm_char_udf.;
        modify &libnm..&comm_char_udf. (&dswriteopts.) work.&comm_char_udf. UPDATEMODE=NOMISSINGCHECK;
        by COMMUNICATION_SK CHAR_UDF_NM;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                processed_dttm="&pdttm"dt;
                replace;
            end;
            when(%SYSRC(_DSENMR))
            do;
                processed_dttm="&pdttm"dt;
                _error_ = 0;
                output;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;

    /* Exit the process if any error */
    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC. ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError = 829;
            %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
        %end;
    %end;
    %else %if &SYSERR.> 4  %then
    %do;
        %let MAError =829;
        %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update CAMPAIGN_CHAR_UDF records SYSERR[&SYSERR.]. Check the log for details;
    %end;
%end;

%if &comm_num_udf. > 0 %then
%do;
    %marowcnt( &comm_num_udf. , comm_nudf_cnt);
%end;

%if &comm_nudf_cnt. > 0 %then
%do;
    /* Communication_Num_Udf: Updates and Inserts for already existing Communication */
    data &libnm..&comm_num_udf.;
        modify &libnm..&comm_num_udf. (&dswriteopts.) work.&comm_num_udf. UPDATEMODE=NOMISSINGCHECK;
        by COMMUNICATION_SK NUM_UDF_NM;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                processed_dttm="&pdttm"dt;
                replace;
            end;
            when(%SYSRC(_DSENMR))
            do;
                processed_dttm="&pdttm"dt;
                _error_ = 0;
                output;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;

    /* Exit the process if any error */
    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC. ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError = 831;
            %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
        %end;
    %end;
    %else %if &SYSERR.> 4  %then
    %do;
        %let MAError =831;
        %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update CAMPAIGN_NM_UDF records SYSERR[&SYSERR.]. Check the log for details;
    %end;
%end;

%if &comm_dt_udf. > 0 %then
%do;
    %marowcnt( &comm_dt_udf. , comm_dudf_cnt);
%end;

%if &comm_dudf_cnt. > 0 %then
%do;
    /* Communication_Date_Udf: Updates and Inserts for already existing Communication */
    data &libnm..&comm_dt_udf;
        modify &libnm..&comm_dt_udf (&dswriteopts.) work.&comm_dt_udf. UPDATEMODE=NOMISSINGCHECK;
        by COMMUNICATION_SK DATE_UDF_NM;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                processed_dttm="&pdttm"dt;
                replace;
            end;
            when(%SYSRC(_DSENMR))
            do;
                processed_dttm="&pdttm"dt;
                _error_ = 0;
                output;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;

    /* Exit the process if any error */
    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC. ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError = 832;
            %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
        %end;
    %end;
    %else %if &SYSERR.> 4  %then
    %do;
        %let MAError =832;
        %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update CAMPAIGN_DT_UDF records SYSERR[&SYSERR.]. Check the log for details;
    %end;
%end;

%put macomudf end Time - %sysfunc(datetime(), datetime24.2);
%mend;
