/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/************************************************************************************/
/* NAME: macptpub.sas                                                               */
/* VERSION: 5.4                                                                   */
/*                                                                                  */
/* DESCRIPTION: Macro to update the CI_Cell_Package and                             */
/*              CI_Cell_Package_Treatment_Cnt table.  CI_Cell_Package is inserted   */
/*             with new records which is not inserted part of the new campaign and  */
/*             communication insert otherwise it updates the existing records.      */
/*             CI_Cell_Package_Treatment_Cnt table is inserted with all the         */
/*             Cell_Package records wether it has got a Treatment_Sk associated     */
/*             with it or not.  Note the Count Columns in the both the tables are   */
/*             not touched part of Publish process and it is 0 for newly inserted   */
/*             records otherwise un-touched                                         */
/* Parameter:                                                                       */
/*        ccp - Physical table name for CI_Cell_Package                             */
/*        ccpt - Physical table name for CI_Cell_Package_Treatment_Count            */
/*                                                                                  */
/* PRODUCT: MA                                                                      */
/* USAGE: %macptpub                                                                 */
/*                                                                                  */
/************************************************************************************/
/*****************************************************************************************/
/*                                                                                       */
/* History:                                                                              */
/*   27APR2007 - First version of the macro.                                             */
/*                                                                                       */
/* DATE       Defect No  Comments                                                        */
/* 18Jun2009  S0583163   Reset the SYSCC value to 100000 in case of error                */
/*                                                                                       */
/* 22Jun2009  S0575393   Allow the missing values from the input set to overwrite the    */
/*                       master table. This is mainly required when default UDF values   */
/*                       are reset to blank.                                             */
/*                                                                                       */
/* 31Mar2010  5.4        %mapxt & %macptpub included in this macro to populate           */
/*                       package_x_treatment and create input set for treatment_cnt table*/
/*                                                                                       */
/* 31Jan2012  S0827991   Fix the commuication occurrence and treatment association       */
/*****************************************************************************************/

%macro macptpub(ccp=, ccpt=);

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put Current Version of the code: MA5.41-Hotfix#2;
    %put Last Modified on 31Jan2012;
    %put MACPTPUB macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

/* The Count columns are not part of the input XML and hence not available in the input data set.  So add that
   separately when a record gets added to the respective cnt table */

/* Cell_Package: Update or Insert */
data &libnm..&ccp.;
    modify &libnm..&ccp. (&dswriteopts.) work.&ccp. UPDATEMODE=NOMISSINGCHECK;
    by CELL_PACKAGE_SK;
    select (_IORC_);
        when (%SYSRC(_SOK))
        do;
            processed_dttm="&pdttm"dt;
            replace;
        end;
        when (%SYSRC(_DSENMR))
        do;
            TOTAL_MARKETING_CONTACT_CNT=0;
            TOTAL_ATTEMPTED_CONTACT_CNT=0;
            TOTAL_RESPONSE_CNT=0;
            TOTAL_SUCCESSFUL_RESPONSE_CNT=0;
            processed_dttm="&pdttm"dt;
            _error_ = 0;
            output;
        end;
        otherwise
        do;
            stop;
        end;
    end;
run;

/* Exit the process if any error */
%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC. ^= 0 %then
    %do;
        %let SYSCC=100000;
        %let MAError = 845;
        %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
%end;
%else %if &SYSERR. > 4  %then
%do;
    %let MAError =845;
    %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update CELL_PACKAGE records SYSERR[&SYSERR.]. Check the log for details;
%end;

/* S0827991- Populate the cpxt and ccpt table only during Full publish and Pre Execution publish.  This is to
   make sure that we don't have any unwanted treatment records recorded in ccpt table which will lead to confusion in reporting */

%put publishtype=&publishtype.;

%if %upcase(&publishtype.) eq PREEXEC or %upcase(&publishtype.) eq FULL %then
%do;
    /* Populate the CI_Package_X_Treatment table */
    %mapxt(cpxt=&cpxt.);

    /* Create treatment_cnt input data set */
    %mapubutl(ccp=&ccp., cp=&cp., cpxt=&cpxt.,ct=&ct., resulttbl=&ccpt.);

    %marowcnt(&ccpt., ccpt_cnt);

    %if &ccpt_cnt > 0 %then
    %do;

        /* Cell_Package_Treatment_Count: Update or Insert */
        data &libnm..&ccpt.;
            modify &libnm..&ccpt. (&dswriteopts.) work.&ccpt. UPDATEMODE=NOMISSINGCHECK;
            by CELL_PACKAGE_SK TREATMENT_SK;
            select (_IORC_);
                when (%SYSRC(_SOK))
                do;
                    processed_dttm="&pdttm"dt;
                    replace;
                end;
                when (%SYSRC(_DSENMR))
                do;
                    TOTAL_MARKETING_CONTACT_CNT=0;
                    TOTAL_ATTEMPTED_CONTACT_CNT=0;
                    TOTAL_RESPONSE_CNT=0;
                    TOTAL_SUCCESSFUL_RESPONSE_CNT=0;
                    processed_dttm="&pdttm"dt;
                    _error_ = 0;
                    output;
                end;
                otherwise
                do;
                    stop;
                end;
            end;
        run;

        /* Exit process if any error */
        %if %symexist(SYSDBRC) %then
        %do;
            %if &SYSDBRC. ^= 0 %then
            %do;
                %let SYSCC=100000;
                %let MAError = 846;
                %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
            %end;
        %end;
        %else %if &SYSERR. > 4  %then
        %do;
            %let MAError =846;
            %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update CELL_PACKAGE_TREATMENT_COUNT records SYSERR[&SYSERR.]. Check the log for details;
        %end;
    %end; /* End for ccpt count check */
%end; /* End for purposetype flag check */

%if &MADebug. eq Y %then
%do;
    %put MACPTPUB End Time - %sysfunc(datetime(), datetime24.2);
%end;

%mend;
