/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/************************************************************************************/
/* NAME: mactcs.sas                                                                 */
/* VERSION: 5.4                                                                   */
/*                                                                                  */
/* DESCRIPTION: Macro to Update/Insert Treatment_Campaign_Set records               */
/* Parameter:                                                                       */
/*       ct_camp_set - Physical table name for TREATMENT_CAMPAIGN_SET               */
/*       ctcsxc      - Physical table name for TRETMENT_CAMPAIGN_SET_X_CAMPAIGN     */
/*       purpose     - Purpose flag (TREATMENT_CAMPAIGN_SET or                      */
/*                                   TREATMENT_CAMPAIGN_SET_DELETION)               */
/*                                                                                  */
/* PRODUCT: MA                                                                      */
/* USAGE: %mactcs                                                                   */
/*                                                                                  */
/************************************************************************************/
/*****************************************************************************************/
/*                                                                                       */
/* History:                                                                              */
/*   30MAR2010 - First version of the macro.                                             */
/*                                                                                       */
/* DATE       Defect No  Comments/updated by                                             */
/* 24May2010  CI5.4      Clean Publish for Treatment_X_Campaign_Set                      */
/* 23Aug2013  S0992864   Add dswriteopts to data modify                                  */
/*****************************************************************************************/

%macro mactcs(ct_camp_set=,ctcsxc=,purpose=);

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put Current Version of the code: 541MA09;
    %put Last Modified on 23Aug2013;
    %put MACTCS macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

/* Populate the Treatment_Campaign_set tables */
%if %upcase(&purpose.) eq TREATMENT_CAMPAIGN_SET %then
%do;
    data &libnm..&ct_camp_set.;
        modify &libnm..&ct_camp_set. (&dswriteopts.) work.&ct_camp_set. UPDATEMODE=NOMISSINGCHECK;
        by TREATMENT_CAMPAIGN_SET_SK;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                processed_dttm="&pdttm"dt;
                replace;
            end;
            when(%SYSRC(_DSENMR))
            do;
                processed_dttm="&pdttm"dt;
                _error_ = 0;
                output;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;

    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError=886;
            %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to update TREATMENT_CAMPAIGN_SET records SYSDBRC[&SYSDBRC]. Check the log for details;
       %end;
    %end;
    %else %if &SQLRC > 4 %then
    %do;
        %let SYSCC=100000;
        %let MAError=886;
        %let MAMsg=Macro[&sysmacroname.]. Data Modify stmt error while attempting to update TREATMENT_CAMPAIGN_SET records SYSDBRC[&SYSDBRC]. Check the log for details;
    %end;

    %if %upcase(&cleanpublish.) eq Y %then
    %do;
        proc sql;
            create table work.tmt_camp_set_sks as
                select distinct treatment_campaign_set_sk
                from &ct_camp_set.;
        quit;

        %if &SQLRC. > 4 or &SYSCC. > 4 %then
        %do;
            %let MAError =885;
            %let MAMsg =Macro[&sysmacroname.]. Error while attempting to delete records . Check the log for details;
            %return;
        %end;

        %mapubdel(sqlvar=treatment_campaign_set_sk, src_tbl=tmt_camp_set_sks, tgt_tbl=&ctcsxc.);

        %if &MAError. ne %str() %then
        %do;
            %return;
        %end;
    %end;

    /* CI_TREAT_CAMP_SET_X_CAMPAIGN: Update or Insert */
    data &libnm..&ctcsxc.;
        modify &libnm..&ctcsxc. (&dswriteopts.) work.&ctcsxc.;
        by TREATMENT_CAMPAIGN_SET_SK CAMPAIGN_SK;
        select(_IORC_);
            when (%SYSRC(_SOK))
            do;
                processed_dttm="&pdttm"dt;
                replace;
            end;
            when (%SYSRC(_DSENMR))
            do;
                processed_dttm="&pdttm"dt;
                _error_ = 0;
                output;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;

    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError=886;
            %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to update TREATMENT_CAMPAIGN_SET records SYSDBRC[&SYSDBRC]. Check the log for details;
       %end;
    %end;
    %else %if &SQLRC > 4 %then
    %do;
        %let SYSCC=100000;
        %let MAError=886;
        %let MAMsg=Macro[&sysmacroname.]. Data Modify stmt error while attempting to update TREATMENT_CAMPAIGN_SET records SYSDBRC[&SYSDBRC]. Check the log for details;
    %end;
%end;
%else %if %upcase(&purpose.) eq TREATMENT_CAMPAIGN_SET_DELETION %then
%do;
     data &libnm..&ct_camp_set.;
        modify &libnm..&ct_camp_set. (&dswriteopts.)
               work.&ct_camp_set.(keep=treatment_campaign_set_sk deleted_flg last_modified_dttm last_modified_by_user_nm);
        by TREATMENT_CAMPAIGN_SET_SK;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                processed_dttm="&pdttm"dt;
                replace;
            end;
            otherwise;
        end;
    run;

    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError=886;
            %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to update TREATMENT_CAMPAIGN_SET records SYSDBRC[&SYSDBRC]. Check the log for details;
       %end;
    %end;
    %else %if &SQLRC > 4 %then
    %do;
        %let SYSCC=100000;
        %let MAError=886;
        %let MAMsg=Macro[&sysmacroname.]. Data Modify stmt error while attempting to update TREATMENT_CAMPAIGN_SET records SYSDBRC[&SYSDBRC]. Check the log for details;
    %end;
%end;

%if &MADebug. eq Y %then
%do;
    %put MACTCS macro End Time - %sysfunc(datetime(), datetime24.2);
%end;
%mend;
