/***********************************************************************
 * madsopts
 *
 * Macro for setting DATA STEP options for MA processing operations
 *
 * Parameters:
 * libnm       :  Macro variable containing the library stmt
 * libnmMacro  :  Macro variable name containing the library stmt
 * operation   :  P  = Publish
 *                CH = Contact History
 ***********************************************************************/

%macro madsopts(libnm, libnmMacro, operation=);

%global dsreadopts dswriteopts dsdelopts dsinsopts;
%let dsreadopts=%str( );
%let dswriteopts=%str( );
%let dsdelopts=%str( );
%let dsinsopts=%str( );

%let dbengine = %upcase(%qscan(&libnm.,3,%str( )));

/* S0948274  CONNECTION=UNIQUE not required by all db's */
%if &dbengine. = NETEZZA %then /* Add CONNECTION=UNIQUE & AUTOCOMMIT=YES for Netezza */
%do;
    %malibupd( %str(&libnm.), &libnmMacro, CONNECTION AUTOCOMMIT , UNIQUE YES);
%end;
%else %if &dbengine. = TERADATA %then /* Add CONNECTION=UNIQUE for Teradata */
%do;
    %malibupd( %str(&libnm.), &libnmMacro, CONNECTION, UNIQUE );
%end;
%else %if &dbengine. = ORACLE %then /* Add UPDATE_LOCK_TYPE = ROW for Oracle */
%do;
    %malibupd( %str(&libnm.), &libnmMacro, UPDATE_LOCK_TYPE, ROW );
%end;

%if &operation eq P %then %do;

    %if &dbengine. eq TERADATA %then %do;
        %let dswriteopts=READ_LOCK_TYPE=TABLE READ_MODE_WAIT=YES READ_ISOLATION_LEVEL=WRITE UPDATE_LOCK_TYPE=TABLE UPDATE_MODE_WAIT=YES UPDATE_ISOLATION_LEVEL=WRITE;
        %let dsreadopts=READ_LOCK_TYPE=TABLE READ_MODE_WAIT=YES READ_ISOLATION_LEVEL=ACCESS;
        %let dsdelopts=READ_LOCK_TYPE=TABLE READ_MODE_WAIT=YES READ_ISOLATION_LEVEL=WRITE UPDATE_LOCK_TYPE=TABLE UPDATE_MODE_WAIT=YES UPDATE_ISOLATION_LEVEL=WRITE;
    %end;

    %if &dbengine. eq DB2 %then %do;
        %let dsdelopts=UPDATE_ISOLATION_LEVEL=RS READ_ISOLATION_LEVEL=RS;
    %end;

    %if &dbengine. eq OLEDB %then %do;
        %let dsdelopts=UPDATE_ISOLATION_LEVEL=RR READ_ISOLATION_LEVEL=RR;
    %end;

%end;

%else %if &operation eq CH %then %do;

    %if &dbengine. eq OLEDB %then %do;
        %let dswriteopts=cntllev=mem UPDATE_ISOLATION_LEVEL=RR;
    %end;
    %if &dbengine. eq TERADATA %then %do;
        %let dsreadopts=READ_LOCK_TYPE=TABLE READ_MODE_WAIT=YES READ_ISOLATION_LEVEL=ACCESS;
    %end;
%end;
/* S0992864 */
%else %if &operation eq DT %then
%do;
    %if &dbengine. eq TERADATA %then
    %do;
        %let dswriteopts=READ_LOCK_TYPE=TABLE READ_MODE_WAIT=YES READ_ISOLATION_LEVEL=WRITE UPDATE_LOCK_TYPE=TABLE UPDATE_MODE_WAIT=YES UPDATE_ISOLATION_LEVEL=WRITE;
    %end;
%end;

%mend;
