/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/* NAME: maexp                                                   */
/* VERSION: 5.41                                                 */
/* DESCRIPTION: Export for MA                                    */
/*                                                               */
/* PRODUCT: MA                                                   */
/* USAGE: %maexp( vards,                                         */
/*            outputName              = ,                        */
/*            outputLocation          = ,                        */
/*            outputType              = ,                        */
/*            appendReplace           =Replace,                  */
/*            headerRow               =Y,                        */
/*            includeLabels           =N,                        */
/*            dedup                   =N,                        */
/*            subjectKeys             =,                         */
/*            appendgroups            =N,                        */
/*            delimchar               =,                         */
/*            quoteRule               ="S",                      */
/*            countOnly               =N,                        */
/*            seedMethod              =none,                     */
/*            memberTable             =,                         */
/*            lowerThreshold          =,                         */
/*            upperThreshold          =,                         */
/*            dbdsoptions             =,                         */
/*            maxValueLength          =,                         */
/*            inputTableDS            =,                         */
/*;           treatmentDelimchar      =,                         */
/*        VarDS - data set with variable information             */
/*                                                               */
/*****************************************************************************************/
/* History-                                                                              */
/* 09/02/2007 - defect S0447121 - changed to DBMS = XLS for all environment              */
/*                                                                                       */
/* 10/24/2007 - defect S0464883 - changed to DBMS=EXCEL for windows and DBMS=XLS  for all*/
/*                              others                                                   */
/* 03/19/2008 - defect S0368146 - modified the error handling  parts , so that files/    */
/*                              datasets will get deleted if export fails for some reason*/
/*                                                                                       */
/* 07/25/2008 - S0523973 Added the code to create work.counts dataset with column        */
/*                       _macount and value 0 , when the export count is 0               */
/*                                                                                       */
/* 07/30/2008 - S0310367 - added code to check if the export path existis , if not then  */
/*                      return proper error message to UI                                */
/*                                                                                       */
/* DATE      Defect No  Comments                                                         */
/*                                                                                       */
/* 17Dec2008 S0520122   Added the code to handle date list in the export                 */
/*                                                                                       */
/* 24Dec2008 S0534301   modified the code to  remove the hardcoded length statement for  */
/*                      variable and proc sql block to create the macro variable         */
/*                                                                                       */
/* 06Feb2008 S0560523   removed conditions before setting the DBMS = EXCEL / XLS Options */
/*                      as DBMS = EXCEL does not work with 64 bit SAS9.2                 */
/*                      but this needs to be checked for DBCS support in SAS9.2          */
/*                                                                                       */
/* 12Feb2009 S0534301   added condition to check if inputTableMeta dataset exists        */
/*                      before proc sql statement                                        */
/*                                                                                       */
/* 26Feb2009 S0504678   Dynamic Treatment support for Selection campaigns                */
/*                                                                                       */
/* 13Mar2009 S0556482   modified to handle tilde character in the format statement       */
/*                                                                                       */
/* 29Apr2009 S0573074   Added Code to check if file exist before existing file import    */
/*                      during export to excel in append mode                            */
/*                                                                                       */
/* 08May2009 S0554913   Added %unquote function to enable macro variable conversion      */
/*                                                                                       */
/* 26May2009 S0401805   Added error handle to check export to readonly location          */
/*                                                                                       */
/* 27May2009 S0312697   Added error handle to check sas dataset open error               */
/*                                                                                       */
/* 02Jul2009 S0528182   Modified Code to set the correct export value for numeric custom */
/*                      tag udf's                                                        */
/*                                                                                       */
/* 03Jul2009 S0532118   if user specify RESPTRACKING_CD as output column name to any     */
/*                      other column name in export definition then the export will      */
/*                      use the actual RESPTRACKING_CD in export                         */
/*                                                                                       */
/* 07Jul2009 S0507380   Added code to remove the dummy row                               */
/*                                                                                       */
/* 09Jul2009 S0570533   Macnvvar macro need not be called when Counts only is selected   */
/*                                                                                       */
/* 17Jul2009 S0524665   Modified to set the length value for Type = 4 & datatype ='C'    */
/*                                                                                       */
/* 27Aug2009 S0598501  Modified to use treatmentDelimchar inplace of hardcoded delimeter */
/*                                                                                       */
/* 04Sep2009 S0606741  Modified to use best. format while reading the treatment list     */
/*                                                                                       */
/* 10Sep2009 S0608126  Added 'origname' column to vards so merging works with aliases    */
/*                                                                                       */
/* 19Nov2009 S0611677  Added datastep to remove RESPTRACKING_CD & PACKAGE_HASH_VAL column*/
/*                     from input matable if exists                                      */
/*                                                                                       */
/* 04Dec2009 S0532118  Do not generate the length statement for RESPTRACKING_CD column   */
/*                                                                                       */
/* 06Apr2010 S0606741  Modified to use best32 while reading the treatment udf tag list   */
/*                                                                                       */
/* 07May2010 S0612456  Modified to convert the dot to space for treatment udf tag list   */
/*                                                                                       */
/* 08Jun2010 S0606031  added a check to use RESPTRACKING_CD from CCCData when input      */
/*                     table is not present                                              */
/*                                                                                       */
/* 10Jun2010 S0658027  Added check row count in existing file before merge               */
/*                     also updated the indentation                                      */
/*                                                                                       */
/* 25Jun2010 S0594364  Use a common macro for getting the row count of tables            */
/*                                                                                       */
/* 16Aug2010 S0692610  create a dummy member table if there is no inputTableDS           */
/*                                                                                       */
/* 19Aug2010 S0693256  New Feature work: MOMA Priorizitation Method                      */
/*                                                                                       */
/* 22Oct2010 S0674127  Several changes: a) Decouple Export and CH input table            */
/*                     b) Create new table for CH processing which will have all subjects*/
/*                     c) Create a member table as before with dedupped subjects         */
/*                     d) Export records are deduped based on the dedup flag but with in */
/*                     cell (one-many relation only)                                     */
/*                     e) The node count will be always the count of dedupped subjects   */
/*                     f) Non-Subject export will alwyas have distinct records           */
/*                     irrespective of the dedup flag                                    */
/* 28Oct2010 S0712827  create chtable when export variables not present                  */
/*                                                                                       */
/* 29Oct2010 S0656256  Do not append file extension to export filename                   */
/*                                                                                       */
/* 10Dec2010 S0721881  Use the max character variable length                             */
/* 23Apr2012 S0854921  Change UNION to UNION ALL                                         */
/* 18Jun2012 S0808215  while generating export records apply length and format RESPT_CD  */
/*                     as the same variable order is later used in the table type exp    */
/* 17Aug2012 S0822815  Modified for Performance enhancement                              */
/* 15Feb2013 S0927125  MOMA performance optimizations                                    */
/*****************************************************************************************/
%macro maexp( vards,
              outputName              = ,
              outputLocation          = ,
              outputType              = ,
              appendReplace           =Replace,
              headerRow               =Y,
              includeLabels           =N,
              dedup                   =N,
              subjectKeys             =,
              appendgroups            =N,
              delimchar               =,
              quoteRule               ="S",
              countOnly               =N,
              seedMethod              =none,
              memberTable             =,
              lowerThreshold          =,
              upperThreshold          =,
              dbdsoptions             =,
              maxValueLength          =,
              inputTableDS            =,
              treatmentDelimchar      =
            );
    %if &MADebug = Y %then
    %do;
        %put MAEXP Current Version of the code: 541MA04;
        %put Last Modified on 17Aug2012;
        %put Parameters passed to maexp;
        %put Variable data set: &vards;
        %put outputName: &outputName;
        %put outputLocation: &outputLocation;
        %put outputType: &outputType;
        %put appendReplace: &appendReplace;
        %put headerRow: &headerRow;
        %put includeLabels: &includeLabels;
        %put dedup: &dedup;
        %put subjectKeys: &subjectKeys;
        %put appendgroups: &appendgroups;
        %put delimchar: &delimchar;
        %put quoteRule: &quoteRule;
        %put countOnly: &countOnly;
        %put seedMethod: &seedMethod;
        %put memberTable: &memberTable;
        %put lowerThreshold: &lowerThreshold;
        %put upperThreshold: &upperThreshold;
        %put dbdsoptions: &dbdsoptions;
        %put inputTableDS: &inputTableDS;
        %put treatmentDelimchar: &treatmentDelimchar;
        %put chTable: &chtable;
        %put subjectExport: &subjectexport;
        %put End maexp parameters.;
    %end;
    %* Section 1;

    %* set and validate various values;
    %let AppendReplace=%upcase(&AppendReplace);
    %let seedMethod=%upcase(&seedMethod);

    %let cccData = %sysfunc(exist(CCCDATA));
    %let haveVars = %sysfunc(exist(&vards));
    %let perCellData = %sysfunc(exist(PERCELLDATA));
    %let totcnt = 0;

    /* S0507380 remove the dummy row */
    %if &haveVars %then
    %do;
        data &vards ;
            length name outputName $32; /* set to maximum column width */
            format name outputName $32.; /* set to maximum column width */
            set &vards(firstobs=2);/* first is dummy - used to create the column with correct data type */
        run;
    %end;

    %if &perCellData %then
    %do;
       /* make sure that ELIGIBILITY is before ELIGIBILITY_1, etc in the final export order */
       proc sort data=PERCELLDATA SORTSEQ=LINGUISTIC(NUMERIC_COLLATION=ON); by PriorityColumn; run;

       /* add the per cell prioritization score variables to the var list so they can be processed as normal */
       data _appendVars;
          length name outputName $ 32; /* maximum width of a column name is 32 */
          format name outputName $32.; /* get rid of proc append warning */
          length dataType $ 2; /* get rid of proc append warning */
          set PerCellData(keep=PriorityColumn rename=(PriorityColumn=name));
          label='';
          outputName=name;
          type=0;
          position=0;  /* irrelevant */
          sortType=0;  /* none */
          sortOrder=0; /* irrelevant */
          format='BEST12.';
          dataType='N'; /* only numeric prioritization columns */
       run;
       proc append base=&vards data=_appendVars force %if &MADebug ^= Y %then nowarn; ; /* final ; is necessary */
       run;
    %end;

    %if &outputtype ne 3 and &outputtype ne 5 %then
    %do;
        %if %length(&outputLocation) > 0 %then
        %do;
            %let outputlocexist =  %sysfunc(fileexist(&outputLocation));
            %if &outputlocexist ne 1 %then
            %do;
                %let SYSCC = 100000; /* Updated from 5 to 100000 part of defect S0570533 */
                %let MAError = 312;
                %let MAMSg=  Macro[&sysmacroname.] The export path &outputLocation does not exist;
                %cleanup;
                %return;
            %end;
        %end;
        %macro cleanup;
        %put Cleaning up;
        %mend;
    %end;
    %else
    %do;
        %macro cleanup;
        %put Cleaning up &outputLocation;
        %mausrexp(&outputLocation,Clear);
        %mend;
        %mausrexp(&outputlocation,Execute);
    %end;

    /* S0598501 */
    /* convert the treatment delimiter string to actual delimiter character */
    /* set default scan modifier */
    %let scanModifier='';
    %if &treatmentDelimchar eq 'export_delimiter_tab' %then
    %do;
        /* Assign a Tab character */
        %let treatmentDelimchar  = '09'x;
        /*if the delimiter is set to TAB then the scan function need Scan modifier to read the tab separated string*/
        %let scanModifier='H';
    %end;
    %else %if &treatmentDelimchar eq 'export_delimiter_comma' %then
        %let treatmentDelimchar  = ',' ;

    %if &CCCData eq 1 %then
    %do;
        /* S0822815 : create Index on CCCData.CCCId */
        proc sql;
            create index CCCID on CCCDATA(CCCID);
        quit;

        filename addcol temp lrecl=32767;
        data _null_;
            file addcol;
            put 'data CCCDATA;';
            put '  set CCCDATA;';
            put '  where cccId ^= -1;';
        run;

        %if &haveVars %then
        %do;
            data _null_;
                file addcol mod;
                set &vards;
                if type=6 or (type=5 and (dataType = 'D' or dataType = 'DT' or dataType = 'T')) then
                do;
                    if dataType = 'D' then put 'format ' name 'DATE9.;';
                    else if dataType = 'DT' then put 'format ' name 'DATETIME20.0;';
                    else put 'format ' name 'TIME.;';
                end;
                /* S0520122 - Added the code to handle date list in the export */
                /* S0528182 - Modified to have common logic for Type 8(Date UDF) & Type 9(Numeric UDF) processing */
                /* S0598501 - Modified to use the treatmentDelimchar inplace of hardcoded delimeter */
                /* S0606741 - Use best. instead of 8. numVal' name '=input(NumExtract' name ',best.);';*/
                if type = 8 or Type = 9  then
                do;
                    put '   length NumStringOut' name '$32767;';
                    put '   ExitLoop' name ' = 1;';
                    put '   LoopCount' name ' = 1;';
                    put '   NumStringIn' name ' = ' name ';';
                    put '   NumStringOut' name '='''';';

                    if format eq '' and Type = 9 then
                        put '   %let Numfmt' name '=best32.;'; /* Default Number format if not specified */
                    else if format eq '' and Type = 8 then
                        put '   %let Numfmt' name '=date9.;';/* Default Date format if not specified */
                    else
                        put '   %let Numfmt' name '=' format ';' ; /* Else use the format specified for the field */

                    put '   do while (ExitLoop' name ');';
                    put "      NumExtract" name "= scan(NumStringIn" name ",LoopCount" name ', "&treatmentDelimchar." ,&scanModifier. );';
                    put '      NumExtract' name ' = left(trim(NumExtract' name '));';
                    put '      if NumExtract' name ' eq '''' then   ExitLoop' name ' = 0;';
                    put '      else';
                    put '      do;';
                    /* S0606741 */
                    put '          format numVal' name '  best32. ;' ;
                    put '          numVal' name '=NumExtract' name ';';

                    put '          NumStr' name '=put(numVal' name ',&Numfmt' name');';
                    put '          NumExtract' name ' =left(trim(NumStr' name '));';
                    /* S0612456 */
                    put "          if NumExtract" name " eq  '.'  then NumExtract" name "=' ' ; ";
                    put '          if LoopCount' name ' = 1 then NumStringOut' name ' =  NumExtract' name ';';
                    put '          else  NumStringOut' name ' =  left(trim(NumStringOut' name ')) || ' "&treatmentDelimchar." ' || NumExtract' name ';';
                    put '          LoopCount' name ' = LoopCount' name ' + 1;';
                    put '       end;';
                    put '   end;';
                    /* drop the new variables created for above processing */
                    put '   drop  ExitLoop' name ' LoopCount' name ' NumExtract' name ' numVal' name ' NumStr' name ' NumStringIn' name'  ' name ';';
                    put '   rename NumStringOut' name ' =' name ';' ;
                end;
            run;
        %end;

        %inc addcol;
        run;
        filename addcol clear;
        /* we have formated the type 8 & 9 values correct in above process, so remove the format for those columns from the vars dataset */
        /* else if the format is specified , the later process will try to apply the format while exporting */
        /* we can not apply a date or numeric format to numbers (which are converted to string by above process) */
        data &vards;
            set &vards ;
            /* S0532118 Also remove the format of RESPTRACKING_CD*/
            if type= 8 or type =9 or outputName = 'RESPTRACKING_CD' then
            format='' ;
        run;
    %end;

    /* S0608126 - store the original name out for later merging */
    data &vards;
        length origname $ 32;
        set &vards;
        origname=upcase(name);
    run;

    %if &countOnly = Y %then
        %if &dedup = N and &membertable eq %then
            %let membertable=justcount;

    /* S0692610 - keep the copy of subjectkey to be used for creating a dummy member table */
    %let subjectkeys_bk=&subjectkeys.;

    %if &inputTableList ne %then
    %do;
        %let lastKey=%scan(&subjectKeys,-1);
        %let dedupKeys=&subjectKeys;
        %let subjectComma=%sysfunc(translate(&subjectKeys,',',' '));
    %end;
    %else
    %do;
        %let lastKey=CCCId;
        %let subjectkeys=;
        %let dedup=N;
    %end;
    %let exportIn=&subjectKeys;
    %let exportOut=;
    %let combinedIn=;
    %let varNames=;
    %if &haveVars = 1 %then
    %do;
        %* S0721881 - use max character variable length 32767 ;
        data _null_;
            length exportIn ExportOut combinedIn varNames $ 32767 ;
            retain combinedIn exportIn "&subjectKeys" ExportOut varNames;
            set &vards end=done;

            if Type = 0 then
            do;
                if exportIn = '' then
                    exportIn = name;
                else if indexw(exportIn, name) = 0 then
                    exportIn = trim(exportIn) || ' ' || name;
            end;

            if exportOut = '' then exportOut = outputName;
            else exportOut = trim(exportOut) || ' ' || outputName;

            if Type = 0 or Type = 4 or Type = 5 or Type = 6 then
            do;
                if combinedIn = '' then combinedIn = name;
                else if indexw(combinedIn, name) = 0 then combinedIn = trim(combinedIn) || ' ' || name;
            end;

            if varNames = '' then varNames=upcase(name);
            else if indexw(varNames,upcase(name))=0 then varNames = trim(varNames) || ' ' || upcase(name);

            if varNames = '' then varNames=upcase(outputName);
            else if indexw(varNames,upcase(outputName))=0  then varNames = trim(varNames) || ' ' || upcase(outputName);

            if done then
            do;
                call symput( 'exportIn',   trim(exportIn)   );
                call symput( 'exportOut',  trim(exportOut)  );
                call symput( 'combinedIn', trim(combinedIn) );
                call symput( 'varNames',   trim(varNames)   );
            end;
        run;
    %end; *haveVars;

    * Section 1.50;
    %let viewSelection=;
    %if &MADebug=Y %then  %put Adding Temporary Variables, combinedIn is &combinedIn;
    %if &haveVars = 1 and &appendGroups = N and &exportIn ne %then
    %do;
        %let combinedIn=%upcase(&combinedIn);
        %* S0721881 - use max character variable length 32767 ;
        data &vards;
            length viewSelection newCombinedIn $ 32767;
            retain i 0 newCombinedIn viewSelection;
            drop i newCombinedIn viewSelection tempvar upname;
            length tempvar upname $ 400;

            set &vards end=done;

            upname=upcase(name);

            if indexw(upcase("&subjectKeys"),upname) = 0 and indexw("&combinedIn",upname) > 0 then
            do;
                tempvar='';
                do while (tempvar='' | indexw("&varNames",tempvar) > 0);
                    tempvar="_"||left(i);
                    i=i+1;
                end;
            end;
            else
            do;
                tempvar=name;
            end;

            if newCombinedIn = ''  then newCombinedIn = tempvar;
            else newCombinedIn = trim(newCombinedIn)||" "||tempvar;

            if tempvar ^= name then
            do;
                if viewSelection = ''   then viewSelection=trim(upname)||" AS "||tempvar;
                else viewSelection=trim(viewSelection)||","||trim(upname)||" AS "||tempvar;

                name=tempvar;
            end;
            else if indexw(upcase("&combinedIn"),upname)>0 then
            do;
                if viewSelection = ''  then viewSelection=trim(upname);
                else viewSelection=trim(viewSelection)||","||trim(upname);
            end;


            if done then
            do;
                i=1;
                tempvar=upcase(scan("&subjectKeys",i," "));
                do while(tempvar ne " ");
                    if indexw(upcase(newCombinedIn),tempvar)=0 then
                    do;
                        %* make sure we keep this variable ;

                        if newCombinedIn = ''   then newCombinedIn = trim(tempvar);
                        else newCombinedIn = trim(tempvar)||" "||trim(newCombinedIn);

                        if viewSelection = ''   then viewSelection=trim(tempvar);
                        else viewSelection=trim(tempvar)||","||trim(viewSelection);
                    end;

                    i=i+1;
                    tempvar=upcase(scan("&subjectKeys",i," "));
                end;

                call symput('combinedIn', trim(newCombinedIn));
                call symput('viewSelection', trim(viewSelection));
            end;
        run;
    %end;
    %else
    %do;
       %let viewSelection=%sysfunc(translate(&combinedIn,',',' '));
    %end;


    %if &MADebug = Y %then
    %do;
        %put Export Input Columns: &exportIn;
        %put Export Output Columns: &exportOut;
        %put Combined Input Columns: &combinedIn;
        %put All Unique Column Names: &varNames;
        %put View Selection: &viewSelection;
    %end;

    %* get the sort criteria, if any;
    %let ordervars=;
    %let sortVars=;
    %if &haveVars = 1  %then
    %do;
        proc sort data=&vards out=vars_sortorder;
            by sortorder;
        run;
        %* S0721881 - use max character variable length 32767 ;
        data _null_;
            length orderVars sortVars $ 32767;
            %if &inputTableList eq   %then
            %do;
                length dedupVars $ 32767;
                retain dedupVars;
            %end;
            retain orderVars sortVars;
            set vars_sortorder end=done;
            where sortOrder ^= 0;
            if sortType = 1  then
            do;
                if orderVars = ''  then
                do;
                    orderVars = name;
                    sortVars = outputname;
                    %if &inputTableList eq  %then
                    %do;
                        dedupVars = name;
                    %end;
                end;
                else
                do;
                    orderVars = trim(orderVars) || ',' || name;
                    sortVars  = trim(sortVars) || ' ' || outputname;
                    %if &inputTableList eq %then
                    %do;
                        dedupVars = trim(dedupVars) || ' ' || name;
                    %end;
                end;
            end;
            else if sortType = 2   then
            do;
                if orderVars = ''  then
                do;
                    orderVars = trim(name) || ' desc';
                    sortVars  = 'descending ' || outputname;
                    %if &inputTableList eq  %then
                    %do;
                        dedupVars = 'descending ' || name;
                    %end;
                end;
                else
                do;
                    orderVars = trim(orderVars) || ',' || trim(name) || ' desc';
                    sortVars  = trim(sortVars ) || ' descending ' || outputname;
                    %if &inputTableList eq  %then
                    %do;
                        dedupVars = trim(dedupVars) || ' descending ' || name;
                    %end;
                end;
            end;
            if done  then
            do;
                call symput( 'orderVars', trim(left(orderVars)) );
                call symput( 'sortVars' , trim(left(sortVars )) );
                %if &inputTableList eq  %then
                %do;
                    call symput( 'dedupKeys' , trim(left(dedupVars)) || ' CCCId' );
                %end;
            end;
        run;
    %end;
    %if &MADebug = Y  %then
    %do;
        %put Order vars: (&orderVars);
        %put Dedup vars: (&dedupKeys);
    %end;
    %if %length(&orderVars) > 0  %then %let Sort = Y;
    %else %let Sort = N;
    %if &appendGroups = Y %then %let sort=N;
    %if &MADebug = Y  %then %put Sort: &Sort;

    %if &SYSCC ne 0 and &SYSCC ne 4 %then
    %do;
        %put failing after stage 1 - SYSCC=&SYSCC;
        %if &MAError eq %then
        %do;
            %let MAError = 301;
            %let MAMsg = Macro[&sysmacroname.] Error processing inputs for export.  SYSCC=&SYSCC;
        %end;
        %cleanup;
        %return;
    %end;

    %* Section 2;
    %let needsCallExpTable=1; /* do not double copy the output table */
    %if &countOnly = Y %then  %let exportRecords=;
    %else %if &outputType = 5 and (&Dedup = N or &sort = N) and &AppendReplace = REPLACE %then %do;
       %let needsCallExpTable=0;
       %let exportRecords=&outputlocation..&outputName;
    %end;
    %else %let exportRecords=exportRecords;

    %* perform output refinement if specified;
    %if &MADebug = Y  %then
    %do;
        %put Before refine output, input table list: &inputTableList;
    %end;
    filename refine temp lrecl=32767;
    %put start refinement;

    /*S0534301   modified the code to  remove the hardcoded length statement     */
    /*(length newInputTableList $ 4000; & its associated lines)                  */

    /*S0504678: New PACKAGE_HASH_VAL added for the default selection */
    data _null_;
        length baseTableName $ 200;
        set &inputTableDS end=done;
        baseTableName=scan(inputTable,2, '.');
        respTrkCd=put(RESPTRACKING_CD,14.);

        /* S0611677  Remove RESPTRACKING_CD,PACKAGE_HASH_VAL columns from input table if exists */
        /* Cause - If we attach a comm to a comm node via cell node , the second comm node matable contains the RESPTRACKING_CD of the first comm node*/

        inputTableName=upcase(baseTableName);
        inputLibName=scan(inputTable,1, '.');
        inputLibName=upcase(inputLibName);

        file refine;

        /* Check if the column exists */
        put 'proc sql noprint ;';
        put '   select  count(name) into :VarExists    ';
        put '   from    dictionary.columns ';
        put '   where   name in ("RESPTRACKING_CD", "PACKAGE_HASH_VAL")';
        put '   and     libname="' inputLibName'" and memname ="' inputTableName'"';
        put ';quit;';

        /* Drop the columns */
        put '%macro RemoveCols();';
        put '%if &VarExists. ne 0 %then';
        put '%do;  ';
        put '   data  ' inputTable  '(drop= RESPTRACKING_CD PACKAGE_HASH_VAL) ;' ;
        put '       set  ' inputTable  ' ;' ;
        put '   run ;' ;
        put '%end;';
        put '%mend;';
        put '%RemoveCols;';

        /* S0822815 : create cccid column as well (this needs to be verified for MAMO integration)*/
        put 'proc sql noprint; ' ;
        put 'create view '  baseTableName ' as select * , "' respTrkCd '" as RESPTRACKING_CD,"' package_hash_val '" as PACKAGE_HASH_VAL ,' CCCId ' as CCCId  from '  inputTable  ;
        if ( tableRefWhereClause ne '') then
        do;
            put   ' where ' tableRefWhereClause;
        end;
        put  ' ; ' ;
        put ' quit; ' ;
    run;

    %inc refine;
    filename refine clear;

    /*S0534301 added proc sql block to create the macro variable
    Added condition to check if inputTableMeta exist */
    /* S0606031 */
    %if %sysfunc(exist(&inputTableDS)) %then
    %do;
        %let inputTableDSExists=1;
        proc sql noprint;
            select scan(inputTable,2, '.') into :inputTableList separated by ' ' from &inputTableDS;
        quit;
    %end;
    %else %let inputTableDSExists=0;

    %put Start section 2.5;
    %if &appendGroups = Y and &perCellData %then
    %do;
       %put Creating Prioritization Data for Marketing Optimization;
       /* S0822815 - create outTableName to be used by maCreatePrioritizationData */
       %if %sysfunc(exist(&inputTableDS)) %then
       %do;
            /* create a percell out table unique name */
            data _Inputtablemeta2;
                set &inputTableDS.;
                PerCell_OutTable='OutTable_' || left(_n_);
                TableName=scan(inputTable,2, '.');
            run;
            /* create PerCellData2 by attaching the percell_out table name */
            proc sql;
                create table PerCellData2 as
                    select      t1.*,
                                t2.PerCell_OutTable
                    from        PerCellData t1
                    inner join  _Inputtablemeta2 t2
                    on          upcase(t1.TableName) = upcase(t2.TableName);
            quit;

            /* update PerCellData table with new columns */
            data PerCellData;
                set PerCellData2;
            run;
            /* create the inputTableList using PerCell_OutTable names */
            proc sql noprint;
                select PerCell_OutTable into :inputTableList separated by ' ' from _Inputtablemeta2 ;
            quit;
        %end;
       filename tincl temp lrecl=32767;

       data _null_;
          file tincl;
          set PerCellData end=done;
          /* make sure we don't drop the response tracking and package hash columns */
          /* S0822815 -  modify outTableName as Mo sorts the input table and creates another table with additional columns hence */
          command='%'||'maCreatePrioritizationData('||TableName||','||PerCell_OutTable||','||PriorityColumn||',&subjectKeys,'||SortCriteria||','||ExtraColumns||' RESPTRACKING_CD PACKAGE_HASH_VAL);';
          put command;
       run;
       %include tincl;
       filename tincl clear;
    %end;

    %put Start Section 2.75;

    %if &MADebug = Y %then %put After refine output, Input file list: &inputTableList;

    %if &seedmethod = EXPORTRANDOM and &exportRecords ne %then
    %do;
        data _null_;
            nototal = 0;
            %let i = 1;
            %let table=%scan(&inputTableList,&i,%str( ));
            %do %while( &table ne );
                %let dsid=%sysfunc(open(&table));
                %let tot=%sysfunc(attrn(&dsid,nobs));
                if &tot > 0 then
                do;
                    set &table nobs=no&i;
                    nototal = nototal + no&i;
                end;
                %let i=%eval(&i+1);
                %let table=%scan(&inputTableList,&i,%str( ));
            %end;
            call symput( 'quickCount', left(put(nototal, f12.)) );
            stop;
        run;
    %end;

    %* create a view to append tables, merge CCCData and order for either dedup or final sort for MA/MO;
    %if &appendGroups = Y %then
    %do;
        proc sql;
            %let i = 1;
            %let table=%scan(&inputTableList,&i,%str( ));
            %do %while( %length(&table) > 0);
            create view merge&i as
                select *, %eval(&i-1) as CCCId from &table order by &subjectComma ;
                /* select *, %eval(&i-1) as CCCId from &table order by &subjectComma, RESPTRACKING_CD;*/ /* S0674127 */
                %let i = %eval(&i+1);
                %let table=%scan(&inputTableList,&i,%str( ));
            %end;
        quit;

        %let n = %eval(&i - 1);
        data merged;
            merge
            %do i = 1 %to &n;
               merge&i
            %end;
            ;
            by &subjectKeys;
/*            by &subjectKeys RESPTRACKING_CD;*/
            if first.&lastKey then output;
            /* if first.&lastKey or first.RESPTRACKING_CD then output;*/ /* S0674127 */
        run;

        %let extraColumns=;
        data _null_;
           set &vards. end=done;
           if _n_ eq 1 then do;
               dcl hash rootkeys(MULTIDATA:"YES");
               rootkeys.definekey('outputName');
               rootkeys.definedata('outputName');
               rootkeys.definedone();
               call missing(outputName); /* avoid unitialized value notes */
           end;
           outputName=upcase(outputName);
           rootkeys.add();
           
           if done then do;
              hasRespCd=rootkeys.find(key:"RESPTRACKING_CD");
              hasPackageHashVal=rootkeys.find(key:"PACKAGE_HASH_VAL");
              if 0 ne hasRespCd or 0 ne hasPackageHashVal then do;
                 if 0 ne hasRespCd and 0 ne hasPackageHashVal then do;
                     call symput("extraColumns",", RESPTRACKING_CD, PACKAGE_HASH_VAL");
                 end; else if 0 ne hasRespCd then do;
                     call symput("extraColumns",", RESPTRACKING_CD");
                 end; else do;
                     call symput("extraColumns",", PACKAGE_HASH_VAL");
                 end;
              end;
           end;
        run;
        proc sql noprint; /* S0504678: Include PACKAGE_HASH_VAL if it is not present in the output column list */
            create view _sasmaexportview as
              select * &extraColumns.
              from merged m
            %if &CCCData eq 1
            %then %do;
               inner join cccdata c on m.cccid = c.cccid
            %end;
            ;
        quit;
    %end;
    %else
    %do;
        /* S0822815: merge the input tables instead of doing a union and use the merged table in exportview creation */
        %if &inputTableList ne %then
        %do;
            data mergedInputTable / view=mergedInputTable ;
                set &inputTableList ;
            run;
        %end;

        /* S0822815: use mergedInputTable in from clause instead of using it in subquery */
        proc sql noprint;
            create table _sasmaExportView (COMPRESS=YES )as
            %if &inputTableList eq %then
            %do;
                select * from cccData
            %end;
            %else
            %do;
                %if &CCCData eq 1  %then
                %do;
                    select &viewSelection,c.* , RESPTRACKING_CD, PACKAGE_HASH_VAL
                %end;
                %else %if &viewSelection ne  %then
                %do;
                    select &viewSelection, RESPTRACKING_CD, PACKAGE_HASH_VAL
                %end;
                %else
                %do;
                    select %sysfunc(translate(&exportIn,',',' ')) ,RESPTRACKING_CD ,PACKAGE_HASH_VAL , CCCId
                %end;
                    from mergedInputTable s
                %if &CCCData eq 1 %then
                %do;
                    inner join cccData c on s.cccid = c.cccid
                %end;
            %end;
            %if &sort = Y or &Dedup = Y  %then
            %do;
                order by
                %if &DeDup=Y and &inputTableList ne %then
                %do;
                    /* S0674127 - The refinement should consider both subject and cell (resptracking_cd) so to retain
                       subjects coming from different cells */
                    &subjectComma, RESPTRACKING_CD
                    %if &sort = Y %then
                    %do;
                        ,
                    %end;
                %end;
                &orderVars
                %if &DeDup=Y  %then , CCCId;
            %end;
            ;
        quit;

    %end;
    /* S0822815 */
    %if &haveVars = 1 %then
    %do;
        /* S0822815 - create a table with the list of columns in the input and their input format names */
        proc contents
            data=_sasmaExportView
            out=_in_vars_fmts(keep = name format rename =(format=in_fmt)) noprint;
        run;
        /* append the in_fmt column in the vars table */

        data &VarDS(drop= rc);
            set &VarDS ;
            length in_fmt $32. ;
            if _n_ = 1 then
            do;
                declare hash hash_informats(dataset: "_in_vars_fmts" , hashexp:10);
                             hash_informats.DefineKey('name');
                             hash_informats.DefineData ('in_fmt') ;
                             hash_informats.DefineDone () ;
            end;
            in_fmt ='';
            out_fmt= upcase(COMPRESS(format, '.1234567890')) ;
            rc = hash_informats.find() ;
        run;
    %end;

    %* get the type and length for all variables being exported;
    %if &inputTableList ne  %then
    %do;
        proc contents data=%scan(&inputTableList,1,%str( )) out=_invars noprint; run;
        data _invars; set _invars; upname = upcase(name); run;
        proc sort data=_invars; by upname; run;
    %end;
    %if &CCCData eq 1 %then
    %do;
        proc contents data=CCCData out=_cccvars noprint; run;
        data _cccvars; set _cccvars; upname = upcase(name); run;
        proc sort data=_cccvars; by upname; run;
    %end;
    %if &haveVars = 1  %then
    %do;
        data _varsSorted; length upname $ 32; set &vards; upname = upcase(origname); exportOrder=_n_; run;
        proc sort data=_varsSorted; by upname; run;
        data _varsExtra;
            merge _varsSorted(in=want)
            %if &inputTableList ne %then
            %do;
                _invars(keep=upname type length rename=(type=vartype))
            %end;
            %if &CCCData eq 1  %then
            %do;
                _cccvars(keep=upname type length rename=(type=vartype))
            %end;
            ;
            by upname;
            if length = . then
            do;
                if type = 1  then length = length(value);
                else length = 40;
            end;
            if want;
        run;
        proc sort data=_varsExtra out=&vards; by exportOrder; run;

        /* S0822815: add the Compress= yes to reduce the IO*/
        filename expstep2 temp lrecl=32767;
        data _null_;
            file expstep2;
            put 'data counts(keep=_macount) ' @;
            stop;
        run;

        %if %length(&exportRecords) gt 0 %then %do;
            data _null_;
                file expstep2 mod;
                set &vards end=done;
                if _n_ = 1 then do;
                   dcl hash rootkeys();
                   rootkeys.definekey('outputName');
                   rootkeys.definedata('outputName');
                   rootkeys.definedone();

                   put "&exportRecords(COMPRESS=yes keep=";
                end;

                if not(missing(outputName)) then do;
                   rc=rootkeys.find();
                   if (0 ne rc) then do;
                       rootkeys.add();
                       put outputName;
                   end;
                end;
                
                if done then do;
                   put ") ";
                end;
            run;
        %end;
        
        data _null_;
            set &VarDS end=eof;
            file expstep2 mod;
            if _n_ = 1 then
            do;
                /* S0674127 - De-couple the member table and the input table for CH processing and create a new table for contact history.
                   This will have all the subjects coming from the upstream nodes and duplicates will be handled with in CH porcess */
                %if &chtable. ne %then
                %do;
                    put "&chtable(keep=&subjectKeys RESPTRACKING_CD PACKAGE_HASH_VAL)";
                %end;
                %if &membertable ne  %then
                %do;
                     put "&membertable(keep=&subjectKeys RESPTRACKING_CD PACKAGE_HASH_VAL)";
                %end;
                put ';';
            end;

            %*** Find length of character format to make output variable this length ;
            %*** if a length is specified in format. ***;
            %*** This section just generates a length statement ***;
            %if &exportRecords ne %then
            %do;
                /* S0532118 - do not generate the length statement for RESPTRACKING_CD column */
                /* S0606031 - if the inputtable is not present then only use the RESPTRACKING_CD from CCCData table */
                /*if outputName ne 'RESPTRACKING_CD' then*/
                /* if not(outputName eq 'RESPTRACKING_CD' and &inputTableDSExists. eq 1 ) then
                do;*/
                    if (type=0 or type=4) and index(format,'$')=1 then
                    do; *** character format specified ***;
                        dotpos = index(format,'.');
                        numpos = indexc(format, '1234567890' );
                        if numpos > 0 and numpos < dotpos then
                            fmtlen = input( substr(format,numpos,dotpos-numpos), best. );
                        else
                        /* S0524665 */
                        do;
                            if dataType = 'C' then
                                fmtlen=length;
                            else
                                fmtlen=0;
                        end;
                        /*fmtlen=0;*/
                        if fmtlen > 0 then
                            put 'length ' outputname '$ ' fmtlen ';' ;
                    end;
                    %if &includeLabels=Y %then
                    %do;
                        if label ne ' ' then
                        do;
                            label = Tranwrd(label,'"', '""');
                            put 'label ' outputname '="' label '";' ;
                        end;
                    %end;

                    if format ^= '' then
                    do;
                        if index(format,'~') eq 1 then
                        do;
                            newformat = substr(format,2);
                            put 'format ' outputname newformat ';';
                        end;
                        else
                            put 'format ' outputname format ';';
                    end;
/*                  end:S0556482*/
                    else
                    do;
                        if vartype = 1 then  put 'length ' outputname length ';';
                        else if dataType = 'C' or dataType = 'U' then  put 'length ' outputname ' $ ' length ';';
                    end;
                /*end;*/
                %if &seedmethod = EXPORTRANDOM %then
                %do;
                    put 'retain insertNow 0 nextSeed 2;';
                %end;
            %end;
        run;

        data _null_;
            set &VarDS end=eof;
            file expstep2 mod;
            if _n_ = 1  then
            do;
                put "retain _macount 0;";
                put "set _sasmaExportView end=done;";
                %if &Dedup = Y %then
                %do;
                    put "by &dedupKeys RESPTRACKING_CD;"; /* S0674127 Include resptracking_cd to remove the duplicates */
                %end;
            end;

            length newValue $ &maxValueLength;

            if type = 1  then
            do;
                newValue = Tranwrd(value,"'", "''");
                put outputname "='" newValue +(-1) "';";
            end;
            /* S0585769 - if RESPTRACKING_CD column is used as one of the column name mapped to */
            /*  any other column name then drop that column and use the actual RESPTRACKING_CD instead */
            /* S0606031 -  drop RESPTRACKING_CD if the inputtableds is present */
            else if outputName ^= name then
            do;
                /* if outputName='RESPTRACKING_CD' then*/
                if (outputName='RESPTRACKING_CD' and &inputTableDSExists. eq 1) then
                do;
                    put 'drop '  name ';' ;
                end;
                /* S0570533 - When exportRecrods is NULL the constructed data step only creats the counts & matable
                dataset. So the macnvvar macro call is not required.  But when it is used all the output columns are
                created with Best format and warning messages printed when character columns are used */
                else
                do;
                    %if &exportRecords. ne %then
                    %do;
                        put '%' 'macnvvar(' name ',' outputName ',' in_fmt  ',' out_fmt  ');';
                    %end;
                end;
            end;

            if eof then
            do;
                 /* S0674127 - Create the same table for CH irrespective of dedup flag */
                %if &chtable. ne %then
                %do;
                    put "output &chtable;";
                %end;

                %if &Dedup = Y  %then
                %do;
                     /* S0674127 - Retain the subjects from accross the cell and remove the duplicates with in the
                        same cell (one-many relation) */
                    %if &exportRecords ne %then
                    %do;
                        put "if first.&lastKey or first.RESPTRACKING_CD then";
                        put "do;";
                        put "    output &exportRecords;";
                        put "end;";
                    %end;
                    /* S0674127 - Member table will have only distinct subjects */
                    %if &membertable ne  %then
                    %do;
                        put "if first.&lastKey then";
                        put "do;";
                        put "    output &memberTable;";
                        put "    _macount + 1;";
                        put "end;";
                    %end;
                %end;
                %else
                %do;
                    %if &exportRecords ne  %then
                    %do;
                        put "output &exportRecords;";
                    %end;
                    %if &membertable ne  %then
                    %do;
                        put "output &memberTable;";
                        put "_macount + 1;";
                    %end;
                %end;

                %if &seedmethod = EXPORTRANDOM and &exportRecords ne %then
                %do;
                    put 'if insertNow = 1 then do;' ;
                    put "set seed point=nextSeed nobs=ns;" ;
                    put "output &exportRecords;";
                    put 'nextSeed + 1;' ;
                    put 'insertNow = 0;' ;
                    put 'end;' ;
                    put 'if insertNow = 0 and nextSeed <= ns ';
                    put "then insertNow = floor( 2 * ranuni(0) * &quickCount / ns );";
                    put 'else insertNow = insertNow - 1;';
                %end;

                put 'if done then do; output Counts; call symput("exportCount", trim(left(put(_macount,f12.))));';
                %if &seedmethod = EXPORTRANDOM and &exportRecords ne  %then
                %do;
                    put 'if nextSeed <= ns then do;  ***Write out rest of seeds;' ;
                    put 's = ns - nextSeed + 1;' ;
                    put 'do s = nextSeed to ns;' ;
                    put "set seed point=s nobs=ns; " ;
                    put "output &exportRecords;";
                    put 'end;';
                    put 'end;';
                %end;
                %if &seedMethod = EXPORTAPPEND and &exportRecords ne  %then
                %do;
                    put 'do until(endseed);';
                    put 'set seed(firstobs=2) end=endseed;';
                    put "output &exportRecords;";
                    put 'end;';
                %end;
                put 'end;';
                put 'run;';
            end;
        run;
        %let exportCount=0;
        %inc expstep2;
        filename expstep2 clear;
        /*S0401805*/ /*S0312697*/
        %if (&SYSCC ne 0 and &SYSCC ne 4) and %symexist(SYSERRORTEXT) %then
        %do;
            /* send back the SYSERRORTEXT to user */
            %let MAERROR=399;/*set a  non used error code so that core can send error text back as is */
            %let MAMsg=Macro[&sysmacroname.].SYSERRORTEXT=[&SYSERRORTEXT];
            %return;
        %end;
    %end;
    %else
    %do;
        %* S0712827 - create chtable also ;
        %* just counting distinct subjects in the input table(s);
        data &memberTable &chtable ;
            set _sasmaExportView;
            keep &subjectKeys RESPTRACKING_CD PACKAGE_HASH_VAL;
        run;
    %end;

    /* S0674127 - Refine the output by removing the duplicates irrespective of the DEDUP flag
       when only Camp/Comm data and no subject data selected for export */
    %if &madebug. eq Y %then %put ExportOut= &exportout.;

    %if &exportRecords ne and &subjectExport. eq N %then
    %do;
        proc sort data=&exportRecords nodupkey;
            by &exportOut.;
        run;
    %end;

    /* S0692610 create a member table with zero rows for the downstream node if the inputTableDS doesn't exists */
    %if &inputTableDSExists. eq 0 %then
    %do;
        data &memberTable.;
            length &subjectkeys_bk. RESPTRACKING_CD PACKAGE_HASH_VAL $10.;
            stop;
        run;
    %end;

    %if &MADebug = Y %then %put Dedup=&dedup and member table=&membertable;
    %if &dedup = N and &membertable ne  %then
    %do;
        %if &subjectKeys ne  %then
        %do;
            proc sort data=&membertable nodupkey; by &subjectKeys; run;
            %macount( &memberTable );
            %let exportCount=&count;
            data counts; _macount=&exportCount; run;
        %end;
    %end;

    %if &countOnly = Y  %then %goto countResult;

    %put RC after section 2=&SYSCC;
    %if &SYSCC ne 0 and &SYSCC ne 4  %then
    %do;
        %if &MAError eq  %then
        %do;
            %let MAError = 302;
            %let MAMsg = Macro[&sysmacroname.] Error merging or dedupping data for export.  SYSCC=&SYSCC;
        %end;
        %cleanup;
        %return;
    %end;

    %* Section 3 - threshold checks;
    %put Start Section 3;
    %if %length(&lowerThreshold) > 0 and &exportCount < &lowerThreshold  %then
    %do;
        %put &exportCount is under lower threshhold of &lowerThreshold;
        %let MAError = 310;
        %let MAMsg   = Macro[&sysmacroname.] There are &exportCount subjects in the communication.  At least &lowerThreshold are required in the audience.;
        %* S0712827 - create chtable also ;
        data &memberTable &chtable ;
            set &memberTable;
            stop;
        run;
        %macount(&membertable);  %*** Have to do this to override with 0 ***;
        %let syscc = 310;
        /* Code to address defect S0368146 - delete the dataset in case of threshold limitation */
        %if %upcase(&appendReplace) ne APPEND AND %sysfunc(exist(&outputlocation..&outputName)) %then
        %do;
            proc datasets library=&outputlocation;
                delete &outputName;
            run;
        %end;
        %cleanup;
        %return;
    %end;
    %if %length(&upperThreshold) > 0 and &exportCount > &upperThreshold  %then
    %do;
        %put &exportCount is over upper threshhold of &upperThreshold;
        %let MAError = 311;
        %let MAMsg   = Macro[&sysmacroname.] There are &exportCount subjects in the communication.  Only &upperThreshold are allowed in the audience.;
        %* S0712827 - create chtable also ;
        data &memberTable &chtable ;
            set &memberTable;
            stop;
        run;
        %macount(&membertable);  %*** Have to do this to override with 0 ***;
        %let syscc = 311;
        /* Code to address defect S0368146 - delete the dataset in case of threshold limitation */
        %if %upcase(&appendReplace) ne APPEND AND %sysfunc(exist(&outputlocation..&outputName)) %then
        %do;
            proc datasets library=&outputlocation;
                delete &outputName;
            run;
        %end;
        %cleanup;
        %return;
    %end;

    %if &SYSCC ne 0 and &SYSCC ne 4  %then
    %do;
        %if &MAError eq  %then
        %do;
            %let MAError = 303;
            %let MAMsg = Macro[&sysmacroname.] Error checking export thresholds.  SYSCC=&SYSCC;
        %end;
        %cleanup;
        %return;
    %end;

    %* Section 4 - assure final sorted order;
    %put Start Section 4;

    %if &Sort = Y and &Dedup = Y  %then
    %do; %* if not dedup the view sorts in the final order;
        %if OutputType = 5 and &AppendReplace = REPLACE  %then %let final=&outputlocation..&outputName;
        %else %let final=_sorted;
        proc sort in=&exportRecords out=&final;
            by &sortVars;
        run;
    %end;
    %else %let final=&exportRecords;

    %if &SYSCC ne 0 and &SYSCC ne 4  %then
    %do;
        %if &MAError eq  %then
        %do;
            %let MAError = 304;
            %let MAMsg = Macro[&sysmacroname.] Error sorting data for export.  SYSCC=&SYSCC;
        %end;
        %cleanup;
        %return;
    %end;

    %* Section 5 - export output;
    %put Start Section 5;

    %if &outputType = 0 or &outputType = 1 or &outputType = 4 or &outputType = 7  %then
    %do;
        %put call maexpfil, MADebug=&MADebug;
        %maexpfil( &final, &vards, &exportOut, &outputlocation/&outputname, &outputType, &headerRow, &appendReplace, %bquote(&delimchar), &quoteRule);
        %put finished maexpfil, syscc=&SYSCC;
        /* If the above macro call resulted in errors, return to the calling macro. */
        %if (&MAError ne ) %then
        %do;
            %cleanup;
            %return;
        %end;
        %else
        %do;
            %if (&SYSCC ne 0 and &SYSCC ne 4) %then
            %do;
                %let MAError = 305;
                %let MAMsg = Macro[&sysmacroname.]. Error during export output.  There was an error in the called macro [MAEXPFIL]. Check the log for details.  SYSCC=&SYSCC;
                /* Code to address defect S0368146 - delete the file in case of error */
                %if %upcase(&appendReplace) ne APPEND AND %sysfunc(fileexist(&outputlocation/&outputname)) %then
                %do;
                    filename delfile "&outputlocation/&outputname";
                    data _null_;
                        rc = fdelete("delfile");
                    run;
                %end;
                %cleanup;
                %return;
            %end;
        %end;
    %end;
    %else %if (&outputType = 3 or &outputType = 5 or &outputType = 6) and &needsCallExpTable. %then
    %do;
        %maexptbl( &final, &outputlocation, &outputName, &outputType, &appendReplace, %str(&dbdsoptions) );
        /* If the above macro call resulted in errors, return to the calling macro. */
        %if (&MAError ne ) %then
        %do;
            %cleanup;
            %return;
        %end;
        %else
        %do;
            %if (&SYSCC ne 0 and &SYSCC ne 4) %then
            %do;
                %let MAError = 305;
                %let MAMsg = Macro[&sysmacroname.] Error during export output.  There was an error in the called macro [MAEXPTBL].  SYSCC=&SYSCC;
                /* Code to address defect S0368146 - delete the table in case of error */
                %if %upcase(&appendReplace) ne APPEND AND %sysfunc(exist(&outputlocation..&outputName)) %then
                %do;
                    proc datasets library=&outputlocation;
                        delete &outputName;
                    run;
                %end;

                %cleanup;
                %return;
            %end;
        %end;
    %end;
    %else %if &outputType = 2  %then
    %do;
        %global excelexport;
        %if &excelexport eq
        %then %let expdbms=XLS;
        %else %let expdbms=&excelexport;
        %let wanted=%lowcase(&expdbms);
        %let suffix=%scan(&outputname, -1, %str(.));
        %let i=%eval(%length(&outputname)-%length(&suffix));
        %if &suffix ne &wanted
        %then %if &suffix = xls or &suffix = xlsx
        %then %let suffix =;
        %let outputname = %substr(&outputname,1,&i)&suffix;
        %let excelName=&outputlocation/&outputname.;
        /* S0554913 Added %unquote function */
        %let excelName=%unquote(&excelName);
        %if %upcase(&AppendReplace) = APPEND   %then
        %do;
            /*S0573074 Added file exists check before import */
            %let fullname=&excelName.&wanted;
            %if %sysfunc(fileexist("&fullname"))= 1 %then
            %do;
                %if &MADEBUG EQ Y %then %put Appending to &fullname so it must be imported first.;
                /*S0560523 Removed the if conditions checking for Operating System*/
                proc import datafile="&excelName" out=excelExisting
                    dbms=&expdbms;
                run;
                /* S0658027 */
                %marowcnt( excelExisting , ImportRowCount);
                %if &ImportRowCount. > 0 %then
                %do;
                    data &final; set excelExisting &final; run;
                %end;
            %end;
            %else %if &MADEBUG EQ Y %then %put Request to append to &fullname but it does not exist.;
        %end;

        proc export data=&final
            outfile="&excelName"
            dbms=&expdbms
            replace;
        run;

        %if (&SYSCC ne 0 and &SYSCC ne 4) %then
        %do;
            %let MAError = 305;
            /* S0594364 */
            %marowcnt( &final , totcnt);
            %put totalcount = &totcnt;
            %if &totcnt > 65536 %then
            %do;
                %let MAMsg = Number of Records is exceeding the limit of 65536 for the Excel.;
            %end;
            %else
            %do;
                %let MAMsg = Macro[&sysmacroname.] Error during export to Excel ().  SYSCC=&SYSCC;
            %end;
            /* Code to address defect S0368146 - delete the excel file in case of error */
            %if %upcase(&appendReplace) ne APPEND and %sysfunc(fileexist(&excelName)) %then
            %do;
                filename delexcel "&excelName";
                data _null_;
                    rc = fdelete("delexcel");
                run;
            %end;

            /*S0401805*/
            %if %symexist(SYSERRORTEXT) %then
            %do;
                /*send back the SYSERRORTEXT to user */
                %let MAERROR=399;/*set a  non used error code so that core can send error text back as is */
                %let MAMsg=Macro[&sysmacroname.].SYSERRORTEXT=[&SYSERRORTEXT];
             %end;
            %cleanup;
            %return;
        %end;
    %end;

    %countResult:

    %cleanup;

/*  create work.counts dataset with column _macount and value 0 , when the export count is 0  */
    %if &exportCount eq 0 %then
    %do;
        data counts;
            _macount = &exportCount;
        run;
    %end;

    data _null_;
        file "&_stpwork.counts.txt";
        set counts;
        put "1";
        put _macount;
    run;
%mend;

%macro maCreatePrioritizationData( inTable, outTable, prioColumn, keyColumns, sortColumns, exColumns );
/* Set up the counting column for the data step.  No sort data means no count. */
%if &sortColumns eq %then %let countMethod=1; %else %let countMethod=%str(_n_);
%if 1 ^= &countMethod %then %do;
   %let sortTable=&outTable;
   proc sort data=&inTable out=&outTable; by &sortColumns; run;
%end; %else %let sortTable=&inTable; /* no sort table so just use input table*/

data &outTable (keep=&keyColumns &prioColumn &exColumns);
   set &sortTable;
   &prioColumn=&countMethod; /* example: e1=_n_ or e2=1 */
run;
%mend;
