/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,    */
/*                    USA.                                       */
/*             All Rights Reserved.                              */
/*  This software is protected by copyright laws and             */
/*               international treaties                          */
/*                                                               */
/* U.S. Government Restricted Rights                             */
/* Use, duplication, or disclosure of this software and related  */
/*  documentation by the U.S. government is subject to the       */
/*   Agreement with SAS Institute and the restrictions set forth */
/*  in FAR 52.227-19, Commercial Computer Software -             */
/*              Restricted Rights ( June 1987 )                  */
/*                                                               */
/*****************************************************************/
/* NAME: maexp                                                   */
/* VERSION: 5.4                                                  */
/*****************************************************************************************/
/* Change Log:                                                                           */
/* Date    DefectNo  Comments                                                            */
/* 13Mar09 S0556482  modified to handle tilde character in the format statement          */
/* 26May09 S0401805  added error handle to check export to readonly location             */
/* 29Oct10 S0656256  Do not append file extension to export filename                     */
/* 04Dec12 S0897280  Do not quote headers when QuoteRule= N                              */
/*****************************************************************************************/
%macro maexpfil( input, vards, varList, output, outputType, headerRow, appendReplace, override , quoteRule);
   %if &MADebug = Y
   %then %do;
      %put Input table: &input;
      %put Variable data set: &vards;
      %put Variable list: &varList;
      %put Output name: &output;
      %put Output type: &outputType (7=Delimited, 4=Positional);
      %put Header row: &headerRow;
      %put Append or replace: &appendReplace;
      %put Override delimiter: &override;
      %put Quote Rule: &quoteRule;
   %end;
   %*-----get filename and delimter for export file-----*;
   %if &outputType=0 /* CSV delimited */
   %then %do;
      %put WARNING: Archaic Export Output Type - 0 (CSV);
      %* S0656256 do not append file extension to export file ;
      %* %let output=&output..csv;
      %if &override eq
      %then %let dlmchar=',';
      %else %if %quote(&override) eq 'export_delimiter_tab'
      %then %let dlmchar=',';
      %else %let dlmchar=%str(&override);
   %end;
   %else %if &outputType=1 /* tab delimited */
   %then %do;
      %put WARNING: Archaic Export Output Type - 1 (TAB);
      %if &override eq %then %let dlmchar='09'x;
   %end;
   %else %do;
      %if %quote(&override) eq 'export_delimiter_comma'
      %then %let dlmchar=%str(",");
      %else %if %quote(&override) eq 'export_delimiter_tab'
      %then %let dlmchar='09'x;
      %else %let dlmchar=%str(&override);
   %end;

   %if &headerRow=Y and %upcase(&appendReplace) = APPEND and %sysfunc(fileexist(&output))
   %then %let headerRow=N; %* do not add header if appending to file that already exists.;

   %* determine file statement for file based exports;
   %let export_filestmt=;
   %let export_filestmt=file "&output";
   %if %upcase(&appendReplace) = APPEND
   %then %let export_filestmt=&export_filestmt mod;

   %if &outputType=0 or &outputType=1 or &outputType=7
   %then %let export_filestmt=&export_filestmt dlm=&dlmchar; %* Delimited or (old) CSV or TAB delimited;

   %let export_filestmt=&export_filestmt recfm=v lrecl=32767;

   filename logfile temp lrecl=32767;

   data _null_;
      set &vards end=done;
      file logfile;
      if _n_ = 1
      then do;
         put 'data ef_vartype (keep=exportOrder sasVarType);';
         put '   length sasVarType $ 2;';
         put '   set &input;';
         put '   if _n_ > 1 then stop;';
      end;

      put '   exportOrder=' exportOrder ';';
      put '   sasVarType=vtypex("' outputname+(-1) '");';
      put '   output;';

      if done then do;
         put 'run;';
      end;
   run;
   %inc logfile;
   filename logfile clear;

   data ef_varsort;
      merge &vards ef_vartype; by exportOrder;

   filename expfile temp lrecl=32767;

   data _null_;
      length _tmpvar $ 2000;
      set &vards end=done;
      file expfile;
      if _n_ = 1
      then do;
         put "data _null_;";

         put "%bquote(&export_filestmt;)";
         put "set &input;";
         put "length _matemp $ 2000;";
         %if &outputType=0 or &outputType=7
         %then %do;
            /* S0897280 - apply quotes only when quote rule is Not "never quote values in export file"*/
            %if &quoteRule ne N %then
            %do;
                put "format _matemp $quote.;";
            %end;
         %end;
         %if &headerRow = Y
         %then %do;
            put "if _n_ = 1 then do;";
         %end;
      end;
      %if &headerRow = Y
      %then %do;
         put "_matemp = '" outputname +(-1) "';";
         %if &outputType = 4
         %then %do;
            put 'put @' position "_matemp @;";
         %end;
         %else %do;
            put "put _matemp @;";
         %end;

         if done then put "put;" / "end;";
      %end;
   run;

   data _null_;
      length _tmpvar $ 2000;
      set ef_varsort end=done;
      file expfile mod;
      if format ^= ''
      then do;
         _tmpvar = '_sasmat' || left(put(_n_,f8.));
/*      begin:S0556482- Modified to remove tilde character from format variable */
        if index(format,'~') eq 1 then
        do;
            notildeformat = substr(format,2);
            put _tmpvar '= left(put(' outputname ', ' notildeformat '));';
        end;
        else
            put _tmpvar '= left(put(' outputname ', ' format '));';
/*      end:S0556482*/
         %if &quoteRule = A
         %then %do;
            put 'format ' _tmpvar ' $quote.;' ;
         %end;
         %else %if &quoteRule ne N
         %then %do;
            %if &outputType=0 or &outputType=7
            %then %do;
               put 'format ' _tmpvar ' $quote.;' ;
            %end;
         %end;
         %else %if &outputType ^= 7
         %then %do;
            _tmpvar = outputname;
         %end;
      end;
      else do;
         if ("&quoteRule"="A" and dataType="N") or dataType="DT" or dataType="D" or dataType="T"
         then do;
            _tmpvar = '_sasmat' || left(put(_n_,f8.));

            length fmt $ 20;
            fmt="BEST12.";
            if dataType="DT" then fmt="DATETIME.";
            else if dataType="D" then fmt="DATE9.";
            else if dataType="T" then fmt="TOD.";

            if sasVarType='C' then
               put _tmpvar '= left(putn(' outputname ', "' fmt +(-1)'"));';
            else
               put _tmpvar '= left(put(' outputname ', ' fmt '));';
         end;
         else _tmpvar = outputname;

         %if &quoteRule=A
         %then %do;
            put 'format ' _tmpvar ' $quote.;' ;
         %end;
         %else %if &quoteRule ne N and &outputType=7
         %then %do;
            if dataType^="N" then put 'format ' _tmpvar '$quote.;';
         %end;
      end;
      %if &outputType = 4
      %then %do;
        if format ^= ''
        then do;
         %if &quoteRule^=A
         %then %do;
            put 'put @' position _tmpvar format '@ ;';
         %end;
         %else %do;
            put 'put @' position _tmpvar '@ ;';
         %end;
        end;
        else do;
           put 'put @' position _tmpvar '@ ;';
        end;
      %end;
      %else %do;
         put 'put ' _tmpvar '@ ;';
      %end;

      if done then put 'put;' / 'run;';

   run;
   %inc expfile;
    /*S0401805*/
    %if (&SYSCC ne 0 and &SYSCC ne 4) and %symexist(SYSERRORTEXT) %then
    %do;
        /*send back the SYSERRORTEXT to user */
        %let MAERROR=399;/*set a  non used error code so that core can send error text back as is */
        %let MAMsg=Macro[&sysmacroname.].SYSERRORTEXT=[&SYSERRORTEXT];
        %return;
    %end;
%mend;
