/*****************************************************************/
/* Copyright (c) 2007 - 2013 by SAS Institute Inc., Cary, NC,    */
/*                    USA.                                       */
/*             All Rights Reserved.                              */
/*  This software is protected by copyright laws and             */
/*               international treaties                          */
/*                                                               */
/*         U.S. Government Restricted Rights                     */
/* Use, duplication, or disclosure of this software and related  */
/*  documentation by the U.S. government is subject to the       */
/*   Agreement with SAS Institute and the restrictions set forth */
/*  in FAR 52.227-19, Commercial Computer Software -             */
/*              Restricted Rights ( June 1987 )                  */
/*                                                               */
/*****************************************************************/
/* NAME: mahcount                                                */
/* VERSION: 6.1                                                  */
/* DESCRIPTION:                                                  */
/*                                                               */
/* PRODUCT: MA                                                   */
/* USAGE: %mahcount( InTable, VarList, IdList, SubjectKeys,      */
/*  OutTable, Type, Mode, Option, IdListSeparator );             */
/*      InTable - Input Table Name                               */
/*      Bins - space separated list of bins, < bin = Bin#        */
/*             missing is automatically provided as bin .        */
/*                                                               */
/*****************************************************************/
/*****************************************************************************/
/* PRISM VERSION: $Revision: 1.1.4.1 $ ;
/* Revision Date: $Date: 2013/07/25 13:07:58 $  ;
/*****************************************************************************/
/*****************************************************************************/
/* HISTORY:                                                                  */
/* DATE       Defect No  Comments/updated by                                 */
/* ---------  ---------  --------------------------------------------------- */
/* 23APR2013             GA                                                  */
/*****************************************************************************/


%macro mahcount(InTable, VarList, SubjectKeys, OutTable, Type, Option, nBins, upperBounds, increments, yearNBins=, yearUpperBounds=, yearIncrements=);
options source source2 mprint;

/* ASSUMPTION:
 * For all input keys, there is not another key that is named
 * (CurrentKey)_Month, (CK)_Day, (CK)_Year
 *
 * The incoming table is also ordered by the SubjectKeys (this makes proc summary processing easier)
 */
%let newVarList=;
data binView / view=binView;
   set &InTable;
   keep &SubjectKeys;
   %let i=1;
   %let var=%scan(&VarList,&i,%str( ));
   %do %while ( %length(&var) > 0 );
      %let nBins = %scan(&nBins,&i,%str( ));
      %let upperBound = %scan(&upperBounds,&i,%str( ));
      %let increment = %scan(&increments,&i,%str( ));

      %if &nBins le 0 %then %let nBins = 16;
      %let oVar=O&var;

      %createBins( &var, &oVar, &nBins, &upperBound, &increment );

      %if ( %upcase(&Type) eq DATE or %upcase(&Type) eq DATETIME ) %then %do;
         %if %upcase(&Type) eq DATE %then %let sVar = &var;
         %else %let sVar=datepart(&var);

         &oVar._day = day(&sVar);
         &oVar._month = month(&sVar);

         %let yBins = %scan(&yearNBins,&i,%str( ));
         %let yUpperBound = %scan(&yearUpperBounds,&i,%str( ));
         %let yIncrement = %scan(&yearIncrements,&i,%str( ));
         %createBins( year(&sVar), &oVar._year, &yBins, &yUpperBound, &yIncrement, useFloor=1 );

         %let newVarList=%trim(&newVarList) &oVar &oVar._day &oVar._month &oVar._year;
      %end; %else
         %let newVarList=%trim(&newVarList) &oVar;

      %let i=%eval(&i+1);
      %let var=%scan(&VarList,&i,%str( ));
   %end;
   keep &newVarList;
run;

/* Do the summary counts on the binned items.  The histogram STP will merge the bins later. */
%mascount( binView, &newVarList, &SubjectKeys, &OutTable, HISTO, &Option );

%mend;

%macro createBins( invar, outvar, nBins, upperBound, increment, useFloor=0 );
   %local i;
   if missing(&invar) then &outvar=.;
   %do i= 1 %to &nBins;
      %if &i eq &nBins %then
         else &outvar=&i;
      %else %if &useFloor %then
         else if &invar lt %sysfunc(floor(&upperBound)) then &outvar=&i;
      %else
         else if &invar lt &upperBound then &outvar=&i;
      ;
      %let upperBound=%sysevalf(&upperBound + &increment);
   %end;
%mend;
