/****************************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.           */
/*                     All Rights Reserved.                                 */
/*       This software is protected by copyright laws and                   */
/*                    international treaties                                */
/*                                                                          */
/*               U.S. Government Restricted Rights                          */
/* Use, duplication, or disclosure of this software and related             */
/*    documentation by the U.S. government is subject to the                */
/*  Agreement with SAS Institute and the restrictions set forth             */
/*       in FAR 52.227-19, Commercial Computer Software -                   */
/*                Restricted Rights ( June 1987 )                           */
/*                                                                          */
/****************************************************************************/
/* NAME:maptast.sas                                                         */
/* VERSION:5.41                                                             */
/* DESCRIPTION: Common Data Interface macro to compute Marketing Contact    */
/* Counts                                                                   */
/*                                                                          */
/* PRODUCT: MA                                                              */
/* USAGE: %maptast                                                          */
/****************************************************************************/
/* Macro variables:                                                         */
/* Name       Desc                                          Value           */
/* ch_tbl     ci_contact_history table name                                 */
/* cp_tbl     ci_cell_package table name                                    */
/* cpt_tbl    ci_cell_package_treatment_cnt table name                      */
/* data_tbl   Input data table                                              */
/* dbengine   Name of the database Engine,Update tables Statments are       */
/*            Engine Dependent                                              */
/* libnm      Library Name for CI Common Data Model                         */
/* newid      Unique new id for creating data processing tables in templib  */
/* PerfFlag   D or T,to decide whether to generate summary by looking at    */
/*            history or just work with the input data set                  */
/*            D- Default, T-True Counts                                     */
/* pth_tbl    ci_presented_treatment_history table name                     */
/* sub_key_sp Space Separated Subject Key Column  List                      */
/* tmplibnm   Temporary Library Name for data processing                    */
/****************************************************************************/
/* Change log                                                               */
/* Date    DefectNo  Comments                                               */
/* 24Mar10 S0651614  New Version for Aster Partition Key Support            */
/* 15Jun12 S0782605  use proc sql to get counts instead of proc summary     */
/****************************************************************************/
%macro maptast(ch_tbl=,cp_tbl=,cpt_tbl=,data_tbl=,dbengine=,libnm=,newid=,
                perfflag=,pth_tbl=,sub_key_sp=,tmplibnm=,);
    /*if PerfFlag = T then create summary by joining the history tables*/
    %if &perfflag. eq T %then
    %do;
        /* Step2a Create a worskset with distinct cell_package_sk & subject_id's from the input set */
        %if &MADebug = Y %then  %put Step2a Start Time:%sysfunc(datetime(),datetime20);
        /*Append the tmp table to TblNames variable*/
        %let TblNames=&TblNames. %sysfunc(strip(pt_DCpk&newid.));
        proc sql;
            create table &tmplibnm..pt_DCpk&newid. ( &MppDBKey. )  as
                select distinct cell_package_sk,&sub_key_cs from &tmplibnm..&data_tbl.
        ;quit;
        %if &MADebug = Y %then  %put Step2a End Time:%sysfunc(datetime(),datetime20);
        %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

        /* Step2b: Get presented treatment history records for input cell_package and subject_id */
        /* add column counted_flag = Y for all records */
        %if &MADebug = Y %then  %put Step2b Start Time:%sysfunc(datetime(),datetime20);
        %let TblNames=&TblNames. %sysfunc(strip(pt_hist&newid.));

        proc sql;
            create table    &tmplibnm..pt_hist&newid. ( &MppDBKey. )  as
                select      pth.cell_package_sk ,
                            pth.treatment_sk ,
                            %let i = 1;
                            %do %while (&i. ne &subCnt.);
                                pth.&&dbsubCol&i ,
                                %let i = %eval(&i+1);
                            %end;
                            pth.presented_treatment_hist_dttm,
                            'Y' as counted_flg
                from        &tmplibnm..pt_DCpk&newid. t
                inner join  &libnm..&pth_tbl. pth
                on          t.cell_package_sk = pth.cell_package_sk
                %let i = 1;
                %do %while (&i. ne &subCnt);
                    and           t.&&dbSubCol&i=pth.&&dbSubCol&i
                    %let i = %eval(&i+1);
                %end;
                where not exists (
                            select  1 from    &tmplibnm..&data_tbl. i
                            where   i.cell_package_sk=pth.cell_package_sk
                            and     i.treatment_sk=pth.treatment_sk
                            %let i = 1;
                            %do %while (&i. ne &subCnt);
                            and     i.&&dbSubCol&i=pth.&&dbSubCol&i
                                %let i = %eval(&i+1);
                            %end;
                            and     i.Presented_Treatment_Hist_Dttm =pth.Presented_Treatment_Hist_Dttm )
                union all
                select      cell_package_sk , treatment_sk , &sub_key_cs. , presented_treatment_hist_dttm ,
                            'N' as counted_flg
                from        &tmplibnm..&data_tbl.

        ;quit;

        %if &MADebug = Y %then  %put Step2b End Time:%sysfunc(datetime(),datetime20);
        %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

         /* Step2d: Get latest contact date time */
        %if &MADebug = Y %then  %put Step2d Start Time:%sysfunc(datetime(),datetime20);
        %let TblNames=&TblNames. %sysfunc(strip(pt_ch_hist&newid.));

        proc sql;
            create table    &tmplibnm..pt_ch_hist&newid. ( &MppDBKey. ) as
                select      ipt.cell_package_sk ,
                            ipt.treatment_sk ,
                            %let i = 1;
                            %do %while (&i. ne &subCnt.);
                                ipt.&&dbsubCol&i ,
                                %let i = %eval(&i+1);
                            %end;
                            ipt.presented_treatment_hist_dttm,
                            ipt.counted_flg,
                            max(ch.contact_dttm) as Ch_Dttm
                from        &tmplibnm..pt_hist&newid. ipt
                inner join  &libnm..&ch_tbl ch
                on          ipt.cell_package_sk = ch.cell_package_sk
                %let i = 1;
                %do %while (&i. ne &subCnt);
                    and           ipt.&&dbSubCol&i=ch.&&dbSubCol&i
                    %let i = %eval(&i+1);
                %end;
                and         ch.contact_dttm <= ipt.presented_treatment_hist_dttm
                group by    ipt.cell_package_sk,ipt.treatment_sk,
                            %let i = 1;
                            %do %while (&i. ne &subCnt.);
                                ipt.&&dbsubCol&i ,
                                %let i = %eval(&i+1);
                            %end;
                            ipt.presented_treatment_hist_dttm,ipt.counted_flg
        ;quit;
        %if &MADebug = Y %then  %put Step2d End Time:%sysfunc(datetime(),datetime20);
        %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

        /* Step2e: Separate counted & uncounted records and create two copies of uncounted one for cell_package level & other for treatment level */
        %if &MADebug = Y %then  %put Step2e Start Time:%sysfunc(datetime(),datetime20);
        %let TblNames=&TblNames. %sysfunc(strip(pt_cp_uncounted&newid.));
        proc sql;
            Create table    &tmplibnm..pt_cp_uncounted&newid. ( &MppDBKey. ) as
                select      t1.* from &tmplibnm..pt_ch_hist&newid. t1
                where       t1.counted_flg = 'N' and
                not exists (select  1  from  &tmplibnm..pt_ch_hist&newid. t2
                            where   t2.cell_package_sk=t1.cell_package_sk
                            %let    i = 1;
                            %do %while (&i. ne &subCnt);
                                and     t2.&&dbSubCol&i=t1.&&dbSubCol&i
                                %let i = %eval(&i+1);
                            %end;
                            and     t2.Ch_Dttm=t1.Ch_Dttm
                            and     t2.counted_flg = 'Y')
        ;quit;

        %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

        %let TblNames=&TblNames. %sysfunc(strip(pt_cpt_uncounted&newid.));
        proc sql;
            Create table    &tmplibnm..pt_cpt_uncounted&newid. ( &MppDBKey. ) as
                select      t1.* from &tmplibnm..pt_ch_hist&newid. t1
                where       t1.counted_flg = 'N' and
                not exists (  select  1  from &tmplibnm..pt_ch_hist&newid. t2
                            where   t2.cell_package_sk=t1.cell_package_sk
                            and     t2.treatment_sk=t1.treatment_sk
                            %let i = 1;
                            %do %while (&i. ne &subCnt);
                            and     t2.&&dbSubCol&i=t1.&&dbSubCol&i
                                %let i = %eval(&i+1);
                            %end;
                            and     t2.Ch_Dttm=t1.Ch_Dttm
                            and     t2.counted_flg = 'Y')
        ;quit;

        %if &MADebug = Y %then  %put Step2e End Time:%sysfunc(datetime(),datetime20);
        %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

/* Step3- Generate Summary tables                           */

        /* Step3a: Generate Cell_Package level summary */
        /* Step3a1: Sort data for Cell_Package level summary */
        %if &MADebug = Y %then  %put Step3a1 Start Time:%sysfunc(datetime(),datetime20);
        %let TblNames=&TblNames. %sysfunc(strip(pt_cp_b4summary&newid.));
        /* Instead of proc sort Using proc sql create table with group by to remove duplicate row for a group condition */
        proc sql;
            create table    &tmplibnm..pt_cp_b4summary&newid. ( &MppDBKey. ) as
                select      cell_package_sk , &sub_key_cs. , Ch_Dttm , count(*) as DummyCnt
                from        &tmplibnm..pt_cp_uncounted&newid.
                group by    cell_package_sk , &sub_key_cs. , Ch_Dttm
        ;quit;
        %if &MADebug = Y %then  %put Step3a1 End Time:%sysfunc(datetime(),datetime20);
        %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

        /* Step3b: Generate Cell_Package treatment level summary */
        /* Step3b1: Sort data for Cell_Package treatment level summary */
        %if &MADebug = Y %then  %put Step3b1 Start Time:%sysfunc(datetime(),datetime20);
        %let TblNames=&TblNames. %sysfunc(strip(pt_cpt_b4summary&newid.));
        /* Instead of proc sort Using proc sql create table with group by to remove duplicate row for a group condition */
        proc sql;
            create table    &tmplibnm..pt_cpt_b4summary&newid. ( &MppDBKey. ) as
                select      cell_package_sk , treatment_sk , &sub_key_cs. , Ch_Dttm , count(*) as DummyCnt
                from        &tmplibnm..pt_cpt_uncounted&newid.
                group by    cell_package_sk , treatment_sk , &sub_key_cs. , Ch_Dttm
        ;quit;
        %if &MADebug = Y %then  %put Step3b1 End Time:%sysfunc(datetime(),datetime20);
        %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

    %end;/*End of PerfFlag= T*/

    /*if PerfFlag = D then create the summary just based on the input data table*/
    %if &perfflag. eq D %then
    %do;
        %let TblNames=&TblNames. %sysfunc(strip(pt_cp_b4summary&newid.));
        /* Instead of proc sort Using proc sql create table with group by to remove duplicate row for a group condition */
        proc sql;
            create table    &tmplibnm..pt_cp_b4summary&newid. ( &MppDBKey. ) as
                select      cell_package_sk , &sub_key_cs. ,count(*) as DummyCnt
                from        &tmplibnm..&data_tbl.
                group by    cell_package_sk , &sub_key_cs.
        ;quit;
        %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

        %let TblNames=&TblNames. %sysfunc(strip(pt_cpt_b4summary&newid.));
        /* Instead of proc sort Using proc sql create table with group by to remove duplicate row for a group condition */
        proc sql;
            create table    &tmplibnm..pt_cpt_b4summary&newid. ( &MppDBKey. ) as
                select      cell_package_sk, treatment_sk ,&sub_key_sp.,count(*) as DummyCnt
                from        &tmplibnm..&data_tbl.
                group by    cell_package_sk, treatment_sk ,&sub_key_sp.
        ;quit;
        %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

    %end;/*End of PerfFlag= D*/

    /* Step3a2: Summaries data for Cell_Package level summary */
    %if &MADebug = Y %then  %put Step3a2 Start Time:%sysfunc(datetime(),datetime20);
    %let TblNames=&TblNames. %sysfunc(strip(pt_cp_summary&newid.));
    %* S0782605 - use proc sql to get counts instead of proc summary ;
    proc sql;
        create table    &tmplibnm..pt_cp_summary&newid. ( &MppDBKey. )  as
             select     CELL_PACKAGE_SK , count(*) as MARKETINGCONTACTCNT
             from       &tmplibnm..PT_CP_B4SUMMARY&newid.
             group by   CELL_PACKAGE_SK ;
    quit;

    %if &MADebug = Y %then  %put Step3a2 End Time:%sysfunc(datetime(),datetime20);

    /* Step3b2: Summaries data for Cell_Package treatment level summary */
    %if &MADebug = Y %then  %put Step3b2 Start Time:%sysfunc(datetime(),datetime20);
    %let TblNames=&TblNames. %sysfunc(strip(pt_cpt_summary&newid.));
    %* S0782605 - use proc sql to get counts instead of proc summary ;
    proc sql;
        create table    &tmplibnm..pt_cpt_summary&newid.  ( &MppDBKey. )  as
            select      CELL_PACKAGE_SK , TREATMENT_SK , count(*)  as MARKETINGCONTACTCNT
            from        &tmplibnm..pt_cpt_b4summary&newid.
            group by    CELL_PACKAGE_SK , TREATMENT_SK ;
    quit;

    %if &MADebug = Y %then  %put Step3b2 End Time:%sysfunc(datetime(),datetime20);

%exit:
%mend;
