/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/************************************************************************************/
/* NAME: maptpub.sas                                                                */
/* VERSION: 5.4                                                                   */
/*                                                                                  */
/* DESCRIPTION: Macro to update the following data model tables                     */
/*              CI_PACKAGE                                                          */
/*              CI_PACKAGE_X_TREATEMENT                                             */
/*                                                                                  */
/*  Parameters:                                                                     */
/*             cp   - Physical table name for Package                               */
/*             cpxt - Physical table name for Package_X_TreatmenT                   */
/*                                                                                  */
/* PRODUCT: MA                                                                      */
/* USAGE: %maptpub                                                                  */
/*                                                                                  */
/************************************************************************************/
/*****************************************************************************************/
/*                                                                                       */
/* History:                                                                              */
/*   27APR2007 - First version of the macro.                                             */
/*                                                                                       */
/* S0504678 - Changes for Treatment_type_UDF & Treatment_Ext tables. CI_TREATMENT        */
/* table population moved to a new script- sinmvp                                        */
/*                                                                                       */
/* DATE       Defect No  Comments                                                        */
/* 30Dec2008  S0547352   The REPLACE statement is removed from the unwanted places       */
/*                                                                                       */
/* 25May2009  S0559332   The STOP statement is removed from the DATA MODIFY block        */
/*                       where only REPLACE or OUTPUT is used                            */
/*                                                                                       */
/* 18Jun2009  S0583163   Reset the SYSCC value to 100000 in case of error                */
/*                                                                                       */
/* 22Jun2009  S0575393   Allow the missing values from the input set to overwrite the    */
/*                       master table. This is mainly required when default UDF values   */
/*                       are reset to blank.                                             */
/*                                                                                       */
/* 30Mar2010  5.4        Added PACKAGE_HASH_VAL column to the by variable                */
/*                       Moved CI_PACKAGE_X_TREATMENT table to a new script              */
/*                                                                                       */
/* 18May2010  S0670074   Remove Package_Hash_Val from CI_Package table                   */
/*****************************************************************************************/

%macro maptpub(cp=);

%put MAPTPUB macro Start Time - %sysfunc(datetime(), datetime24.2);

/* CI_PACKAGE: Update if exists otherwise directly insert */
data &libnm..&cp.;
    modify &libnm..&cp.(&dswriteopts.) work.&cp.;
    by PACKAGE_SK ;
    select(_IORC_);
        when (%SYSRC(_DSENMR))
        do;
            _error_= 0;
            output;
        end;
        otherwise;
    end;
run;

/* Exit the process if any error */
%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC. ^= 0 %then
    %do;
        %let SYSCC=100000;
        %let MAError = 847;
        %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
%end;
%else %if &SYSERR. > 4  %then
%do;
    %let MAError =847;
    %let MAMsg =Macro[&sysmacroname.]. Data Modify Stmt error while attempting to update Change_Type table . Check the log for details;
%end;

/* The CI_PACKAGE_X_TREATMENT table update moved to a new script mapxt.sas. Now this table has got a new column
   contributing_cell_package_sk which reference to CI_CELL_PACKAGE, there needs to be populated after the cell_package
   table */

%put MAPTPUB macro end Time - %sysfunc(datetime(), datetime24.2);
%mend;
