/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/************************************************************************************/
/* NAME: mapub.sas                                                                  */
/* VERSION: 5.4.9                                                                   */
/*                                                                                  */
/* DESCRIPTION: Macro to publish the campaign records and subsequently insert       */
/*              records to all relevant tables where there is a new record for the  */
/*              new campaign insert in this process.                                */
/* Parameter:                                                                       */
/*    camp          - Physical table name for Campaign                              */
/*    camp_char_udf - Physical table name for Campaign_Char_Udf                     */
/*    camp_num_udf  - Physical table name for Campaign_Num_Udf                      */
/*    camp_dt_udf   - Physical table name for Campaign_Date_udf                     */
/*    camp_ext      - Physical table name for Campaign_Ext                          */
/*    comm          - Physical table name for Communication                         */
/*    comm_char_udf - Physical table name for Communication_Char_udf                */
/*    comm_num_udf  - Physical table name for Communication_Num_udf                 */
/*    comm_dt_udf   - Physical table name for Communication_Date_udf                */
/*    comm_ext      - Physical table name for Communication_Ext                     */
/*    ccell         - Physical table name for Marketing_Cell                        */
/*    ccp           - Physical table name for Cell_Package                          */
/*    ccpt          - Physical table name for Cell_Package_Treatment_Count          */
/*    ccl           - Physical table name for Change_log                            */
/*                                                                                  */
/* PRODUCT: MA                                                                      */
/* USAGE: %mapub                                                                    */
/*                                                                                  */
/************************************************************************************/
/*****************************************************************************/
* PRISM VERSION: $Revision: 1.1.2.7.2.9.2.11.4.1.4.5 $ ;
* Revision Date: $Date: 2013/08/14 04:20:35 $ ;
/*****************************************************************************/
/************************************************************************************/
/*                                                                                  */
/* History:                                                                         */
/*   27APR2007 - First version of the macro.                                        */
/*                                                                                  */
/* S0479619 - Added new CAMPAIGN_FOLDER_TXT column to CI_CAMPAIGN table             */
/* S0490547 - 5 new Budget columns added to CI_Communication table                  */
/*                                                                                  */
/* DATE       Defect No  Comments                                                   */
/* 14Oct2008  S0504678   a)Removed SEQ_NO and Check_List_Flg from Campaign UDF tbls */
/*                       b)Added PACKAGE_SK & CONTROL_GROUP_TYPE_CD to treament_cnt */
/*                       table part of the 5.3 data model changes.                  */
/*                                                                                  */
/* 5Feb2009   S0549213   Retain the orginal column length from input table, which   */
/*                       gets truncated to 200 char length when used tranwrd func.  */
/*                                                                                  */
/* 12Feb2009  S0506508   Add new Campaign_Group_Sk to Campaign table                */
/*                                                                                  */
/* 18Jun2009  S0583163   Reset the SYSCC value to 100000 in case of error           */
/*                                                                                  */
/* 22Jun2009  S0575393   Allow the missing values from the input set to overwrite   */
/*                       the master table. This is mainly required when default     */
/*                       UDF values are reset to blank.                             */
/*                                                                                  */
/* 16Jul2009  S0554379   a) Fix the campaign_cd column name in the version flag     */
/*                       update sql code block                                      */
/*                       b) Eliminate the additional white space created from the   */
/*                       put statement for all character variable                   */
/*                                                                                  */
/* 29Jan2010  a2ma54     Two new columns added                                      */
/*                       CI_COMMUNICATION#COMMUNICATION_REOCCUR_TYPE_CD             */
/*                       CI_CELL_PACKAGE#MAX_TREATMENT_CNT                          */
/*                                                                                  */
/* 26Mar2010  a2ma54     a)New column added for Offer arbitration                   */
/*                       CI_CELL_PACKAGE#PACKAGE_HASH_VAL                           */
/*                       b)Moved Treatment_Cnt table processing to macptpub         */
/*                                                                                  */
/* 09Apr2010  S0658600   Create new cell_package base table for preparing           */
/*                       treatment_cnt input set                                    */
/*                                                                                  */
/* 12May2010  5.4Iteration5 Publish DELETED_FLG to CI_COMMUNICATION table           */
/*                                                                                  */
/* 18May2010  S0670074   Remove Package_Hash_Val from cell_package table            */
/*                                                                                  */
/* 15Jul2010  S0684325   Control_Group_Type_Cd added to marketing_cell table        */
/*                                                                                  */
/* 29Jul2010  S0648079   Additional condition check for setting current_version_flg */
/*                                                                                  */
/* 16Sep2010  Task-TA559 New column CI_COMMUNICATIONS#BUDGET_UNIT_USAGE_AMT         */
/*                                                                                  */
/* 03Jan2012  S0767119   Performance Improvements. Append stmt added                */
/*                                                                                  */
/* 06Jan2012  S0782404   Fix the CAMP/COMM UDF table population                     */
/* 12Feb2013  S0924649   541MA07: : -ERROR: BY VARIABLES ARE NOT PROPERLY SORT...   */
/* 26Feb2013  S0937584   541MA07: Publish error on DTTM fields for CI_CAMPAIGN...   */
/* 14Aug2013  S0996040   Remove &dswriteopts from PROC APPEND                       */
/************************************************************************************/

%macro mapub(camp= , camp_char_udf= , camp_num_udf= , camp_dt_udf= , camp_ext= , comm= ,
                 comm_char_udf= , comm_num_udf= , comm_dt_udf= , comm_ext= , ccell= , ccp= , ccpt=, ccl=);


/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put Current Version of the code: 541MA09;
    %put Last Modified on 14Aug2013  ;
    %put MAPUB macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

/* Create an additional column in all Comm Udf input tables
   also create a Base table to use in the Transpose procedure */

/* S0782404 Create _CNT variables to process the UDF table only when records exists */

data work.&camp_char_udf. work.CampCudfBase (drop=insertflg);
    set work.&camp_char_udf. NOBS=cnt;
    insertflg = 0;
    call symputx('camp_cudf_cnt',cnt,'G');
run;

data work.&camp_num_udf. work.CampNudfBase (drop=insertflg);
    set work.&camp_num_udf. NOBS=cnt;
    insertflg = 0;
    call symputx('camp_nudf_cnt',cnt,'G');
run;

data work.&camp_dt_udf. work.CampDudfBase (drop=insertflg);
    set work.&camp_dt_udf. NOBS=cnt;
    insertflg = 0;
    call symputx('camp_dudf_cnt',cnt,'G');
run;

data work.&comm_char_udf. work.CommCudfBase (drop=insertflg);
    set work.&comm_char_udf. NOBS=cnt;
    insertflg = 0;
    call symputx('comm_cudf_cnt',cnt,'G');
run;

data work.&comm_num_udf. work.CommNudfBase (drop=insertflg);
    set work.&comm_num_udf. NOBS=cnt;
    insertflg = 0;
    call symputx('comm_nudf_cnt',cnt,'G');
run;

data work.&comm_dt_udf. work.CommDudfBase (drop=insertflg);
    set work.&comm_dt_udf. NOBS=cnt;
    insertflg = 0;
    call symputx('comm_dudf_cnt',cnt,'G');
run;

data work.&comm.;
    set work.&comm.;
    insertflg=0;
run;

/* Create a new cell_package input set. This is required for two reasons
   i) The cell_package table record is removed from the input set once it is inserted to the target table. This
   leads to not having 0 reocrds when creating input set for the cell_package_treatment_cnt table.
   ii) The cell_package_treatment_cnt table is updated only once as against for each campaign/communicaiton
*/
proc sort data=work.&ccp. out=work.&ccp._base;
    by cell_package_sk package_sk;
run;

/* Exit the process if any error */
%if &SYSERR. > 4 %then
%do;
    %let MAError=723;
    %let MAMsg=Macro[&sysmacroname.]. Data Step failed - while attempting to create temp dataset. Check the log for details.;
    %return;
%end;

/* Pre-process the Cell_Package input table to get Treatment_Sk and _Cd to
   insert in to CI_Cell_Package_Treatment_Cnt table */
/*%mapubutl(ccp=&ccp., cp=&cp., cpxt=&cpxt.,ct=&ct., resulttbl=&ccpt.,utlflg=C );*/

/* S0549213: Retain the orginal length of the following columns */
data _null_;
    set work.&camp.;
    call symput('_ln_camp_nm',put(sum(vlength(campaign_nm),countc(campaign_nm,'"')),Best.));
    call symput('_ln_camp_desc',put(sum(vlength(campaign_desc),countc(campaign_desc,'"')),Best.));
    call symput('_ln_camp_o_nm',put(sum(vlength(campaign_owner_nm),countc(campaign_owner_nm,'"')),Best.));
    call symput('_ln_modify_by',put(sum(vlength(last_modified_by_user_nm),countc(last_modified_by_user_nm,'"')),Best.));
    call symput('_ln_approved_by',put(sum(vlength(approval_given_by_nm),countc(approval_given_by_nm,'"')),Best.));
    call symput('_ln_bc_nm',put(sum(vlength(business_context_nm),countc(business_context_nm,'"')),Best.));
run;

data _null_;
    file &campsrc. lrecl=32767;
    set work.&camp. end=done;

    length _camp_nm $ &_ln_camp_nm.;
    length _camp_desc $ &_ln_camp_desc.;
    length _camp_o_nm $ &_ln_camp_o_nm.;
    length _modify_by $ &_ln_modify_by.;
    length _approved_by $ &_ln_approved_by.;
    length _bc_nm $ &_ln_bc_nm.;

    _camp_nm=tranwrd(campaign_nm,'"','""');
    _camp_desc=tranwrd(campaign_desc,'"','""');
    _camp_o_nm=tranwrd(campaign_owner_nm,'"','""');
    _modify_by=tranwrd(last_modified_by_user_nm,'"','""');
    _approved_by=tranwrd(approval_given_by_nm,'"','""');
    _bc_nm=tranwrd(business_context_nm,'"','""');

    if _n_=1 then
    do;
        put '%macro publish_master;';
    end;


    put ' proc sql; ';
    put '   SELECT COUNT(CAMPAIGN_SK) into :camp_upd_cnt FROM ' "&libnm..&camp" ' WHERE CAMPAIGN_SK = ' CAMPAIGN_SK ';';
    put ' quit; ';

    put '%if &camp_upd_cnt > 0 %then';
    put '%do ;';

    /* update campaign table */
    /* S0506508: Campaign_Group_SK is added part of the Campaign table */
    /* S0554379: Fix campagin_cd column name & white space created in put statement */
    put ' proc sql;';
    put ' update '"&libnm..&camp";
    put '    SET ';
    put '        CAMPAIGN_CD ="' CAMPAIGN_CD +(-1) '",';
    put '        CAMPAIGN_GROUP_SK =' CAMPAIGN_GROUP_SK ',';
    put '        CAMPAIGN_FOLDER_TXT ="' CAMPAIGN_FOLDER_TXT +(-1) '",';
    put '        CAMPAIGN_VERSION_NO =' CAMPAIGN_VERSION_NO',';
    put '        CURRENT_VERSION_FLG  ="' CURRENT_VERSION_FLG +(-1) '",';
    put '        CAMPAIGN_NM  ="' _camp_nm +(-1) '",';
    put '        CAMPAIGN_DESC ="' _camp_desc +(-1) '",';
    put '        CAMPAIGN_STATUS_CD   ="' CAMPAIGN_STATUS_CD +(-1) '",';
    put '        CAMPAIGN_OWNER_NM  ="' _camp_o_nm +(-1) '",';
    put '        MIN_BUDGET_OFFER_AMT =' MIN_BUDGET_OFFER_AMT',';
    put '        MAX_BUDGET_OFFER_AMT =' MAX_BUDGET_OFFER_AMT',';
    put '        MIN_BUDGET_AMT=' MIN_BUDGET_AMT',';
    put '        MAX_BUDGET_AMT =' MAX_BUDGET_AMT',';
    put '        START_DTTM="' START_DTTM '"dt,';
    put '        END_DTTM ="' END_DTTM'"dt,';
    put '        RUN_DTTM ="' RUN_DTTM'"dt,';
    put '        LAST_MODIFIED_DTTM  ="' LAST_MODIFIED_DTTM'"dt,';
    put '        LAST_MODIFIED_BY_USER_NM="' _modify_by +(-1) '",';
    put '        APPROVAL_DTTM ="' APPROVAL_DTTM'"dt,';
    put '        APPROVAL_GIVEN_BY_NM ="' _approved_by +(-1) '",';
    put '        DELETED_FLG  ="' DELETED_FLG +(-1) '",';
    put '        BUSINESS_CONTEXT_NM ="' _bc_nm +(-1) '",';
    put '        CAMPAIGN_TYPE_CD="' CAMPAIGN_TYPE_CD +(-1) '",';
    put '        PROCESSED_DTTM="&pdttm."dt';
    put '    where (CAMPAIGN_SK = ' CAMPAIGN_SK');';
    put 'quit;';
    put ' ';
    put ' ';

    /* If there are errors, initialize the global variables and return. */
    /* change the Error code according to the xl sheet */
    put '%if %symexist(SYSDBRC) %then';
    put '%do;';
    put '     %if &SYSDBRC ^= 0 %then';
    put '     %do;';
    put '         %let SYSCC=100000;';
    put '         %let MAError=800;';
    put '         %let MAMsg = Macro[&sysmacroname.]. Publish failed for ["'%nrbquote(_camp_nm)'"] with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];';
    put '         %goto eof_camp_' _n_ ';';
    put '     %end;';
    put '%end;';
    put '%else %if &sqlrc > 4  %then';
    PUT '%do;';
    put '    %let SYSCC=100000;';
    put '    %let MAError =800;';
    put '    %let MAMsg = Macro[&sysmacroname.]. PROC SQL error while attempting to update CAMPAIGN ["'%nrbquote( _camp_nm )'"] records SQLRC[&sqlrc]. Check the log for details;';
    put '    %goto eof_camp_' _n_ ';';
    put '%end;';
    put ' ';
    put ' ';
    put '%end;';
    put '%if &camp_upd_cnt = 0 %then';
    put '%do ;'; /* if no rec found for update then insert */
    /* insert into campaign table and subsequently in Communication,Marketing_Cell and cell_package table */
    put '    proc sql;';
    put '        update '"&libnm..&camp"; /* S0648079: Update the current version flag correctly */
    put '            SET ';
    put '            CURRENT_VERSION_FLG  ="N"';
    put '            Where UPCASE(CAMPAIGN_CD) =UPCASE(TRIM(LEFT("' CAMPAIGN_CD +(-1) '"))) and ';
    put '                  UPCASE(CAMPAIGN_NM) =UPCASE(TRIM(LEFT("' _camp_nm +(-1) '"))) and ';
    put '                  UPCASE(CAMPAIGN_FOLDER_TXT) =UPCASE(TRIM(LEFT("' CAMPAIGN_FOLDER_TXT +(-1) '"))) and ';
    put '                  UPCASE(BUSINESS_CONTEXT_NM) =UPCASE(TRIM(LEFT("' _bc_nm +(-1) '"))) and';
    put '                  CURRENT_VERSION_FLG="Y" ;';
    put '  ';
    put '  ';
    put '        insert into ' "&libnm..&camp"' ( ';
    put '            CAMPAIGN_SK,';
    put '            CAMPAIGN_GROUP_SK,';
    put '            CAMPAIGN_CD,';
    put '            CAMPAIGN_FOLDER_TXT,';
    put '            CAMPAIGN_VERSION_NO,';
    put '            CURRENT_VERSION_FLG,';
    put '            CAMPAIGN_NM,';
    put '            CAMPAIGN_DESC,';
    put '            CAMPAIGN_STATUS_CD,';
    put '            CAMPAIGN_OWNER_NM,';
    put '            MIN_BUDGET_OFFER_AMT,';
    put '            MAX_BUDGET_OFFER_AMT,';
    put '            MIN_BUDGET_AMT,';
    put '            MAX_BUDGET_AMT,';
    put '            START_DTTM,';
    put '            END_DTTM,';
    put '            RUN_DTTM,';
    put '            LAST_MODIFIED_DTTM,';
    put '            LAST_MODIFIED_BY_USER_NM,';
    put '            APPROVAL_DTTM,';
    put '            APPROVAL_GIVEN_BY_NM,';
    put '            DELETED_FLG,';
    put '            BUSINESS_CONTEXT_NM,';
    put '            CAMPAIGN_TYPE_CD,';
    put '            PROCESSED_DTTM) ';
    put '        values( ';
    put '            ' CAMPAIGN_SK' ,';
    put '            ' CAMPAIGN_GROUP_SK' ,';
    put '            "' CAMPAIGN_CD +(-1) '",';
    put '            "' CAMPAIGN_FOLDER_TXT +(-1) '",';
    put '            ' CAMPAIGN_VERSION_NO',';
    put '            "' CURRENT_VERSION_FLG +(-1) '",';
    put '            "' _camp_nm +(-1) '",';
    put '            "' _camp_desc +(-1) '",';
    put '            "' CAMPAIGN_STATUS_CD +(-1) '",';
    put '            "' _camp_o_nm +(-1) '",';
    put '            ' MIN_BUDGET_OFFER_AMT',';
    put '            ' MAX_BUDGET_OFFER_AMT',';
    put '            ' MIN_BUDGET_AMT',';
    put '            ' MAX_BUDGET_AMT',';
    put '            "' START_DTTM'"dt,';
    put '            "' END_DTTM'"dt ,';
    put '            "' RUN_DTTM'"dt,';
    put '            "' LAST_MODIFIED_DTTM'"dt,';
    put '            "' _modify_by +(-1) '",';
    put '            "' APPROVAL_DTTM'"dt,';
    put '            "' _approved_by +(-1) '",';
    put '            "' DELETED_FLG +(-1) '",';
    put '            "' _bc_nm +(-1) '",';
    put '            "' CAMPAIGN_TYPE_CD +(-1) '",';
    put '            "&pdttm"dt  ) ;';
    put '    quit;';
    put ' ';
    put ' ';

    /* If there are errors, initialize the global variables and return. */
    put '    %if %symexist(SYSDBRC) %then';
    put '    %do;';
    put '         %if &SYSDBRC ^= 0 %then';
    put '         %do;';
    put '             %let SYSCC=100000;';
    put '             %let MAError=801;';
    put '             %let MAMsg = Macro[&sysmacroname.]. Publish failed for ["'%nrbquote(_camp_nm )'"] with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];';
    put '             %goto eof_camp_' _n_ ';';
    put '         %end;';
    put '     %end;';
    put '    %else %if &sqlrc > 4  %then';
    put '    %do;';
    put '        %let SYSCC=100000;';
    put '        %let MAError =801;';
    put '        %let MAMsg = Macro[&sysmacroname.]. PROC SQL error while attempting to insert CAMPAIGN ["'%nrbquote( _camp_nm )'"] records SQLRC[&sqlrc]. Check the log for details;';
    put '        %goto eof_camp_' _n_ ';';
    put '    %end;';
    put ' ';
    put ' ';

    /* S0782404 New flag is set to avoid the repeat processing of the same UDF records */
    put ' %let initial_pub=Y;';

    /* Start of Communication Table processing:*/

    /* S0782404: At any given time only one campaign will be published and during initial publish i.e, campaign records insert */
    /* it is safe to insert all related camp_udf, comm, comm_udf records without creating a subset with insertflag. This change is */
    /* applied to all UDF table insert */
    put '   data work.commInsert;';
    put '      set work.&comm.;';
    put '      PROCESSED_DTTM="&pdttm"dt;';
    put '   run;';

    /* If there are errors, initialize the global variables and return. */
    put '    %if &SYSERR > 4  %then ';
    put '    %do;';
    put '        %let MAError =723;';
    put '        %let MAMsg = Macro[&sysmacroname.]. Data Step failed - while attempting to create temp dataset. Check the log for details.;';
    put '        %goto eof_camp_' _n_ ';';
    put '    %end;';
    put ' ';
    put ' ';

    /* S0767119: Performance improvement */
    put 'PROC APPEND BASE=&libnm..&comm (&dsinsopts.) DATA=work.commInsert FORCE;';
    put ' ';
    put ' ';

    /* If there are errors, initialize the global variables and return. */
    put '    %if %symexist(SYSDBRC) %then';
    put '    %do;';
    put '         %if &SYSDBRC ^= 0 %then';
    put '         %do;';
    put '             %let SYSCC=100000;';
    put '             %let MAError=802;';
    put '             %let MAMsg = Macro[&sysmacroname.]. Publish failed for ["'%nrbquote( _camp_nm )'"] with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];';
    put '             %goto eof_camp_' _n_ ';';
    put '         %end;';
    put '    %end;';
    put '    %else %if &sqlrc > 4  %then';
    put '    %do;';
    put '        %let SYSCC=100000;';
    put '        %let MAError=802;';
    put '        %let MAMsg = Macro[&sysmacroname.]. PROC SQL error while attempting to insert COMMUNICATION records for CAMPAIGN ["'%nrbquote( _camp_nm )'"] SQLRC[&sqlrc]. Check the log for details;';
    put '        %goto eof_camp_' _n_ ';';
    put '    %end;';
    put ' ';
    put ' ';

    /* COMM UDF Processing: Communication_Char_UDF*/
    %if &comm_cudf_cnt. > 0 %then
    %do;
        put '   data work.commudfInsert;';
        put '       set ' "work.&comm_char_udf.;";
        put '       PROCESSED_DTTM="&pdttm"dt;';
        put '   run;';
        put ' ';


        put 'PROC APPEND BASE=&libnm..&comm_char_udf (&dsinsopts.) DATA=work.commudfInsert FORCE;';
        put ' ';

        /* If there are errors, initialize the global variables and return. */
        put '    %if %symexist(SYSDBRC) %then';
        put '    %do;';
        put '         %if &SYSDBRC ^= 0 %then';
        put '         %do;';
        put '             %let SYSCC=100000;';
        put '             %let MAError=803;';
        put '             %let MAMsg = Macro[&sysmacroname.]. Publish failed for ["'%nrbquote( _camp_nm )'"] with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];';
        put '             %goto eof_camp_' _n_ ';';
        put '         %end;';
        put '    %end;';
        put '    %else %if &sqlrc > 4  %then';
        put '    %do;';
        put '        %let SYSCC=100000;';
        put '        %let MAError=803;';
        put '        %let MAMsg = Macro[&sysmacroname.]. PROC SQL error while attempting to insert COMMUNICATION_CHAR_UDF  records for  CAMPAIGN ["'%nrbquote( _camp_nm )'"] SQLRC[&sqlrc]. Check the log for details;';
        put '        %goto eof_camp_' _n_ ';';
        put '    %end;';
        put ' ';
        put ' ';
    %end;

   /* Communication_Num_UDF  for all new communication inserted above */
    %if &comm_nudf_cnt. > 0 %then
    %do;
        put '   data work.commudfInsert;';
        put '       set ' "work.&comm_num_udf.;";
        put '       PROCESSED_DTTM="&pdttm"dt;';
        put '   run;';
        put ' ';
        put ' ';

        put 'PROC APPEND BASE=&libnm..&comm_num_udf (&dsinsopts.) DATA=work.commudfInsert FORCE;';
        put ' ';

        /* If there are errors, initialize the global variables and return. */
        put '    %if %symexist(SYSDBRC) %then';
        put '    %do;';
        put '         %if &SYSDBRC ^= 0 %then';
        put '         %do;';
        put '             %let SYSCC=100000;';
        put '             %let MAError=804;';
        put '             %let MAMsg = Macro[&sysmacroname.]. Publish failed for ["'%nrbquote( _camp_nm )'"] with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];';
        put '             %goto eof_camp_' _n_ ';';
        put '         %end;';
        put '    %end;';
        put '    %else %if &sqlrc > 4  %then';
        put '    %do;';
        put '        %let SYSCC=100000;';
        put '        %let MAError=804;';
        put '        %let MAMsg =Macro[&sysmacroname.]. PROC SQL error while attempting to insert COMMUNICATION_NUM_UDF  records for CAMPAIGN ["'%nrbquote( _camp_nm )'"] SQLRC[&sqlrc]. Check the log for details;';
        put '        %goto eof_camp_' _n_ ';';
        put '    %end;';
        put ' ';
        put ' ';
    %end;

    /* Communication_Date_UDF  for all new communication inserted above */
    %if &comm_dudf_cnt. > 0 %then
    %do;
        put '   data work.commudfInsert;';
        put '       set ' "work.&comm_dt_udf.;";
        put '       PROCESSED_DTTM="&pdttm"dt;';
        put '   run;';
        put ' ';

        put 'PROC APPEND BASE=&libnm..&comm_dt_udf (&dsinsopts.) DATA=work.commudfInsert FORCE;';
        put ' ';

        /* If there are errors, initialize the global variables and return. */
        put '    %if %symexist(SYSDBRC) %then';
        put '    %do;';
        put '         %if &SYSDBRC ^= 0 %then';
        put '         %do;';
        put '             %let SYSCC=100000;';
        put '             %let MAError=805;';
        put '             %let MAMsg = Macro[&sysmacroname.]. Publish failed for ["'%nrbquote( _camp_nm )'"] with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];';
        put '             %goto eof_camp_' _n_ ';';
        put '         %end;';
        put '    %end;';
        put '    %else %if &sqlrc > 4  %then';
        put '    %do;';
        put '        %let SYSCC=100000;';
        put '        %let MAError =805; ';
        put '        %let MAMsg =Macro[&sysmacroname.]. PROC SQL error while attempting to insert COMMUNICATION_DATE_UDF  records for CAMPAIGN ["'%nrbquote( _camp_nm )'"] SQLRC[&sqlrc]. Check the log for details;';
        put '        %goto eof_camp_' _n_ ';';
        put '    %end;';
        put ' ';
        put ' ';
    %end;

    /* End of Communication and Communication UDF inserts */

    /* Marketing_Cell: Subset to get all Marketing_Cell records for the new Campaign inserted above */

    /* Marketing_Cell: Inserts */
    put '    proc sql;';
    put '        insert into ' "&libnm..&ccell" ' ( ';
    put '            MARKETING_CELL_SK,';
    put '            CAMPAIGN_SK,';
    put '            MARKETING_CELL_CD,';
    put '            MARKETING_CELL_NM,';
    put '            MARKETING_CELL_DESC,';
    put '            CONTROL_GROUP_TYPE_CD,';
    put '            PROCESSED_DTTM)';
    put '        select ';
    put '            MARKETING_CELL_SK,';
    put '            CAMPAIGN_SK,';
    put '            MARKETING_CELL_CD,';
    put '            MARKETING_CELL_NM,';
    put '            MARKETING_CELL_DESC,';
    put '            CONTROL_GROUP_TYPE_CD,';
    put '            "&pdttm"dt ';
    put '        from ' "work.&ccell.;";
    put '    quit;';
    put ' ';
    put ' ';

    /* If there are errors, initialize the global variables and return. */
    put '    %if %symexist(SYSDBRC) %then';
    put '    %do;';
    put '         %if &SYSDBRC ^= 0 %then';
    put '         %do;';
    put '             %let SYSCC=100000;';
    put '             %let MAError=806;';
    put '             %let MAMsg = Macro[&sysmacroname.]. Publish failed for ["'%nrbquote( _camp_nm )'"] with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];';
    put '             %goto eof_camp_' _n_ ';';
    put '         %end;';
    put '    %end;';
    put '    %else %if &sqlrc > 4  %then';
    put '    %do;';
    put '        %let SYSCC=100000;';
    put '        %let MAError =806;';
    put '        %let MAMsg =Macro[&sysmacroname.]. PROC SQL error while attempting to insert Marketing_Cell records for CAMPAIGN ["'%nrbquote( _camp_nm )'"] SQLRC[&sqlrc]. Check the log for details;';
    put '        %goto eof_camp_' _n_ ';';
    put '    %end;';
    put ' ';
    put ' ';

    /* Campaign_Char_UDF:  */
    %if &camp_cudf_cnt. > 0 %then
    %do;
        put '    data work.campudf ;';
        put '        set ' "work.&camp_char_udf.;";
        put '       PROCESSED_DTTM="&pdttm"dt;';
        put '   run;';
        put ' ';

        /* Campaign_Char_UDF: Insert to target table */
        put 'PROC APPEND BASE=&libnm..&camp_char_udf. (&dsinsopts.) DATA=work.campudf FORCE;';
        put ' ';

        /* If there are errors, initialize the global variables and return. */
        put '    %if %symexist(SYSDBRC) %then';
        put '    %do;';
        put '         %if &SYSDBRC ^= 0 %then';
        put '         %do;';
        put '             %let SYSCC=100000;';
        put '             %let MAError=807;';
        put '             %let MAMsg = Macro[&sysmacroname.]. Publish failed for ["'%nrbquote( _camp_nm )'"] with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];';
        put '             %goto eof_camp_' _n_ ';';
        put '         %end;';
        put '    %end;';
        put '    %else %if &sqlrc > 4  %then';
        put '    %do;';
        put '        %let SYSCC=100000;';
        put '        %let MAError =807;';
        put '        %let MAMsg =Macro[&sysmacroname.]. PROC SQL error while attempting to insert CAMPAIGN_CHAR_UDF records for CAMPAIGN ["'%nrbquote( _camp_nm )'"] SQLRC[&sqlrc]. Check the log for details;';
        put '        %goto eof_camp_' _n_ ';';
        put '    %end;';
        put ' ';
        put ' ';
    %end;

    /* Campaign_num_udf: */
    %if &camp_nudf_cnt. > 0 %then
    %do;
        put '    data work.campudf ;';
        put '        set ' "work.&camp_num_udf.;";
        put '       PROCESSED_DTTM="&pdttm"dt;';
        put '   run;';
        put ' ';

        /* Campaign_num_udf: Insert to target table */
        put 'PROC APPEND BASE=&libnm..&camp_num_udf. (&dsinsopts.) DATA=work.campudf FORCE;';
        put ' ';

        /* If there are errors, initialize the global variables and return. */
        put '    %if %symexist(SYSDBRC) %then';
        put '    %do;';
        put '         %if &SYSDBRC ^= 0 %then';
        put '         %do;';
        put '             %let SYSCC=100000;';
        put '             %let MAError=808;';
        put '             %let MAMsg = Macro[&sysmacroname.]. Publish failed for ["'%nrbquote( _camp_nm )'"] with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];';
        put '             %goto eof_camp_' _n_ ';';
        put '         %end;';
        put '    %end;';
        put '    %else %if &sqlrc > 4  %then';
        put '    %do;';
        put '        %let SYSCC=100000;';
        put '        %let MAError =808;';
        put '        %let MAMsg =Macro[&sysmacroname.]. PROC SQL error while attempting to insert CAMPAIGN_NUM_UDF records for CAMPAIGN ["'%nrbquote( _camp_nm )'"] SQLRC[&sqlrc]. Check the log for details;';
        put '        %return;';
        put '        %goto eof_camp_' _n_ ';';
        put '    %end;';
        put ' ';
        put ' ';
    %end;

    /* Campaign_date_udf: Create Subset */
    %if &camp_dudf_cnt. > 0 %then
    %do;
        /* Campaign_date_udf: Create Subset */
        put '    data work.campudf ;';
        put '        set ' "work.&camp_dt_udf.;";
        put '       PROCESSED_DTTM="&pdttm"dt;';
        put '   run;';
        put ' ';

        /* Campaign_date_udf: Insert to target table */

        put 'PROC APPEND BASE=&libnm..&camp_dt_udf. (&dsinsopts.) DATA=work.campudf FORCE;';
        put ' ';

        /* If there are errors, initialize the global variables and return. */
        put '    %if %symexist(SYSDBRC) %then';
        put '    %do;';
        put '         %if &SYSDBRC ^= 0 %then';
        put '         %do;';
        put '             %let SYSCC=100000;';
        put '             %let MAError=809;';
        put '             %let MAMsg = Macro[&sysmacroname.]. Publish failed for ["'%nrbquote( _camp_nm )'"] with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];';
        put '             %goto eof_camp_' _n_ ';';
        put '         %end;';
        put '    %end;';
        put '    %else %if &sqlrc > 4  %then';
        put '    %do;';
        put '        %let SYSCC=100000;';
        put '        %let MAError =809;';
        put '        %let MAMsg =Macro[&sysmacroname.]. PROC SQL error while attempting to insert CAMPAIGN_DATE_UDF records for CAMPAIGN ["'%nrbquote( _camp_nm )'"] SQLRC[&sqlrc]. Check the log for details;';
        put '        %goto eof_camp_' _n_ ';';
        put '    %end;';
        put ' ';
        put ' ';
    %end;

    /* Cell_Package: Subset Cell_Package based on campaign_sk */
    put '    data work.ccpSubset ' "work.&ccp" ';';
    put '        length insertflg 8;';
    put '        set ' "work.&ccp" ';';
    put '        if CAMPAIGN_SK = ' CAMPAIGN_SK 'then ';
    put '        do;';
    put '            insertflg=1;';
    put '            output work.ccpsubset;';
    put '        end;';
    put '        else';
    put '        do;';
    put '            insertflg=0;';
    put '            output ' "work.&ccp" ';';
    put '        end;';
    put '    run;';
    put ' ';
    put ' ';

    /* If there are errors, initialize the global variables and return. */
    put '    %if &SYSERR > 4  %then ';
    put '    %do;';
    put '        %let MAError =723;';
    put '        %let MAMsg = Macro[&sysmacroname.]. Data Step failed - while attempting to create temp dataset. Check the log for details.;';
    put '        %goto eof_camp_' _n_ ';';
    put '    %end;';
    put ' ';
    put ' ';

    /* Cell_Package: Insert subset table to target Cell_Package table */
    put 'data work.ccpInsert;';
    put '   set work.ccpsubset;';
    put '   TOTAL_MARKETING_CONTACT_CNT = 0;';
    put '   TOTAL_ATTEMPTED_CONTACT_CNT = 0;';
    put '   TOTAL_RESPONSE_CNT = 0;';
    put '   TOTAL_SUCCESSFUL_RESPONSE_CNT = 0;';
    put '   PROCESSED_DTTM = "&pdttm"dt;';
    put 'run;';
    put ' ';

    put 'proc append base=&libnm..&ccp (&dsinsopts.) data=work.ccpInsert FORCE;';
    put ' ';

    /* If there are errors, initialize the global variables and return. */
    put '    %if %symexist(SYSDBRC) %then';
    put '    %do;';
    put '         %if &SYSDBRC ^= 0 %then';
    put '         %do;';
    put '             %let SYSCC=100000;';
    put '             %let MAError=810;';
    put '             %let MAMsg = Macro[&sysmacroname.]. Publish failed for ["'%nrbquote( _camp_nm )'"] with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];';
    put '             %goto eof_camp_' _n_ ';';
    put '         %end;';
    put '    %end;';
    put '    %else %if &sqlrc > 4  %then';
    put '    %do;';
    put '        %let SYSCC=100000;';
    put '        %let MAError =810;';
    put '        %let MAMsg =Macro[&sysmacroname.]. PROC SQL error while attempting to insert CELL_PACKAGE records for CAMPAIGN ["'%nrbquote( _camp_nm )'"] SQLRC[&sqlrc]. Check the log for details;';
    put '        %goto eof_camp_' _n_ ';';
    put '    %end;';
    put ' ';
    put ' ';

    /* The CI_CELL_PACKAGE_TREATMENT_CNT table update moved out from this place to macptpub macro processing. Now instead
       populating this table per campaign/communication it will be processed only once at the end of publish process.
       This is introduced populate the ci_package_x_treatment once after populating all cell_package records and
       create the correct input set for treatment_cnt table.
    */

    /* End of update / insert Single Campaign record and all related table inserts */
    put '%end;';

    /* GOTO label to exit the campaign level process */
    put '%eof_camp_' _n_ ':';
    put ' ';
    put ' ';

    if done then
    do;
         put '%mend;';
         put '%publish_master;';
    end;
run;

%if &SYSERR. > 4  %then
%do;
    %let MAError =721;
    %let MAMsg =Macro[&sysmacroname.]. Erro creating the Dynamic Source Code. Check the log for details;
    %return;
%end;

%include &campsrc.;

/* CI_CHANGE_LOG: Update if exists otherwise directly insert */

/* This step is to round the fractions of seconds to nearest second to avoid any unwanted
   unique constraint issues */
data work.&ccl.;
    set work.&ccl.;
    CHANGE_DTTM=ROUND(CHANGE_DTTM);
run;

data &libnm..&ccl.;
    modify &libnm..&ccl. (&dswriteopts.) work.&ccl. UPDATEMODE=NOMISSINGCHECK;
    by CAMPAIGN_SK CHANGE_TYPE_CD CHANGE_DTTM;
    select(_IORC_);
        when (%SYSRC(_SOK))
        do;
            replace;
        end;
        when (%SYSRC(_DSENMR))
        do;
            _error_ = 0;
            output;
        end;
        otherwise
        do;
            stop;
        end;
    end;
run;

/* Exit the process if any error */
%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC. ^= 0 %then
    %do;
        %let MAError = 848;
        %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
%end;
%else
%if &SYSERR. > 4  %then
%do;
    %let MAError =848;
    %let MAMsg =Macro[&sysmacroname.]. Data Modify Stmt error while attempting to update Change_Log table . Check the log for details;
%end;

%if &MADebug. eq Y %then
%do;
    %put MAPUB-Macro End Time - %sysfunc(datetime(), datetime24.2);
%end;

%mend;
