/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/************************************************************************************/
/* NAME: mapubch.sas                                                                */
/* VERSION: 5.4.1                                                                   */
/*                                                                                  */
/* DESCRIPTION: Macro to populate CI_Channel table                                  */
/* Parameter:                                                                       */
/*       cch - Physical table name for ci_channel                                   */
/*                                                                                  */
/* PRODUCT: MA                                                                      */
/* USAGE: %mapubch                                                                  */
/*                                                                                  */
/************************************************************************************/
/*****************************************************************************************/
/*                                                                                       */
/* History:                                                                              */
/*   30Jun2010 - First version of the macro.                                             */
/*                                                                                       */
/* DATE       Defect No  Comments/updated by                                             */
/* 30Jun2010  S0681555   CI_Channel added part of the publish                            */
/*****************************************************************************************/

%macro mapubch(cch=);

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put Current Version of the code: 5.4.1;
    %put Last Modified on 30Jun2010;
    %put MAPUBCH macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

/* Populate the CI_Channel table */
data &libnm..&cch.;
    modify &libnm..&cch.(&dswriteopts.) work.&cch.;
    by CHANNEL_CD;
    select (_IORC_);
        when(%SYSRC(_DSENMR))
        do;
            processed_dttm="&pdttm"dt;
            _error_ = 0;
            output;
        end;
        otherwise;
    end;
run;

%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC. ^= 0 %then
    %do;
        %let SYSCC=100000;
        %let MAError=886;
        %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to update CI_Channel records SYSDBRC[&SYSDBRC]. Check the log for details;
   %end;
%end;
%else %if &SYSERR. > 4 %then
%do;
    %let SYSCC=100000;
    %let MAError=886;
    %let MAMsg=Macro[&sysmacroname.]. Data Modify stmt error while attempting to update CI_Channel records SYSDBRC[&SYSDBRC]. Check the log for details;
%end;

%if &MADebug. eq Y %then
%do;
    %put MAPUBCH macro End Time - %sysfunc(datetime(), datetime24.2);
%end;
%mend;
