/*****************************************************************/
/* Copyright (c) 2004 - 2011 by SAS Institute Inc., Cary, NC,    */
/*                    USA.                                       */
/*             All Rights Reserved.                              */
/*  This software is protected by copyright laws and             */
/*               international treaties                          */
/*                                                               */
/*              U.S. Government Restricted Rights                */
/* Use, duplication, or disclosure of this software and related  */
/*  documentation by the U.S. government is subject to the       */
/*   Agreement with SAS Institute and the restrictions set forth */
/*  in FAR 52.227-19, Commercial Computer Software -             */
/*              Restricted Rights ( June 1987 )                  */
/*                                                               */
/*****************************************************************/

/*******************************************************************************/
/* NAME: mapubdel.sas                                                          */
/* VERSION: 5.4                                                              */
/* DESCRIPTION: Refresh the Campaign and all associated tables in the CI Common*/
/*              data model and keep only the latest and greatest information   */
/*                                                                             */
/* Parameters:                                                                 */
/*           sqlvar = Name of the variable used in the SQL where clause  and   */
/*                    in modify stmt by variable                               */
/*           val    = Value used in the byvar                                  */
/*           src_tbl= Input table name with out the libref                     */
/*           tgt_tbl= Target table name from where the records will be deleted */
/*                                                                             */
/* PRODUCT: CI                                                                 */
/* USAGE: %mapubdel                                                            */
/*                                                                             */
/*******************************************************************************/

/*****************************************************************************************/
/*History:                                                                               */
/*dd-mon-yy                                                                              */
/* 21Dec2009 - First Version of the Code                                                 */
/*                                                                                       */
/* DATE       Defect No  Comments                                                        */
/* 12Jan2010  S0492899   Refresh Common Data model tables                                */
/*                                                                                       */
/* 07Apr2010  S0658031   Package_Hash_Val include part of the delete condition check     */
/*                                                                                       */
/* 10Jan2012  S0823726   Fix the DBKEY useage for SPDS issue                             */
/* 04Dec2012  S0779213   Fix deadlocks around Oracle DELETE processing.    
/*****************************************************************************************/

%macro mapubdel(sqlvar=, val=, src_tbl=, tgt_tbl=, );

/* Exit the macro if there are errors from the previous execution */
%if &MAError. ne %str() %then
%do;
    %return;
%end;

%if &MADebug. eq Y %then
%do;
    %put MAPUBDEL Current Version of the code: F0MA541 HF06;
    %put Last Modified on 04Dec2012;
    %put MAPUBDEL macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

%if &src_tbl eq %str() %then
%do;
    proc sql;
        delete from &libnm..&tgt_tbl.
            where &sqlvar.= &val.;
    quit;

    %if &SQLRC. > 4 %then
    %do;
        %let SYSCC=100000;
        %let MAError =885;
        %let MAMsg =Macro[&sysmacroname.]. Error while attempting to delete records . Check the log for details;
    %end;

%end;
%else %if &src_tbl ne %str() %then
%do;
    /* S0767119 - Performance Improvement */
    %let done=0;
    %let maxkey=0;

    %do %until(&done.=1);
        %let maxkey = %eval(&maxkey+1);
        %let var&maxkey.=%qscan(%bquote(&sqlvar.),&maxkey.,',');
        %if  &&var&maxkey. eq %str() %then
        %do;
            %let done=1;
            %let maxkey=%eval(&maxkey - 1);
        %end;
    %end; /* S0823743 */

    %marowcnt(&src_tbl., deleteCnt);

    %do j = 1 %to &deleteCnt;

        %do i=1 %to &maxkey.;
            %let key&i=;
        %end;

        data _null_;
            obsnum=&j;
            set work.&src_tbl. point=obsnum;
            %do i=1 %to &maxkey.;
                call symput("key&i.", left(&&var&i));
            %end;
            stop;
        run;

        proc sql;
            delete from &libnm..&tgt_tbl. tt
            where
            %do i=1 %to &maxkey;
                %if &i > 1 %then
                %do;
                    and
                %end;
                tt.&&var&i. = &&key&i.
            %end;
            ;

        %if &SQLRC. > 4 %then
        %do;
            %let SYSCC=100000;
            %let MAError =885;
            %let MAMsg =Macro[&sysmacroname.]. Error while attempting to delete records . Check the log for details;
            %return;
        %end;

    %end;

    quit;
%end;

/* Check for any errors */
%if &SYSERR. > 4 %then
%do;
    %let SYSCC=100000;
    %let MAError =885;
    %let MAMsg =Macro[&sysmacroname.]. Error while attempting to delete records . Check the log for details;
%end;

%mend;
