/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/************************************************************************************/
/* NAME: mapubutl                                                                   */
/* VERSION: 5.4.4                                                                   */
/*                                                                                  */
/* DESCRIPTION: This Util macro is used to create any subset data by joing multiple */
/*              tables.  Currently it is used to crete the input table for          */
/*              CI_Cell_Package_Treatment_Cnt table.                                */
/* Parameters:                                                                      */
/*        ccp - Physical table name for CI_Cell_Package                             */
/*        ccpt - Physical table name for CI_Cell_Package_Treatment_Count            */
/*        cp   - Physical table name for Package                                    */
/*        cpxt - Physical table name for Package_X_Treatmen                         */
/*        ct   - Physical table name for Treatment or the                           */
/*               table contains the newly inserted Treatment_Sk                     */
/*        resulttbl - Result temp table name to store the resulted records          */
/*        utlflg - Flag to indicate the specific operation                          */
/*                 C - to create result table with treatment_sk and treatment_cd    */
/*                     for the matching package_cd from cell_package table          */
/*                     i.e, for newly inserted cell_package_sk find the associated  */
/*                     treatment variables                                          */
/*                 T  - to create result table with all cell_package variables for  */
/*                      newly inserted treatment for the existing package_cd        */
/*                                                                                  */
/* PRODUCT: MA                                                                      */
/* USAGE: %mapubutl                                                                 */
/*                                                                                  */
/************************************************************************************/
/************************************************************************************/
/*                                                                                  */
/* History:                                                                         */
/*   27APR2007 - First version of the macro.                                        */
/*                                                                                  */
/* DATE       Defect No  Comments                                                   */
/* 14Oct2008  S0504678   Added PACKAGE_SK and CONTROL_GROUP_TYPE_CD to treament_cnt */
/*                       table part of the 5.3 data model changes.                  */
/*                                                                                  */
/* 13Jul2009  S0583163   Reset the SYSCC value to 100000 in case of error           */
/*                                                                                  */
/* 26Mar2010  a2ma54     New column added for Offer arbitration Package_hash_val    */
/*                       The UTLFLG parameter is removed from the signature         */
/*                                                                                  */
/* 09Apr2010  S0658600   Use the new cell_package input table for preparing         */
/*                       treatment_cnt input set                                    */
/*                                                                                  */
/* 18May2010  S0670074   Remove Package_Hash_Val from cell_package table            */
/*                                                                                  */
/* 30Jun2010  S0677342   Remove duplicate records after the final merge             */
/*                                                                                  */
/* 31Jan2012  S0827991   Fix the commuication occurrence and treatment association  */
/************************************************************************************/

%macro mapubutl(ccp=, cp=, cpxt=, ct=, resulttbl=);

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put  Current Version of the code: MA5.41 Hotfix#2;
    %put  Last Modified on 31Jan2012;
    %put  MAPUBUTL macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

%put Start Time - %sysfunc(datetime(), datetime24.2);

/* Note: The utlflg parameter is not used in the revised code */

/* S0827991: The PREEXEC publish XML will have all the communication data in it even when a single
   communication is selected for execution. To have the correct treatment record populated in ccpt
   table use the ExecuteCommSK param to retain only records relevant to this communication and ignore
   the rest */
%if %upcase(&publishtype.) eq PREEXEC and &executecommsk. > 0 %then
%do;
    data work.&ccp._base;
        set work.&ccp._base;
        where communication_sk=&executecommsk.;
        by cell_package_sk package_sk;
    run;
%end;

%generateInClause(PACKAGE_SK, work.&ccp._base, _whTable, 100, fileRef=inClause);
proc sql noprint;
    select whClause into :whClause1 - :whClause9999 from _whTable;
quit;
%let numWhObs = &SQLOBS;

/* Step 1: Create treatment_cnt input table: When a campaign associated with the treatment.

   Due to performance reasons MA CORE passes only the change data in treatment table (i.e., only when
   there is a new treatment or change to the existing treatment).  So the input cell_package table
   is joined with the target CI_PACKAGE_X_TREATMENT and target CI_TREATMENT table to find out the
   Treatment_SK rather than working with the input package_x_treatment table.
*/

    proc sql;
        create table work.ccpt_new_ccc_step1 as
            select cpxt.PACKAGE_SK, cpxt.CONTRIBUTING_CELL_PACKAGE_SK, ctt.TREATMENT_SK, ctt.TREATMENT_CD
            from &libnm..&cpxt. (&dsreadopts.) cpxt
            inner join &libnm..&ct. (&dsreadopts.) ctt on (cpxt.TREATMENT_SK = CTT.TREATMENT_SK)
            where
            %if &numWhObs le 0 %then 0=1;
            %else %do i = 1 %to &numWhObs;
            %if &i gt 1 %then OR;
            cpxt.PACKAGE_SK IN (&&whClause&i)
            %end;
        ;
    quit;

    proc sql;
        create table work.ccpt_new_ccc as
            select
                    CCP.CELL_PACKAGE_SK,
                    CT.TREATMENT_SK,
                    CCP.PACKAGE_SK,
                    CCP.PACKAGE_CD,
                    CT.TREATMENT_CD,
                    CCP.CAMPAIGN_SK,
                    CCP.CAMPAIGN_CD,
                    CCP.CAMPAIGN_NM,
                    CCP.CHANNEL_CD,
                    CCP.COMMUNICATION_SK,
                    CCP.COMMUNICATION_CD,
                    CCP.COMMUNICATION_NM,
                    CCP.COMMUNICATION_OCCURRENCE_NO,
                    CCP.MARKETING_CELL_SK,
                    CCP.MARKETING_CELL_CD,
                    CCP.MARKETING_CELL_NM,
                    CCP.MARKETING_CELL_DESC,
                    CCP.SUBJECT_TYPE_NM,
                    CCP.CONTROL_GROUP_TYPE_CD,
                    CCP.TRACK_PRESENTED_TREATMENTS_FLG
              from work.&ccp._base ccp
                   inner join work.ccpt_new_ccc_step1 ct
                   on ( ccp.CELL_PACKAGE_SK=ct.CONTRIBUTING_CELL_PACKAGE_SK and
                        ccp.PACKAGE_SK=ct.PACKAGE_SK )
        ;
    quit;

/* This Step2 looks to be not correct with respect to CDM for 54 as we have the contributing_cell_package_sk which
   gives the correct association of occurrence and treatment.  So removing this step2 completely */

/* Step 2: create treatment_cnt input table: When a new treatment added to the existing package.
   Find out all cell_package_sk associated with this package (to which a new treatment is added)
   from data model CI_CELL_PACKAGE table and insert those records in to treatment_cnt table
   with new treatment_sk */

/* S0504678 Added PACKAGE_SK and CONTROL_GROUP_TYPE_CD part of 5.3 data model changes */
/*
    proc sql;
        create table work.ccpt_new_ccc_step2 as
        select
            CCP.CELL_PACKAGE_SK,
            CCP.PACKAGE_SK,
            CCP.PACKAGE_CD,
            CCP.CAMPAIGN_SK,
            CCP.CAMPAIGN_CD,
            CCP.CAMPAIGN_NM,
            CCP.CHANNEL_CD,
            CCP.COMMUNICATION_SK,
            CCP.COMMUNICATION_CD,
            CCP.COMMUNICATION_NM,
            CCP.MARKETING_CELL_SK,
            CCP.MARKETING_CELL_CD,
            CCP.MARKETING_CELL_NM,
            CCP.MARKETING_CELL_DESC,
            CCP.SUBJECT_TYPE_NM,
            CCP.CONTROL_GROUP_TYPE_CD,
            CCP.TRACK_PRESENTED_TREATMENTS_FLG
        from &libnm..&ccp. (&dsreadopts.) ccp
        where
            %if &numWhObs le 0 %then 0=1;
            %else %do i = 1 %to &numWhObs;
                %put whClause&i = &&whClause&i;
                %if &i gt 1 %then OR;
                ccp.PACKAGE_SK IN (&&whClause&i)
            %end;

        ;

    proc sql;
        create table work.ccpt_existing_pkcd as
            select
            CCP.CELL_PACKAGE_SK,
            CT.TREATMENT_SK,
            CCP.PACKAGE_SK,
            CCP.PACKAGE_CD,
            CT.TREATMENT_CD,
            CCP.CAMPAIGN_SK,
            CCP.CAMPAIGN_CD,
            CCP.CAMPAIGN_NM,
            CCP.CHANNEL_CD,
            CCP.COMMUNICATION_SK,
            CCP.COMMUNICATION_CD,
            CCP.COMMUNICATION_NM,
            CCP.MARKETING_CELL_SK,
            CCP.MARKETING_CELL_CD,
            CCP.MARKETING_CELL_NM,
            CCP.MARKETING_CELL_DESC,
            CCP.SUBJECT_TYPE_NM,
            CCP.CONTROL_GROUP_TYPE_CD,
            CCP.TRACK_PRESENTED_TREATMENTS_FLG
            from work.ccpt_new_ccc_step2 ccp inner join
                 work.&cpxt. cpxt on (ccp.package_sk=cpxt.package_sk) inner join
                 work.&ct. ct on (CPXT.TREATMENT_SK = CT.TREATMENT_SK)
            ;

    quit;

*/
    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError = 861;
            %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
        %end;
    %end;
    %else %if &SQLRC. > 4  %then
    %do;
        %let SYSCC=100000;
        %let MAError =861;
        %let MAMsg = Macro[&sysmacroname.]. Proc SQL error while attempting create result table for CI_Cell_Package_Treatment_Count SQLRC[&sqlrc]. Check the log for details;
        %return;
    %end;


/* Now we have three input temp tables
    i) work.ccp - CI_Cell_Package input table from xml stream.
    ii) work.ccpt_new_ccc - generated input table for new campaign/campaign_cell/communication
    iii) work.ccpt_existing_pkcd - generated input table for existing package */

    proc sort data=work.&ccp._base;
        by CELL_PACKAGE_SK;
    run;

    proc sort data=work.ccpt_new_ccc;
        by CELL_PACKAGE_SK;
    run;

/*    proc sort data=work.ccpt_existing_pkcd;*/
/*        by CELL_PACKAGE_SK;*/
/*    run;*/

    /* Check this merge step to make sure the records correct */
    data work.&resulttbl.;
        merge work.&ccp._base (in=a)
              work.ccpt_new_ccc (in=b);
        by CELL_PACKAGE_SK;
        if a and b then
            output;
    run;

/*    data work.&resulttbl.;*/
/*        merge work.ccpt_existing_pkcd*/
/*              work.&resulttbl.;*/
/*        by CELL_PACKAGE_SK;*/
/*    run;*/

    /* S0677342: Selecting the same treatment with same UDF values multiple times in a communicaiton node will
       result in duplicate records in the final input set of cell_package_treatment_cnt tables, which needs to
       removed */
    proc sort data=work.&resulttbl. nodupkey;
        by cell_package_sk treatment_sk;
    run;

    %if &SYSERR. > 4  %then
    %do;
        %let MAError =862;
        %let MAMsg = Macro[&sysmacroname.]. Data Step error while attempting create result table for CI_Cell_Package_Treatment_Count SYSERR[&SYSERR]. Check the log for details;
        %return;
    %end;


%mend;


%macro generateInClause( column, inTable, outTable, limitPerIn, fileRef=fTempRef,format=%str(F10.0));
proc sql noprint;
   create table work._INCLAUSE_1(replace=yes) as
   select distinct(&column) as &column format=&format from &inTable;
quit;

filename &fileRef temp lrecl=32767;

%let nRows = &SQLOBS;
%if &nRows le 0 %then %do;
   data _null_;
      file &fileRef;
      put "stop;";
   run;
%end; %else %do;
%let nLoops = 1;
%if &nRows gt &limitPerIn %then %do;
   data _null_;
      nLoops=int(&nRows/&limitPerIn);
      mVal = mod(&nRows,&limitPerIn);
      if ( mVal > 0 ) then nLoops + 1;
      call symput("nLoops",trim(left(nLoops)));
   run;
%end;

%do i = 1 %to &nLoops;
   %let var=;

   data _INCLAUSE_V / view=_INCLAUSE_V;
      set work._INCLAUSE_1(firstobs=%eval(((&i-1)*&limitPerIn) + 1));
      if ( _n_ > &limitPerIn ) then stop;
   run;
   proc sql noprint;
      select &column into :var SEPARATED BY ',' from _INCLAUSE_V;
   quit;
   %put var=&var;
   data _null_;
      file &fileRef mod;
      put "whClause='&var';";
      put "output;";
   run;

%end;
%end;

data &outTable;
   length whClause $ 32767;
   %include &fileRef;
run;
filename &fileRef clear;
%mend;
