/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/************************************************************************************/
/* NAME: mapubwp.sas                                                                */
/* VERSION: 5.4                                                                     */
/*                                                                                  */
/*                                                                                  */
/* DESCRIPTION: Wrapper macro for PUBLISH stored process to initialize all the      */
/*              table names for the CI Common Model and convert XML input file      */
/*              to the respective SAS tables.  This also calls the relevant macros  */
/*              in the order of preferance (Parent table inserted first before      */
/*              any child table is processed ).                                     */
/* Parameters:                                                                      */
/*            reportlib - libname statement for publish tables                      */
/*                                                                                  */
/* PRODUCT: MA                                                                      */
/* USAGE: %mapubwp                                                                  */
/*                                                                                  */
/************************************************************************************/
/*****************************************************************************************/
/*                                                                                       */
/* History:                                                                              */
/*   27APR2007 - First version of the macro.                                             */
/*                                                                                       */
/* S0482197 - added %unquote function when passing libstmt to macrtlib macro             */
/*                                                                                       */
/* S0482199 - Changed the dbengine condition to check != SASSPDS and blank so that       */
/* this can take care of any new databases getting added in the future. E.g. Netezza     */
/*                                                                                       */
/* S0504678 - Changes for Treatment_type_UDF & Treatment_Ext tables                      */
/* matrnudf.sas is changed to a generic macro so that the Campaign/Communication         */
/* and Treatment extension tables updated using this                                     */
/*                                                                                       */
/* DATE       Defect No  Comments                                                        */
/* 01Sep2008  S0506508   Changes for Campaign_Group publish                              */
/*                                                                                       */
/* 14Oct2008  S0504678   Added Treatment Delete_Flg code block                           */
/*                                                                                       */
/* 14Oct2008  S0506508   Added CAMP_GRP_CHECKLIST_ITEM / UDF / EXTN table publish        */
/*                                                                                       */
/* 20Oct2008  S0504678  a) Purpose parameter check included for all possible values      */
/*                         CAMPAIGN/CAMPAIGN_DELETION/CAMPAIGN_GROUP/COMMUNICATION_EXPORT*/
/*                         TREATMENT_DELETION                                            */
/*                      b) Added CAMP_CHECKLIST_ITEM/ UDF / EXTN tables                  */
/*                                                                                       */
/* 5Feb2009   S0551066  Added new CAMPAIGN_GROUP UDF and Extn tables                     */
/*                                                                                       */
/* 31Mar2009  S0569710  a) Added MO_OPTIMIZATION_SCENARIO/CI_CAMP_GRP_X_MO_OPT_SCENARIO  */
/*                      b) Campaign_Group_Deletion handling                              */
/*                                                                                       */
/* 15May2009  S0578891  Prefix CI_ to the MO_OPTIMIZATION_SCENARIO table name            */
/*                                                                                       */
/* 09Jun2009  S0492899  Dynamic Treatment table names added for delete processing and    */
/*                      new set of macro to refresh the data model tables based on       */
/*                      cleanpublish parameter                                           */
/*                                                                                       */
/* 10Jun2009  S0571190  Dynamic Treatment records excluded from populating Treatment_Ext */
/*                                                                                       */
/* 13Jul2009  S0583163  Reset the SYSCC value to 100000 in case of error                 */
/*                                                                                       */
/* 16Jul2009  S0554379  Delete flag applied to all the versions of the campaign          */
/*                                                                                       */
/* 11Sep2009  S0605941  Communication_Status_Cd required to be updated when PURPOSE flag */
/*                      set to COMMUNICATION_EXPORT                                      */
/*                                                                                       */
/* 12Jan2010  S0492899  Refresh Common Data model tables                                 */
/*                                                                                       */
/* 19Jan2010  S0637486  Campaign_Group publish should not invoke publish delete          */
/*                                                                                       */
/* 30Mar2010  5.4       Added CI_TREATMENT_CAMPAIGN_SET processing.                      */
/*                      New Purpose values are: TREATMENT_CAMPAIGN_SET &                 */
/*                      TREATMENT_CAMPAIGN_SET_DELETION                                  */
/*                                                                                       */
/* 12May2010  5.4Iteration5 Publish DELETED_FLG to CI_COMMUNICATION table                */
/*                                                                                       */
/* 30Jun2010  S0681555  CI_CHANNEL table population                                      */
/*                                                                                       */
/* 08Dec2010  S0722605  Campaign Delete_Flg update based on NM/Cd/Folder_Txt/BC Name     */
/*                                                                                       */
/* 06Jan2012  S0782404  Add new flags to avoid repeat processing of UDF tables           */
/* 26Feb2013  S0937584   541MA07: Publish error on DTTM fields for CI_CAMPAIGN...        */
/*****************************************************************************************/

%macro mapubwp(reportlib=);

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put MAPUBWP Current Version of the code: 541MA07;
    %put Last Modified on 26Feb2013;
    %put MAPUBWP macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

/* Step 1: Variable Definitions */

%global libnm pubsrc pdttm lib_nm dbengine;

/* S0782404 */
%global camp_cudf_cnt camp_nudf_cnt camp_dudf_cnt comm_cudf_cnt comm_nudf_cnt comm_dudf_cnt;

%let dbengine = %upcase(%qscan(&reportlib.,3,%str( ) %str(%")));
%madsopts(%str(&reportlib), reportlib, operation=P);

/* Input XML Stream */
%let InStream=Pubmeta;

/* XML Map file */
%let InStreamMap=Publish;

/* Dynamic source file name */
%let campsrc=campsrc;
%let commsrc=commsrc;

/* Define filename reference for the Campaign and Commuication Dynamic source code */
filename &campsrc. catalog "work.CIpublish.camp_publish.Source";
filename &commsrc. catalog "work.CIpublish.comm_publish.Source";

/* Static Map file name */
%let pubmap=PubMap;

data _null_;
    call symput('Mappath',pathname("MAMISC"));
run;

filename &pubmap. "&mappath./mapublish.map";

/* S0782404 Initial Publish indicator flag */
%let initial_pub=N;

/* CI Common Model Table Names */

/* Campaign */
%let camp=CI_CAMPAIGN;
%let camp_char_udf= CI_CAMPAIGN_CHAR_UDF;
%let camp_num_udf= CI_CAMPAIGN_NUM_UDF;
%let camp_dt_udf= CI_CAMPAIGN_DATE_UDF;
%let camp_ext= CI_CAMPAIGN_EXT;
%let camp_st= CI_CAMPAIGN_STATUS;

%let c_clst_item=CI_CAMP_CHECKLIST_ITEM;
%let c_clst_char_udf=CI_CAMP_CHECKLIST_CHAR_UDF;
%let c_clst_num_udf=CI_CAMP_CHECKLIST_NUM_UDF;
%let c_clst_dt_udf=CI_CAMP_CHECKLIST_DATE_UDF;
%let c_clst_ext=CI_CAMP_CHECKLIST_ITEM_EXT;

/* Communication */
%let comm=CI_COMMUNICATION;
%let comm_char_udf= CI_COMMUNICATION_CHAR_UDF;
%let comm_num_udf= CI_COMMUNICATION_NUM_UDF;
%let comm_dt_udf= CI_COMMUNICATION_DATE_UDF;
%let comm_ext= CI_COMMUNICATION_EXT;
%let comm_st= CI_COMMUNICATION_STATUS;
/* 5.4Iteration5 */
%let comm_deleted=DELETED_COMMUNICATION;

/* Campaign Cell */
%let ccell=CI_MARKETING_CELL;

/* Cell_Package  */
%let ccp=CI_CELL_PACKAGE;
%let ccpt=CI_CELL_PACKAGE_TREATMENT_CNT;

/* Package */
%let cp=CI_PACKAGE;
%let cpxt=CI_PACKAGE_X_TREATMENT;
%let ccgt=CI_CONTROL_GROUP_TYPE;

/* Treatment */
%let ct=CI_TREATMENT;
%let ct_char_udf=CI_TREATMENT_CHAR_UDF;
%let ct_num_udf=CI_TREATMENT_NUM_UDF;
%let ct_dt_udf=CI_TREATMENT_DATE_UDF;
%let ct_ext=CI_TREATMENT_EXT;

/* Response */
%let cr=CI_RESPONSE;
%let crt=CI_RESPONSE_TYPE;
%let crxcp=CI_RESPONSE_X_CELL_PACKAGE;
%let crcr=CI_RESPONSE_CHANNEL_RESPONSE;

/* Change Log */
%let ccl=CI_CHANGE_LOG;
%let cct=CI_CHANGE_TYPE;

/* Campaign_Group */
%let ccg=CI_CAMPAIGN_GROUP;
/*%let ccxcg=CI_CAMPAIGN_X_CAMPAIGN_GROUP;*/
%let cg_clst_item=CI_CAMP_GRP_CHECKLIST_ITEM;
%let cg_clst_char_udf=CI_CAMP_GRP_CLIST_CHAR_UDF;
%let cg_clst_num_udf=CI_CAMP_GRP_CLIST_NUM_UDF;
%let cg_clst_dt_udf=CI_CAMP_GRP_CLIST_DATE_UDF;
%let cg_clst_ext=CI_CAMP_GRP_CLIST_ITEM_EXT;
%let cg_char_udf=CI_CAMPAIGN_GROUP_CHAR_UDF;
%let cg_num_udf=CI_CAMPAIGN_GROUP_NUM_UDF;
%let cg_dt_udf=CI_CAMPAIGN_GROUP_DATE_UDF;
%let cg_ext=CI_CAMPAIGN_GROUP_EXT;
%let opt_sc=CI_MO_OPTIMIZATION_SCENARIO;
%let cgxoptsc=CI_CAMP_GRP_X_MO_OPT_SCENARIO;

/* S0492899 Dynamic Treatment */
%let dt_attr=CI_DYNAMIC_TREATMENT_ATTRIBUTE;
%let dt_ext=CI_DYNAMIC_TREATMENT_ATTR_EXT;

/* Treatment_Campaign_Set  */
%let ct_camp_set=CI_TREATMENT_CAMPAIGN_SET;
%let ctcsxc=CI_TREAT_CAMP_SET_X_CAMPAIGN;

/* CI_Channel */
%let cch=CI_CHANNEL;

/* Process DateTime */
%let pdttm=%sysfunc(datetime(),datetime20);

%if &SYSERR. ne 0 %then
%do;
    %let MAError=720;
    %let MAMsg=Macro[&sysmacroname.]. Error while attempting to define variables. Check the log for details.;
    %return;
%end;


/* Execute and validate the publish table library */

%if (&dbengine. ne %str()) and (&dbengine. ne SASSPDS) %then
%do;

    %let reportlib= %sysfunc(strip(&reportlib.)); /* Remove spaces after semicolom */

    %let semicolpos= %index(&reportlib.,%str(;));
    %let liblength= %length(&reportlib.);

    %if &semicolpos. > 0 %then
    %do;
        /* Remove the semicolon if it part of the libname before concatenate reread..*/
        %if %substr(%superq(reportlib),&liblength.) eq %str(;) %then
            %let reportlib=%substr(%superq(reportlib),1,%eval(&liblength.-1));
    %end;

    %let reportlib= &reportlib %str(reread_exposure=YES);
%end;

%macrtlib(%unquote(&reportlib.), MAPUBWP);

%if &MAError. ne %str() %then
%do;
    %return;
%end;

%let libnm=%upcase(%qscan(&reportlib.,2));

/* This new variable lib_nm added to print the correct libname from macrtlib macro incase of errors */
%let lib_nm=&libnm.;

/* Execute and validate the Input XML library */
%let xml_lib = libname &instream xml xmlmap=&pubmap;

%macrtlib(&xml_lib., MAPUBWP);

/*libname &instream. xml xmlmap=&pubmap.;*/

%if &MAError. ne %str() %then
%do;
    %return;
%end;

/* Create SAS data set out of the XML input streatm */
proc copy in=&instream out=WORK;
    %if %upcase(&purpose.) eq CAMPAIGN_DELETION %then
    %do;
        select &camp.;
    %end;
    %else %if %upcase(&purpose.) eq COMMUNICATION_EXPORT %then
    %do;
        select &camp. &comm.;
    %end;
    /* The check campaignid eq CAMPAIGNGROUP can be removed once the purpose parameter is fully implemented */
    %else %if %upcase(&purpose.) eq CAMPAIGN_GROUP %then
    %do;
    /* S0551066: Added new campaign_group udf tables */
    /* S0569710: Added new MO_OPTIMIZATION_SCENARIO and CI_CAMP_GRP_X_MO_OPT_SCENARIO tables*/
        select &ccg. &cg_clst_item.
               &cg_clst_char_udf. &cg_clst_num_udf.  &cg_clst_dt_udf.
               &cg_char_udf. &cg_num_udf. &cg_dt_udf.
               &opt_sc. &cgxoptsc.;
    %end;
    %else %if %upcase(&purpose.) eq TREATMENT_DELETION %then
    %do;
        select &ct.;
    %end;
    %else %if %upcase(&purpose.) eq CAMPAIGN_GROUP_DELETION %then
    %do;
        select &ccg.;
    %end;
    /* Treatment_Campaign_Set added part of 5.4 Offer arbitration project */
    %else %if %upcase(&purpose.) eq TREATMENT_CAMPAIGN_SET %then
    %do;
        select &ct_camp_set. &ctcsxc.;
    %end;
    %else %if %upcase(&purpose.) eq TREATMENT_CAMPAIGN_SET_DELETION %then
    %do;
        select &ct_camp_set.;
    %end;
    %else %if %upcase(&purpose.) eq CHANNEL %then
    %do;
        select &cch.; /* S0681555 */
    %end;
    /* If purpose=CAMPAIGN then read all the tables from the inputXML */
run;

%if &SYSERR > 4 %then
%do;
    %let MAError=722;
    %let MAMsg=Macro[&sysmacroname.]. PROC Copy failed - while attempting to create dataset out of XML stream. Check the log for details.;
    %return;
%end;

libname &instream; /* Clear the PUBMETA library referrence */

/* Check to see whether this publish call is to update the deleted_flg */
/* S0554379 Deleted Flag applied to all the previous versions of the campaign */
%if %upcase(&purpose.) eq CAMPAIGN_DELETION %then
%do;
    /* S0722605 Added additional variable check on the where clause to update only the intended campaign deleted flag */
    data _null_;
    set work.&camp.;
        call symput('_ln_camp_nm',put(sum(vlength(campaign_nm),countc(campaign_nm,'"')),Best.));
        call symput('_ln_modify_by',put(sum(vlength(last_modified_by_user_nm),countc(last_modified_by_user_nm,'"')),Best.));
        call symput('_ln_bc_nm',put(sum(vlength(business_context_nm),countc(business_context_nm,'"')),Best.));
    run;

    data _null_;
       file &campsrc. lrecl=32767;
       set work.&camp.;

       length _camp_nm $ &_ln_camp_nm.;
       length _modify_by $ &_ln_modify_by.;
       length _bc_nm $ &_ln_bc_nm.;


       _camp_nm=tranwrd(campaign_nm,'"','""');
       _modify_by=tranwrd(last_modified_by_user_nm,'"','""');
       _bc_nm=tranwrd(business_context_nm,'"','""');

       put ' proc sql; ';
       put '     update &libnm..&camp. (&dswriteopts.)';
       put '         SET DELETED_FLG="Y", ';
       put '             LAST_MODIFIED_DTTM="' last_modified_dttm '"dt, ';
       put '             LAST_MODIFIED_BY_USER_NM="' _modify_by +(-1) '" ';
       put '            Where UPCASE(CAMPAIGN_CD) =UPCASE(TRIM(LEFT("' CAMPAIGN_CD +(-1) '"))) and ';
       put '                  UPCASE(CAMPAIGN_NM) =UPCASE(TRIM(LEFT("' _camp_nm +(-1) '"))) and ';
       put '                  UPCASE(CAMPAIGN_FOLDER_TXT) =UPCASE(TRIM(LEFT("' CAMPAIGN_FOLDER_TXT +(-1) '"))) and ';
       put '                  UPCASE(BUSINESS_CONTEXT_NM) =UPCASE(TRIM(LEFT("' _bc_nm +(-1) '")));';
       put ' quit;';
    run;

    %include &campsrc.;

    %if &SQLRC > 4 %then
    %do;
        %let SYSCC=100000;
        %let MAError=800;
        %let MAMsg=Macro[&sysmacroname.]. Error while attempting to Update CAMPAIGN records SQLRC[&sqlrc]. Check the log for details;
    %end;

    %return; /* This return statement is to terminate the publish process after updating the deleted_flg */

%end; /* End of updateing Campaign DELETED_FLG */

/* Communication_Export process.  Update only the CI_COMMUNICATION.Export_DTTM, CI_CAMPAIGN.RUN_DTTM & LAST_MODIFIED_DTTM*/
%if %upcase(&purpose.) eq COMMUNICATION_EXPORT %then
%do;

    /* Update RUN_DTTM and LAST_MODIFIED_DTTM in CI_CAMPAIGN table after Communication Export */
    data &libnm..&camp.;
        modify &libnm..&camp. (&dswriteopts.) work.&camp.(keep=campaign_sk run_dttm last_modified_dttm);
        by CAMPAIGN_SK;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                processed_dttm="&pdttm"dt;
                replace;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;


    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError=800;
            %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update CAMPAIGN records SYSDBRC[&SYSDBRC]. Check the log for details;
       %end;
    %end;
    %else %if &SQLRC > 4 %then
    %do;
        %let SYSCC=100000;
        %let MAError=800;
        %let MAMsg=Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update CAMPAIGN records SQLRC[&sqlrc]. Check the log for details;
    %end;

    /* Update EXPORT_DTTM and COMMUNICATION_STATUS_CD in CI_COMMUNICATION table */
    data &libnm..&comm.;
        modify &libnm..&comm. (&dswriteopts.) work.&comm.(keep=communication_sk export_dttm communication_status_cd); /* S0605941: Communication_Status_Cd is added */
        by COMMUNICATION_SK;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                processed_dttm="&pdttm"dt;
                replace;
            end;
            otherwise;
        end;
    run;

    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError=812;
            %let MAMsg = Macro[&sysmacroname.]. Publish failed for Communication with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
       %end;
    %end;
    %else %if &SQLRC > 4 %then
    %do;
        %let SYSCC=100000;
        %let MAError=812;
        %let MAMsg=Macro[&sysmacroname.]. Publish failed for Communication with reason code SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;

    %return; /* Exit the process after updating Run/Export DTTM */
%end; /* End of updating Communication Export_DTTM */

/* S0569710: Campaign_Group_Deletion */
%if %upcase(&purpose.) eq CAMPAIGN_GROUP_DELETION %then
%do;
     data &libnm..&ccg.;
        modify &libnm..&ccg. (&dswriteopts.)
               work.&ccg.(keep=campaign_group_sk deleted_flg last_modified_dttm last_modified_by_user_nm);
        by CAMPAIGN_GROUP_SK;
        select (_IORC_);
            when(%SYSRC(_SOK))
            do;
                processed_dttm="&pdttm"dt;
                replace;
            end;
            otherwise
            do;
                stop;
            end;
        end;
    run;

    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError=884;
            %let MAMsg = Macro[&sysmacroname.]. Data Modify stmt error while attempting to Update CAMPAIGN_GROUP records SYSDBRC[&SYSDBRC]. Check the log for details;
       %end;
    %end;
    %else %if &SQLRC > 4 %then
    %do;
        %let SYSCC=100000;
        %let MAError=884;
        %let MAMsg=Macro[&sysmacroname.]. PROC SQL error while attempting to update CAMPAIGN_GROUP records SQLRC[&sqlrc]. Check the log for details;
    %end;

    %return; /* This return statement is to terminate the publish process after updating the deleted_flg */
%end;

/* S0504678 - Treatment Delete publish changes */
%if %upcase(&purpose.) eq TREATMENT_DELETION %then
%do;
    proc sql;
        update &libnm..&ct. _tt
            set DELETED_FLG='Y',
                PROCESSED_DTTM="&pdttm."dt
            where exists
                ( select treatment_sk from &ct. _st
                    where _st.treatment_sk = _tt.treatment_sk);
    quit;


    %if %symexist(SYSDBRC) %then
    %do;
        %if &SYSDBRC ^= 0 %then
        %do;
            %let SYSCC=100000;
            %let MAError=800;
            %let MAMsg = Macro[&sysmacroname.]. PROC SQL error while attempting to update CAMPAIGN records SYSDBRC[&SYSDBRC]. Check the log for details;
       %end;
    %end;
    %else %if &SQLRC > 4 %then
    %do;
        %let SYSCC=100000;
        %let MAError=800;
        %let MAMsg=Macro[&sysmacroname.]. PROC SQL error while attempting to update CAMPAIGN records SQLRC[&sqlrc]. Check the log for details;
    %end;

    %return; /* This return statement is to terminate the publish process after updating the deleted_flg */
%end; /* End of updateing Treatment DELETE_FLG */

/* 5.4 Offer Arbitration changes */
%if %upcase(&purpose.) eq TREATMENT_CAMPAIGN_SET or %upcase(&purpose.) eq TREATMENT_CAMPAIGN_SET_DELETION %then
%do;
    %mactcs(ct_camp_set=&ct_camp_set., ctcsxc=&ctcsxc., purpose=&purpose.);

    /* Exit from the publish process */
    %return;

%end;

/* S0492899 - Delete wrapper script which invokes table delete in a sequence so that it doesn't cause any constraint error */
/* S0637486 - Campaign_Group publish should not invoke publish delete. In other way only the Campaign publish
              should go through the Publish delete path */
%if %upcase(&cleanpublish.) eq Y and %upcase(&purpose.) eq CAMPAIGN %then
%do;
    %madelwp(purpose=&purpose.);

    /* Exit from the publish process. If the clean publish is called and ther are errors  */
    %if &MAError. ne %str() %then
    %do;
        %return;
    %end;
%end;

/* S0506508 - Campaign_Group publish changes */
/* S0782404 - Added _Cnt check */
%if %upcase(&purpose.) eq CAMPAIGN_GROUP %then
%do;
    /* a)Publish CI_Campaign_Group and UDF tables */
    /* S0569710: Added new MO_Optimization_Scenario and CI_Camp_Grp_x_mo_opt_Scenario table */
    %macamgp(ccg=&ccg., cg_char_udf=&cg_char_udf., cg_num_udf=&cg_num_udf., cg_dt_udf=&cg_dt_udf.,
             opt_sc= &opt_sc., cgxoptsc= &cgxoptsc. );

    /* b) S0551066 Populate the CI_CAMPAIGN_GROUP_EXT tabl */
    %if &cg_cudf_cnt. > 0 or &cg_nudf_cnt. > 0 or  &cg_dudf_cnt. > 0%then
    %do;
        %matrnudf(char_intbl=&cg_char_udf., num_intbl=&cg_num_udf., dt_intbl=&cg_dt_udf.,
                  byvar=%str(CAMPAIGN_GROUP_SK), ext_tbl=&cg_ext., parent_tbl=CAMP_GRP,
                  nullcheck=%str(CAMPAIGN_GROUP_SK ne .) );
    %end;

    /* c) S0506508 Publish CI_CAMP_GRP_CHECKLIST_ITEM and associated UDF tables */
    %macgclst(cg_clst_item=&cg_clst_item., cg_clst_char_udf=&cg_clst_char_udf., cg_clst_num_udf= &cg_clst_num_udf.,
              cg_clst_dt_udf=&cg_clst_dt_udf.);

    /* d) S0506508 Populate the CI_CAMP_GRP_CHECKLIST_EXT tables */
    %if &cgclst_cudf_cnt. > 0 or &cgclst_nudf_cnt. > 0 or  &cgclst_dudf_cnt. > 0%then
    %do;
        %matrnudf(char_intbl=&cg_clst_char_udf., num_intbl=&cg_clst_num_udf., dt_intbl=&cg_clst_dt_udf.,
                  byvar=%str(CAMPAIGN_GROUP_SK UDF_SEQ_NO), ext_tbl=&cg_clst_ext., parent_tbl=CAMP_GRP_CLIST,
                  nullcheck=%str(CAMPAIGN_GROUP_SK ne . and UDF_SEQ_NO ne .) );
    %end;

    %return;
%end; /* End of updating CAMPAIGN_GROUP tables */


/* New Feature: Populate CI_Channel table */
%if %upcase(&purpose.) eq CHANNEL %then
%do;
    %mapubch(cch=&cch.);

    %return;
%end;

/* Insert sourrending tables like status, response, package tables etc.
   before touching the parent tables to avoid referential constriant issue */

/* Macro to update Campaign/Communication Status tables */
%mastspub(camp_st=&camp_st., comm_st=&comm_st., cct=&cct., ccgt=&ccgt.);

/* Macro to update the Treatment and Treatment_UDF tables */
%matmtudf(ct=&ct., ct_char_udf=&ct_char_udf., ct_num_udf=&ct_num_udf., ct_dt_udf=&ct_dt_udf.);

/* Macro to update the Treatment Extention table */
/* S0571190 - Exclude the Dynamic Treatment records (Treatment_Hash_val = X) getting populated to Treatment_Ext table */
/* S0782404 - Added _Cnt check */
%if &tmt_cudf_cnt. > 0 or &tmt_nudf_cnt. > 0 or &tmt_dudf_cnt. > 0 %then
%do;
    %matrnudf(char_intbl=&ct_char_udf., num_intbl=&ct_num_udf., dt_intbl=&ct_dt_udf., byvar=TREATMENT_SK, ext_tbl=&ct_ext., parent_tbl=TREATMENT,
              nullcheck=%str(TREATMENT_SK ne . ), exclude_dytmt=Y );
%end;

/* Macro to update Package table  */
%maptpub(cp=&cp.);

/* Publish parent macro call to first update campaign records one by one and if no match found
   insert the same recrod in to Campaign table and subsequently insert records in to all relevant child tables where
   there is a new record for this new campaign */
%mapub(camp= &camp., camp_char_udf=&camp_char_udf., camp_num_udf=&camp_num_udf., camp_dt_udf=&camp_dt_udf.,
           camp_ext=&camp_ext., comm=&comm., comm_char_udf=&comm_char_udf., comm_num_udf=&comm_num_udf.,
           comm_dt_udf=&comm_dt_udf., comm_ext=&comm_ext., ccell=&ccell., ccp=&ccp., ccpt=&ccpt., ccl=&ccl.);

/* Macro to update/insert the Campaign_Cell table */
%macelpub(ccell=&ccell.);

/* Macro to update Communication related tables.  This first updates communication record one by one and
   if there is no match found then insert the single record and subsequently insert records to
   all relevant child tables where there is a matching record for the newly inserted communication record*/
%macompub(comm=&comm., comm_char_udf=&comm_char_udf., comm_num_udf=&comm_num_udf., comm_dt_udf=&comm_dt_udf.,
          comm_ext=&comm_ext., ccell=&ccell., ccp=&ccp., ccpt=&ccpt.);

/* Macro to update Campaign UDF tables.  It updates the existing records and no match found then inserts it */
%macamudf(camp_char_udf=&camp_char_udf., camp_num_udf=&camp_num_udf., camp_dt_udf=&camp_dt_udf.);

/* Macro to update Campaign Extention table */
%if &camp_cudf_cnt. > 0 or &camp_nudf_cnt. > 0 or &camp_dudf_cnt. > 0 %then
%do;
    %matrnudf(char_intbl=CampCudfBase, num_intbl=CampNudfBase, dt_intbl=CampDudfBase, byvar=CAMPAIGN_SK, ext_tbl=&camp_ext., parent_tbl=CAMPAIGN,
              nullcheck=%str(CAMPAIGN_SK ne .) );
%end;

/* Macro to update Camapign_CheckList UDF tables. */
%macclst(c_clst_item=&c_clst_item., c_clst_char_udf=&c_clst_char_udf., c_clst_num_udf=&c_clst_num_udf.,
         c_clst_dt_udf=&c_clst_dt_udf. );

/* Macro to update the Campaign Checklist Item Extension table */
%if &cclst_cudf_cnt. > 0 or &cclst_nudf_cnt. > 0 or &cclst_dudf_cnt. > 0 %then
%do;
    %matrnudf(char_intbl=&c_clst_char_udf., num_intbl=&c_clst_num_udf., dt_intbl=&c_clst_dt_udf., byvar=%str(CAMPAIGN_SK UDF_SEQ_NO),
              ext_tbl=&c_clst_ext., parent_tbl=CAMPAIGN_CHECKLIST, nullcheck=%str(CAMPAIGN_SK ne . AND UDF_SEQ_NO ne .) );
%end;

/* Macro to update Communication UDF tables.  It updates the existing records and no match found then inserts it */
%macomudf(comm_char_udf=&comm_char_udf., comm_num_udf=&comm_num_udf., comm_dt_udf=&comm_dt_udf.);

/* Macro to update the Communication Extention table */
%if &comm_cudf_cnt. > 0 or &comm_nudf_cnt. > 0 or &comm_dudf_cnt. > 0 %then
%do;
    %matrnudf(char_intbl=CommCudfBase, num_intbl=CommNudfBase, dt_intbl=CommDudfBase, byvar=COMMUNICATION_SK, ext_tbl=&comm_ext., parent_tbl=COMMUNICATION,
              nullcheck=%str(COMMUNICATION_SK ne .) );
%end;

/* Macro to update Cell_Package and Package_Treatment_Count table */
%macptpub(ccp=&ccp., ccpt=&ccpt.);

/* Macro to update Response, Response_Type, Response_X_Cell_Package and CI_Response_Channel_Response tables */
%marespub(cr=&cr., crt=&crt., crxcp=&crxcp., crcr=&crcr.);

%if &MADebug. eq Y %then
%do;
    %put MAPUBWP macro End Time - %sysfunc(datetime(), datetime24.2);
%end;

%mend;
