/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/************************************************************************************/
/* NAME: mapxt.sas                                                                */
/* VERSION: 5.4                                                                   */
/*                                                                                  */
/* DESCRIPTION: Macro to update the following data model tables                     */
/*              CI_PACKAGE_X_TREATEMENT                                             */
/*                                                                                  */
/*  Parameters:                                                                     */
/*             cpxt - Physical table name for Package_X_Treatment                   */
/*                                                                                  */
/* PRODUCT: MA                                                                      */
/* USAGE: %maptpub                                                                  */
/*                                                                                  */
/************************************************************************************/
/*****************************************************************************/
* PRISM VERSION: $Revision: 1.1.4.3.4.1.4.3 $ ;                              
* Revision Date: $Date: 2013/02/12 14:51:54 $ ;                                  
/*****************************************************************************/
/*****************************************************************************************/
/*                                                                                       */
/* History:                                                                              */
/*   30MAR2010 - First version of the macro.                                             */
/*                                                                                       */
/* 31Jan2012  S0827991   Fix the commuication occurrence and treatment association       */
/* 12Feb2013  S0924638   541MA07: CREATE TABLE (FOO) AS SELECT (...) FROM (FOO) can ...  */
/*****************************************************************************************/

%macro mapxt(cpxt=);

/* Some DEBUG information */
%if &MADebug. eq Y %then
%do;
    %put Current Version of the code: 541MA07;
    %put Last Modified on 12Feb2013;
    %put MAPXT macro Start Time - %sysfunc(datetime(), datetime24.2);
%end;

/* S0827991: The PREEXEC publish XML will have all the communication data in it even when a single
   communication is selected for execution. To have the correct package & treatment record populated in cpxt
   table use the ExecuteCommSK param to retain only records relevant to this communication and ignore
   the rest */
%if %upcase(&publishtype.) eq PREEXEC and &executecommsk. > 0 %then
%do;
    proc sort data=work.&cpxt.;
        by contributing_cell_package_sk;
        
    proc sort data=work.&ccp._base;
        by cell_package_sk;
        
    data work.&cpxt(keep=package_sk package_hash_val treatment_sk contributing_cell_package_sk sequence_no) ;
        merge 
            work.&cpxt.(in=a) 
            work.&ccp._base(in=b rename=(cell_package_sk=contributing_cell_package_sk));
        by contributing_cell_package_sk;
        if a and b;
    run;
    
    proc sort data=work.&cpxt. nodupkey;
        by package_sk package_hash_val treatment_sk contributing_cell_package_sk sequence_no;
%end;

/* CI_PACKAGE_X_TREATMENT: Update or Insert */
data &libnm..&cpxt.;
    modify &libnm..&cpxt. (&dswriteopts.) work.&cpxt. UPDATEMODE=NOMISSINGCHECK;
    by PACKAGE_SK PACKAGE_HASH_VAL TREATMENT_SK CONTRIBUTING_CELL_PACKAGE_SK SEQUENCE_NO;
    select(_IORC_);
        when (%SYSRC(_SOK))
        do;
            /* No need to replace; all columns accounted for in BY statement
             * replace just creates a lock we don't need.
             * replace;
             */
        end;
        when (%SYSRC(_DSENMR))
        do;
            _error_ = 0;
            output;
        end;
        otherwise
        do;
            stop;
        end;
    end;
run;

/* Exit the process if any error */
%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC. ^= 0 %then
    %do;
        %let SYSCC=100000;
        %let MAError = 838;
        %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
%end;
%else %if &SYSERR. > 4  %then
%do;
    %let MAError =838;
    %let MAMsg =Macro[&sysmacroname.]. Data Modify Stmt error while attempting to update Package_X_Treatment table . Check the log for details;
%end;

%if &MADebug. eq Y %then
%do;
    %put MAPXT End Time - %sysfunc(datetime(), datetime24.2);
%end;

%mend;
