/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/************************************************************************************/
/* NAME: marespub.sas                                                               */
/* VERSION: 5.4                                                                     */
/*                                                                                  */
/* DESCRIPTION: Macro to process the following data model tables                    */
/*              CI_RESPONSE                                                         */
/*              CI_RESPONSE_TYPE                                                    */
/*              CI_REPONSE_X_CELL_PACKAGE                                           */
/*              CI_RESPONSE_CHANNEL_RESPONSE                                        */
/* Parameters:                                                                      */
/*             cr    - Physical table name for Response                             */
/*             crt   - Physical table name for Response_type                        */
/*             crxcp - Physical table name for Response_X_Cell_Package              */
/*             crcr  - Physical table name for Response_Channel_Response            */
/*                                                                                  */
/* PRODUCT: MA                                                                      */
/* USAGE: %marespub                                                                 */
/*                                                                                  */
/************************************************************************************/
/*****************************************************************************************/
/*                                                                                       */
/* History:                                                                              */
/*   27APR2007 - First version of the macro.                                             */
/*                                                                                       */
/* DATE       Defect No  Comments                                                        */
/* 25May2009  S0559332   The STOP statement is removed from the DATA MODIFY block        */
/*                       where only REPLACE or OUTPUT is used                            */
/*                                                                                       */
/* 22Jun2009  S0575393   Allow the missing values from the input set to overwrite the    */
/*                       master table. This is mainly required when default UDF values   */
/*                       are reset to blank.                                             */
/* 23Aug2013  S0992864   Add dswriteopts to data modify                                  */
/*****************************************************************************************/

%macro marespub(cr=, crt=, crxcp=, crcr=);
%put Start Time - %sysfunc(datetime(), datetime24.2);

/* CI_RESPONSE: Update if exists otherwise directly insert */
data &libnm..&cr.;
    modify &libnm..&cr. (&dswriteopts.) work.&cr. UPDATEMODE=NOMISSINGCHECK;
    by RESPONSE_CD;
    select(_IORC_);
        when (%SYSRC(_SOK))
        do;
            replace;
        end;
        when (%SYSRC(_DSENMR))
        do;
            _error_ = 0;
            output;
        end;
        otherwise
        do;
            stop;
        end;
    end;
run;

/* Exit the process if any error */
%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC. ^= 0 %then
    %do;
        %let SYSCC=100000;
        %let MAError = 840;
        %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
%end;
%else %if &SYSERR. > 4  %then
%do;
    %let MAError =840;
    %let MAMsg =Macro[&sysmacroname.]. Data Modify Stmt error while attempting to update Response table . Check the log for details;
%end;

/* CI_RESPONSE_TYPE Update or Insert */
data &libnm..&crt.;
    modify &libnm..&crt. (&dswriteopts.) work.&crt. UPDATEMODE=NOMISSINGCHECK;
    by RESPONSE_TYPE_CD;
    select(_IORC_);
        when (%SYSRC(_SOK))
        do;
            replace;
        end;
        when (%SYSRC(_DSENMR))
        do;
            _error_ = 0;
            output;
        end;
        otherwise
        do;
            stop;
        end;
    end;
run;

/* Exit the process if any error */
%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC. ^= 0 %then
    %do;
        %let SYSCC=100000;
        %let MAError = 841;
        %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
%end;
%else %if &SYSERR. > 4  %then
%do;
    %let MAError =841;
    %let MAMsg =Macro[&sysmacroname.]. Data Modify Stmt error while attempting to update Response_Type table . Check the log for details;
%end;


/* CI_Response_X_Cell_Package: Update or Insert */
data &libnm..&crxcp.;
    modify &libnm..&crxcp. (&dswriteopts.) work.&crxcp. UPDATEMODE=NOMISSINGCHECK;
    by RESPONSE_CD CELL_PACKAGE_SK;
    select(_IORC_);
        when (%SYSRC(_SOK))
        do;
            replace;
        end;
        when (%SYSRC(_DSENMR))
        do;
            _error_ = 0;
            output;
        end;
        otherwise
        do;
            stop;
        end;
    end;
run;

/* Exit the process if any error */
%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC. ^= 0 %then
    %do;
        %let SYSCC=100000;
        %let MAError = 842;
        %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
%end;
%else
%if &SYSERR. > 4  %then
%do;
    %let MAError =842;
    %let MAMsg =Macro[&sysmacroname.]. Data Modify Stmt error while attempting to update CI_RESPONSE_X_CELL_Package. Check the log for details;
%end;

/* CI_RESPONSE_CHANNEL_RESPONSE: Insert */
/* All the three attributes are Primary Key here.  So It is only insert */
/* S0559332 - STOP statement removed */
data &libnm..&crcr.;
    modify &libnm..&crcr. (&dswriteopts.) work.&crcr.;
    by RESPONSE_CD RESPONSE_CHANNEL_CD RESPONSE_CHANNEL_RESPONSE_CD;
    select(_IORC_);
        when (%SYSRC(_DSENMR))
        do;
            _error_ = 0;
            output;
        end;
        otherwise;
    end;
run;

/* Exit the process if any error */
%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC. ^= 0 %then
    %do;
        %let SYSCC=100000;
        %let MAError = 844;
        %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
%end;
%else %if &SYSERR. > 4  %then
%do;
    %let MAError =844;
    %let MAMsg =Macro[&sysmacroname.]. Data Modify Stmt error while attempting to update CI_RESPONSE_CHANNEL_RESPONSE. Check the log for details;
%end;

%put end Time - %sysfunc(datetime(), datetime24.2);
%mend;
