/****************************************************************************/
/* Copyright (c) 2007 - 2012 by SAS Institute Inc., Cary, NC,USA.           */
/*                     All Rights Reserved.                                 */
/*       This software is protected by copyright laws and                   */
/*                    international treaties                                */
/*                                                                          */
/*               U.S. Government Restricted Rights                          */
/* Use, duplication, or disclosure of this software and related             */
/*    documentation by the U.S. government is subject to the                */
/*  Agreement with SAS Institute and the restrictions set forth             */
/*       in FAR 52.227-19, Commercial Computer Software -                   */
/*                Restricted Rights ( June 1987 )                           */
/*                                                                          */
/****************************************************************************/
/* NAME:marhast.sas                                                         */
/* VERSION:5.41                                                             */
/* DESCRIPTION: Common Data Interface macro to compute response counts      */
/*                                                                          */
/* PRODUCT: MA                                                              */
/* USAGE: %marhast                                                          */
/****************************************************************************/
/* Macro variables:                                                         */
/* Name       Desc                                    Value                 */
/* ch_tbl     ci_contact_history table name                                 */
/* cp_tbl     ci_cell_package table name                                    */
/* cpt_tbl    ci_cell_package_treatment_cnt table name                      */
/* crxcp_tbl  ci_response_x_cell_package table name                         */
/* data_tbl   Input Data table                                              */
/* dbengine   Name of the database Engine,Update tables Statments are       */
/*            Engine Dependent                                              */
/* libnm      Library Name for CI Common Data Model                         */
/* newid      Unique new id for creating data processing tables in templib  */
/* PerfFlag   D or T,to decide whether to generate summary by looking at    */
/*            history or just work with the input data set                  */
/*            D- Default, T-True Counts                                     */
/* pth_tbl    ci_presented_treatment_history table name                     */
/* rh_tbl     ci_response_history table Name                                */
/* sub_key_sp Space Separated Subject Key Column  List                      */
/* s_rsp_cd   Successfull response Type Code                                */
/* tmplibnm   Temporary Library Name for data processing                    */
/****************************************************************************/
/* Change log                                                               */
/* Date    DefectNo  Comments                                               */
/* 24Mar10 S0651614  New Version for Aster Partition Key Support            */
/* 18Jun12 S0782605  use proc sql to get counts instead of proc summary     */
/****************************************************************************/
%macro marhast(ch_tbl=,cp_tbl=,cpt_tbl=,crxcp_tbl=,data_tbl=,dbengine=,libnm=,
        newid=,perfflag=,pth_tbl=,rh_tbl=,sub_key_sp=,s_rsp_cd=,tmplibnm=,);
    /* Create workset using input table to generate the summary    */
    /* if Perf flag = Y then it create two worksets one for tpt_flg= Y and other for tpt_flg= N  */
    /* if Perf flag = N then it will have one workset for all cell packages irrespective of tpt_flg */

    /*if PerfFlag = T (True Counts)then create summary by joining the history tables*/
    %if &perfflag. eq T %then
    %do;
        /* Step2a:Create WorkSet joining input with cell_package to get track_presented_treatments_flg */
        %if &MADebug = Y %then  %put Step1 Start Time:%sysfunc(datetime(),datetime20);
        /*Append the tmp table to TblNames variable*/
        %let TblNames=&TblNames. %sysfunc(strip(rhWorkSet&newid.));
        proc sql;
            create table    &tmplibnm..rhWorkSet&newid. ( &MppDBKey. ) as
                select      t.cell_package_sk ,
                            t.treatment_sk,
                            %let i = 1;
                            %do %while (&i. ne &subCnt.);
                                t.&&dbSubCol&i ,
                                %let i = %eval(&i+1);
                            %end;
                            t.response_cd,
                            t.response_dttm,
                            cp.track_presented_treatments_flg as tpt_flg
                from        &tmplibnm..&data_tbl. t
                inner join  &libnm..&cp_tbl. cp
                on          t.cell_package_sk = cp.cell_package_sk
        ;quit;
        %if &MADebug = Y %then  %put Step1 End Time:%sysfunc(datetime(),datetime20);
        %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

        /*Step2b Create separate copies of dataset for Treatment_flag Y and N*/
        %let TblNames=&TblNames. %sysfunc(strip(rhWorkY&newid.));
        proc sql;
            create table &tmplibnm..rhWorkY&newid. ( &MppDBKey. ) as
                select * from &tmplibnm..rhWorkSet&newid. where tpt_flg = 'Y';
        quit;
        %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

        %let TblNames=&TblNames. %sysfunc(strip(rhWorkN&newid.));
        proc sql;
            create table &tmplibnm..rhWorkN&newid. ( &MppDBKey. ) as
                select * from &tmplibnm..rhWorkSet&newid. where tpt_flg = 'N';
        quit;
        %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

        /* Step2c: Check if there are records in rhWorkSet_y */
        %let cnt_tpt_flg_y=;
        proc sql noprint;
            select count(*) into :cnt_tpt_flg_y from &tmplibnm..rhWorkY&newid. ;
        quit;
        %if &cnt_tpt_flg_y. > 0 %then
        %do;
            /*Step2d Create a worskset with distinct cell_package_sk & subject_id's from rhWorkSet_y ;*/
            %let TblNames=&TblNames. %sysfunc(strip(rh_DiCpsk&newid.));
            proc sql;
                create table &tmplibnm..rh_DiCpsk&newid. ( &MppDBKey. ) as
                    select distinct cell_package_sk , &sub_key_cs from &tmplibnm..rhWorkY&newid. ;
            quit;
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            /* Step2e: Get the response history for the input_cell_package_sk having Tpt_flag= Y */
            /* add column counted_flag = Y for all records; */
            %if &MADebug = Y %then  %put Step 2e Start Time:%sysfunc(datetime(),datetime20);
            %let TblNames=&TblNames. %sysfunc(strip(rh_history&newid.));
            proc sql;
                create table    &tmplibnm..rh_history&newid. ( &MppDBKey. ) as
                    select      rh.cell_package_sk ,
                                rh.treatment_sk ,
                                %let i = 1;
                                %do %while (&i. ne &subCnt.);
                                    rh.&&dbSubCol&i ,
                                    %let i = %eval(&i+1);
                                %end;
                                rh.response_cd,
                                rh.response_dttm,
                                'Y' as counted_flg
                    from        &tmplibnm..rh_DiCpsk&newid. rhw
                    inner join  &libnm..&rh_tbl. rh
                    on          rh.cell_package_sk =rhw.cell_package_sk
                    %let i = 1;
                    %do %while (&i. ne &subCnt);
                        and           rh.&&dbSubCol&i=rhw.&&dbSubCol&i
                        %let i = %eval(&i+1);
                    %end;
                    where not exists (  select  1
                                from    &tmplibnm..rhWorkY&newid. i
                                where   i.cell_package_sk=rh.cell_package_sk
                                and     i.treatment_sk=rh.treatment_sk
                                %let i = 1;
                                %do %while (&i. ne &subCnt);
                                and     i.&&dbSubCol&i=rh.&&dbSubCol&i
                                    %let i = %eval(&i+1);
                                %end;
                                and     i.response_dttm =rh.response_dttm
                                and     i.response_cd = rh.response_cd)
                    union all
                    select      cell_package_sk , treatment_sk , &sub_key_cs. ,
                                response_cd , response_dttm, 'N' as counted_flg
                    from        &tmplibnm..rhWorkY&newid.
            ;quit;
            %if &MADebug = Y %then  %put Step 2e End Time:%sysfunc(datetime(),datetime20);
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            /* Step2g: Check if there are records in rh_history for Treatment_sk = -1 ;*/
            %let cnt_trt_sk_1=;
            proc sql noprint;
                select count(*) into :cnt_trt_sk_1 from &tmplibnm..rh_history&newid. where Treatment_sk = -1
            ;quit;

            %if &cnt_trt_sk_1. > 0 %then
            %do;
                /* Step2h: Convert (-1) Treatment_sk into corrosponding Treatment_sk for that cell_package ;*/
                %if &MADebug = Y %then  %put Step2e Start Time:%sysfunc(datetime(),datetime20);
                %let TblNames=&TblNames. %sysfunc(strip(rh_history_maptrtsk&newid.));
                proc sql;
                    create table    &tmplibnm..rh_history_maptrtsk&newid. ( &MppDBKey. ) as
                        select      t.cell_package_sk ,
                                    cpt.treatment_sk as treatment_sk,
                                    %let i = 1;
                                    %do %while (&i. ne &subCnt.);
                                        t.&&dbSubCol&i ,
                                        %let i = %eval(&i+1);
                                    %end;
                                    t.response_cd,
                                    t.response_dttm,
                                    counted_flg
                        from        &tmplibnm..rh_history&newid. t
                        inner join  &libnm..&cpt_tbl. cpt
                        on          cpt.cell_package_sk = t.cell_package_sk
                        and         t.treatment_sk =-1
                ;quit;
                %if &MADebug = Y %then  %put Step2e End Time:%sysfunc(datetime(),datetime20);
                %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

                /* Step2i: Convert (-1) Treatment_sk into corrosponding Treatment_sk for that cell_package ;*/
                /*remove the treatment_sk = -1 records*/
                proc sql;
                    delete from &tmplibnm..rh_history&newid. where treatment_sk =-1;
                quit;
                %if &SQLRC ne 0  %then %do;%let MAError =738; %goto exit; %end;

                /*add the mapped treatment records*/
                proc sql;
                    insert into &tmplibnm..rh_history&newid.
                        select * from &tmplibnm..rh_history_maptrtsk&newid.
                quit;
                %if &SQLRC ne 0  %then %do;%let MAError =739; %goto exit; %end;

            %end;
            /* Step2j: Get Response Type Codes */
            %if &MADebug = Y %then  %put Step2j Start Time:%sysfunc(datetime(),datetime20);
            %let TblNames=&TblNames. %sysfunc(strip(rh_xr_history&newid.));
            proc sql;
                create table    &tmplibnm..rh_xr_history&newid. ( &MppDBKey. ) as
                    select      rh.*,
                                xr.response_type_cd
                    from        &tmplibnm..rh_history&newid. rh
                    inner join  &libnm..&crxcp_tbl. xr
                    on          xr.cell_package_sk = rh.cell_package_sk
                    and         xr.response_cd = rh.response_cd
            ;quit;
            %if &MADebug = Y %then  %put Step2j End Time:%sysfunc(datetime(),datetime20);
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            /* Step2k: Get latest Presented Treatment Date Time ;*/
            %if &MADebug = Y %then  %put Step2k Start Time:%sysfunc(datetime(),datetime20);
            %let TblNames=&TblNames. %sysfunc(strip(rh_xr_pt_hist&newid.));
            proc sql;
                create table    &tmplibnm..rh_xr_pt_hist&newid. ( &MppDBKey. ) as
                    select      rx.cell_package_sk ,
                                rx.treatment_sk ,
                                %let i = 1;
                                %do %while (&i. ne &subCnt.);
                                    rx.&&dbSubCol&i ,
                                    %let i = %eval(&i+1);
                                %end;
                                rx.response_cd ,
                                rx.response_type_cd,
                                rx.response_dttm ,
                                rx.counted_flg,
                                max(pt.presented_treatment_hist_dttm) as pt_dttm
                    from        &tmplibnm..rh_xr_history&newid. rx
                    inner join  &libnm..&pth_tbl. pt
                    on          pt.cell_package_sk = rx.cell_package_sk
                    and         pt.treatment_sk=rx.treatment_sk
                    %let i = 1;
                    %do %while (&i. ne &subCnt);
                        and           pt.&&dbSubCol&i=rx.&&dbSubCol&i
                        %let i = %eval(&i+1);
                    %end;
                    and         pt.presented_treatment_hist_dttm  <= rx.response_dttm
                    group by    rx.cell_package_sk , rx.treatment_sk ,
                                %let i = 1;
                                %do %while (&i. ne &subCnt.);
                                    rx.&&dbSubCol&i ,
                                    %let i = %eval(&i+1);
                                %end;
                                rx.response_cd, rx.response_type_cd, rx.response_dttm, rx.counted_flg
            ;quit;
            %if &MADebug = Y %then  %put Step2k End Time:%sysfunc(datetime(),datetime20);
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            /* Step2l: Get latest Contact Date Time ;*/
            %if &MADebug = Y %then  %put Step2l Start Time:%sysfunc(datetime(),datetime20);
            %let TblNames=&TblNames. %sysfunc(strip(rh_xr_pt_ch_hist&newid.));
            proc sql;
                create table    &tmplibnm..rh_xr_pt_ch_hist&newid. ( &MppDBKey. ) as
                    select      rxp.cell_package_sk ,
                                rxp.treatment_sk ,
                                %let i = 1;
                                %do %while (&i. ne &subCnt.);
                                    rxp.&&dbSubCol&i ,
                                    %let i = %eval(&i+1);
                                %end;
                                rxp.response_cd,
                                rxp.response_type_cd,
                                rxp.response_dttm,
                                rxp.pt_dttm,
                                rxp.counted_flg,
                                max(ch.contact_dttm) as ch_dttm
                    from        &tmplibnm..rh_xr_pt_hist&newid. rxp
                    left join   &libnm..&ch_tbl. ch
                    on          ch.cell_package_sk=rxp.cell_package_sk
                    %let i = 1;
                    %do %while (&i. ne &subCnt);
                        and           ch.&&dbSubCol&i=rxp.&&dbSubCol&i
                        %let i = %eval(&i+1);
                    %end;
                    and         ch.contact_dttm <= rxp.pt_dttm
                    group by    rxp.cell_package_sk , rxp.treatment_sk ,
                                %let i = 1;
                                %do %while (&i. ne &subCnt.);
                                    rxp.&&dbSubCol&i ,
                                    %let i = %eval(&i+1);
                                %end;
                                rxp.response_cd , rxp.response_type_cd , rxp.response_dttm , rxp.pt_dttm , rxp.counted_flg
            ;quit;
            %if &MADebug = Y %then  %put Step2l End Time:%sysfunc(datetime(),datetime20);
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            /* Step2m: Separate counted & uncounted records and create two copies of uncounted one for cell_package level & other for treatment level;*/
            %if &MADebug = Y %then  %put Step2m Start Time:%sysfunc(datetime(),datetime20);
            /*data &tmplibnm..rh_counted&newid. &tmplibnm..rh_cp_uncounted&newid. &tmplibnm..rh_cpt_uncounted&newid. ; */
            %let TblNames=&TblNames. %sysfunc(strip(rh_cp_uncounted&newid.));
            proc sql;
                create table    &tmplibnm..rh_cp_uncounted&newid. ( &MppDBKey. ) as
                    select      t1.* from &tmplibnm..rh_xr_pt_ch_hist&newid. t1
                    where       counted_flg = 'N' and
                    not exists (  select  1
                                from    &tmplibnm..rh_xr_pt_ch_hist&newid. t2
                                where   t2.cell_package_sk=t1.cell_package_sk
                                %let    i = 1;
                                %do %while (&i. ne &subCnt);
                                    and     t2.&&dbSubCol&i=t1.&&dbSubCol&i
                                    %let i = %eval(&i+1);
                                %end;
                                and     t2.response_cd=t1.response_cd
                                and     t2.Ch_Dttm=t1.Ch_Dttm
                                and     t2.counted_flg = 'Y')
            ;quit;

            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            %let TblNames=&TblNames. %sysfunc(strip(rh_cpt_uncounted&newid.));
            proc sql;
                create table    &tmplibnm..rh_cpt_uncounted&newid. ( &MppDBKey. ) as
                    select      t1.* from &tmplibnm..rh_xr_pt_ch_hist&newid. t1
                    where       t1.counted_flg = 'N' and
                    not exists (  select  1
                                from    &tmplibnm..rh_xr_pt_ch_hist&newid. t2
                                where   t2.cell_package_sk=t1.cell_package_sk
                                and     t2.treatment_sk = t1.treatment_sk
                                %let    i = 1;
                                %do %while (&i. ne &subCnt);
                                    and     t2.&&dbSubCol&i=t1.&&dbSubCol&i
                                    %let i = %eval(&i+1);
                                %end;
                                and     t2.response_cd=t1.response_cd
                                and     t2.Ch_Dttm=t1.Ch_Dttm
                                and     t2.counted_flg = 'Y')
            ;quit;

            %if &MADebug = Y %then  %put Step2m End Time:%sysfunc(datetime(),datetime20);
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

/* Step3- Generate Summary tables for Cell Packages With Treatment Flag = Y */

            /* Step3a:  Generate cell Package level summary for cell_package_sk having tpt_flg = Y when PerfFlag = T*/

            /* Step3a1: Prepare dataset to Count TOTAL_RESPONSE_CNT & TOTAL_SUCCESSFUL_RESPONSE_CNT at Cell  Package Level */
            %let TblNames=&TblNames. %sysfunc(strip(rh_cp_b4summary&newid.));

            /*Instead of proc sort Using proc sql create table with group by to remove duplicate row for a group condition*/
            proc sql;
                create table    &tmplibnm..rh_cp_b4summary&newid. ( &MppDBKey. ) as
                    select      cell_package_sk , &sub_key_cs. , response_cd, response_type_cd , Ch_Dttm , count(*) as DummyCnt
                    from        &tmplibnm..rh_cp_uncounted&newid.
                    group by    cell_package_sk , &sub_key_cs. , response_cd , response_type_cd , Ch_Dttm
            ;quit;
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            /* Step3a2: count TOTAL_RESPONSE_CNT  at cell package level */
            %let TblNames=&TblNames. %sysfunc(strip(cp_total_resp&newid.));
            %* S0782605 - use proc sql to get counts instead of proc summary ;
            proc sql;
                create table    &tmplibnm..cp_total_resp&newid. (  &MppDBKey. ) as
                    select      CELL_PACKAGE_SK , count(*) as T_RESP_CNT
                    from        &tmplibnm..rh_cp_b4summary&newid.
                    group by    CELL_PACKAGE_SK ;
            quit;

            /* Step3a3: count TOTAL_SUCCESSFUL_RESPONSE_CNT at cell package level */
            %let TblNames=&TblNames. %sysfunc(strip(cp_success_resp&newid.));
            proc sql;
                create table    &tmplibnm..cp_success_resp&newid. ( &MppDBKey. ) as
                    select      CELL_PACKAGE_SK , count(*) as S_RESP_CNT
                    from        &tmplibnm..rh_cp_b4summary&newid.
                    where       RESPONSE_TYPE_CD = "&s_rsp_cd"
                    group by    CELL_PACKAGE_SK ;
            quit;

            /* Step3b: Generate Cell Package Treatment Level Summary for cell_package_sk having tpt_flg = Y when PerfFlag = T*/

            /* Step3b1: Prepare dataset to Count TOTAL_RESPONSE_CNT & TOTAL_SUCCESSFUL_RESPONSE_CNT at Treatment Level */
            %let TblNames=&TblNames. %sysfunc(strip(rh_cpt_b4summary&newid.));

            /*Instead of proc sort Using proc sql create table with group by to remove duplicate row for a group condition*/
            proc sql;
                create table    &tmplibnm..rh_cpt_b4summary&newid. ( &MppDBKey. ) as
                    select      cell_package_sk , treatment_sk , &sub_key_cs. , response_cd , response_type_cd , Ch_Dttm , count(*) as DummyCnt
                    from        &tmplibnm..rh_cpt_uncounted&newid.
                    group by    cell_package_sk , treatment_sk , &sub_key_cs. , response_cd , response_type_cd , Ch_Dttm
            ;quit;
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

            /* Step3b2: count TOTAL_RESPONSE_CNT  at Treatment level */
            %let TblNames=&TblNames. %sysfunc(strip(cpt_total_resp&newid.));
            %* S0782605 - use proc sql to get counts instead of proc summary ;
            proc sql;
                create table    &tmplibnm..cpt_total_resp&newid. ( &MppDBKey. ) as
                    select      CELL_PACKAGE_SK , TREATMENT_SK , count(*) as T_RESP_CNT
                    from        &tmplibnm..rh_cpt_b4summary&newid.
                    group by    CELL_PACKAGE_SK , TREATMENT_SK ;
            quit;

            /* Step3b3: TOTAL_SUCCESSFUL_RESPONSE_CNT at Treatment level */
            %let TblNames=&TblNames. %sysfunc(strip(cpt_success_resp&newid.));
            proc sql;
                create table    &tmplibnm..cpt_success_resp&newid.( &MppDBKey. ) as
                    select      CELL_PACKAGE_SK , TREATMENT_SK , count(*) as S_RESP_CNT
                    from        &tmplibnm..rh_cpt_b4summary&newid.
                    where       Response_type_cd = "&s_rsp_cd"
                    group by    CELL_PACKAGE_SK , TREATMENT_SK ;
            quit;

            /* Step3c: Merge the summary tables                 */

            /* Step3c1: Merge the cell package level summary    */
            %let TblNames=&TblNames. %sysfunc(strip(cp_resp_count&newid.));
            data &tmplibnm..cp_resp_count&newid. ( &MppDBKey. );
                merge &tmplibnm..cp_total_resp&newid.(in=tr) &tmplibnm..cp_success_resp&newid. (in=sr);
                by cell_package_sk;
            run;

            /* Step3c2: Merge the treatment level summary tables */
            %let TblNames=&TblNames. %sysfunc(strip(cpt_resp_count&newid.));
            data &tmplibnm..cpt_resp_count&newid. ( &MppDBKey. );
                merge &tmplibnm..cpt_total_resp&newid.(in=tr) &tmplibnm..cpt_success_resp&newid. (in=sr);
                by cell_package_sk treatment_sk;
            run;
        %end;
    %end;

    %if &perfflag. eq D %then
    %do;
       /*Create work dataset from input dataset;*/
        %let TblNames=&TblNames. %sysfunc(strip(rhWorkN&newid.));
        proc sql;
            create table &tmplibnm..rhWorkN&newid. ( &MppDBKey. ) as
                select  cell_package_sk,treatment_sk, &sub_key_cs.,response_cd,response_dttm,'N' as tpt_flg
                from        &tmplibnm..&data_tbl.
        quit;
        %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;
    %end;

/* Step4- Generate Summary tables for Cell Packages With Treatment Flag = N (or when Perfflag=D)*/

    /* Step4a: Check if there are records in rhWorkSet_n */
    %let cnt_tpt_flg_N=;
    proc sql noprint;
        select count(*) into :cnt_tpt_flg_N from &tmplibnm..rhWorkN&newid.
    ;quit;

    %if &cnt_tpt_flg_N. > 0 %then
    %do;
        /* Step4b: Check if there are records in rhWorkSet_n with Treatment_sk = -1 */
        proc sql noprint;
            select count(*) into :cnt_trt_sk_1 from &tmplibnm..rhWorkN&newid. t where t.treatment_sk =-1
        ;quit;

        %if &cnt_trt_sk_1. > 0 %then
        %do;
            /* Step4c: Convert the Treatment_SK -1 into actual treatments for that cell package ;*/
            %let TblNames=&TblNames. %sysfunc(strip(rhWorkSet_maptrtsk_N&newid.));
            proc sql;
                create table    &tmplibnm..rhWorkSet_maptrtsk_N&newid. ( &MppDBKey. ) as
                    select      t.cell_package_sk ,
                                cpt.treatment_sk,
                                %let i = 1;
                                %do %while (&i. ne &subCnt.);
                                    t.&&dbSubCol&i ,
                                    %let i = %eval(&i+1);
                                %end;
                                t.response_cd,
                                t.response_dttm
                    from        &tmplibnm..rhWorkN&newid. t
                    inner join  &libnm..&cpt_tbl. cpt
                    on          cpt.cell_package_sk = t.cell_package_sk
                    where       t.treatment_sk =-1
            ;quit;
            %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;
            /*Step4d Remove the treatment_sk = -1 records from rhWorkSet_n      */
            /*    and add mapped treatment_sk records from rhWorkSet_maptrtsk_N */
            proc sql;
                delete from &tmplibnm..rhWorkN&newid. where treatment_sk =-1;
            quit;
            %if &SQLRC ne 0  %then %do;%let MAError =738; %goto exit; %end;
            proc sql;
                insert into &tmplibnm..rhWorkN&newid.
                    select  cell_package_sk ,
                                treatment_sk,
                                &sub_key_cs.,
                                response_cd,
                                response_dttm,
                                'N' as tpt_flg
                    from &tmplibnm..rhWorkSet_maptrtsk_N&newid.
            ;quit;
            %if &SQLRC ne 0  %then %do;%let MAError =739; %goto exit; %end;
        %end;

        /* Step4e: join the rhWorkSet_n table to Cell_package_X table to get Response_type_cd */
        %let TblNames=&TblNames. %sysfunc(strip(rh_perf_yn_tpt_n&newid.));
        proc sql;
            create table    &tmplibnm..rh_perf_yn_tpt_n&newid. ( &MppDBKey. ) as
                select      t.*,
                            xr.response_type_cd
                from        &tmplibnm..rhWorkN&newid. t
                inner join  &libnm..&crxcp_tbl. xr
                on          xr.cell_package_sk = t.cell_package_sk
                and         xr.response_cd = t.response_cd
        ;quit;
        %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

/* Step5- Generate Summary tables for Cell Packages With Treatment Flag = N */

        /* Step5a: Prepare dataset to Count TOTAL_RESPONSE_CNT & TOTAL_SUCCESSFUL_RESPONSE_CNT at Cell_Package Level */
        %let TblNames=&TblNames. %sysfunc(strip(rh_cp_b4summary_n&newid.));
        proc sql;
            create table    &tmplibnm..rh_cp_b4summary_n&newid. ( &MppDBKey. ) as
                select      cell_package_sk , &sub_key_cs. , response_cd , response_type_cd, count(*) as DummyCnt
                from        &tmplibnm..rh_perf_yn_tpt_n&newid.
                group by    cell_package_sk , &sub_key_cs. , response_cd , response_type_cd
        ;quit;
        %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

        /* Step5b: TOTAL_RESPONSE_CNT  at cell package level */
        %let TblNames=&TblNames. %sysfunc(strip(cp_total_resp_n&newid.));
        %* S0782605 - use proc sql to get counts instead of proc summary ;
        proc sql;
            create table    &tmplibnm..cp_total_resp_n&newid.( &MppDBKey. ) as
                select      cell_package_sk , count(*) as T_RESP_CNT
                from        &tmplibnm..rh_cp_b4summary_n&newid.
                group by    cell_package_sk ;
        quit;

        /* Step5c: TOTAL_SUCCESSFUL_RESPONSE_CNT at cell package level */
        %let TblNames=&TblNames. %sysfunc(strip(cp_success_resp_n&newid.));
        proc sql;
            create table    &tmplibnm..cp_success_resp_n&newid.( &MppDBKey. ) as
                select      cell_package_sk , count(*) as S_RESP_CNT
                from        &tmplibnm..rh_cp_b4summary_n&newid.
                where       RESPONSE_TYPE_CD = "&s_rsp_cd"
                group by    cell_package_sk ;
        quit;

        /* Step5d: Prepare dataset to Count TOTAL_RESPONSE_CNT & TOTAL_SUCCESSFUL_RESPONSE_CNT at Treatment Level */
        %let TblNames=&TblNames. %sysfunc(strip(rh_cpt_b4summary_n&newid.));
        proc sql;
            create table &tmplibnm..rh_cpt_b4summary_n&newid. ( &MppDBKey. ) as
                select      cell_package_sk , treatment_sk , &sub_key_cs. , response_cd , response_type_cd , count(*) as DummyCnt
                from        &tmplibnm..rh_perf_yn_tpt_n&newid.
                group by    cell_package_sk , treatment_sk , &sub_key_cs. , response_cd , response_type_cd
        ;quit;
        %if &SQLRC ne 0  %then %do;%let MAError =736; %goto exit; %end;

        /* Step5e: TOTAL_RESPONSE_CNT  at Treatment level */
        %let TblNames=&TblNames. %sysfunc(strip(cpt_total_resp_n&newid.));
        %* S0782605 - use proc sql to get counts instead of proc summary ;
        proc sql;
            create table    &tmplibnm..cpt_total_resp_n&newid.( &MppDBKey. ) as
                select      cell_package_sk , treatment_sk , count(*) as T_RESP_CNT
                from        &tmplibnm..rh_cpt_b4summary_n&newid.
                group by    cell_package_sk , treatment_sk ;
        quit;

        /* Step5f: TOTAL_SUCCESSFUL_RESPONSE_CNT at Treatment level */
        %let TblNames=&TblNames. %sysfunc(strip(cpt_success_resp_n&newid.));
        proc sql;
            create table    &tmplibnm..cpt_success_resp_n&newid.( &MppDBKey. ) as
                select      cell_package_sk , treatment_sk , count(*) as S_RESP_CNT
                from        &tmplibnm..rh_cpt_b4summary_n&newid.
                where       Response_type_cd = "&s_rsp_cd"
                group by    cell_package_sk , treatment_sk ;
        quit;

        /* Step5g: Merge Cell_Package Summary columns */
        /*option nodsnferr used to avoid error occurance in case one of the merging tables not present */
        option nodsnferr;
        %let TblNames=&TblNames. %sysfunc(strip(cp_resp_count_n&newid.));
        data &tmplibnm..cp_resp_count_n&newid. ( &MppDBKey. );
            merge &tmplibnm..cp_total_resp_n&newid.(in=tr) &tmplibnm..cp_success_resp_n&newid. (in=sr);
            by cell_package_sk;
        run;

        /* Step5h: Merge Cell_Package_Treatment Summary columns */
        %let TblNames=&TblNames. %sysfunc(strip(cpt_resp_count_n&newid.));
        data &tmplibnm..cpt_resp_count_n&newid. ( &MppDBKey. ) ;
            merge &tmplibnm..cpt_total_resp_n&newid.(in=tr) &tmplibnm..cpt_success_resp_n&newid. (in=sr);
            by cell_package_sk;
        run;
    %end;

/* Step6- Merge the Summary tables from Step3 & Step5           */

    /* Step6a: Merge Cell_Package Summary tables(summary of treatment_flag Y + summary of treatment_flag N) */
    /*option nodsnferr used to avoid error occurance in case one of the data tables not present */
    option nodsnferr;
    %let TblNames=&TblNames. %sysfunc(strip(cp_responses&newid.));
    data &tmplibnm..cp_responses&newid. ( &MppDBKey. );
        set &tmplibnm..cp_resp_count&newid. &tmplibnm..cp_resp_count_n&newid. ;
    run;

    /* Step6b: Merge Cell_Package_treatment Summary tables(summary of treatment_flag Y + summary of treatment_flag N) */
    %let TblNames=&TblNames. %sysfunc(strip(cpt_responses&newid.));
    data &tmplibnm..cpt_responses&newid. ( &MppDBKey. );
        set &tmplibnm..cpt_resp_count&newid. &tmplibnm..cpt_resp_count_n&newid.;
    run;
%exit:
%mend;
