/****************************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.           */
/*                     All Rights Reserved.                                 */
/*       This software is protected by copyright laws and                   */
/*                    international treaties                                */
/*                                                                          */
/*               U.S. Government Restricted Rights                          */
/* Use, duplication, or disclosure of this software and related             */
/*    documentation by the U.S. government is subject to the                */
/*  Agreement with SAS Institute and the restrictions set forth             */
/*       in FAR 52.227-19, Commercial Computer Software -                   */
/*                Restricted Rights ( June 1987 )                           */
/*                                                                          */
/****************************************************************************/
/* NAME: marowcnt.sas                                                       */
/* VERSION: 5.4                                                           */
/* DESCRIPTION: Use a common macro for getting the row count of tables      */
/*                                                                          */
/* PRODUCT: MA                                                              */
/* USAGE: %marowcnt                                                         */
/****************************************************************************/
/* Macro variables:                                                         */
/* Name       Desc                                    Value                 */
/* table      table name                                                    */
/* countvar   Count variable name to store the row count                    */
/****************************************************************************/
/* Change log                                                               */
/* Date    DefectNo  Comments                                               */
/* 22Apr10 S0594364  Initial Version                                        */
/* 14Aug13 S0999690  add dsreadopts                                         */
/****************************************************************************/
%macro marowcnt(table, countvar);

    %if %symexist(dsreadopts) eq 0 %then
    %do;
        %let dsreadopts = %str( );
    %end;

    %global &countvar;
    %if %sysfunc( exist( &table ) ) %then
    %do;
        %let gotCount=N;
        data _null_;
            if _n_ = 1 then
            do;
                if nc < 1e15 then
                do;       *** a spds or sas dataset ***;
                    call symput( 'gotCount', 'Y' );
                    call symput( "&countvar", left(put(nc,best12.)));
                end;
                stop;
            end;
            set &table nobs=nc;   *** Moved set to bottom in Hotfix 2 ***;
        run;
        %if &gotCount = N %then
        %do;   *** a database table ***;
            %let nc=-1;
            proc sql noprint;
                select count(*) into :&countvar from &table (&dsreadopts.);
            quit;
        %end;
    %end;
    %else
    %do;
        %let &countvar=-1;
    %end;
%mend;
