/*****************************************************************/
/* Copyright (c) 2007 - 2013 by SAS Institute Inc., Cary, NC,    */
/*                    USA.                                       */
/*             All Rights Reserved.                              */
/*  This software is protected by copyright laws and             */
/*               international treaties                          */
/*                                                               */
/*         U.S. Government Restricted Rights                     */
/* Use, duplication, or disclosure of this software and related  */
/*  documentation by the U.S. government is subject to the       */
/*   Agreement with SAS Institute and the restrictions set forth */
/*  in FAR 52.227-19, Commercial Computer Software -             */
/*              Restricted Rights ( June 1987 )                  */
/*                                                               */
/*****************************************************************/
/* NAME: mascount                                                */
/* VERSION: 6.1                                                  */
/* DESCRIPTION:                                                  */
/*                                                               */
/* PRODUCT: MA                                                   */
/* USAGE: %mamcount( InTable, VarList, IdList, SubjectKeys,      */
/*  OutTable, Type, Mode, Option, IdListSeparator );             */
/*        InTable -                                              */
/*        VarList -                                              */
/*        IdList -                                               */
/*        SubjectKeys -                                          */
/*        OutTable -                                             */
/*        Type -                                                 */
/*        Mode -                                                 */
/*        Option -                                               */
/*        IdListSeparator -                                      */
/*                                                               */
/*****************************************************************/
/*****************************************************************************/
/* PRISM VERSION: $Revision: 1.1.4.1 $ ;
/* Revision Date: $Date: 2013/07/25 13:07:58 $  ;
/*****************************************************************************/
/*****************************************************************************/
/* HISTORY:                                                                  */
/* DATE       Defect No  Comments/updated by                                 */
/* ---------  ---------  --------------------------------------------------- */
/* 23APR2013             GA                                                  */
/*****************************************************************************/



%macro mascount(InTable, VarList, SubjectKeys, OutTable, Type, Option);

/* Check if there was an error in the stored process.
   Return from the macro if there is one. */
%if (&MAError ne %str()) %then %do;
    %return;
%end;

%if %UPCASE(&Option) eq SAMETABLE %then %do;

   %if &MAdebug = Y
   %then %put Variables are the in subject table. One step PROC SUMMARY.;

   proc summary data=&InTable chartype;
       class &VarList / missing groupinternal;
       types &VarList;
       output out=_macount(compress=yes);
   run;

    /* If there are errors, initialize the global variables and return. */
    %if &syserr > 4 %then %do;
        %let MAError = 101;
        %let MAMsg = Macro[&sysmacroname.]. Error in one-step PROC SUMMARY, SYSERR[&syserr];
        %return;
    %end;
%end;
%else %do;
   %if &MAdebug = Y
   %then %put Variables potentially many-to-one with subject table. Two step PROC SUMMARY.;

   %let keyCount=0;
   %let subjectVar=%scan(&SubjectKeys, 1, %str( ));
   %do %while(%length(&subjectVar) > 0);
      %let keyCount=%eval(&keyCount+1);
      %let subjectVar=%scan(&subjectKeys, &keyCount+1, %str( ));
   %end;

   %let lastKey=%scan(&SubjectKeys,-1,%str( ));

   %if &keyCount > 1 %then %do;
      data work.view2 /view=work.view2;
         retain _makey 0;
         set &inTable; by &SubjectKeys;
         if first.&lastKey then _makey+1;
         keep _makey &VarList;
      run;

/*
        proc sort data=&InTable out=inkeys nodupkey; by &SubjectKeys; run;
        data keymap;
                set inkeys;
                _makey = _n_;
                keep _makey &SubjectKeys;
        run;
        proc sql noprint;
             create view work.view2 as
             select
             %let i=1;
             %let var=%scan(&VarList, 1, %str( ));
             %do %while (%length(&var) > 0);
                i.&var,
                %let i=%eval(&i+1);
                %let var=%scan(&VarList, &i, %str( ));
             %end;
             m._makey
             from &InTable i, keymap m
             where
             %let i=1;
             %let subjectVar=%scan(&SubjectKeys, 1, %str( ));
             %do %while (%length(&subjectVar) > 0);
                 %if &i > 1 %then and ;
                i.&subjectVar = m.&subjectVar
                %let i=%eval(&i+1);
                %let subjectVar=%scan(&subjectkeys, &i, %str( ));
             %end;
             order by _makey
    ;
        quit;
*/
    /* If there are errors, initialize the global variables and return. */
      %if &sqlrc > 4  %then %do;
         %let MAError = 102;
         %let MAMsg = Macro[&sysmacroname.]. PROC SQL error in creating view - WORK.VIEW2, SQLRC[&sqlrc].;
         %return;
      %end;

      %let inTable = work.view2;
      %let SubjectKeys = _makey;
   %end;

   proc summary data=&InTable chartype;
      class &VarList &SubjectKeys / missing groupinternal;
      by &SubjectKeys;
      types (&VarList) * (&SubjectKeys);
      output out=_idsum(compress=yes drop=_freq_);
   run;

   /* If there are errors, initialize the global variables and return. */
   %if &syserr > 4 %then %do;
      %let MAError = 103;
      %let MAMsg = Macro[&sysmacroname.]. Error in two-step PROC SUMMARY in creating _IDSUM, SYSERR[&syserr];
      %return;
   %end;


   proc summary data=_idsum(rename=(_type_=_idtype)) nway;
      class _idtype &VarList / missing groupinternal;
      output out=_macount(compress=yes);
   run;

   /* If there are errors, initialize the global variables and return. */
   %if &syserr > 4 %then %do;
      %let MAError = 104;
      %let MAMsg = Macro[&sysmacroname.]. Error in two-step PROC SUMMARY in creating _MACOUNT, SYSERR[&syserr];
      %return;
   %end;

%end;


 data &OutTable;

 %if %UPCASE(&Type) eq CHAR %then %do;
     array vars{*} $ 200 &VarList;
 %end;
 %else %do;
     array vars{*} 8 &VarList;
 %end;

     set _macount;

%if %UPCASE(&Option) eq SAMETABLE %then %do;
     pos = index( _type_, '1' );
%end;
%else %do;
     pos = index( _idtype, '1' );
%end;

     fmt = vformat( vars(pos) );

 %if %UPCASE(&Type) eq HISTO %then %do;
    Group1 = vars(pos);
    keep Group1;
 %end;
 %else %if %UPCASE(&Type) eq CHAR %then %do;
    length Value $ 200 unformatted $ 200 formatted $ 200;
    Value = vars(pos);
    formatted = putc( Value, fmt );
    rename Value=unformatted;
    keep Value formatted;
 %end;
 %else %do;
    Value = vars(pos);
    length formatted $ 200;
    formatted = putn( Value, fmt );
    rename Value=unformatted;
    keep Value formatted;
 %end;

    length Rule $ 8;
    Rule = 'Any';
    rename _FREQ_=Count1;
    keep _FREQ_ Rule;

    %let i=1;
    %let sTable=%scan(&outTable,&i,%str( ));
    %do %while(%length(&sTable) > 0);
        if pos = &i then output &sTable;

        %let i=%eval(&i+1);
        %let sTable=%scan(&outTable,&i,%str( ));
    %end;
 run;

  /* If there are errors, initialize the global variables and return. */
    %if &syserr > 4 %then %do;
        %let MAError = 106;
        %let MAMsg = Macro[&sysmacroname.]. PROC APPEND - error in appending [BASE] to [&OutTable.], SYSERR[&syserr];
        %return;
    %end;

%mend;
