/*****************************************************************/
/* Copyright (c) 2004 - 2012 by SAS Institute Inc., Cary, NC,    */
/*                    USA.                                       */
/*             All Rights Reserved.                              */
/*  This software is protected by copyright laws and             */
/*               international treaties                          */
/*                                                               */
/*       U.S. Government Restricted Rights                       */
/* Use, duplication, or disclosure of this software and related  */
/*  documentation by the U.S. government is subject to the       */
/*   Agreement with SAS Institute and the restrictions set forth */
/*  in FAR 52.227-19, Commercial Computer Software -             */
/*              Restricted Rights ( June 1987 )                  */
/*                                                               */
/*****************************************************************/
/* NAME: maseq                                                   */
/* VERSION: 5.4  Backport from 6.1                               */
/* DESCRIPTION:                                                  */
/*                                                               */
/* PRODUCT: MA                                                   */
/* USAGE: %maseq( table, sequence, number, limit= );             */
/*        table - two level name of sequence table               */
/*        sequence - name of sequence                            */
/*        number - increment sequence by this                    */
/*        limit= # of retries when table is locked (default 30)  */
/*                                                               */
/*****************************************************************/
%macro maseq(table, sequence, number, limit=30);
    %if %sysfunc(exist( &table ) ) eq 0
    %then %do;
        data &table;
            length Sequence $ 20;
            Sequence = 'CampNum' ; Number = 0; output;
            Sequence = 'CampCode'; Number = 0; output;
            Sequence = 'CommNum' ; Number = 0; output;
            Sequence = 'CommCode'; Number = 0; output;
            Sequence = 'FlowNum' ; Number = 0; output;
        run;
    %end;

    %let try=1;
%TryLock:
    %if &try lt &limit %then %do;
        /* hide errors acquiring the lock until the last run */
        filename bitbuckt temp;
        proc printto log=bitbuckt new; run;
    %end;
    lock &table;
    %if &try lt &limit %then %do;
        proc printto; run;
        filename bitbuckt clear;
    %end;

    /* increment the limit now so that if we loop back due to a lock
       acquisition failure then the number has increased */
    %if &SYSLCKRC gt 0 %then %do;
        %if &try le &limit %then %do;
            %put Unable to acquire lock, sleeping (&SYSLCKRC).;
            data _null_;
                s=sleep(1,1);
            run;
            %let try=%eval(&try+1);
            %goto TryLock;
        %end;
        %else %goto LockFailed;
    %end;

    %let seqCheck = 0;
    proc sql noprint; select count(*) into :seqCheck from &table where Sequence="&sequence"; quit;
    %if &SeqCheck = 0
    %then %do;
        proc sql noprint;
           insert into &table (Sequence, Number) values ("&sequence", 0);
        quit;
    %end;
    %if &number eq
    %then %let number = 1;
    %let id=;
    %put Try &try to get &sequence;
    data &table;
        modify &table;
        if Sequence="&sequence";
        number + &number;
        if _iorc_ = %sysrc( _DSENOM  )
        then do; call symput( 'id', '' ); call sleep(3); end;
        else call symput( 'id', left( put( number, f16. ) ) );
    run;
    lock &table clear;
    %let newid=&id;
%LockFailed:
%mend;
