/*****************************************************************/
/* Copyright (c) 2004 - 2013 by SAS Institute Inc., Cary, NC,    */
/*                    USA.                                       */
/*             All Rights Reserved.                              */
/*  This software is protected by copyright laws and             */
/*               international treaties                          */
/*                                                               */
/*       U.S. Government Restricted Rights                       */
/* Use, duplication, or disclosure of this software and related  */
/*  documentation by the U.S. government is subject to the       */
/*   Agreement with SAS Institute and the restrictions set forth */
/*  in FAR 52.227-19, Commercial Computer Software -             */
/*              Restricted Rights ( June 1987 )                  */
/*                                                               */
/*****************************************************************/
/* NAME: mashmacr                                                */
/* VERSION: 6.1                                                  */
/* DESCRIPTION:                                                  */
/*                                                               */
/* PRODUCT: MA                                                   */
/* USAGE: %mashmacr;                                             */
/* This macro contains other shared macros for use by stored     */
/* processes                                                     */
/*****************************************************************/
/*****************************************************************************/
/* PRISM VERSION: $Revision: 1.1.4.1 $ ;
/* Revision Date: $Date: 2013/07/25 13:07:58 $  ;
/*****************************************************************************/
/*****************************************************************************/
/* HISTORY:                                                                  */
/* DATE       Defect No  Comments/updated by                                 */
/* ---------  ---------  --------------------------------------------------- */
/* 23APR2013             GA                                                  */
/*****************************************************************************/

%macro mashmacr;

%macro init_libraries;
   filename tmp temp lrecl=32767;
   data _null_;
      file tmp;
      set INVOKE_LIBRARIES(firstobs=2);
      put libstmt ";";
   run;
   %include tmp;
   filename tmp clear;
%mend;

%macro catch_all_err(mname);
    %if %length(&MAError)>0 %then %do;
        %return;
    %end;
    %else %do;
        %if (&SYSCC. ge 2 and &SYSCC. ne 4) %then %do;
            %let MAError = 000;
            %let MAMsg = Error in Stored Process[&mname]. Check the log for details.;
            %return;
        %end;
    %end;
%mend;

/* Dump the CODE column from tablename into fname */
%macro dump_code(tablename, fname, mod=);
    data _null_;
        file &fname &mod;
        set &tablename;
        put code;
    run;
%mend;

%macro column_exists(DATASET, COLNAME, OUTVAR);
   %local dsid colIndex rc;
   %let dsid=%sysfunc(open(&DATASET.));
   %let colIndex=%sysfunc(varnum(&dsid,&COLNAME.));
   %let rc=%sysfunc(close(&dsid.));
   %let &OUTVAR=%sysfunc(ifn(%eval(&colIndex. gt 0),1,0));
%mend;

/* Name an index for subject_keys.
 * The macro variable named by IDX_DATA_STEP_VAR will contain the data step
 * statement for the right hand side of the index= statement.
 * The macro variable named by IDX_OUT_VAR will contain the index name.
 */

%macro generate_index_stmt(COLUMN_NAMES, IDX_DATA_STEP_VAR, IDX_OUT_VAR, MPK_IDX_NAME=_key);
%let COLUMN_NAMES=%kcmpres(&COLUMN_NAMES);
%local isMultipart;

%let isMultipart=0;
%is_multipart(&COLUMN_NAMES.,isMultipart);

/* Single part keys *must* be named the same as the column name.
 */
%if &isMultipart. %then %do;
   %let &IDX_DATA_STEP_VAR=&MPK_IDX_NAME=(&COLUMN_NAMES.);
   %let &IDX_OUT_VAR=&MPK_IDX_NAME;
%end; %else %do;
   %let &IDX_DATA_STEP_VAR=&COLUMN_NAMES.;
   %let &IDX_OUT_VAR=&COLUMN_NAMES.;
%end;
%mend;

%macro is_multipart(COLUMN_NAMES, OUT_VAR);
%let &OUT_VAR=%eval( %kscan(&COLUMN_NAMES.,1) ne %kscan(&COLUMN_NAMES.,-1));
%mend;

%mend;
