/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/************************************************************************************/
/* NAME: mastspub.sas                                                               */
/* VERSION: 5.4                                                                   */
/*                                                                                  */
/* DESCRIPTION: Macro to update or insert Campaign/Communication Status table.      */
/* Parameters:                                                                      */
/*            camp_st - Physical table name for Campaign_Status                     */
/*            comm_st - Physical table name for Communication_Status                */
/*             cct    - Physical table name for Change_Type                         */
/*             ccgt   - Physical table name for Control_Group_Type                  */
/* PRODUCT: MA                                                                      */
/* USAGE: %mastspub                                                                 */
/*                                                                                  */
/************************************************************************************/
/*****************************************************************************************/
/*                                                                                       */
/* History:                                                                              */
/*   27APR2007 - Revised first version of the macro.                                     */
/*                                                                                       */
/*                                                                                       */
/* DATE       Defect No  Comments                                                        */
/* 30Dec2008  S0547352   The replace statement is removed from the unwanted places       */
/*                                                                                       */
/* 25May2009  S0559332   The STOP statement is removed from the DATA MODIFY block        */
/*                       where only REPLACE or OUTPUT is used                            */
/*                                                                                       */
/* 18Jun2009  S0583163   Reset the SYSCC value to 100000 in case of error                */
/* 23Aug2013  S0992864   Add dswriteopts to data modify                                  */
/*****************************************************************************************/

%macro mastspub(camp_st=, comm_st=, cct=, ccgt=);

%put MASTSPUB Start Time - %sysfunc(datetime(), datetime24.2);

/* CI_CAMPAIGN_STATUS: Only Inserts */
data &libnm..&camp_st.;
    modify &libnm..&camp_st. (&dswriteopts.) work.&camp_st.;
    by CAMPAIGN_STATUS_CD;
    select(_IORC_);
        when (%SYSRC(_DSENMR))
        do;
            _error_= 0;
            output;
        end;
        otherwise;
    end;
run;

/* Exit the process if there is any error */
%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC ^= 0 %then
    %do;
        %let SYSCC=100000;
        %let MAError = 835;
        %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
%end;
%else %if &SYSERR. > 4  %then
%do;
    %let MAError =835;
    %let MAMsg =Macro[&sysmacroname.]. Data Modify Stmt error while attempting to update CAMPAIGN_STATUS table . Check the log for details;
%end;

/* CI_COMMUNICATION_STATUS: Only Inserts */
data &libnm..&comm_st.;
    modify &libnm..&comm_st. (&dswriteopts.) work.&comm_st.;
    by COMMUNICATION_STATUS_CD;
    select(_IORC_);
        when (%SYSRC(_DSENMR))
        do;
            _error_= 0;
            output;
        end;
        otherwise;
    end;
run;

/* Exit the process if there is any error */
%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC ^= 0 %then
    %do;
        %let SYSCC=100000;
        %let MAError = 836;
        %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
%end;
%else %if &SYSERR. > 4  %then
%do;
    %let MAError =836;
    %let MAMsg =Macro[&sysmacroname.]. Data Modify Stmt error while attempting to update COMMUNICATION_STATUS table . Check the log for details;
%end;


/* CI_CHANGE_TYPE: Only Inserts */
data &libnm..&cct.;
    modify &libnm..&cct. (&dswriteopts.) work.&cct.;
    by CHANGE_TYPE_CD;
    select(_IORC_);
        when (%SYSRC(_DSENMR))
        do;
            _error_=0;
            output;
        end;
        otherwise;
    end;
run;

/* Exit the process if any error */
%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC ^= 0 %then
    %do;
        %let SYSCC=100000;
        %let MAError = 837;
        %let MAMsg = Macro[&sysmacroname.]. SYSDBRC=[&SYSDBRC] and SYSDBMSG=[&SYSDBMSG];
    %end;
%end;
%else %if &SYSERR. > 4  %then
%do;
    %let MAError =837;
    %let MAMsg =Macro[&sysmacroname.]. Data Modify Stmt error while attempting to update Package table . Check the log for details;
%end;

/* CI_CONTROL_GROUP_TYPE: Only Inserts */
data &libnm..&ccgt.;
    modify &libnm..&ccgt. (&dswriteopts.) work.&ccgt.;
    by CONTROL_GROUP_TYPE_CD;
    select(_IORC_);
        when (%SYSRC(_DSENMR))
        do;
            _error_= 0;
            output;
        end;
        otherwise;
    end;
run;

/* Exit the process if any error */
%if %symexist(SYSDBRC) %then
%do;
    %if &SYSDBRC ^= 0 %then
    %do;
        %let SYSCC=100000;
        %let MAError = 849;
        %let MAMsg = Macro[&sysmacroname.]. SYSDBRC[&SYSDBRC] and SYSDBMSG[&SYSDBMSG];
    %end;
%end;
%else %if &SYSERR. > 4  %then
%do;
    %let MAError =849;
    %let MAMsg =Macro[&sysmacroname.]. Data Modify Stmt error while attempting to update Control_Group_Type table . Check the log for details;
%end;

%put MASTSPUB end Time - %sysfunc(datetime(), datetime24.2);
%mend;
