/*****************************************************************/
/* Copyright (c) 2007 - 2012 by SAS Institute Inc., Cary, NC,    */
/*                    USA.                                       */
/*             All Rights Reserved.                              */
/*  This software is protected by copyright laws and             */
/*               international treaties                          */
/*                                                               */
/*              U.S. Government Restricted Rights                */
/* Use, duplication, or disclosure of this software and related  */
/*  documentation by the U.S. government is subject to the       */
/*   Agreement with SAS Institute and the restrictions set forth */
/*  in FAR 52.227-19, Commercial Computer Software -             */
/*              Restricted Rights ( June 1987 )                  */
/*                                                               */
/*****************************************************************/
/* NAME: matblxst                                                */
/* VERSION: F0MA541                                              */
/* DESCRIPTION: Test if tables, directories, or libraries exist  */
/*                                                               */
/* PRODUCT: MA                                                   */
/* USAGE: %matblxst( LibnameTable, TableNamesTable,              */
/*                   DirectoriesTable, LibrariesTable,           */
/*                   outputPrefix );                             */
/* LibnameTable - table that contains libnames that need to be   */
/*                instantiated for the table test                */
/* TableNamesTable - table that contains the fully qualified     */
/*                   table names to test                         */
/* DirectoriesTable - table that contains directories to test    */
/* LibrariesTable - table that contains export librefs to test   */
/* outputPrefix - place to put the output files, which will      */
/*                contain a line for each object that failed the */
/*                respective test                                */
/* badTables.txt - fully qualified missing table names           */
/* badDirectories.txt - missing directories                      */
/* badLibraries.txt - missing libraries                          */
/*                                                               */
/*****************************************************************/
%macro matblxst(LibnameTable, TableNamesTable, DirectoriesTable, LibrariesTable, outputPrefix);

   %global MADebug;

   %if &MADebug = Y
   %then %do;
       %put LibnameTable=&LibnameTable;
       %put TableNamesTable=&TableNamesTable;
       %put DirectoriesTable=&DirectoriesTable;
       %put LibrariesTable=&LibrariesTable;
       %put outputPrefix=&outputPrefix;
   %end;

   /* step one, assign any passed in libnames */
   filename libs temp lrecl=32767;
   data _null_;
       file libs;
       set &LibnameTable(firstobs=2);
       put libstmt;
   run;
   %include libs;
   filename libs clear;
   %if (&SYSCC ne 0 and &SYSCC ne 4) %then %return;


   filename tout "&outputPrefix.badTables.txt";
   data _null_;
       file tout;
       set &TableNamesTable(firstobs=2);
       if (exist(name))=0 then put name;
   run;
   filename tout clear;
   %if (&SYSCC ne 0 and &SYSCC ne 4) %then %return;

   /* step three, verify any passed in directories, dumping invalid names to the output file */

   filename tout "&outputPrefix.badDirectories.txt";
   data _null_;
      file tout;
      set &DirectoriesTable(firstobs=2);
      if (fileexist(name))=0 then put name;
   run;
   filename tout clear;
   %if (&SYSCC ne 0 and &SYSCC ne 4) %then %return;

   /* step four, verify any passed in export libraries, assigning as needed */
   /* note:passing the check means the library does not exist */
   filename tout "&outputPrefix.badLibraries.txt";
   filename tempinc temp lrecl=32767;
   data _null_;
      length c $ 32767;
      file tempinc;
      set &LibrariesTable(firstobs=2);
      c='%'||'mausrexp('||trim(name)||',Execute);'; put c;
      c='data _null_; file tout mod;'; put c;
      c='if libref("'||trim(name)||'") then put "'||trim(name)||'";'; put c;
      c='run;';put c;
      c='%'||'mausrexp('||trim(name)||',Clear);';put c;
   run;
   %include tempinc;
   filename tempinc clear;
   filename tout clear;
   %if (&SYSCC ne 0 and &SYSCC ne 4) %then %return;
%mend;
