/*****************************************************************/
/*    Copyright (c) 2012 by SAS Institute Inc., Cary, NC,USA.    */
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */ 
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */ 
/*  Agreement with SAS Institute and the restrictions set forth  */ 
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/ 
/* NAME: ma_metadata_histogram                                   */ 
/* VERSION: 6.1                                                  */ 
/* DESCRIPTION:                                                  */ 
/*                                                               */ 
/* PRODUCT: MA                                                   */ 
/* USAGE: %ma_metadata_histogram( );                             */ 
/*                                                               */ 
/*****************************************************************/ 
/*****************************************************************************/
/* PRISM VERSION: $Revision: 1.1.6.1 $ ;                              
/* Revision Date: $Date: 2013/07/25 13:07:58 $  ;                                  
/*****************************************************************************/
/*****************************************************************************/
/* HISTORY:                                                                  */
/* DATE       Defect No  Comments/updated by                                 */
/* ---------  ---------  --------------------------------------------------- */
/* 23APR2013             GA                                                  */
/*****************************************************************************/


/* this keeps the main.html out off the result package */
filename temp temp;
%let _ODSOPTIONS=%str(body=temp);

*ProcessBody;

/* Reset SYSCC to truly reflect the SAS condition code from this stored process. */
%let syscc = 0;

%stpbegin;
%maspinit(xmlstream=mameta);
%mashmacr;
/* TODO: rewrite this for combination nominal and date histograms */
%macro process_one(first, varid, intable, outtable, rule, type, midpoints);
   proc sort data=&intable; by group1; run;
   %catch_all_err(&_PROGRAM);
   %if %length(&MAError)>0 %then %return;
   
   /* create the midpoints table for merging */
   data tmp_table;
      format midpoint BEST20.;
      format group1 BEST20.;
      format count BEST20.;
      
      count=0;
   
      %let i = 1;
      %let mpoint = %scan(&midpoints, &i,%str( ));
      %do %while( %length(&mpoint)>0 );
         midpoint=&mpoint;
         group1=&i;
         output;
         
         %let i=%eval(&i+1);
         %let mpoint = %scan(&midpoints, &i,%str( ));
      %end;
   run;
   
   data tmp_table(keep=variable rule
                  %if %length(&type) > 0 %then type;
                  midpoint count
                   );
      format variable $200.; /* TODO: make this passed/configured */
      format rule $8.;
      %if %length(&type) > 0 %then %do;
         format type $8.;
      %end;
      format midpoint BEST20.;
      format count BEST20.;
      
      merge tmp_table &intable(rename=(count1=count)); by group1;
      
      variable="&varid";
      rule="&rule";
      %if %length(&type) > 0 %then %do;
         type="&type";
      %end;
   run;
      
   %if &first = Y %then %do;
      data &outtable(compress=binary);
         set tmp_table;
      run;
   %end; %else %do;
      proc append base=&outtable data=tmp_table; run;
   %end;
%mend;

%macro process_tables;
   filename tmp temp lrecl=32767;
   data _null_;
      format first $1.;
      file tmp;
      set METADATA_TABLES(firstobs=2); by output_table;
      if first.output_table then first='Y';
      else first='N';

      put "%" "process_one(" first ", %" "bquote(" varid ")," input_table ", &TMPLIBREF.." output_table "," rule "," type "," midpoints ");";
      /* TODO: how to process this right
      put "%" "catch_all_err(&_PROGRAM);";
      put "%if %length(\&MAError)>0 %then %return;";
      */

   run;
   %include tmp;


   filename tmp clear;
%mend;

%macro runmacros;
   %init_libraries;
   %catch_all_err(&_PROGRAM);
   %if %length(&MAError)>0 %then %return;

   %process_tables;
   %catch_all_err(&_PROGRAM);
   %if %length(&MAError)>0 %then %return;
%mend;

%runmacros;

/* Write out execution status to the status file. */
%mastatus(&_stpwork.status.txt);
%stpend;

libname _all_ clear;
