/*****************************************************************/
/*    Copyright (c) 2012 by SAS Institute Inc., Cary, NC,USA.    */
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */ 
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */ 
/*  Agreement with SAS Institute and the restrictions set forth  */ 
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/ 
/* NAME: ma_metadata_nominal                                     */ 
/* VERSION: 6.1                                                  */ 
/* DESCRIPTION:                                                  */ 
/*                                                               */ 
/* PRODUCT: MA                                                   */ 
/* USAGE: %ma_metadata_nominal(  );                              */ 
/*                                                               */ 
/*****************************************************************/ 
/*****************************************************************************/
/* PRISM VERSION: $Revision: 1.1.6.1 $ ;                              
/* Revision Date: $Date: 2013/07/25 13:07:58 $  ;                                  
/*****************************************************************************/
/*****************************************************************************/
/* HISTORY:                                                                  */
/* DATE       Defect No  Comments/updated by                                 */
/* ---------  ---------  --------------------------------------------------- */
/* 23APR2013             GA                                                  */
/*****************************************************************************/


/* this keeps the main.html out off the result package */
filename temp temp;
%let _ODSOPTIONS=%str(body=temp);

*ProcessBody;

/* Reset SYSCC to truly reflect the SAS condition code from this stored process. */
%let syscc = 0;

%stpbegin;
%maspinit(xmlstream=mameta);
%mashmacr;

options source source2 mprint;

%global MAError MAMsg;

%macro create_tmpview( varid, intablename, colname, coltype, fmt, type );
   %put type=&type;
   data tmpview / view=tmpview;
      length variable $ 200; *TODO: generate this length dynamically;
      length rule $ 8;
      length formattedValue $ 200;
      format formattedValue $200.;
      length count 8;
      drop tmp;
      
      %let putfn=;
      %if (%upcase(&coltype) eq C) %then %do;
         format &colname $200.;
         %if %length(&fmt) = 0 %then %let fmt=%str($200.);
         %let putfn=putc;
      %end; %else %do;
         format &colname BEST12.;
         %if %length(&fmt) = 0 %then %let fmt=%str(BEST.);
         %let putfn=putn;
      %end;
           
      %if (%upcase(&type) eq VALUES) %then %do;
         count=-1;
         set &intablename(rename=(unformatted=&colname));
         informat &colname;
         formattedValue=left(put(&colname,&fmt));
      %end; %else %do;
         %let fmtcol=&colname;
         set &intablename(rename=(
            unformatted=&colname
            count1=count
            formatted=tmp
         ));
         informat &colname;
         informat tmp;
         if missing(tmp) then formattedValue=put(&colname,&fmt.);
         else formattedValue=left(&putfn.(&colname,"&fmt"));
      %end;
      
      label &colname=' ';
      label formattedValue=' ';
      label count=' ';
      
      variable="&varid";
      rule='Any';
   run;
%mend;

%macro create_metatable;
   filename tmp temp lrecl=32767;
   data _null_;
      file tmp;
      set METADATA_TABLES(firstobs=2); by output_table;
      
      put "%" "create_tmpview(%" "bquote(" id ")," input_table "," output_colname "," type "," output_format "," counts ");";
      if first.output_table then
         put "data &TMPLIBREF.." output_table "(compress=binary); set tmpview; run;";
      else put "proc append base=&TMPLIBREF.." output_table " data=tmpview; run;";
   run;
   %include tmp;
   filename tmp clear;
%mend;

%macro run_macros;
   %catch_all_err(&_PROGRAM);
   %if %length(&MAError)>0 %then %return;
   
   %init_libraries;
   %catch_all_err(&_PROGRAM);
   %if %length(&MAError)>0 %then %return;
   
   %create_metatable;   
   %catch_all_err(&_PROGRAM);
   %if %length(&MAError)>0 %then %return;
%mend;

%run_macros;

/* Write out execution status to the status file. */
%mastatus(&_stpwork.status.txt);
%stpend;

libname _all_ clear;
