/*****************************************************************/
/*    Copyright (c) 2012 by SAS Institute Inc., Cary, NC,USA.    */
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */ 
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */ 
/*  Agreement with SAS Institute and the restrictions set forth  */ 
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/ 
/* NAME: ma_metadata_univariate                                  */ 
/* VERSION: 6.1                                                  */ 
/* DESCRIPTION:                                                  */ 
/*                                                               */ 
/* PRODUCT: MA                                                   */ 
/* USAGE: %ma_metadata_univariate( );                            */ 
/*                                                               */ 
/*****************************************************************/ 
/*****************************************************************************/
/* PRISM VERSION: $Revision: 1.1.6.1 $ ;                              
/* Revision Date: $Date: 2013/07/25 13:07:58 $  ;                                  
/*****************************************************************************/
/*****************************************************************************/
/* HISTORY:                                                                  */
/* DATE       Defect No  Comments/updated by                                 */
/* ---------  ---------  --------------------------------------------------- */
/* 23APR2013             GA                                                  */
/*****************************************************************************/


/* this keeps the main.html out off the result package */
filename temp temp;
%let _ODSOPTIONS=%str(body=temp);

*ProcessBody;

/* Reset SYSCC to truly reflect the SAS condition code from this stored process. */
%let syscc = 0;

%stpbegin;
%maspinit(xmlstream=mameta);
%mashmacr;

%macro process_one(first, intable, outtable, name, varid, subjectID);
   data _null_; *tmp_table(keep=variable min max mean);
      file univar mod;
      format variable $200.;
      format min BEST20.;
      format max BEST20.;
      format mean BEST20.;
	  format &name._min BEST20.;
	  format &name._max BEST20.;
	  format &name._mean BEST20.;
      
      set &intable;
      min=&name._min;
      max=&name._max;
	  if missing(&name._mean) and not(missing(&name._min)) and not(missing(&name._max)) then do;
	     mean=(&name._min/2) + (&name._max/2);
	  end; else mean=&name._mean;

      variable="&varid";
      *output tmp_table;
      
      /* send data to the record file */
      put "&subjectID "; /* the extra space ensures that "" is not treated as a subject of 'quote' */
      put "&varid";
      put min;
      put max;
      put mean;
      stop;
   run;
%mend;

%macro process_tables;
   filename tmp temp lrecl=32767;
   data _null_;
      format first $1.;
      file tmp;
      set METADATA_TABLES(firstobs=2); by output_table;
      if first.output_table then first='Y';
      else first='N';

      put "%" "process_one(" first "," input_table ", &TMPLIBREF.." output_table "," column_name ",%" "bquote(" varid "));";
   run;
   %include tmp;
   filename tmp clear;
%mend;

%macro runmacros;
   %init_libraries;
   %catch_all_err(&_PROGRAM);
   %if %length(&MAError)>0 %then %return;

   %process_tables;
   %catch_all_err(&_PROGRAM);
   %if %length(&MAError)>0 %then %return;
%mend;

filename univar "&_stpwork.univariate.txt" lrecl=32767;
%runmacros;

data _null_;
   file univar mod;
   put "__END_OF_SEQUENCE__";
run;

filename univar clear;

/* Write out execution status to the status file. */
%mastatus(&_stpwork.status.txt);
%stpend;

libname _all_ clear;
