/*****************************************************************/
/*    Copyright (c) 2012 by SAS Institute Inc., Cary, NC,USA.    */
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */ 
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */ 
/*  Agreement with SAS Institute and the restrictions set forth  */ 
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/ 
/* NAME: ma_metadata_update                                      */ 
/* VERSION: 6.1                                                  */ 
/* DESCRIPTION:                                                  */ 
/*                                                               */ 
/* PRODUCT: MA                                                   */ 
/* USAGE: %ma_metadata_update(  );                               */ 
/* MODE = UPDATE, (anything else)                                */
/*                                                               */ 
/*****************************************************************/ 
/*****************************************************************************/
/* PRISM VERSION: $Revision: 1.1.6.1 $ ;                              
/* Revision Date: $Date: 2013/07/25 13:07:58 $  ;                                  
/*****************************************************************************/
/*****************************************************************************/
/* HISTORY:                                                                  */
/* DATE       Defect No  Comments/updated by                                 */
/* ---------  ---------  --------------------------------------------------- */
/* 23APR2013             GA                                                  */
/*****************************************************************************/


/* this keeps the main.html out off the result package */
filename temp temp;
%let _ODSOPTIONS=%str(body=temp);

*ProcessBody;

/* Reset SYSCC to truly reflect the SAS condition code from this stored process. */
%let syscc = 0;

%stpbegin;

%global MADebug;
%let MADebug=Y;

%maspinit(xmlstream=mameta);
%mashmacr;

%macro update_table( sourcetable, desttable );
   proc sql noprint;
      create table keytable(index=(variable)) as
         select distinct variable from &sourcetable;
         
      delete from &desttable where variable in ( select variable from keytable );
   quit;
   
   proc append base=&desttable data=&sourcetable; run;
%mend;

%macro process_one(first, last, intable, outtable);
   %if &first = Y %then %do;
      data tmptable(compress=binary);
         set &intable;
      run;
   %end; %else %do;
      proc append base=tmptable data=&intable; run;
   %end;
   
   %if &last = Y %then %do;
      %if %upcase(&MODE) = UPDATE and %sysfunc(exist(&outtable)) %then %do;
         %update_table( tmptable, &outtable );
      %end; %else %do;
         data &outtable(compress=binary);
            set tmptable;
         run;
      %end;
   %end;
%mend;

%macro process_views;
   filename tmp temp lrecl=32767;
   data _null_;
      format last $1.;
      format first $1.;
      
      file tmp;
      set METADATA_TABLES(firstobs=2); by output_table;
      
      if first.output_table then first='Y';
      else first='N';
      
      if last.output_table then last='Y';
      else last='N';
      
      /* Hard coded MAMeta here - output of this STP always go to MAMeta */
      put "%" "process_one(" first "," last "," input_table ", MAMeta." output_table ");";      
   run;
   %include tmp;
   filename tmp clear;
%mend;

%macro runmacros;
   %init_libraries;
   %catch_all_err(&_PROGRAM);
   %if %length(&MAError)>0 %then %return;

   %process_views;
   %catch_all_err(&_PROGRAM);
   %if %length(&MAError)>0 %then %return;
%mend;

%runmacros;

/* Write out execution status to the status file. */
%mastatus(&_stpwork.status.txt);
%stpend;

libname _all_ clear;
