/*****************************************************************/
/* Copyright (c) 2007 - 2011 by SAS Institute Inc., Cary, NC,USA.*/
/*                     All Rights Reserved.                      */
/*       This software is protected by copyright laws and        */
/*                    international treaties                     */
/*                                                               */
/*               U.S. Government Restricted Rights               */
/* Use, duplication, or disclosure of this software and related  */
/*    documentation by the U.S. government is subject to the     */
/*  Agreement with SAS Institute and the restrictions set forth  */
/*       in FAR 52.227-19, Commercial Computer Software -        */
/*                Restricted Rights ( June 1987 )                */
/*                                                               */
/*****************************************************************/
/*******************************************************************************************/
/* NAME: ma_publish.sas                                                                    */
/* VERSION: 5.4                                                                            */
/* DESCRIPTION:  This stored process is used to publish the report model                   */
/*               ReportLib - Report Database libname statement.                            */
/* Parameters:                                                                             */
/*            reportlib - libname statement for publish tables                             */
/*            campaignid - A new value CampaignGroup is expected in this variable to       */
/*                         publish only the Campaign Group tables                          */
/*            purpose - CAMPAIGN/CAMPAIGN_DELETION/CAMPAIGN_GROUP/COMMUNICATION_EXPORT/    */
/*                      TREATMENT_DELETE                                                   */
/*                      CAMPAIGN: Publish full Campaign                                    */
/*                      CAMPAIGN_DELETION: Update Campaign Delete flag and DTTM columns    */
/*                      CAMPAIGN_GROUP: Publish only Camapign Groups                       */  
/*                      COMMUNICATION_EXPORT: Update Export/RUN/Last Modified DTTM in comm */
/*                      TREATMENT_DELETION: Update Treatment Delete Flag                   */
/*                                                                                         */
/*            bulkload - Bulkload options from the BC metadata                             */
/*            dbmstemp - Y/N for Database global temporary schema option                   */
/*            dbtempschema - DB temp schema for table upload                               */
/*                                                                                         */
/* PRODUCT: MA                                                                             */
/* USAGE: ma_publish                                                                       */
/*                                                                                         */
/*******************************************************************************************/
/*******************************************************************************************/
/*                                                                                         */
/* History:                                                                                */
/* DATE       Defect No  Comments                                                          */
/* 01Sep2008   S0506508   Added campaignid part of global var for Campaign Group publish   */
/*                                                                                         */
/* 14Oct2008   S0504678   Added PURPOSE variable which will inform the type of process     */
/*                                                                                         */
/* 09Jun2009   S0492899   Added cleanpublish var to enable the data model refresh process  */
/*                                                                                         */
/* 02Feb2010  S0616965   Clear any open library connection                                 */
/*                                                                                         */
/* 16Sep2010  S0702561   Special handling for Aster clean publish                          */
/*                                                                                         */
/* 23Nov2010  S0719271   use option missing='.' for publish process                        */
/*                                                                                         */
/* 31Jan2012  S0827991   Two new param Publishtype and ExecuteCommSk added to fix the      */
/*                       commuication occurrence and treatment association                 */
/*******************************************************************************************/

/* Initialize the variable which could have null values */
/* S0663045 - Added bulkload/dbmstemp/dbtempschema etc variables for Aster special processing */
%global reportlib campaignid purpose cleanpublish dbdsoptions dbmstemp dbtempschema dbtmplib unqintbl; 

/* S0827991 - publishtype can have one of these values INCREMENTAL/FULL/PREEXEC and 
   executecommsk will have either null/<valid communication_sk> */
%global publishtype executecommsk;

/* This keeps the main.html out off the result package */
filename temp temp;
%let _ODSOPTIONS=%str(body=temp);

*ProcessBody;

/* Reset syscc to truely reflect any errors happen in this stored process */
%let SYSCC=0;

%stpbegin;

/* Setting up the system options based on the MADEBUG global macro variable */
%maspinit;

%*S0719271 - set option missing='.' to enable dynamic source code create correct sql statements ;
option missing='.';

/* Mask the single quotes in the dbdsoptions */
%syscall stpsrv_unquote2(dbdsoptions);

/* Publish Wrapper macro to create input table from input stream and call child macros in sequence */
%mapubwp(reportlib=&reportlib.);

/* Write out execution status to a file */
%mastatus( &_stpwork.status.txt );

/* De-assign the publish library reference */
%maclear( &reportlib. );

%stpend;

/* Clear the fileref. */
filename temp clear;
